@@ -35,6 +35,12 @@ module Sequel
 
       # Error class for exceptions raised by the connection.
       class Error < StandardError
+        attr_reader :sqlstate
+
+        def initialize(message, sqlstate)
+          @sqlstate = sqlstate
+          super(message)
+        end
       end
 
       # Create the underlying IBM_DB connection.
@@ -69,11 +75,16 @@ module Sequel
         IBM_DB.getErrormsg(@conn, IBM_DB::DB_CONN)
       end
 
+      # Return the related error message for the connection.
+      def error_sqlstate
+        IBM_DB.getErrorstate(@conn, IBM_DB::DB_CONN)
+      end
+
       # Execute the given SQL on the database, and return a Statement instance
       # holding the results.
       def execute(sql)
         stmt = IBM_DB.exec(@conn, sql)
-        raise Error, error_msg unless stmt
+        raise Error.new(error_msg, error_sqlstate) unless stmt
         Statement.new(stmt)
       end
 
@@ -83,7 +94,7 @@ module Sequel
         stmt = @prepared_statements[ps_name].last
         res = stmt.execute(*values)
         unless res
-          raise Error, "Error executing statement #{ps_name}: #{error_msg}"
+          raise Error.new("Error executing statement #{ps_name}: #{error_msg}", error_sqlstate)
         end
         stmt
       end
@@ -98,7 +109,7 @@ module Sequel
         else
           err = error_msg
           err = "Error preparing #{ps_name} with SQL: #{sql}" if error_msg.nil? || error_msg.empty?
-          raise Error, err
+          raise Error.new(err, error_sqlstate)
         end
       end
 
@@ -146,8 +157,14 @@ module Sequel
         IBM_DB.field_precision(@stmt, key)
       end
 
-      # Free the memory related to this result set.
+      # Free the memory related to this statement.
       def free
+        IBM_DB.free_stmt(@stmt)
+      end
+
+      # Free the memory related to this result set, only useful for prepared
+      # statements which have a different result set on every call.
+      def free_result
         IBM_DB.free_result(@stmt)
       end
 
@@ -165,12 +182,6 @@ module Sequel
       # Hash of connection procs for converting
       attr_reader :conversion_procs
 
-      def initialize(opts={})
-        super
-        @conversion_procs = DB2_TYPES.dup
-        @conversion_procs[:timestamp] = method(:to_application_timestamp)
-      end
-
       # REORG the related table whenever it is altered.  This is not always
       # required, but it is necessary for compatibilty with other Sequel
       # code in many cases.
@@ -199,7 +210,7 @@ module Sequel
       end
 
       # Execute the given SQL on the database.
-      def execute(sql, opts={}, &block)
+      def execute(sql, opts=OPTS, &block)
         if sql.is_a?(Symbol)
           execute_prepared_statement(sql, opts, &block)
         else
@@ -211,14 +222,14 @@ module Sequel
 
       # Execute the given SQL on the database, returning the last inserted
       # identity value.
-      def execute_insert(sql, opts={})
+      def execute_insert(sql, opts=OPTS)
         synchronize(opts[:server]) do |c|
           if sql.is_a?(Symbol)
             execute_prepared_statement(sql, opts)
           else
             _execute(c, sql, opts)
           end
-          _execute(c, "SELECT IDENTITY_VAL_LOCAL() FROM SYSIBM.SYSDUMMY1", opts){|stmt| i = stmt.fetch_array.first.to_i; stmt.free; i}
+          _execute(c, "SELECT IDENTITY_VAL_LOCAL() FROM SYSIBM.SYSDUMMY1", opts){|stmt| i = stmt.fetch_array.first.to_i; i}
         end
       rescue Connection::Error => e
         raise_error(e)
@@ -240,28 +251,19 @@ module Sequel
             log_sql << sql
             log_sql << ")"
           end
-          stmt = log_yield(log_sql, args){conn.execute_prepared(ps_name, *args)}
-          if block_given?
-            begin
+          begin
+            stmt = log_yield(log_sql, args){conn.execute_prepared(ps_name, *args)}
+            if block_given?
               yield(stmt)
-            ensure
-              stmt.free
+            else  
+              stmt.affected
             end
-          else  
-            stmt.affected
+          ensure
+            stmt.free_result if stmt
           end
         end
       end
 
-      # Convert smallint type to boolean if convert_smallint_to_bool is true
-      def schema_column_type(db_type)
-        if Sequel::IBMDB.convert_smallint_to_bool && db_type =~ /smallint/i 
-          :boolean
-        else
-          super
-        end
-      end
-
       # On DB2, a table might need to be REORGed if you are testing existence
       # of it.  This REORGs automatically if the database raises a specific
       # error that indicates it should be REORGed.
@@ -287,31 +289,38 @@ module Sequel
       def _execute(conn, sql, opts)
         stmt = log_yield(sql){conn.execute(sql)}
         if block_given?
-          begin
-            yield(stmt)
-          ensure
-            stmt.free
-          end
+          yield(stmt)
         else  
           stmt.affected
         end
+      ensure
+        stmt.free if stmt
+      end
+
+      def adapter_initialize
+        @conversion_procs = DB2_TYPES.dup
+        @conversion_procs[:timestamp] = method(:to_application_timestamp)
       end
 
       # IBM_DB uses an autocommit setting instead of sending SQL queries.
       # So starting a transaction just turns autocommit off.
-      def begin_transaction(conn, opts={})
+      def begin_transaction(conn, opts=OPTS)
         log_yield(TRANSACTION_BEGIN){conn.autocommit = false}
+        set_transaction_isolation(conn, opts)
       end
 
       # This commits transaction in progress on the
       # connection and sets autocommit back on.
-      def commit_transaction(conn, opts={})
+      def commit_transaction(conn, opts=OPTS)
         log_yield(TRANSACTION_COMMIT){conn.commit}
       end
     
-      # Close the given connection.
-      def disconnect_connection(conn)
-        conn.close
+      def database_error_classes
+        [Connection::Error]
+      end
+
+      def database_exception_sqlstate(exception, opts)
+        exception.sqlstate
       end
 
       # Don't convert smallint to boolean for the metadata
@@ -346,9 +355,18 @@ module Sequel
 
       # This rolls back the transaction in progress on the
       # connection and sets autocommit back on.
-      def rollback_transaction(conn, opts={})
+      def rollback_transaction(conn, opts=OPTS)
         log_yield(TRANSACTION_ROLLBACK){conn.rollback}
       end
+
+      # Convert smallint type to boolean if convert_smallint_to_bool is true
+      def schema_column_type(db_type)
+        if Sequel::IBMDB.convert_smallint_to_bool && db_type =~ /smallint/i 
+          :boolean
+        else
+          super
+        end
+      end
     end
     
     class Dataset < Sequel::Dataset
@@ -374,17 +392,17 @@ module Sequel
         
         private
         # Execute the prepared statement with arguments instead of the given SQL.
-        def execute(sql, opts={}, &block)
+        def execute(sql, opts=OPTS, &block)
           super(prepared_statement_name, {:arguments=>bind_arguments}.merge(opts), &block)
         end
         
         # Execute the prepared statment with arguments instead of the given SQL.
-        def execute_dui(sql, opts={}, &block)
+        def execute_dui(sql, opts=OPTS, &block)
           super(prepared_statement_name, {:arguments=>bind_arguments}.merge(opts), &block)
         end
 
         # Execute the prepared statement with arguments instead of the given SQL.
-        def execute_insert(sql, opts={}, &block)
+        def execute_insert(sql, opts=OPTS, &block)
           super(prepared_statement_name, {:arguments=>bind_arguments}.merge(opts), &block)
         end
 
@@ -409,7 +427,6 @@ module Sequel
       # Fetch the rows from the database and yield plain hashes.
       def fetch_rows(sql)
         execute(sql) do |stmt|
-          offset = @opts[:offset]
           columns = []
           convert = convert_smallint_to_bool
           cps = db.conversion_procs
@@ -422,7 +439,6 @@ module Sequel
             columns << [key, cps[type]]
           end
           cols = columns.map{|c| c.at(0)}
-          cols.delete(row_number_column) if offset
           @columns = cols
 
           while res = stmt.fetch_array
@@ -430,7 +446,6 @@ module Sequel
             res.zip(columns).each do |v, (k, pr)|
               row[k] = ((pr ? pr.call(v) : v) if v)
             end
-            row.delete(row_number_column) if offset
             yield row
           end
         end
