@@ -30,98 +30,119 @@ module Sequel
     # Contains procs keyed on sub adapter type that extend the
     # given database object so it supports the correct database type.
     DATABASE_SETUP = {:postgresql=>proc do |db|
-        Sequel.ts_require 'adapters/jdbc/postgresql'
+        JDBC.load_gem(:Postgres)
+        org.postgresql.Driver
+        Sequel.require 'adapters/jdbc/postgresql'
         db.extend(Sequel::JDBC::Postgres::DatabaseMethods)
         db.dataset_class = Sequel::JDBC::Postgres::Dataset
-        JDBC.load_gem('postgres')
         org.postgresql.Driver
       end,
       :mysql=>proc do |db|
-        Sequel.ts_require 'adapters/jdbc/mysql'
+        JDBC.load_gem(:MySQL)
+        com.mysql.jdbc.Driver
+        Sequel.require 'adapters/jdbc/mysql'
         db.extend(Sequel::JDBC::MySQL::DatabaseMethods)
         db.extend_datasets Sequel::MySQL::DatasetMethods
-        JDBC.load_gem('mysql')
         com.mysql.jdbc.Driver
       end,
       :sqlite=>proc do |db|
-        Sequel.ts_require 'adapters/jdbc/sqlite'
+        JDBC.load_gem(:SQLite3)
+        org.sqlite.JDBC
+        Sequel.require 'adapters/jdbc/sqlite'
         db.extend(Sequel::JDBC::SQLite::DatabaseMethods)
         db.extend_datasets Sequel::SQLite::DatasetMethods
         db.set_integer_booleans
-        JDBC.load_gem('sqlite3')
         org.sqlite.JDBC
       end,
       :oracle=>proc do |db|
-        Sequel.ts_require 'adapters/jdbc/oracle'
+        Java::oracle.jdbc.driver.OracleDriver
+        Sequel.require 'adapters/jdbc/oracle'
         db.extend(Sequel::JDBC::Oracle::DatabaseMethods)
         db.dataset_class = Sequel::JDBC::Oracle::Dataset
         Java::oracle.jdbc.driver.OracleDriver
       end,
       :sqlserver=>proc do |db|
-        Sequel.ts_require 'adapters/jdbc/sqlserver'
+        com.microsoft.sqlserver.jdbc.SQLServerDriver
+        Sequel.require 'adapters/jdbc/sqlserver'
         db.extend(Sequel::JDBC::SQLServer::DatabaseMethods)
         db.extend_datasets Sequel::MSSQL::DatasetMethods
         db.send(:set_mssql_unicode_strings)
         com.microsoft.sqlserver.jdbc.SQLServerDriver
       end,
       :jtds=>proc do |db|
-        Sequel.ts_require 'adapters/jdbc/jtds'
-        db.extend(Sequel::JDBC::MSSQL::DatabaseMethods)
+        JDBC.load_gem(:JTDS)
+        Java::net.sourceforge.jtds.jdbc.Driver
+        Sequel.require 'adapters/jdbc/jtds'
+        db.extend(Sequel::JDBC::JTDS::DatabaseMethods)
         db.dataset_class = Sequel::JDBC::JTDS::Dataset
         db.send(:set_mssql_unicode_strings)
-        JDBC.load_gem('jtds')
         Java::net.sourceforge.jtds.jdbc.Driver
       end,
       :h2=>proc do |db|
-        Sequel.ts_require 'adapters/jdbc/h2'
+        JDBC.load_gem(:H2)
+        org.h2.Driver
+        Sequel.require 'adapters/jdbc/h2'
         db.extend(Sequel::JDBC::H2::DatabaseMethods)
         db.dataset_class = Sequel::JDBC::H2::Dataset
-        JDBC.load_gem('h2')
         org.h2.Driver
       end,
       :hsqldb=>proc do |db|
-        Sequel.ts_require 'adapters/jdbc/hsqldb'
+        JDBC.load_gem(:HSQLDB)
+        org.hsqldb.jdbcDriver
+        Sequel.require 'adapters/jdbc/hsqldb'
         db.extend(Sequel::JDBC::HSQLDB::DatabaseMethods)
         db.dataset_class = Sequel::JDBC::HSQLDB::Dataset
-        # Current gem is 1.8.1.3, but Sequel supports 2.2.5
         org.hsqldb.jdbcDriver
       end,
       :derby=>proc do |db|
-        Sequel.ts_require 'adapters/jdbc/derby'
+        JDBC.load_gem(:Derby)
+        org.apache.derby.jdbc.EmbeddedDriver
+        Sequel.require 'adapters/jdbc/derby'
         db.extend(Sequel::JDBC::Derby::DatabaseMethods)
         db.dataset_class = Sequel::JDBC::Derby::Dataset
-        JDBC.load_gem('derby')
         org.apache.derby.jdbc.EmbeddedDriver
       end,
       :as400=>proc do |db|
-        Sequel.ts_require 'adapters/jdbc/as400'
+        com.ibm.as400.access.AS400JDBCDriver
+        Sequel.require 'adapters/jdbc/as400'
         db.extend(Sequel::JDBC::AS400::DatabaseMethods)
         db.dataset_class = Sequel::JDBC::AS400::Dataset
         com.ibm.as400.access.AS400JDBCDriver
       end,
       :"informix-sqli"=>proc do |db|
-        Sequel.ts_require 'adapters/jdbc/informix'
+        com.informix.jdbc.IfxDriver
+        Sequel.require 'adapters/jdbc/informix'
         db.extend(Sequel::JDBC::Informix::DatabaseMethods)
         db.extend_datasets Sequel::Informix::DatasetMethods
         com.informix.jdbc.IfxDriver
       end,
       :db2=>proc do |db|
-        Sequel.ts_require 'adapters/jdbc/db2'
+        com.ibm.db2.jcc.DB2Driver
+        Sequel.require 'adapters/jdbc/db2'
         db.extend(Sequel::JDBC::DB2::DatabaseMethods)
-        db.extend_datasets Sequel::DB2::DatasetMethods
+        db.dataset_class = Sequel::JDBC::DB2::Dataset
         com.ibm.db2.jcc.DB2Driver
       end,
       :firebirdsql=>proc do |db|
-        Sequel.ts_require 'adapters/jdbc/firebird'
+        org.firebirdsql.jdbc.FBDriver
+        Sequel.require 'adapters/jdbc/firebird'
         db.extend(Sequel::JDBC::Firebird::DatabaseMethods)
         db.extend_datasets Sequel::Firebird::DatasetMethods
         org.firebirdsql.jdbc.FBDriver
       end,
       :jdbcprogress=>proc do |db|
-        Sequel.ts_require 'adapters/jdbc/progress'
+        com.progress.sql.jdbc.JdbcProgressDriver
+        Sequel.require 'adapters/jdbc/progress'
         db.extend(Sequel::JDBC::Progress::DatabaseMethods)
         db.extend_datasets Sequel::Progress::DatasetMethods
         com.progress.sql.jdbc.JdbcProgressDriver
+      end,
+      :cubrid=>proc do |db|
+        Java::cubrid.jdbc.driver.CUBRIDDriver
+        Sequel.require 'adapters/jdbc/cubrid'
+        db.extend(Sequel::JDBC::Cubrid::DatabaseMethods)
+        db.extend_datasets Sequel::Cubrid::DatasetMethods
+        Java::cubrid.jdbc.driver.CUBRIDDriver
       end
     }
     
@@ -129,9 +150,14 @@ module Sequel
     # works for PostgreSQL, MySQL, and SQLite.
     def self.load_gem(name)
       begin
-        Sequel.tsk_require "jdbc/#{name}"
+        require "jdbc/#{name.to_s.downcase}"
       rescue LoadError
         # jdbc gem not used, hopefully the user has the .jar in their CLASSPATH
+      else
+        if defined?(::Jdbc) && ( ::Jdbc.const_defined?(name) rescue nil )
+          jdbc_module = ::Jdbc.const_get(name) # e.g. Jdbc::SQLite3
+          jdbc_module.load_driver if jdbc_module.respond_to?(:load_driver)
+        end
       end
     end
 
@@ -151,27 +177,9 @@ module Sequel
       # fetching rows.
       attr_accessor :convert_types
 
-      # Call the DATABASE_SETUP proc directly after initialization,
-      # so the object always uses sub adapter specific code.  Also,
-      # raise an error immediately if the connection doesn't have a
-      # uri, since JDBC requires one.
-      def initialize(opts)
-        super
-        @connection_prepared_statements = {}
-        @connection_prepared_statements_mutex = Mutex.new
-        @convert_types = typecast_value_boolean(@opts.fetch(:convert_types, true))
-        raise(Error, "No connection string specified") unless uri
-        
-        resolved_uri = jndi? ? get_uri_from_jndi : uri
-
-        if match = /\Ajdbc:([^:]+)/.match(resolved_uri) and prok = DATABASE_SETUP[match[1].to_sym]
-          @driver = prok.call(self)
-        end        
-      end
-      
       # Execute the given stored procedure with the give name. If a block is
       # given, the stored procedure should return rows.
-      def call_sproc(name, opts = {})
+      def call_sproc(name, opts = OPTS)
         args = opts[:args] || []
         sql = "{call #{name}(#{args.map{'?'}.join(',')})}"
         synchronize(opts[:server]) do |conn|
@@ -226,17 +234,25 @@ module Sequel
               raise(Sequel::DatabaseError, 'driver.new.connect returned nil: probably bad JDBC connection string') unless c
               c
             rescue JavaSQL::SQLException, NativeException, StandardError => e2
-              e.message << "\n#{e2.class.name}: #{e2.message}"
-              raise e
+              unless e2.message == e.message
+                e2.message << "\n#{e.class.name}: #{e.message}"
+              end
+              raise e2
             end
           end
         end
         setup_connection(conn)
       end
+
+      # Close given adapter connections, and delete any related prepared statements.
+      def disconnect_connection(c)
+        @connection_prepared_statements_mutex.synchronize{@connection_prepared_statements.delete(c)}
+        c.close
+      end
       
       # Execute the given SQL.  If a block is given, if should be a SELECT
       # statement or something else that returns rows.
-      def execute(sql, opts={}, &block)
+      def execute(sql, opts=OPTS, &block)
         return call_sproc(sql, opts, &block) if opts[:sproc]
         return execute_prepared_statement(sql, opts, &block) if [Symbol, Dataset].any?{|c| sql.is_a?(c)}
         synchronize(opts[:server]) do |conn|
@@ -248,13 +264,7 @@ module Sequel
               when :ddl
                 log_yield(sql){stmt.execute(sql)}
               when :insert
-                log_yield(sql) do
-                  if requires_return_generated_keys?
-                    stmt.executeUpdate(sql, JavaSQL::Statement.RETURN_GENERATED_KEYS)
-                  else
-                    stmt.executeUpdate(sql)
-                  end
-                end
+                log_yield(sql){execute_statement_insert(stmt, sql)}
                 last_insert_id(conn, opts.merge(:stmt=>stmt))
               else
                 log_yield(sql){stmt.executeUpdate(sql)}
@@ -264,21 +274,21 @@ module Sequel
         end
       end
       alias execute_dui execute
-      
+
       # Execute the given DDL SQL, which should not return any
       # values or rows.
-      def execute_ddl(sql, opts={})
+      def execute_ddl(sql, opts=OPTS)
         execute(sql, {:type=>:ddl}.merge(opts))
       end
       
       # Execute the given INSERT SQL, returning the last inserted
       # row id.
-      def execute_insert(sql, opts={})
+      def execute_insert(sql, opts=OPTS)
         execute(sql, {:type=>:insert}.merge(opts))
       end
       
       # Use the JDBC metadata to get the index information for the table.
-      def indexes(table, opts={})
+      def indexes(table, opts=OPTS)
         m = output_identifier_meth
         im = input_identifier_meth
         schema, table = schema_and_table(table)
@@ -301,7 +311,7 @@ module Sequel
       end
       
       # All tables in this database
-      def tables(opts={})
+      def tables(opts=OPTS)
         get_tables('TABLE', opts)
       end
       
@@ -309,31 +319,61 @@ module Sequel
       # using the :uri, :url, or :database options.  You don't
       # need to worry about this if you use Sequel.connect
       # with the JDBC connectrion strings.
-      def uri(opts={})
+      def uri(opts=OPTS)
         opts = @opts.merge(opts)
         ur = opts[:uri] || opts[:url] || opts[:database]
         ur =~ /^\Ajdbc:/ ? ur : "jdbc:#{ur}"
       end
 
       # All views in this database
-      def views(opts={})
+      def views(opts=OPTS)
         get_tables('VIEW', opts)
       end
 
       private
          
+      # Call the DATABASE_SETUP proc directly after initialization,
+      # so the object always uses sub adapter specific code.  Also,
+      # raise an error immediately if the connection doesn't have a
+      # uri, since JDBC requires one.
+      def adapter_initialize
+        @connection_prepared_statements = {}
+        @connection_prepared_statements_mutex = Mutex.new
+        @convert_types = typecast_value_boolean(@opts.fetch(:convert_types, true))
+        raise(Error, "No connection string specified") unless uri
+        
+        resolved_uri = jndi? ? get_uri_from_jndi : uri
+
+        if match = /\Ajdbc:([^:]+)/.match(resolved_uri) and prok = DATABASE_SETUP[match[1].to_sym]
+          @driver = prok.call(self)
+        end        
+      end
+      
       # Yield the native prepared statements hash for the given connection
       # to the block in a thread-safe manner.
       def cps_sync(conn, &block)
         @connection_prepared_statements_mutex.synchronize{yield(@connection_prepared_statements[conn] ||= {})}
       end
 
-      # Close given adapter connections, and delete any related prepared statements.
-      def disconnect_connection(c)
-        @connection_prepared_statements_mutex.synchronize{@connection_prepared_statements.delete(c)}
-        c.close
+      def database_error_classes
+        [NativeException]
       end
-      
+
+      def database_exception_sqlstate(exception, opts)
+        if database_exception_use_sqlstates?
+          while exception.respond_to?(:cause)
+            exception = exception.cause
+            return exception.getSQLState if exception.respond_to?(:getSQLState)
+          end
+        end
+        nil
+      end
+
+      # Whether the JDBC subadapter should use SQL states for exception handling, true by default.
+      def database_exception_use_sqlstates?
+        true
+      end
+
       # Raise a disconnect error if the SQL state of the cause of the exception indicates so.
       def disconnect_error?(exception, opts)
         cause = exception.respond_to?(:cause) ? exception.cause : exception
@@ -347,9 +387,9 @@ module Sequel
       # statement, use that statement instead of creating another.
       # Otherwise, prepare a new statement for the connection, bind the
       # variables, and execute it.
-      def execute_prepared_statement(name, opts={})
+      def execute_prepared_statement(name, opts=OPTS)
         args = opts[:arguments]
-        if Dataset === name
+        if name.is_a?(Dataset)
           ps = name
           name = ps.prepared_statement_name
         else
@@ -361,7 +401,7 @@ module Sequel
             cps = cps[1]
           else
             log_yield("CLOSE #{name}"){cps[1].close} if cps
-            cps = log_yield("PREPARE#{" #{name}:" if name} #{sql}"){conn.prepareStatement(sql)}
+            cps = log_yield("PREPARE#{" #{name}:" if name} #{sql}"){prepare_jdbc_statement(conn, sql, opts)}
             cps_sync(conn){|cpsh| cpsh[name] = [sql, cps]} if name
           end
           i = 0
@@ -380,8 +420,8 @@ module Sequel
               when :ddl
                 log_yield(msg, args){cps.execute}
               when :insert
-                log_yield(msg, args){cps.executeUpdate}
-                last_insert_id(conn, opts.merge(:prepared=>true))
+                log_yield(msg, args){execute_prepared_statement_insert(cps)}
+                last_insert_id(conn, opts.merge(:prepared=>true, :stmt=>cps))
               else
                 log_yield(msg, args){cps.executeUpdate}
               end
@@ -394,6 +434,16 @@ module Sequel
         end
       end
 
+      # Execute the prepared insert statement
+      def execute_prepared_statement_insert(stmt)
+        stmt.executeUpdate
+      end
+      
+      # Execute the insert SQL using the statement
+      def execute_statement_insert(stmt, sql)
+        stmt.executeUpdate(sql)
+      end
+      
       # Gets the connection from JNDI.
       def get_connection_from_jndi
         jndi_name = JNDI_URI_REGEXP.match(uri)[1]
@@ -431,7 +481,8 @@ module Sequel
       # Support fractional seconds for Time objects used in bound variables
       def java_sql_timestamp(time)
         ts = java.sql.Timestamp.new(time.to_i * 1000)
-        ts.setNanos(RUBY_VERSION >= '1.9.0' ? time.nsec : time.usec * 1000)
+        # Work around jruby 1.6 ruby 1.9 mode bug
+        ts.setNanos((RUBY_VERSION >= '1.9.0' && time.nsec != 0) ? time.nsec : time.usec * 1000)
         ts
       end 
       
@@ -460,6 +511,11 @@ module Sequel
         end
       end
 
+      # Created a JDBC prepared statement on the connection with the given SQL.
+      def prepare_jdbc_statement(conn, sql, opts)
+        conn.prepareStatement(sql)
+      end
+
       # Java being java, you need to specify the type of each argument
       # for the prepared statement, and bind it individually.  This
       # guesses which JDBC method to use, and hopefully JRuby will convert
@@ -477,7 +533,7 @@ module Sequel
         when TrueClass, FalseClass
           cps.setBoolean(i, arg)
         when NilClass
-          cps.setString(i, nil)
+          set_ps_arg_nil(cps, i)
         when DateTime
           cps.setTimestamp(i, java_sql_datetime(arg))
         when Date
@@ -492,6 +548,11 @@ module Sequel
           cps.setObject(i, arg)
         end
       end
+
+      # Use setString with a nil value by default, but this doesn't work on all subadapters.
+      def set_ps_arg_nil(cps, i)
+        cps.setString(i, nil)
+      end
       
       # Return the connection.  Used to do configuration on the
       # connection object before adding it to the connection pool.
@@ -500,7 +561,7 @@ module Sequel
       end
       
       # Parse the table schema for the given table.
-      def schema_parse_table(table, opts={})
+      def schema_parse_table(table, opts=OPTS)
         m = output_identifier_meth(opts[:dataset])
         im = input_identifier_meth(opts[:dataset])
         ds = dataset
@@ -513,14 +574,19 @@ module Sequel
           next if schema_parse_table_skip?(h, schema)
           pks << h[:column_name]
         end
+        schemas = []
         metadata(:getColumns, nil, schema, table, nil) do |h|
           next if schema_parse_table_skip?(h, schema)
           s = {:type=>schema_column_type(h[:type_name]), :db_type=>h[:type_name], :default=>(h[:column_def] == '' ? nil : h[:column_def]), :allow_null=>(h[:nullable] != 0), :primary_key=>pks.include?(h[:column_name]), :column_size=>h[:column_size], :scale=>h[:decimal_digits]}
           if s[:db_type] =~ DECIMAL_TYPE_RE && s[:scale] == 0
             s[:type] = :integer
           end
+          schemas << h[:table_schem] unless schemas.include?(h[:table_schem])
           ts << [m.call(h[:column_name]), s]
         end
+        if schemas.length > 1
+          raise Error, 'Schema parsing in the jdbc adapter resulted in columns being returned for a table with the same name in multiple schemas.  Please explicitly qualify your table with a schema.'
+        end
         ts
       end
       
@@ -539,13 +605,6 @@ module Sequel
       ensure
         stmt.close if stmt
       end
-
-      # This method determines whether or not to add
-      # Statement.RETURN_GENERATED_KEYS as an argument when inserting rows.
-      # Sub-adapters that require this should override this method.
-      def requires_return_generated_keys?
-        false
-      end
     end
     
     class Dataset < Sequel::Dataset
@@ -564,17 +623,17 @@ module Sequel
         
         # Execute the prepared SQL using the stored type and
         # arguments derived from the hash passed to call.
-        def execute(sql, opts={}, &block)
+        def execute(sql, opts=OPTS, &block)
           super(self, {:arguments=>bind_arguments}.merge(opts), &block)
         end
         
         # Same as execute, explicit due to intricacies of alias and super.
-        def execute_dui(sql, opts={}, &block)
+        def execute_dui(sql, opts=OPTS, &block)
           super(self, {:arguments=>bind_arguments}.merge(opts), &block)
         end
         
         # Same as execute, explicit due to intricacies of alias and super.
-        def execute_insert(sql, opts={}, &block)
+        def execute_insert(sql, opts=OPTS, &block)
           super(self, {:arguments=>bind_arguments, :type=>:insert}.merge(opts), &block)
         end
       end
@@ -587,17 +646,17 @@ module Sequel
         private
         
         # Execute the database stored procedure with the stored arguments.
-        def execute(sql, opts={}, &block)
+        def execute(sql, opts=OPTS, &block)
           super(@sproc_name, {:args=>@sproc_args, :sproc=>true}.merge(opts), &block)
         end
         
         # Same as execute, explicit due to intricacies of alias and super.
-        def execute_dui(sql, opts={}, &block)
+        def execute_dui(sql, opts=OPTS, &block)
           super(@sproc_name, {:args=>@sproc_args, :sproc=>true}.merge(opts), &block)
         end
         
         # Same as execute, explicit due to intricacies of alias and super.
-        def execute_insert(sql, opts={}, &block)
+        def execute_insert(sql, opts=OPTS, &block)
           super(@sproc_name, {:args=>@sproc_args, :sproc=>true, :type=>:insert}.merge(opts), &block)
         end
       end
@@ -636,6 +695,8 @@ module Sequel
       JAVA_BUFFERED_READER  = Java::JavaIo::BufferedReader
       JAVA_BIG_DECIMAL      = Java::JavaMath::BigDecimal
       JAVA_BYTE_ARRAY       = Java::byte[]
+      JAVA_UUID             = Java::JavaUtil::UUID
+      JAVA_HASH_MAP         = Java::JavaUtil::HashMap
 
       # Handle type conversions for common Java types.
       class TYPE_TRANSLATOR
@@ -645,7 +706,7 @@ module Sequel
         def decimal(v) BigDecimal.new(v.to_string) end
         def byte_array(v) Sequel::SQL::Blob.new(String.from_java_bytes(v)) end
         def blob(v) Sequel::SQL::Blob.new(String.from_java_bytes(v.getBytes(1, v.length))) end
-        def clob(v) Sequel::SQL::Blob.new(v.getSubString(1, v.length)) end
+        def clob(v) v.getSubString(1, v.length) end
         def buffered_reader(v)
           lines = ""
           c = false
@@ -656,6 +717,8 @@ module Sequel
           end
           lines
         end
+        def uuid(v) v.to_string end
+        def hash_map(v) v.to_hash end
       end
       TYPE_TRANSLATOR_INSTANCE = tt = TYPE_TRANSLATOR.new
 
@@ -668,6 +731,8 @@ module Sequel
       BYTE_ARRAY_METHOD = tt.method(:byte_array)
       BLOB_METHOD = tt.method(:blob)
       CLOB_METHOD = tt.method(:clob)
+      UUID_METHOD = tt.method(:uuid)
+      HASH_MAP_METHOD = tt.method(:hash_map)
 
       # Convert the given Java timestamp to an instance of Sequel.datetime_class.
       def convert_type_timestamp(v)
@@ -695,6 +760,10 @@ module Sequel
           BLOB_METHOD
         when JAVA_SQL_CLOB
           CLOB_METHOD
+        when JAVA_UUID
+          UUID_METHOD
+        when JAVA_HASH_MAP
+          HASH_MAP_METHOD
         else
           false
         end
@@ -714,17 +783,13 @@ module Sequel
         i = 0
         meta.getColumnCount.times{cols << [output_identifier(meta.getColumnLabel(i+=1)), i]}
         columns = cols.map{|c| c.at(0)}
-        if opts[:offset] && offset_returns_row_number_column?
-          rn = row_number_column
-          columns.delete(rn)
-        end
         @columns = columns
         ct = @convert_types
         if (ct.nil? ? db.convert_types : ct)
           cols.each{|c| c << nil}
-          process_result_set_convert(cols, result, rn, &block)
+          process_result_set_convert(cols, result, &block)
         else
-          process_result_set_no_convert(cols, result, rn, &block)
+          process_result_set_no_convert(cols, result, &block)
         end
       ensure
         result.close
@@ -748,7 +813,7 @@ module Sequel
       #   the result of as the column's conversion proc to speed up
       #   later processing.  If the conversion proc exists, call it
       #   and return the result, otherwise, return the object.
-      def process_result_set_convert(cols, result, rn)
+      def process_result_set_convert(cols, result)
         while result.next
           row = {}
           cols.each do |n, i, p|
@@ -770,18 +835,16 @@ module Sequel
               v
             end
           end
-          row.delete(rn) if rn
           yield row
         end
       end
 
       # Yield rows without calling any conversion procs.  This
       # may yield Java values and not ruby values.
-      def process_result_set_no_convert(cols, result, rn)
+      def process_result_set_no_convert(cols, result)
         while result.next
           row = {}
           cols.each{|n, i| row[n] = result.getObject(i)}
-          row.delete(rn) if rn
           yield row
         end
       end
