@@ -4,6 +4,7 @@ module Sequel
     module H2
       # Instance methods for H2 Database objects accessed via JDBC.
       module DatabaseMethods
+        extend Sequel::Database::ResetIdentifierMangling
         PRIMARY_KEY_INDEX_RE = /\Aprimary_key/i.freeze
       
         # Commit an existing prepared transaction with the given transaction
@@ -47,7 +48,7 @@ module Sequel
         
         # If the :prepare option is given and we aren't in a savepoint,
         # prepare the transaction for a two-phase commit.
-        def commit_transaction(conn, opts={})
+        def commit_transaction(conn, opts=OPTS)
           if (s = opts[:prepare]) && _trans(conn)[:savepoint_level] <= 1
             log_connection_execute(conn, "PREPARE COMMIT #{s}")
           else
@@ -85,6 +86,12 @@ module Sequel
             sql = "ALTER TABLE #{quote_schema_table(table)} ALTER COLUMN #{quote_identifier(op[:name])} #{type_literal(op)}"
             column_definition_order.each{|m| send(:"column_definition_#{m}_sql", sql, op)}
             sql
+          when :drop_constraint
+            if op[:type] == :primary_key
+              "ALTER TABLE #{quote_schema_table(table)} DROP PRIMARY KEY"
+            else
+              super(table, op)
+            end
           else
             super(table, op)
           end
@@ -96,8 +103,19 @@ module Sequel
           uri == 'jdbc:h2:mem:' ? o.merge(:max_connections=>1) : o
         end
       
+        DATABASE_ERROR_REGEXPS = {
+          /Unique index or primary key violation/ => UniqueConstraintViolation,
+          /Referential integrity constraint violation/ => ForeignKeyConstraintViolation,
+          /Check constraint violation/ => CheckConstraintViolation,
+          /NULL not allowed for column/ => NotNullConstraintViolation,
+          /Deadlock detected\. The current transaction was rolled back\./ => SerializationFailure,
+        }.freeze
+        def database_error_regexps
+          DATABASE_ERROR_REGEXPS
+        end
+
         # Use IDENTITY() to get the last inserted id.
-        def last_insert_id(conn, opts={})
+        def last_insert_id(conn, opts=OPTS)
           statement(conn) do |stmt|
             sql = 'SELECT IDENTITY();'
             rs = log_yield(sql){stmt.executeQuery(sql)}
@@ -110,9 +128,9 @@ module Sequel
           PRIMARY_KEY_INDEX_RE
         end
 
-        # Treat clob as string instead of blob
-        def schema_column_type(db_type)
-          db_type == 'clob' ? :string : super
+        # H2 does not support named column constraints.
+        def supports_named_column_constraints?
+          false
         end
 
         # Use BIGINT IDENTITY for identity columns that use bigint, fixes
