@@ -7,12 +7,13 @@ module Sequel
     module Informix
       # Database instance methods for Informix databases accessed via JDBC.
       module DatabaseMethods
+        extend Sequel::Database::ResetIdentifierMangling
         include Sequel::Informix::DatabaseMethods
         
         private
         
         # TODO: implement
-        def last_insert_id(conn, opts={})
+        def last_insert_id(conn, opts=OPTS)
           nil
         end
       end
