@@ -7,6 +7,7 @@ module Sequel
     module Oracle
       # Instance methods for Oracle Database objects accessed via JDBC.
       module DatabaseMethods
+        extend Sequel::Database::ResetIdentifierMangling
         PRIMARY_KEY_INDEX_RE = /\Asys_/i.freeze
 
         include Sequel::Oracle::DatabaseMethods
@@ -21,6 +22,15 @@ module Sequel
         
         private
 
+        # Oracle exception handling with SQLState is less accurate than with regexps.
+        def database_exception_use_sqlstates?
+          false
+        end
+
+        def disconnect_error?(exception, opts)
+          super || exception.message =~ /\AClosed Connection/
+        end
+
         def last_insert_id(conn, opts)
           unless sequence = opts[:sequence]
             if t = opts[:table]
@@ -75,12 +85,13 @@ module Sequel
         private
 
         JAVA_BIG_DECIMAL = ::Sequel::JDBC::Dataset::JAVA_BIG_DECIMAL
+        JAVA_BIG_DECIMAL_CONSTRUCTOR = java.math.BigDecimal.java_class.constructor(Java::long).method(:new_instance)
 
         class ::Sequel::JDBC::Dataset::TYPE_TRANSLATOR
           def oracle_decimal(v)
             if v.scale == 0
               i = v.long_value
-              if v.equals(JAVA_BIG_DECIMAL.new(i))
+              if v.equals(JAVA_BIG_DECIMAL_CONSTRUCTOR.call(i))
                 i
               else
                 decimal(v)
@@ -97,10 +108,16 @@ module Sequel
           db.to_application_timestamp(v.to_string)
         end
       
+        def convert_type_oracle_timestamptz(v)
+          convert_type_oracle_timestamp(db.synchronize(@opts[:server]){|c| v.timestampValue(c)})
+        end
+      
         def convert_type_proc(v)
           case v
           when JAVA_BIG_DECIMAL
             ORACLE_DECIMAL_METHOD
+          when Java::OracleSql::TIMESTAMPTZ
+            method(:convert_type_oracle_timestamptz)
           when Java::OracleSql::TIMESTAMP
             method(:convert_type_oracle_timestamp)
           else
