@@ -6,18 +6,19 @@ module Sequel
     # support via ODBC.
     module MSSQL
       module DatabaseMethods
+        extend Sequel::Database::ResetIdentifierMangling
         include Sequel::MSSQL::DatabaseMethods
         LAST_INSERT_ID_SQL='SELECT SCOPE_IDENTITY()'.freeze
         
         # Return the last inserted identity value.
-        def execute_insert(sql, opts={})
+        def execute_insert(sql, opts=OPTS)
           synchronize(opts[:server]) do |conn|
             begin
               log_yield(sql){conn.do(sql)}
               begin
                 s = log_yield(LAST_INSERT_ID_SQL){conn.run(LAST_INSERT_ID_SQL)}
-                if (rows = s.fetch_all) and (row = rows.first)
-                  Integer(row.first)
+                if (rows = s.fetch_all) and (row = rows.first) and (v = row.first)
+                  Integer(v)
                 end
               ensure
                 s.drop if s
