@@ -14,21 +14,18 @@ module Sequel
           opts[:password]
         )
       end
+
+      def disconnect_connection(c)
+        c.disconnect
+      end
       
-      def execute(sql, opts={})
+      def execute(sql, opts=OPTS)
         synchronize(opts[:server]) do |conn|
           r = log_yield(sql){conn.execute(sql)}
           yield(r) if block_given?
           r
         end
       end
-      alias_method :do, :execute
-
-      private
-
-      def disconnect_connection(c)
-        c.disconnect
-      end
     end
     
     class Dataset < Sequel::Dataset
