@@ -20,7 +20,11 @@ rescue LoadError => e
         else
           # Raise an error if no valid string escaping method can be found.
           def escape_string(obj)
-            raise Sequel::Error, "string escaping not supported with this postgres driver.  Try using ruby-pg, ruby-postgres, or postgres-pr."
+            if Sequel::Postgres.force_standard_strings
+              str.gsub("'", "''")
+            else
+              raise Sequel::Error, "string escaping not supported with this postgres driver.  Try using ruby-pg, ruby-postgres, or postgres-pr."
+            end
           end
         end
       end
@@ -87,72 +91,24 @@ module Sequel
   Dataset::NON_SQL_OPTIONS << :cursor
   module Postgres
     CONVERTED_EXCEPTIONS << PGError
-    
-    NAN             = 0.0/0.0
-    PLUS_INFINITY   = 1.0/0.0
-    MINUS_INFINITY  = -1.0/0.0
-    NAN_STR             = 'NaN'.freeze
-    PLUS_INFINITY_STR   = 'Infinity'.freeze
-    MINUS_INFINITY_STR  = '-Infinity'.freeze
-    TRUE_STR = 't'.freeze
-    DASH_STR = '-'.freeze
-    
-    TYPE_TRANSLATOR = tt = Class.new do
-      def boolean(s) s == TRUE_STR end
+
+    PG_TYPES[17] = Class.new do
       def bytea(s) ::Sequel::SQL::Blob.new(Adapter.unescape_bytea(s)) end
-      def integer(s) s.to_i end
-      def float(s) 
-        case s
-        when NAN_STR
-          NAN
-        when PLUS_INFINITY_STR
-          PLUS_INFINITY
-        when MINUS_INFINITY_STR
-          MINUS_INFINITY
-        else
-          s.to_f 
-        end
-      end
-      def date(s) ::Date.new(*s.split(DASH_STR).map{|x| x.to_i}) end
-    end.new
+    end.new.method(:bytea)
 
-    # Hash with type name strings/symbols and callable values for converting PostgreSQL types.
-    # Non-builtin types that don't have fixed numbers should use this to register
-    # conversion procs.
-    PG_NAMED_TYPES = {} unless defined?(PG_NAMED_TYPES)
+    @use_iso_date_format = true
 
-    # Hash with integer keys and callable values for converting PostgreSQL types.
-    PG_TYPES = {} unless defined?(PG_TYPES)
-
-    {
-      [16] => tt.method(:boolean),
-      [17] => tt.method(:bytea),
-      [20, 21, 22, 23, 26] => tt.method(:integer),
-      [700, 701] => tt.method(:float),
-      [790, 1700] => ::BigDecimal.method(:new),
-      [1083, 1266] => ::Sequel.method(:string_to_time),
-    }.each do |k,v|
-      k.each{|n| PG_TYPES[n] = v}
-    end
-    
     class << self
       # As an optimization, Sequel sets the date style to ISO, so that PostgreSQL provides
       # the date in a known format that Sequel can parse faster.  This can be turned off
       # if you require a date style other than ISO.
-      attr_reader :use_iso_date_format
+      attr_accessor :use_iso_date_format
     end
 
-    # Modify the type translator for the date type depending on the value given.
-    def self.use_iso_date_format=(v)
-      PG_TYPES[1082] = v ? TYPE_TRANSLATOR.method(:date) : Sequel.method(:string_to_date)
-      @use_iso_date_format = v
-    end
-    self.use_iso_date_format = true
-    
     # PGconn subclass for connection specific methods used with the
     # pg, postgres, or postgres-pr driver.
     class Adapter < ::PGconn
-      DISCONNECT_ERROR_RE = /\Acould not receive data from server: Software caused connection abort/
+      DISCONNECT_ERROR_RE = /\Acould not receive data from server/
       
       self.translate_results = false if respond_to?(:translate_results=)
       
@@ -166,7 +122,7 @@ module Sequel
       def check_disconnect_errors
         begin
           yield
-        rescue PGError =>e
+        rescue PGError => e
           disconnect = false
           begin
             s = status
@@ -177,8 +133,11 @@ module Sequel
           disconnect ||= !status_ok
           disconnect ||= e.message =~ DISCONNECT_ERROR_RE
           disconnect ? raise(Sequel.convert_exception_class(e, Sequel::DatabaseDisconnectError)) : raise
+        rescue IOError, Errno::EPIPE, Errno::ECONNRESET => e
+          disconnect = true
+          raise(Sequel.convert_exception_class(e, Sequel::DatabaseDisconnectError))
         ensure
-          block if status_ok
+          block if status_ok && !disconnect
         end
       end
 
@@ -190,7 +149,7 @@ module Sequel
         begin
           block_given? ? yield(q) : q.cmd_tuples
         ensure
-          q.clear if q
+          q.clear if q && q.respond_to?(:clear)
         end
       end
 
@@ -213,24 +172,11 @@ module Sequel
       
       set_adapter_scheme :postgres
 
-      # A hash of conversion procs, keyed by type integer (oid) and
-      # having callable values for the conversion proc for that type.
-      attr_reader :conversion_procs
-
-      # Whether infinite timestamps should be converted on retrieval.  By default, no
+      # Whether infinite timestamps/dates should be converted on retrieval.  By default, no
       # conversion is done, so an error is raised if you attempt to retrieve an infinite
-      # timestamp.  You can set this to :nil to convert to nil, :string to leave
+      # timestamp/date.  You can set this to :nil to convert to nil, :string to leave
       # as a string, or :float to convert to an infinite float.
-      attr_accessor :convert_infinite_timestamps
-      
-      # Add the primary_keys and primary_key_sequences instance variables,
-      # so we can get the correct return values for inserted rows.
-      def initialize(*args)
-        super
-        @convert_infinite_timestamps = false
-        @primary_keys = {}
-        @primary_key_sequences = {}
-      end
+      attr_reader :convert_infinite_timestamps
 
       # Convert given argument so that it can be used directly by pg.  Currently, pg doesn't
       # handle fractional seconds in Time/DateTime or blobs with "\0", and it won't ever
@@ -288,23 +234,85 @@ module Sequel
         conn.instance_variable_set(:@db, self)
         conn.instance_variable_set(:@prepared_statements, {}) if SEQUEL_POSTGRES_USES_PG
         connection_configuration_sqls.each{|sql| conn.execute(sql)}
-        @conversion_procs ||= get_conversion_procs(conn)
         conn
       end
       
+      # Set whether to allow infinite timestamps/dates.  Make sure the
+      # conversion proc for date reflects that setting.
+      def convert_infinite_timestamps=(v)
+        @convert_infinite_timestamps = case v
+        when Symbol
+          v
+        when 'nil'
+          :nil
+        when 'string'
+          :string
+        when 'float'
+          :float
+        when String
+          typecast_value_boolean(v)
+        else
+          false
+        end
+
+        pr = old_pr = @use_iso_date_format ? TYPE_TRANSLATOR.method(:date) : Sequel.method(:string_to_date)
+        if v
+          pr = lambda do |val|
+            case val
+            when *INFINITE_TIMESTAMP_STRINGS
+              infinite_timestamp_value(val)
+            else
+              old_pr.call(val)
+            end
+          end
+        end
+        conversion_procs[1082] = pr
+      end
+
+      # Disconnect given connection
+      def disconnect_connection(conn)
+        begin
+          conn.finish
+        rescue PGError, IOError
+        end
+      end
+
+      if SEQUEL_POSTGRES_USES_PG && Object.const_defined?(:PG) && ::PG.const_defined?(:Constants) && ::PG::Constants.const_defined?(:PG_DIAG_SCHEMA_NAME)
+        # Return a hash of information about the related PGError (or Sequel::DatabaseError that
+        # wraps a PGError), with the following entries:
+        #
+        # :schema :: The schema name related to the error
+        # :table :: The table name related to the error
+        # :column :: the column name related to the error
+        # :constraint :: The constraint name related to the error
+        # :type :: The datatype name related to the error
+        #
+        # This requires a PostgreSQL 9.3+ server and 9.3+ client library,
+        # and ruby-pg 0.16.0+ to be supported.
+        def error_info(e)
+          e = e.wrapped_exception if e.is_a?(DatabaseError)
+          r = e.result
+          h = {}
+          h[:schema] = r.error_field(::PG::PG_DIAG_SCHEMA_NAME)
+          h[:table] = r.error_field(::PG::PG_DIAG_TABLE_NAME)
+          h[:column] = r.error_field(::PG::PG_DIAG_COLUMN_NAME)
+          h[:constraint] = r.error_field(::PG::PG_DIAG_CONSTRAINT_NAME)
+          h[:type] = r.error_field(::PG::PG_DIAG_DATATYPE_NAME)
+          h
+        end
+      end
+      
       # Execute the given SQL with the given args on an available connection.
-      def execute(sql, opts={}, &block)
+      def execute(sql, opts=OPTS, &block)
         synchronize(opts[:server]){|conn| check_database_errors{_execute(conn, sql, opts, &block)}}
       end
 
       if SEQUEL_POSTGRES_USES_PG
-        # +copy_table+ uses PostgreSQL's +COPY+ SQL statement to return formatted
+        # +copy_table+ uses PostgreSQL's +COPY TO STDOUT+ SQL statement to return formatted
         # results directly to the caller.  This method is only supported if pg is the
         # underlying ruby driver.  This method should only be called if you want
-        # results returned to the client.  If you are using +COPY FROM+ or +COPY TO+
-        # with a filename, you should just use +run+ instead of this method.  This
-        # method does not currently support +COPY FROM STDIN+, but that may be supported
-        # in the future.
+        # results returned to the client.  If you are using +COPY TO+
+        # with a filename, you should just use +run+ instead of this method.
         #
         # The table argument supports the following types:
         #
@@ -324,25 +332,9 @@ module Sequel
         # If a block is provided, the method continually yields to the block, one yield
         # per row.  If a block is not provided, a single string is returned with all
         # of the data.
-        def copy_table(table, opts={})
-          sql = if table.is_a?(String)
-            sql = table
-          else
-            if opts[:options] || opts[:format]
-              options = " ("
-              options << "FORMAT #{opts[:format]}" if opts[:format]
-              options << "#{', ' if opts[:format]}#{opts[:options]}" if opts[:options]
-              options << ')'
-            end
-            table = if table.is_a?(::Sequel::Dataset)
-              "(#{table.sql})"
-            else
-              literal(table)
-            end
-           sql = "COPY #{table} TO STDOUT#{options}"
-          end
-          synchronize(opts[:server]) do |conn| 
-            conn.execute(sql)
+        def copy_table(table, opts=OPTS)
+          synchronize(opts[:server]) do |conn|
+            conn.execute(copy_table_sql(table, opts))
             begin
               if block_given?
                 while buf = conn.get_copy_data
@@ -360,6 +352,58 @@ module Sequel
           end 
         end
 
+        # +copy_into+ uses PostgreSQL's +COPY FROM STDIN+ SQL statement to do very fast inserts 
+        # into a table using input preformatting in either CSV or PostgreSQL text format.
+        # This method is only supported if pg 0.14.0+ is the underlying ruby driver.
+        # This method should only be called if you want
+        # results returned to the client.  If you are using +COPY FROM+
+        # with a filename, you should just use +run+ instead of this method.
+        #
+        # The following options are respected:
+        #
+        # :columns :: The columns to insert into, with the same order as the columns in the
+        #             input data.  If this isn't given, uses all columns in the table.
+        # :data :: The data to copy to PostgreSQL, which should already be in CSV or PostgreSQL
+        #          text format.  This can be either a string, or any object that responds to
+        #          each and yields string.
+        # :format :: The format to use.  text is the default, so this should be :csv or :binary.
+        # :options :: An options SQL string to use, which should contain comma separated options.
+        # :server :: The server on which to run the query.
+        #
+        # If a block is provided and :data option is not, this will yield to the block repeatedly.
+        # The block should return a string, or nil to signal that it is finished.
+        def copy_into(table, opts=OPTS)
+          data = opts[:data]
+          data = Array(data) if data.is_a?(String)
+
+          if block_given? && data
+            raise Error, "Cannot provide both a :data option and a block to copy_into"
+          elsif !block_given? && !data
+            raise Error, "Must provide either a :data option or a block to copy_into"
+          end
+
+          synchronize(opts[:server]) do |conn|
+            conn.execute(copy_into_sql(table, opts))
+            begin
+              if block_given?
+                while buf = yield
+                  conn.put_copy_data(buf)
+                end
+              else
+                data.each{|buff| conn.put_copy_data(buff)}
+              end
+            rescue Exception => e
+              conn.put_copy_end("ruby exception occurred while copying data into PostgreSQL")
+            ensure
+              conn.put_copy_end unless e
+              while res = conn.get_result
+                raise e if e
+                check_database_errors{res.check}
+              end
+            end
+          end 
+        end
+
         # Listens on the given channel (or multiple channels if channel is an array), waiting for notifications.
         # After a notification is received, or the timeout has passed, stops listening to the channel. Options:
         #
@@ -381,12 +425,16 @@ module Sequel
         # * the channel the notification was sent to (as a string)
         # * the backend pid of the notifier (as an integer),
         # * and the payload of the notification (as a string or nil).
-        def listen(channels, opts={}, &block)
+        def listen(channels, opts=OPTS, &block)
           check_database_errors do
             synchronize(opts[:server]) do |conn|
               begin
                 channels = Array(channels)
-                channels.each{|channel| conn.execute("LISTEN #{channel}")}
+                channels.each do |channel|
+                  sql = "LISTEN "
+                  dataset.send(:identifier_append, sql, channel)
+                  conn.execute(sql)
+                end
                 opts[:after_listen].call(conn) if opts[:after_listen]
                 timeout = opts[:timeout] ? [opts[:timeout]] : []
                 if l = opts[:loop]
@@ -410,16 +458,10 @@ module Sequel
         end
       end
 
-      # Reset the database's conversion procs, requires a server query if there
-      # any named types.
-      def reset_conversion_procs
-        synchronize{|conn| @conversion_procs = get_conversion_procs(conn)}
-      end
-
       # If convert_infinite_timestamps is true and the value is infinite, return an appropriate
       # value based on the convert_infinite_timestamps setting.
       def to_application_timestamp(value)
-        if c = convert_infinite_timestamps
+        if convert_infinite_timestamps
           case value
           when *INFINITE_TIMESTAMP_STRINGS
             infinite_timestamp_value(value)
@@ -442,6 +484,20 @@ module Sequel
         end
       end
 
+      # Execute the prepared statement name with the given arguments on the connection.
+      def _execute_prepared_statement(conn, ps_name, args, opts)
+        conn.exec_prepared(ps_name, args)
+      end
+
+      # Add the primary_keys and primary_key_sequences instance variables,
+      # so we can get the correct return values for inserted rows.
+      def adapter_initialize
+        @use_iso_date_format = typecast_value_boolean(@opts.fetch(:use_iso_date_format, Postgres.use_iso_date_format))
+        initialize_postgres_adapter
+        conversion_procs[1082] = TYPE_TRANSLATOR.method(:date) if @use_iso_date_format
+        self.convert_infinite_timestamps = @opts[:convert_infinite_timestamps]
+      end
+
       # Convert exceptions raised from the block into DatabaseErrors.
       def check_database_errors
         begin
@@ -454,18 +510,20 @@ module Sequel
       # Set the DateStyle to ISO if configured, for faster date parsing.
       def connection_configuration_sqls
         sqls = super
-        sqls << "SET DateStyle = 'ISO'" if Postgres.use_iso_date_format
+        sqls << "SET DateStyle = 'ISO'" if @use_iso_date_format
         sqls
       end
 
-      # Disconnect given connection
-      def disconnect_connection(conn)
-        begin
-          conn.finish
-        rescue PGError
+      def database_error_classes
+        [PGError]
+      end
+
+      def database_exception_sqlstate(exception, opts)
+        if exception.respond_to?(:result) && (result = exception.result)
+          result.error_field(::PGresult::PG_DIAG_SQLSTATE)
         end
       end
-      
+
       # Execute the prepared statement with the given name on an available
       # connection, using the given args.  If the connection has not prepared
       # a statement with the given name yet, prepare it.  If the connection
@@ -473,7 +531,7 @@ module Sequel
       # deallocate that statement first and then prepare this statement.
       # If a block is given, yield the result, otherwise, return the number
       # of rows changed.
-      def execute_prepared_statement(conn, name, opts={}, &block)
+      def execute_prepared_statement(conn, name, opts=OPTS, &block)
         ps = prepared_statement(name)
         sql = ps.prepared_sql
         ps_name = name.to_s
@@ -484,8 +542,8 @@ module Sequel
 
         unless conn.prepared_statements[ps_name] == sql
           conn.execute("DEALLOCATE #{ps_name}") if conn.prepared_statements.include?(ps_name)
-          conn.prepared_statements[ps_name] = sql
           conn.check_disconnect_errors{log_yield("PREPARE #{ps_name} AS #{sql}"){conn.prepare(ps_name, sql)}}
+          conn.prepared_statements[ps_name] = sql
         end
 
         log_sql = "EXECUTE #{ps_name}"
@@ -495,28 +553,14 @@ module Sequel
           log_sql << ")"
         end
 
-        q = conn.check_disconnect_errors{log_yield(log_sql, args){conn.exec_prepared(ps_name, args)}}
+        q = conn.check_disconnect_errors{log_yield(log_sql, args){_execute_prepared_statement(conn, ps_name, args, opts)}}
         begin
           block_given? ? yield(q) : q.cmd_tuples
         ensure
-          q.clear
+          q.clear if q && q.respond_to?(:clear)
         end
       end
 
-      # Return the conversion procs hash to use for this database.
-      def get_conversion_procs(conn)
-        procs = PG_TYPES.dup
-        procs[1184] = procs[1114] = method(:to_application_timestamp)
-        unless (pgnt = PG_NAMED_TYPES).empty?
-          conn.execute("SELECT oid, typname FROM pg_type where typtype = 'b' AND typname IN ('#{pgnt.keys.map{|type| conn.escape_string(type.to_s)}.join("', '")}')") do |res|
-            res.ntuples.times do |i|
-              procs[res.getvalue(i, 0).to_i] ||= pgnt[res.getvalue(i, 1).untaint.to_sym]
-            end
-          end
-        end
-        procs
-      end
-
       # Return an appropriate value for the given infinite timestamp string.
       def infinite_timestamp_value(value)
         case convert_infinite_timestamps
@@ -534,6 +578,22 @@ module Sequel
         conn.execute(sql)
       end
 
+      # If the value is an infinite value (either an infinite float or a string returned by
+      # by PostgreSQL for an infinite timestamp), return it without converting it if
+      # convert_infinite_timestamps is set.
+      def typecast_value_date(value)
+        if convert_infinite_timestamps
+          case value
+          when *INFINITE_DATETIME_VALUES
+            value
+          else
+            super
+          end
+        else
+          super
+        end
+      end
+
       # If the value is an infinite value (either an infinite float or a string returned by
       # by PostgreSQL for an infinite timestamp), return it without converting it if
       # convert_infinite_timestamps is set.
@@ -581,7 +641,7 @@ module Sequel
       #
       # This is untested with the prepared statement/bound variable support,
       # and unlikely to work with either.
-      def use_cursor(opts={})
+      def use_cursor(opts=OPTS)
         clone(:cursor=>{:rows_per_fetch=>1000}.merge(opts))
       end
 
@@ -604,20 +664,15 @@ module Sequel
 
           private
           
-          # PostgreSQL most of the time requires type information for each of
-          # arguments to a prepared statement.  Handle this by allowing the
-          # named argument to have a __* suffix, with the * being the type.
-          # In the generated SQL, cast the bound argument to that type to
-          # elminate ambiguity (and PostgreSQL from raising an exception).
           def prepared_arg(k)
-            y, type = k.to_s.split("__")
+            y = k
             if i = prepared_args.index(y)
               i += 1
             else
               prepared_args << y
               i = prepared_args.length
             end
-            LiteralString.new("#{prepared_arg_placeholder}#{i}#{"::#{type}" if type}")
+            LiteralString.new("#{prepared_arg_placeholder}#{i}")
           end
 
           # Always assume a prepared argument.
@@ -634,12 +689,12 @@ module Sequel
           private
           
           # Execute the given SQL with the stored bind arguments.
-          def execute(sql, opts={}, &block)
+          def execute(sql, opts=OPTS, &block)
             super(sql, {:arguments=>bind_arguments}.merge(opts), &block)
           end
           
           # Same as execute, explicit due to intricacies of alias and super.
-          def execute_dui(sql, opts={}, &block)
+          def execute_dui(sql, opts=OPTS, &block)
             super(sql, {:arguments=>bind_arguments}.merge(opts), &block)
           end
         end
@@ -659,18 +714,18 @@ module Sequel
           
           # Execute the stored prepared statement name and the stored bind
           # arguments instead of the SQL given.
-          def execute(sql, opts={}, &block)
+          def execute(sql, opts=OPTS, &block)
             super(prepared_statement_name, opts, &block)
           end
           
           # Same as execute, explicit due to intricacies of alias and super.
-          def execute_dui(sql, opts={}, &block)
+          def execute_dui(sql, opts=OPTS, &block)
             super(prepared_statement_name, opts, &block)
           end
         end
         
         # Execute the given type of statement with the hash of values.
-        def call(type, bind_vars={}, *values, &block)
+        def call(type, bind_vars=OPTS, *values, &block)
           ps = to_prepared_statement(type, values)
           ps.extend(BindArgumentMethods)
           ps.call(bind_vars, &block)
@@ -741,12 +796,12 @@ module Sequel
       
       # Use the driver's escape_bytea
       def literal_blob_append(sql, v)
-        sql << APOS << db.synchronize{|c| c.escape_bytea(v)} << APOS
+        sql << APOS << db.synchronize(@opts[:server]){|c| c.escape_bytea(v)} << APOS
       end
       
       # Use the driver's escape_string
       def literal_string_append(sql, v)
-        sql << APOS << db.synchronize{|c| c.escape_string(v)} << APOS
+        sql << APOS << db.synchronize(@opts[:server]){|c| c.escape_string(v)} << APOS
       end
       
       # For each row in the result set, yield a hash with column name symbol
@@ -765,7 +820,7 @@ module Sequel
   end
 end
 
-if SEQUEL_POSTGRES_USES_PG
+if SEQUEL_POSTGRES_USES_PG && !ENV['NO_SEQUEL_PG']
   begin
     require 'sequel_pg'
   rescue LoadError
