@@ -1,9 +1,11 @@
-Sequel.require 'adapters/utils/emulate_offset_with_row_number'
+Sequel.require %w'emulate_offset_with_row_number split_alter_table', 'adapters/utils'
 
 module Sequel
   Dataset::NON_SQL_OPTIONS << :disable_insert_output
   module MSSQL
     module DatabaseMethods
+      extend Sequel::Database::ResetIdentifierMangling
+
       AUTO_INCREMENT = 'IDENTITY(1,1)'.freeze
       SERVER_VERSION_RE = /^(\d+)\.(\d+)\.(\d+)/.freeze
       SERVER_VERSION_SQL = "SELECT CAST(SERVERPROPERTY('ProductVersion') AS varchar)".freeze
@@ -12,12 +14,21 @@ module Sequel
       SQL_ROLLBACK = "IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION".freeze
       SQL_ROLLBACK_TO_SAVEPOINT = 'IF @@TRANCOUNT > 0 ROLLBACK TRANSACTION autopoint_%d'.freeze
       SQL_SAVEPOINT = 'SAVE TRANSACTION autopoint_%d'.freeze
+      MSSQL_DEFAULT_RE = /\A(?:\(N?('.*')\)|\(\((-?\d+(?:\.\d+)?)\)\))\z/
+      FOREIGN_KEY_ACTION_MAP = {0 => :no_action, 1 => :cascade, 2 => :set_null, 3 => :set_default}.freeze
+
+      include Sequel::Database::SplitAlterTable
       
       # Whether to use N'' to quote strings, which allows unicode characters inside the
       # strings.  True by default for compatibility, can be set to false for a possible
       # performance increase.  This sets the default for all datasets created from this
       # Database object.
-      attr_accessor :mssql_unicode_strings
+      attr_reader :mssql_unicode_strings
+
+      def mssql_unicode_strings=(v)
+        @mssql_unicode_strings = v
+        reset_default_dataset
+      end
 
       # The types to check for 0 scale to transform :decimal types
       # to :integer.
@@ -33,8 +44,49 @@ module Sequel
         false
       end
 
+      # Return foreign key information using the system views, including
+      # :name, :on_delete, and :on_update entries in the hashes.
+      def foreign_key_list(table, opts=OPTS)
+        m = output_identifier_meth
+        im = input_identifier_meth
+        schema, table = schema_and_table(table)
+        current_schema = m.call(get(Sequel.function('schema_name')))
+        fk_action_map = FOREIGN_KEY_ACTION_MAP
+        ds = metadata_dataset.from(:sys__foreign_keys___fk).
+          join(:sys__foreign_key_columns___fkc, :constraint_object_id => :object_id).
+          join(:sys__all_columns___pc, :object_id => :fkc__parent_object_id,     :column_id => :fkc__parent_column_id).
+          join(:sys__all_columns___rc, :object_id => :fkc__referenced_object_id, :column_id => :fkc__referenced_column_id).
+          where{{object_schema_name(:fk__parent_object_id) => im.call(schema || current_schema)}}.
+          where{{object_name(:fk__parent_object_id) => im.call(table)}}.
+          select{[:fk__name, 
+                  :fk__delete_referential_action, 
+                  :fk__update_referential_action, 
+                  :pc__name___column, 
+                  :rc__name___referenced_column, 
+                  object_schema_name(:fk__referenced_object_id).as(:schema), 
+                  object_name(:fk__referenced_object_id).as(:table)]}.
+          order(:name, :fkc__constraint_column_id)
+        h = {}
+        ds.each do |row|
+          if r = h[row[:name]]
+            r[:columns] << m.call(row[:column])
+            r[:key] << m.call(row[:referenced_column])
+          else
+            referenced_schema = m.call(row[:schema])
+            referenced_table = m.call(row[:table])
+            h[row[:name]] = { :name      => m.call(row[:name]), 
+                              :table     => (referenced_schema == current_schema) ? referenced_table : :"#{referenced_schema}__#{referenced_table}",
+                              :columns   => [m.call(row[:column])], 
+                              :key       => [m.call(row[:referenced_column])], 
+                              :on_update => fk_action_map[row[:update_referential_action]], 
+                              :on_delete => fk_action_map[row[:delete_referential_action]] }
+          end
+        end
+        h.values
+      end
+
       # Use the system tables to get index information
-      def indexes(table, opts={})
+      def indexes(table, opts=OPTS)
         m = output_identifier_meth
         im = input_identifier_meth
         indexes = {}
@@ -84,18 +136,26 @@ module Sequel
 
       # Microsoft SQL Server supports using the INFORMATION_SCHEMA to get
       # information on tables.
-      def tables(opts={})
+      def tables(opts=OPTS)
         information_schema_tables('BASE TABLE', opts)
       end
 
       # Microsoft SQL Server supports using the INFORMATION_SCHEMA to get
       # information on views.
-      def views(opts={})
+      def views(opts=OPTS)
         information_schema_tables('VIEW', opts)
       end
-
+      
       private
       
+      # Add dropping of the default constraint to the list of SQL queries.
+      # This is necessary before dropping the column or changing its type.
+      def add_drop_default_constraint_sql(sqls, table, column)
+        if constraint = default_constraint_name(table, column)
+          sqls << "ALTER TABLE #{quote_schema_table(table)} DROP CONSTRAINT #{constraint}"
+        end
+      end
+
       # MSSQL uses the IDENTITY(1,1) column for autoincrementing columns.
       def auto_increment_sql
         AUTO_INCREMENT
@@ -106,6 +166,10 @@ module Sequel
         case op[:op]
         when :add_column
           "ALTER TABLE #{quote_schema_table(table)} ADD #{column_definition_sql(op)}"
+        when :drop_column
+          sqls = []
+          add_drop_default_constraint_sql(sqls, table, op[:name])
+          sqls << super
         when :rename_column
           "sp_rename #{literal("#{quote_schema_table(table)}.#{quote_identifier(op[:name])}")}, #{literal(op[:new_name].to_s)}, 'COLUMN'"
         when :set_column_type
@@ -113,9 +177,7 @@ module Sequel
           if sch = schema(table)
             if cs = sch.each{|k, v| break v if k == op[:name]; nil}
               cs = cs.dup
-              if constraint = default_constraint_name(table, op[:name])
-                sqls << "ALTER TABLE #{quote_schema_table(table)} DROP CONSTRAINT #{constraint}"
-              end
+              add_drop_default_constraint_sql(sqls, table, op[:name])
               cs[:default] = cs[:ruby_default]
               op = cs.merge!(op)
               default = op.delete(:default)
@@ -148,9 +210,17 @@ module Sequel
         SQL_BEGIN
       end
       
+      # Handle MSSQL specific default format.
+      def column_schema_normalize_default(default, type)
+        if m = MSSQL_DEFAULT_RE.match(default)
+          default = m[1] || m[2]
+        end
+        super(default, type)
+      end
+
       # Commit the active transaction on the connection, does not commit/release
       # savepoints.
-      def commit_transaction(conn, opts={})
+      def commit_transaction(conn, opts=OPTS)
         log_connection_execute(conn, commit_transaction_sql) unless _trans(conn)[:savepoint_level] > 1
       end
 
@@ -175,16 +245,27 @@ module Sequel
         run(ds.into(name).sql)
       end
     
+      DATABASE_ERROR_REGEXPS = {
+        /Violation of UNIQUE KEY constraint/ => UniqueConstraintViolation,
+        /conflicted with the (FOREIGN KEY.*|REFERENCE) constraint/ => ForeignKeyConstraintViolation,
+        /conflicted with the CHECK constraint/ => CheckConstraintViolation,
+        /column does not allow nulls/ => NotNullConstraintViolation,
+        /was deadlocked on lock resources with another process and has been chosen as the deadlock victim/ => SerializationFailure,
+      }.freeze
+      def database_error_regexps
+        DATABASE_ERROR_REGEXPS
+      end
+
       # The name of the constraint for setting the default value on the table and column.
-      def default_constraint_name(table, column)
-        from(:sysobjects___c_obj).
-          join(:syscomments___com, :id=>:id).
-          join(:sysobjects___t_obj, :id=>:c_obj__parent_obj).
-          join(:sysconstraints___con, :constid=>:c_obj__id).
-          join(:syscolumns___col, :id=>:t_obj__id, :colid=>:colid).
-          where{{c_obj__uid=>user_id{}}}.
-          where(:c_obj__xtype=>'D', :t_obj__name=>table.to_s, :col__name=>column.to_s).
-          get(:c_obj__name)
+      # The SQL used to select default constraints utilizes MSSQL catalog views which were introduced in 2005.
+      # This method intentionally does not support MSSQL 2000.
+      def default_constraint_name(table, column_name)
+        if server_version >= 9000000
+          table_name = schema_and_table(table).compact.join('.')
+          self[:sys__default_constraints].
+            where{{:parent_object_id => Sequel::SQL::Function.new(:object_id, table_name), col_name(:parent_object_id, :parent_column_id) => column_name.to_s}}.
+            get(:name)
+        end
       end
 
       # The SQL to drop an index for the table.
@@ -207,7 +288,7 @@ module Sequel
         m = output_identifier_meth
         metadata_dataset.from(:information_schema__tables___t).
           select(:table_name).
-          filter(:table_type=>type, :table_schema=>(opts[:schema]||default_schema||'dbo').to_s).
+          filter(:table_type=>type, :table_schema=>(opts[:schema]||'dbo').to_s).
           map{|x| m.call(x[:table_name])}
       end
 
@@ -236,8 +317,10 @@ module Sequel
       # The closest MSSQL equivalent of a boolean datatype is the bit type.
       def schema_column_type(db_type)
         case db_type
-        when /\A(bit)\z/io
+        when /\A(?:bit)\z/io
           :boolean
+        when /\A(?:(?:small)?money)\z/io
+          :decimal
         else
           super
         end
@@ -251,19 +334,23 @@ module Sequel
         m2 = input_identifier_meth(opts[:dataset])
         tn = m2.call(table_name.to_s)
         table_id = get{object_id(tn)}
-        pk_index_id = metadata_dataset.from(:sysindexes).
+        info_sch_sch = opts[:information_schema_schema]
+        inf_sch_qual = lambda{|s| info_sch_sch ? Sequel.qualify(info_sch_sch, s) : Sequel.expr(s)}
+        sys_qual = lambda{|s| info_sch_sch ? Sequel.qualify(info_sch_sch, Sequel.qualify(Sequel.lit(''), s)) : Sequel.expr(s)}
+
+        pk_index_id = metadata_dataset.from(sys_qual.call(:sysindexes)).
           where(:id=>table_id, :indid=>1..254){{(status & 2048)=>2048}}.
           get(:indid)
-        pk_cols = metadata_dataset.from(:sysindexkeys___sik).
-          join(:syscolumns___sc, :id=>:id, :colid=>:colid).
+        pk_cols = metadata_dataset.from(sys_qual.call(:sysindexkeys).as(:sik)).
+          join(sys_qual.call(:syscolumns).as(:sc), :id=>:id, :colid=>:colid).
           where(:sik__id=>table_id, :sik__indid=>pk_index_id).
           select_order_map(:sc__name)
-        ds = metadata_dataset.from(:information_schema__tables___t).
-         join(:information_schema__columns___c, :table_catalog=>:table_catalog,
+        ds = metadata_dataset.from(inf_sch_qual.call(:information_schema__tables).as(:t)).
+         join(inf_sch_qual.call(:information_schema__columns).as(:c), :table_catalog=>:table_catalog,
               :table_schema => :table_schema, :table_name => :table_name).
          select(:column_name___column, :data_type___db_type, :character_maximum_length___max_chars, :column_default___default, :is_nullable___allow_null, :numeric_precision___column_size, :numeric_scale___scale).
          filter(:c__table_name=>tn)
-        if schema = opts[:schema] || default_schema
+        if schema = opts[:schema]
           ds.filter!(:c__table_schema=>schema)
         end
         ds.map do |row|
@@ -316,7 +403,8 @@ module Sequel
       DELETE_CLAUSE_METHODS = Dataset.clause_methods(:delete, %w'with delete from output from2 where')
       INSERT_CLAUSE_METHODS = Dataset.clause_methods(:insert, %w'with insert into columns output values')
       SELECT_CLAUSE_METHODS = Dataset.clause_methods(:select, %w'with select distinct limit columns into from lock join where group having order compounds')
-      UPDATE_CLAUSE_METHODS = Dataset.clause_methods(:update, %w'with update table set output from where')
+      UPDATE_CLAUSE_METHODS = Dataset.clause_methods(:update, %w'with update limit table set output from where')
+      UPDATE_CLAUSE_METHODS_2000 = Dataset.clause_methods(:update, %w'update table set output from where')
       NOLOCK = ' WITH (NOLOCK)'.freeze
       UPDLOCK = ' WITH (UPDLOCK)'.freeze
       WILDCARD = LiteralString.new('*').freeze
@@ -333,6 +421,7 @@ module Sequel
       APOS_RE = Dataset::APOS_RE
       DOUBLE_APOS = Dataset::DOUBLE_APOS
       INTO = Dataset::INTO
+      DOUBLE_BRACKET_CLOSE = ']]'.freeze
       DATEPART_SECOND_OPEN = "CAST((datepart(".freeze
       DATEPART_SECOND_MIDDLE = ') + datepart(ns, '.freeze
       DATEPART_SECOND_CLOSE = ")/1000000000.0) AS double precision)".freeze
@@ -353,14 +442,17 @@ module Sequel
       CASE_INSENSITIVE_COLLATION = 'Latin1_General_CI_AS'.freeze
       DEFAULT_TIMESTAMP_FORMAT = "'%Y-%m-%dT%H:%M:%S%N%z'".freeze
       FORMAT_DATE = "'%Y%m%d'".freeze
+      CROSS_APPLY = 'CROSS APPLY'.freeze
+      OUTER_APPLY = 'OUTER APPLY'.freeze
+
+      Sequel::Dataset.def_mutation_method(:disable_insert_output, :output, :module=>self)
 
       # Allow overriding of the mssql_unicode_strings option at the dataset level.
-      attr_accessor :mssql_unicode_strings
+      attr_writer :mssql_unicode_strings
 
-      # Copy the mssql_unicode_strings option from the +db+ object.
-      def initialize(db, opts={})
-        super
-        @mssql_unicode_strings = db.mssql_unicode_strings
+      # Use the database's mssql_unicode_strings setting if the dataset hasn't overridden it.
+      def mssql_unicode_strings
+        defined?(@mssql_unicode_strings) ? @mssql_unicode_strings : (@mssql_unicode_strings = db.mssql_unicode_strings)
       end
 
       # MSSQL uses + for string concatenation, and LIKE is case insensitive by default.
@@ -401,18 +493,38 @@ module Sequel
         end
       end
       
+      # Uses CROSS APPLY to join the given table into the current dataset.
+      def cross_apply(table)
+        join_table(:cross_apply, table)
+      end
+
       # Disable the use of INSERT OUTPUT
       def disable_insert_output
         clone(:disable_insert_output=>true)
       end
 
-      # Disable the use of INSERT OUTPUT, modifying the receiver
-      def disable_insert_output!
-        mutation_method(:disable_insert_output)
+      # MSSQL treats [] as a metacharacter in LIKE expresions.
+      def escape_like(string)
+        string.gsub(/[\\%_\[\]]/){|m| "\\#{m}"}
       end
-
+   
+      # There is no function on Microsoft SQL Server that does character length
+      # and respects trailing spaces (datalength respects trailing spaces, but
+      # counts bytes instead of characters).  Use a hack to work around the
+      # trailing spaces issue.
+      def emulated_function_sql_append(sql, f)
+        case f.f
+        when :char_length
+          literal_append(sql, SQL::Function.new(:len, Sequel.join([f.args.first, 'x'])) - 1)
+        when :trim
+          literal_append(sql, SQL::Function.new(:ltrim, SQL::Function.new(:rtrim, f.args.first)))
+        else
+          super
+        end
+      end
+      
       # MSSQL uses the CONTAINS keyword for full text search
-      def full_text_search(cols, terms, opts = {})
+      def full_text_search(cols, terms, opts = OPTS)
         terms = "\"#{terms.join('" OR "')}\"" if terms.is_a?(Array)
         filter("CONTAINS (?, ?)", cols, terms)
       end
@@ -447,6 +559,11 @@ module Sequel
         lock_style(:dirty)
       end
 
+      # Uses OUTER APPLY to join the given table into the current dataset.
+      def outer_apply(table)
+        join_table(:outer_apply, table)
+      end
+
       # Include an OUTPUT clause in the eventual INSERT, UPDATE, or DELETE query.
       #
       # The first argument is the table to output into, and the second argument
@@ -472,15 +589,9 @@ module Sequel
         clone({:output => output})
       end
 
-      # An output method that modifies the receiver.
-      def output!(into, values)
-        mutation_method(:output, into, values)
-      end
-
-      # MSSQL uses [] to quote identifiers.  MSSQL does not support
-      # escaping of ], so you cannot use that character in an identifier.
+      # MSSQL uses [] to quote identifiers.
       def quoted_identifier_append(sql, name)
-        sql << BRACKET_OPEN << name.to_s << BRACKET_CLOSE
+        sql << BRACKET_OPEN << name.to_s.gsub(/\]/, DOUBLE_BRACKET_CLOSE) << BRACKET_CLOSE
       end
       
       # The version of the database server.
@@ -549,7 +660,7 @@ module Sequel
       # dataset.  If OUTPUT is already set, use existing returning values.  If OUTPUT
       # is only set to return a single columns, return an array of just that column.
       # Otherwise, return an array of hashes.
-      def _import(columns, values, opts={})
+      def _import(columns, values, opts=OPTS)
         if opts[:return] == :primary_key && !@opts[:output]
           output(nil, [SQL::QualifiedIdentifier.new(:inserted, first_primary_key)])._import(columns, values, opts)
         elsif @opts[:output]
@@ -636,6 +747,18 @@ module Sequel
         end
       end
 
+      # Handle CROSS APPLY and OUTER APPLY JOIN types
+      def join_type_sql(join_type)
+        case join_type
+        when :cross_apply
+          CROSS_APPLY
+        when :outer_apply
+          OUTER_APPLY
+        else
+          super
+        end
+      end
+
       # MSSQL uses a literal hexidecimal number for blob strings
       def literal_blob_append(sql, v)
         sql << HEX_START << v.unpack(HSTAR).first
@@ -690,6 +813,7 @@ module Sequel
           end
         end
       end
+      alias update_limit_sql select_limit_sql
 
       # Support different types of locking styles
       def select_lock_sql(sql)
@@ -722,10 +846,15 @@ module Sequel
       alias delete_output_sql output_sql
       alias update_output_sql output_sql
 
-      # MSSQL supports the OUTPUT clause for UPDATE statements.
-      # It also allows prepending a WITH clause.
+      # MSSQL supports the OUTPUT and TOP clause for UPDATE statements.
+      # It also allows prepending a WITH clause.  For MSSQL 2000
+      # and below, exclude WITH and TOP.
       def update_clause_methods
-        UPDATE_CLAUSE_METHODS
+        if is_2005_or_later?
+          UPDATE_CLAUSE_METHODS
+        else
+          UPDATE_CLAUSE_METHODS_2000
+        end
       end
 
       # Only include the primary table in the main update clause
