@@ -1,3 +1,6 @@
+Sequel.require 'adapters/utils/split_alter_table'
+Sequel.require 'adapters/utils/replace'
+
 module Sequel
   Dataset::NON_SQL_OPTIONS << :insert_ignore
   Dataset::NON_SQL_OPTIONS << :update_ignore
@@ -29,10 +32,15 @@ module Sequel
     # Methods shared by Database instances that connect to MySQL,
     # currently supported by the native and JDBC adapters.
     module DatabaseMethods
+      extend Sequel::Database::ResetIdentifierMangling
+
       AUTO_INCREMENT = 'AUTO_INCREMENT'.freeze
       CAST_TYPES = {String=>:CHAR, Integer=>:SIGNED, Time=>:DATETIME, DateTime=>:DATETIME, Numeric=>:DECIMAL, BigDecimal=>:DECIMAL, File=>:BINARY}
       COLUMN_DEFINITION_ORDER = [:collate, :null, :default, :unique, :primary_key, :auto_increment, :references]
       PRIMARY = 'PRIMARY'.freeze
+      MYSQL_TIMESTAMP_RE = /\ACURRENT_(?:DATE|TIMESTAMP)?\z/
+
+      include Sequel::Database::SplitAlterTable
       
       # MySQL's cast rules are restrictive in that you can't just cast to any possible
       # database type.
@@ -54,12 +62,12 @@ module Sequel
       # Use the Information Schema's KEY_COLUMN_USAGE table to get
       # basic information on foreign key columns, but include the
       # constraint name.
-      def foreign_key_list(table, opts={})
+      def foreign_key_list(table, opts=OPTS)
         m = output_identifier_meth
         im = input_identifier_meth
         ds = metadata_dataset.
           from(:INFORMATION_SCHEMA__KEY_COLUMN_USAGE).
-          where(:TABLE_NAME=>im.call(table)).
+          where(:TABLE_NAME=>im.call(table), :TABLE_SCHEMA=>Sequel.function(:DATABASE)).
           exclude(:CONSTRAINT_NAME=>'PRIMARY').
           exclude(:REFERENCED_TABLE_NAME=>nil).
           select(:CONSTRAINT_NAME___name, :COLUMN_NAME___column, :REFERENCED_TABLE_NAME___table, :REFERENCED_COLUMN_NAME___key)
@@ -86,7 +94,7 @@ module Sequel
       #
       # By default partial indexes are not included, you can use the
       # option :partial to override this.
-      def indexes(table, opts={})
+      def indexes(table, opts=OPTS)
         indexes = {}
         remove_indexes = []
         m = output_identifier_meth
@@ -110,8 +118,10 @@ module Sequel
 
       # Get version of MySQL server, used for determined capabilities.
       def server_version
-        m = /(\d+)\.(\d+)\.(\d+)/.match(get(SQL::Function.new(:version)))
-        @server_version ||= (m[1].to_i * 10000) + (m[2].to_i * 100) + m[3].to_i
+        @server_version ||= begin
+          m = /(\d+)\.(\d+)\.(\d+)/.match(get(SQL::Function.new(:version)))
+          (m[1].to_i * 10000) + (m[2].to_i * 100) + m[3].to_i
+        end
       end
       
       # MySQL supports CREATE TABLE IF NOT EXISTS syntax.
@@ -144,7 +154,7 @@ module Sequel
       #
       # Options:
       # * :server - Set the server to use
-      def tables(opts={})
+      def tables(opts=OPTS)
         full_tables('BASE TABLE', opts)
       end
       
@@ -161,24 +171,23 @@ module Sequel
       #
       # Options:
       # * :server - Set the server to use
-      def views(opts={})
+      def views(opts=OPTS)
         full_tables('VIEW', opts)
       end
       
       private
       
-      # Use MySQL specific syntax for rename column, set column type, and
-      # drop index cases.
-      def alter_table_sql(table, op)
+      # Use MySQL specific syntax for some alter table operations.
+      def alter_table_op_sql(table, op)
         case op[:op]
         when :add_column
           if related = op.delete(:table)
-            sql = super(table, op)
+            sql = super
             op[:table] = related
             op[:key] ||= primary_key_from_schema(related)
-            [sql, "ALTER TABLE #{quote_schema_table(table)} ADD FOREIGN KEY (#{quote_identifier(op[:name])})#{column_references_sql(op)}"]
+            sql << ", ADD FOREIGN KEY (#{quote_identifier(op[:name])})#{column_references_sql(op)}"
           else
-            super(table, op)
+            super
           end
         when :rename_column, :set_column_type, :set_column_null, :set_column_default
           o = op[:op]
@@ -189,31 +198,66 @@ module Sequel
           opts[:null] = o == :set_column_null ? op[:null] : opts[:allow_null]
           opts[:default] = o == :set_column_default ? op[:default] : opts[:ruby_default]
           opts.delete(:default) if opts[:default] == nil
-          "ALTER TABLE #{quote_schema_table(table)} CHANGE COLUMN #{quote_identifier(op[:name])} #{column_definition_sql(op.merge(opts))}"
-        when :drop_index
-          "#{drop_index_sql(table, op)} ON #{quote_schema_table(table)}"
+          opts.delete(:primary_key)
+          unless op[:type] || opts[:type]
+            raise Error, "cannot determine database type to use for CHANGE COLUMN operation"
+          end
+          opts = op.merge(opts)
+          opts.delete(:auto_increment) if op[:auto_increment] == false
+          "CHANGE COLUMN #{quote_identifier(op[:name])} #{column_definition_sql(opts)}"
         when :drop_constraint
-          type = case op[:type]
+          case op[:type]
           when :primary_key
-            return "ALTER TABLE #{quote_schema_table(table)} DROP PRIMARY KEY"
+            "DROP PRIMARY KEY"
           when :foreign_key
-            'FOREIGN KEY'
+            name = op[:name] || foreign_key_name(table, op[:columns])
+            "DROP FOREIGN KEY #{quote_identifier(name)}"
           when :unique
-            'INDEX'
-          else
-            raise(Error, "must specify constraint type via :type=>(:foreign_key|:primary_key|:unique) when dropping constraints on MySQL")
+            "DROP INDEX #{quote_identifier(op[:name])}"
           end
-          "ALTER TABLE #{quote_schema_table(table)} DROP #{type} #{quote_identifier(op[:name])}"
         when :add_constraint
           if op[:type] == :foreign_key
             op[:key] ||= primary_key_from_schema(op[:table])
           end
-          super(table, op)
+          super
         else
-          super(table, op)
+          super
         end
       end
 
+      # MySQL server requires table names when dropping indexes.
+      def alter_table_sql(table, op)
+        case op[:op]
+        when :drop_index
+          "#{drop_index_sql(table, op)} ON #{quote_schema_table(table)}"
+        when :drop_constraint
+          if op[:type] == :primary_key
+            if (pk = primary_key_from_schema(table)).length == 1
+              return [alter_table_sql(table, {:op=>:rename_column, :name=>pk.first, :new_name=>pk.first, :auto_increment=>false}), super]
+            end
+          end
+          super
+        else
+          super
+        end
+      end
+
+      # Handle MySQL specific default format.
+      def column_schema_normalize_default(default, type)
+        if column_schema_default_string_type?(type)
+          return if [:date, :datetime, :time].include?(type) && MYSQL_TIMESTAMP_RE.match(default)
+          default = "'#{default.gsub("'", "''").gsub('\\', '\\\\')}'"
+        end
+        super(default, type)
+      end
+
+      # Don't allow combining adding foreign key operations with other
+      # operations, since in some cases adding a foreign key constraint in
+      # the same query as other operations results in MySQL error 150.
+      def combinable_alter_table_op?(op)
+        super && !(op[:op] == :add_constraint && op[:type] == :foreign_key) && !(op[:op] == :drop_constraint && op[:type] == :primary_key)
+      end
+
       # The SQL queries to execute on initial connection
       def mysql_connection_setting_sqls
         sqls = []
@@ -247,7 +291,7 @@ module Sequel
 
       # Use XA START to start a new prepared transaction if the :prepare
       # option is given.
-      def begin_transaction(conn, opts={})
+      def begin_transaction(conn, opts=OPTS)
         if (s = opts[:prepare]) && (th = _trans(conn))[:savepoint_level] == 0
           log_connection_execute(conn, "XA START #{literal(s)}")
           th[:savepoint_level] += 1
@@ -270,7 +314,7 @@ module Sequel
       
       # Prepare the XA transaction for a two-phase commit if the
       # :prepare option is given.
-      def commit_transaction(conn, opts={})
+      def commit_transaction(conn, opts=OPTS)
         if (s = opts[:prepare]) && _trans(conn)[:savepoint_level] <= 1
           log_connection_execute(conn, "XA END #{literal(s)}")
           log_connection_execute(conn, "XA PREPARE #{literal(s)}")
@@ -280,12 +324,13 @@ module Sequel
       end
 
       # Use MySQL specific syntax for engine type and character encoding
-      def create_table_sql(name, generator, options = {})
+      def create_table_sql(name, generator, options = OPTS)
         engine = options.fetch(:engine, Sequel::MySQL.default_engine)
         charset = options.fetch(:charset, Sequel::MySQL.default_charset)
         collate = options.fetch(:collate, Sequel::MySQL.default_collate)
         generator.constraints.sort_by{|c| (c[:type] == :primary_key) ? -1 : 1}
 
+        # Proc for figuring out the primary key for a given table.
         key_proc = lambda do |t|
           if t == name 
             if pk = generator.primary_key_name
@@ -298,12 +343,17 @@ module Sequel
           end
         end
 
+        # Manually set the keys, since MySQL requires one, it doesn't use the primary
+        # key if none are specified.
         generator.constraints.each do |c|
           if c[:type] == :foreign_key
             c[:key] ||= key_proc.call(c[:table])
           end
         end
 
+        # Split column constraints into table constraints in some cases:
+        # * foreign key - Always
+        # * unique, primary_key - Only if constraint has a name
         generator.columns.each do |c|
           if t = c.delete(:table)
             same_table = t == name
@@ -315,13 +365,23 @@ module Sequel
               generator.constraints.unshift(:type=>:unique, :columns=>Array(k))
             end
 
-            generator.foreign_key([c[:name]], t, c.merge(:name=>nil, :type=>:foreign_key, :key=>key))
+            generator.foreign_key([c[:name]], t, c.merge(:name=>c[:foreign_key_constraint_name], :type=>:foreign_key, :key=>key))
           end
         end
 
         "#{super}#{" ENGINE=#{engine}" if engine}#{" DEFAULT CHARSET=#{charset}" if charset}#{" DEFAULT COLLATE=#{collate}" if collate}"
       end
 
+      DATABASE_ERROR_REGEXPS = {
+        /Duplicate entry .+ for key/ => UniqueConstraintViolation,
+        /foreign key constraint fails/ => ForeignKeyConstraintViolation,
+        /cannot be null/ => NotNullConstraintViolation,
+        /Deadlock found when trying to get lock; try restarting transaction/ => SerializationFailure,
+      }.freeze
+      def database_error_regexps
+        DATABASE_ERROR_REGEXPS
+      end
+
       # Backbone of the tables and views support using SHOW FULL TABLES.
       def full_tables(type, opts)
         m = output_identifier_meth
@@ -359,7 +419,7 @@ module Sequel
       end
 
       # Rollback the currently open XA transaction
-      def rollback_transaction(conn, opts={})
+      def rollback_transaction(conn, opts=OPTS)
         if (s = opts[:prepare]) && _trans(conn)[:savepoint_level] <= 1
           log_connection_execute(conn, "XA END #{literal(s)}")
           log_connection_execute(conn, "XA PREPARE #{literal(s)}")
@@ -369,9 +429,18 @@ module Sequel
         end
       end
 
-      # MySQL treats integer primary keys as autoincrementing.
-      def schema_autoincrementing_primary_key?(schema)
-        super and schema[:db_type] =~ /int/io
+      # Recognize MySQL set type.
+      def schema_column_type(db_type)
+        case db_type
+        when /\Aset/io
+          :set
+        when /\Amediumint/io
+          :integer
+        when /\Amediumtext/io
+          :string
+        else
+          super
+        end
       end
 
       # Use the MySQL specific DESCRIBE syntax to get a table description.
@@ -392,6 +461,21 @@ module Sequel
         end
       end
 
+      # MySQL can combine multiple alter table ops into a single query.
+      def supports_combining_alter_table_ops?
+        true
+      end
+
+      # MySQL supports CREATE OR REPLACE VIEW.
+      def supports_create_or_replace_view?
+        true
+      end
+
+      # MySQL does not support named column constraints.
+      def supports_named_column_constraints?
+        false
+      end
+
       # Respect the :size option if given to produce
       # tinyblob, mediumblob, and longblob if :tiny,
       # :medium, or :long is given.
@@ -449,10 +533,11 @@ module Sequel
       PAREN_CLOSE = Dataset::PAREN_CLOSE
       NOT_SPACE = Dataset::NOT_SPACE
       FROM = Dataset::FROM
-      INSERT = Dataset::INSERT
       COMMA = Dataset::COMMA
       LIMIT = Dataset::LIMIT
       GROUP_BY = Dataset::GROUP_BY
+      ESCAPE = Dataset::ESCAPE
+      BACKSLASH = Dataset::BACKSLASH
       REGEXP = 'REGEXP'.freeze
       LIKE = 'LIKE'.freeze
       BINARY = 'BINARY '.freeze
@@ -467,7 +552,6 @@ module Sequel
       EMPTY_COLUMNS = " ()".freeze
       EMPTY_VALUES = " VALUES ()".freeze
       IGNORE = " IGNORE".freeze
-      REPLACE = 'REPLACE'.freeze
       ON_DUPLICATE_KEY_UPDATE = " ON DUPLICATE KEY UPDATE ".freeze
       EQ_VALUES = '=VALUES('.freeze
       EQ = '='.freeze
@@ -478,7 +562,11 @@ module Sequel
       EXPLAIN_EXTENDED = 'EXPLAIN EXTENDED '.freeze
       BACKSLASH_RE = /\\/.freeze
       QUAD_BACKSLASH = "\\\\\\\\".freeze
-      
+      BLOB_START = "0x".freeze
+      HSTAR = "H*".freeze
+
+      include Sequel::Dataset::Replace
+
       # MySQL specific syntax for LIKE/REGEXP searches, as well as
       # string concatenation.
       def complex_expression_sql_append(sql, op, args)
@@ -499,6 +587,10 @@ module Sequel
           sql << SPACE
           sql << BINARY if [:~, :'!~', :LIKE, :'NOT LIKE'].include?(op)
           literal_append(sql, args.at(1))
+          if [:LIKE, :'NOT LIKE', :ILIKE, :'NOT ILIKE'].include?(op)
+            sql << ESCAPE
+            literal_append(sql, BACKSLASH)
+          end
           sql << PAREN_CLOSE
         when :'||'
           if args.length > 1
@@ -531,7 +623,7 @@ module Sequel
       
       # Return the results of an EXPLAIN query as a string. Options:
       # :extended :: Use EXPLAIN EXPTENDED instead of EXPLAIN if true.
-      def explain(opts={})
+      def explain(opts=OPTS)
         # Load the PrettyTable class, needed for explain output
         Sequel.extension(:_pretty_table) unless defined?(Sequel::PrettyTable)
 
@@ -546,27 +638,22 @@ module Sequel
       end
 
       # Adds full text filter
-      def full_text_search(cols, terms, opts = {})
+      def full_text_search(cols, terms, opts = OPTS)
         filter(full_text_sql(cols, terms, opts))
       end
       
       # MySQL specific full text search syntax.
-      def full_text_sql(cols, terms, opts = {})
+      def full_text_sql(cols, terms, opts = OPTS)
         terms = terms.join(' ') if terms.is_a?(Array)
         SQL::PlaceholderLiteralString.new((opts[:boolean] ? MATCH_AGAINST_BOOLEAN : MATCH_AGAINST), [Array(cols), terms])
       end
 
-      # MySQL allows HAVING clause on ungrouped datasets.
-      def having(*cond, &block)
-        _filter(:having, *cond, &block)
-      end
-      
       # Transforms an CROSS JOIN to an INNER JOIN if the expr is not nil.
       # Raises an error on use of :full_outer type, since MySQL doesn't support it.
-      def join_table(type, table, expr=nil, table_alias={}, &block)
+      def join_table(type, table, expr=nil, opts=OPTS, &block)
         type = :inner if (type == :cross) && !expr.nil?
         raise(Sequel::Error, "MySQL doesn't support FULL OUTER JOIN") if type == :full_outer
-        super(type, table, expr, table_alias, &block)
+        super(type, table, expr, opts, &block)
       end
       
       # Transforms :natural_inner to NATURAL LEFT JOIN and straight to
@@ -628,23 +715,7 @@ module Sequel
       def quoted_identifier_append(sql, c)
         sql << BACKTICK << c.to_s.gsub(BACKTICK_RE, DOUBLE_BACKTICK) << BACKTICK
       end
-      
-      # Execute a REPLACE statement on the database.
-      def replace(*values)
-        execute_insert(replace_sql(*values))
-      end
 
-      # MySQL specific syntax for REPLACE (aka UPSERT, or update if exists,
-      # insert if it doesn't).
-      def replace_sql(*values)
-        clone(:replace=>true).insert_sql(*values)
-      end
-
-      # Replace multiple rows in a single query.
-      def multi_replace(*values)
-        clone(:replace=>true).multi_insert(*values)
-      end
-      
       # MySQL can emulate DISTINCT ON with its non-standard GROUP BY implementation,
       # though the rows returned cannot be made deterministic through ordering.
       def supports_distinct_on?
@@ -672,6 +743,11 @@ module Sequel
         false
       end
     
+      # MySQL supports pattern matching via regular expressions
+      def supports_regexp?
+        true
+      end
+
       # MySQL does support fractional timestamps in literal timestamps, but it
       # ignores them.  Also, using them seems to cause problems on 1.9.  Since
       # they are ignored anyway, not using them is probably best.
@@ -736,11 +812,6 @@ module Sequel
         sql << IGNORE if opts[:update_ignore]
       end
 
-      # If this is an replace instead of an insert, use replace instead
-      def insert_insert_sql(sql)
-        sql << (@opts[:replace] ? REPLACE : INSERT)
-      end
-
       # MySQL supports INSERT ... ON DUPLICATE KEY UPDATE
       def insert_on_duplicate_key_update_sql(sql)
         if update_cols = opts[:on_duplicate_key_update]
@@ -793,17 +864,31 @@ module Sequel
       def limit_sql(sql)
         if l = @opts[:limit]
           sql << LIMIT
-          literal_append(sql, @opts[:limit])
+          literal_append(sql, l)
         end
       end
       alias delete_limit_sql limit_sql
       alias update_limit_sql limit_sql
 
+      # MySQL uses a preceding X for hex escaping strings
+      def literal_blob_append(sql, v)
+        sql << BLOB_START << v.unpack(HSTAR).first
+      end
+
       # Use 0 for false on MySQL
       def literal_false
         BOOL_FALSE
       end
 
+      # Raise error for infinitate and NaN values
+      def literal_float(v)
+        if v.infinite? || v.nan?
+          raise InvalidValue, "Infinite floats and NaN values are not valid on MySQL"
+        else
+          super
+        end
+      end
+
       # SQL fragment for String.  Doubles \ and ' by default.
       def literal_string_append(sql, v)
         sql << APOS << v.gsub(BACKSLASH_RE, QUAD_BACKSLASH).gsub(APOS_RE, DOUBLE_APOS) << APOS
