@@ -1,6 +1,7 @@
 module Sequel
   module Progress
     module DatabaseMethods
+      extend Sequel::Database::ResetIdentifierMangling
 
       # Progress uses the :progress database type.
       def database_type
@@ -9,7 +10,7 @@ module Sequel
     end
   
     module DatasetMethods
-      SELECT_CLAUSE_METHODS = Dataset.clause_methods(:select, %w'limit distinct columns from join where group order having compounds')
+      SELECT_CLAUSE_METHODS = Dataset.clause_methods(:select, %w'select limit distinct columns from join where group order having compounds')
 
       # Progress requires SQL standard datetimes
       def requires_sql_standard_datetimes?
