@@ -1,15 +1,25 @@
+Sequel.require 'adapters/utils/replace'
+
 module Sequel
   module SQLite
     # No matter how you connect to SQLite, the following Database options
     # can be used to set PRAGMAs on connections in a thread-safe manner:
     # :auto_vacuum, :foreign_keys, :synchronous, and :temp_store.
     module DatabaseMethods
+      extend Sequel::Database::ResetIdentifierMangling
+
       AUTO_VACUUM = [:none, :full, :incremental].freeze
       PRIMARY_KEY_INDEX_RE = /\Asqlite_autoindex_/.freeze
       SYNCHRONOUS = [:off, :normal, :full].freeze
       TABLES_FILTER = "type = 'table' AND NOT name = 'sqlite_sequence'".freeze
       TEMP_STORE = [:default, :file, :memory].freeze
       VIEWS_FILTER = "type = 'view'".freeze
+      TRANSACTION_MODE = {
+        :deferred => "BEGIN DEFERRED TRANSACTION".freeze,
+        :immediate => "BEGIN IMMEDIATE TRANSACTION".freeze,
+        :exclusive => "BEGIN EXCLUSIVE TRANSACTION".freeze,
+        nil => Sequel::Database::SQL_BEGIN,
+      }.freeze
 
       # Whether to use integers for booleans in the database.  SQLite recommends
       # booleans be stored as integers, but historically Sequel has used 't'/'f'.
@@ -35,6 +45,18 @@ module Sequel
         pragma_set(:case_sensitive_like, !!value ? 'on' : 'off') if sqlite_version >= 30203
       end
 
+      # A symbol signifying the value of the default transaction mode
+      attr_reader :transaction_mode
+
+      # Set the default transaction mode.
+      def transaction_mode=(value)
+        if TRANSACTION_MODE.include?(value)
+          @transaction_mode = value
+        else
+          raise Error, "Invalid value for transaction_mode.  Please specify one of :deferred, :immediate, :exclusive, nil"
+        end
+      end
+
       # SQLite uses the :sqlite database type.
       def database_type
         :sqlite
@@ -55,7 +77,7 @@ module Sequel
 
       # Return the array of foreign key info hashes using the foreign_key_list PRAGMA,
       # including information for the :on_update and :on_delete entries.
-      def foreign_key_list(table, opts={})
+      def foreign_key_list(table, opts=OPTS)
         m = output_identifier_meth
         h = {}
         metadata_dataset.with_sql("PRAGMA foreign_key_list(?)", input_identifier_meth.call(table)).each do |row|
@@ -70,7 +92,7 @@ module Sequel
       end
 
       # Use the index_list and index_info PRAGMAs to determine the indexes on the table.
-      def indexes(table, opts={})
+      def indexes(table, opts=OPTS)
         m = output_identifier_meth
         im = input_identifier_meth
         indexes = {}
@@ -101,7 +123,7 @@ module Sequel
 
       # Set the integer_booleans option using the passed in :integer_boolean option.
       def set_integer_booleans
-        @integer_booleans = typecast_value_boolean(@opts[:integer_booleans])
+        @integer_booleans = @opts.has_key?(:integer_booleans) ? typecast_value_boolean(@opts[:integer_booleans]) : true
       end
       
       # The version of the server as an integer, where 3.6.19 = 30619.
@@ -121,6 +143,11 @@ module Sequel
         sqlite_version >= 30300
       end
       
+      # SQLite 3.6.19+ supports deferrable foreign key constraints.
+      def supports_deferrable_foreign_key_constraints?
+        sqlite_version >= 30619
+      end
+
       # SQLite 3.6.8+ supports savepoints. 
       def supports_savepoints?
         sqlite_version >= 30608
@@ -136,7 +163,7 @@ module Sequel
       # SQLite supports timezones in timestamps, since it just stores them as strings,
       # but it breaks the usage of SQLite's datetime functions.
       def use_timestamp_timezones?
-        defined?(@use_timestamp_timezones) ? @use_timestamp_timezones : (@use_timestamp_timezones = true)
+        defined?(@use_timestamp_timezones) ? @use_timestamp_timezones : (@use_timestamp_timezones = false)
       end
 
       # A symbol signifying the value of the synchronous PRAGMA.
@@ -155,7 +182,7 @@ module Sequel
       #
       # Options:
       # * :server - Set the server to use.
-      def tables(opts={})
+      def tables(opts=OPTS)
         tables_and_views(TABLES_FILTER, opts)
       end
       
@@ -175,7 +202,7 @@ module Sequel
       #
       # Options:
       # * :server - Set the server to use.
-      def views(opts={})
+      def views(opts=OPTS)
         tables_and_views(VIEWS_FILTER, opts)
       end
 
@@ -197,6 +224,7 @@ module Sequel
             ops.each{|op| alter_table_sql_list(table, [op]).flatten.each{|sql| execute_ddl(sql)}}
           end
         end
+      ensure
         self.foreign_keys = true if fks
       end
 
@@ -231,11 +259,13 @@ module Sequel
           when :primary_key
             duplicate_table(table){|columns| columns.each{|s| s[:primary_key] = nil}}
           when :foreign_key
-            duplicate_table(table, :no_foreign_keys=>true)
-          when :unique
-            duplicate_table(table)
+            if op[:columns]
+              duplicate_table(table, :skip_foreign_key_columns=>op[:columns])
+            else
+              duplicate_table(table, :no_foreign_keys=>true)
+            end
           else
-            raise Error, "Unsupported :type option for drop_constraint: #{op[:type].inspect}"
+            duplicate_table(table)
           end
         when :add_constraint
           duplicate_table(table, :constraints=>[op])
@@ -246,8 +276,15 @@ module Sequel
         end
       end
 
+      def begin_new_transaction(conn, opts)
+        mode = opts[:mode] || @transaction_mode
+        sql = TRANSACTION_MODE[mode] or raise Error, "transaction :mode must be one of: :deferred, :immediate, :exclusive, nil"
+        log_connection_execute(conn, sql)
+        set_transaction_isolation(conn, opts)
+      end
+
       # A name to use for the backup table
-      def backup_table_name(table, opts={})
+      def backup_table_name(table, opts=OPTS)
         table = table.gsub('`', '')
         (opts[:times]||1000).times do |i|
           table_name = "#{table}_backup#{i}"
@@ -260,6 +297,12 @@ module Sequel
         sql << " DEFAULT (#{literal(column[:default])})" if column.include?(:default)
       end
     
+      # Add null/not null SQL fragment to column creation SQL.
+      def column_definition_null_sql(sql, column)
+        column = column.merge(:null=>false) if column[:primary_key]
+        super(sql, column)
+      end
+    
       # Array of PRAGMA SQL statements based on the Database options that should be applied to
       # new connections.
       def connection_pragmas
@@ -277,6 +320,21 @@ module Sequel
         ps
       end
 
+      # SQLite support creating temporary views.
+      def create_view_prefix_sql(name, options)
+        "CREATE #{'TEMPORARY 'if options[:temp]}VIEW #{quote_schema_table(name)}"
+      end
+
+      DATABASE_ERROR_REGEXPS = {
+        /is not unique\z/ => UniqueConstraintViolation,
+        /foreign key constraint failed\z/ => ForeignKeyConstraintViolation,
+        /\A(SQLITE ERROR 19 \(CONSTRAINT\) : )?constraint failed\z/ => ConstraintViolation,
+        /may not be NULL\z/ => NotNullConstraintViolation,
+      }.freeze
+      def database_error_regexps
+        DATABASE_ERROR_REGEXPS
+      end
+
       # The array of column schema hashes for the current columns in the table
       def defined_columns_for(table)
         cols = parse_pragma(table, {})
@@ -290,7 +348,7 @@ module Sequel
       # Duplicate an existing table by creating a new table, copying all records
       # from the existing table into the new table, deleting the existing table
       # and renaming the new table to the existing table's name.
-      def duplicate_table(table, opts={})
+      def duplicate_table(table, opts=OPTS)
         remove_cached_schema(table)
         def_columns = defined_columns_for(table)
         old_columns = def_columns.map{|c| c[:name]}
@@ -316,6 +374,12 @@ module Sequel
           if ocp = opts[:old_columns_proc]
             fks.delete_if{|c| ocp.call(c[:columns].dup) != c[:columns]}
           end
+          
+          # Skip any foreign key columns where a constraint for those
+          # foreign keys is being dropped.
+          if sfkc = opts[:skip_foreign_key_columns]
+            fks.delete_if{|c| c[:columns] == sfkc}
+          end
 
           constraints.concat(fks.each{|h| h[:type] = :foreign_key})
         end
@@ -373,7 +437,7 @@ module Sequel
           row.delete(:cid)
           row[:allow_null] = row.delete(:notnull).to_i == 0
           row[:default] = row.delete(:dflt_value)
-          row[:primary_key] = row.delete(:pk).to_i == 1
+          row[:primary_key] = row.delete(:pk).to_i > 0
           row[:default] = nil if blank_object?(row[:default]) || row[:default] == 'NULL'
           row[:db_type] = row.delete(:type)
           row[:type] = schema_column_type(row[:db_type])
@@ -383,7 +447,7 @@ module Sequel
       
       # SQLite treats integer primary keys as autoincrementing (alias of rowid).
       def schema_autoincrementing_primary_key?(schema)
-        super and schema[:db_type].downcase == 'integer'
+        super && schema[:db_type].downcase == 'integer'
       end
 
       # SQLite supports schema parsing using the table_info PRAGMA, so
@@ -411,8 +475,11 @@ module Sequel
     
     # Instance methods for datasets that connect to an SQLite database
     module DatasetMethods
+      include Dataset::Replace
+
       SELECT_CLAUSE_METHODS = Dataset.clause_methods(:select, %w'select distinct columns from join where group having compounds order limit')
       CONSTANT_MAP = {:CURRENT_DATE=>"date(CURRENT_TIMESTAMP, 'localtime')".freeze, :CURRENT_TIMESTAMP=>"datetime(CURRENT_TIMESTAMP, 'localtime')".freeze, :CURRENT_TIME=>"time(CURRENT_TIMESTAMP, 'localtime')".freeze}
+      EMULATED_FUNCTION_MAP = {:char_length=>'length'.freeze}
       EXTRACT_MAP = {:year=>"'%Y'", :month=>"'%m'", :day=>"'%d'", :hour=>"'%H'", :minute=>"'%M'", :second=>"'%f'"}
       NOT_SPACE = Dataset::NOT_SPACE
       COMMA = Dataset::COMMA
@@ -428,16 +495,27 @@ module Sequel
       DOUBLE_BACKTICK = '``'.freeze
       BLOB_START = "X'".freeze
       HSTAR = "H*".freeze
+      DATE_OPEN = "date(".freeze
+      DATETIME_OPEN = "datetime(".freeze
 
-      # SQLite does not support pattern matching via regular expressions.
-      # SQLite is case insensitive (depending on pragma), so use LIKE for
-      # ILIKE.
+      def cast_sql_append(sql, expr, type)
+        if type == Time or type == DateTime
+          sql << DATETIME_OPEN
+          literal_append(sql, expr)
+          sql << PAREN_CLOSE
+        elsif type == Date
+          sql << DATE_OPEN
+          literal_append(sql, expr)
+          sql << PAREN_CLOSE
+        else
+          super
+        end
+      end
+
+      # SQLite doesn't support a NOT LIKE b, you need to use NOT (a LIKE b).
+      # It doesn't support xor or the extract function natively, so those have to be emulated.
       def complex_expression_sql_append(sql, op, args)
         case op
-        when :~, :'!~', :'~*', :'!~*'
-          raise Error, "SQLite does not support pattern matching via regular expressions"
-        when :ILIKE
-          super(sql, :LIKE, args.map{|a| SQL::Function.new(:upper, a)})
         when :"NOT LIKE", :"NOT ILIKE"
           sql << NOT_SPACE
           complex_expression_sql_append(sql, (op == :"NOT ILIKE" ? :ILIKE : :LIKE), args)
@@ -562,7 +640,7 @@ module Sequel
           col
         end
       end
-      
+
       # SQL fragment specifying a list of identifiers
       def identifier_list(columns)
         columns.map{|i| quote_identifier(i)}.join(COMMA)
@@ -588,7 +666,9 @@ module Sequel
         SELECT_CLAUSE_METHODS
       end
       
-      # Support FOR SHARE locking when using the :share lock style.
+      # SQLite does not support FOR UPDATE, but silently ignore it
+      # instead of raising an error for compatibility with other
+      # databases.
       def select_lock_sql(sql)
         super unless @opts[:lock] == :update
       end
