@@ -1,3 +1,4 @@
+require 'swift/db/mysql'
 Sequel.require 'adapters/shared/mysql'
 
 module Sequel
@@ -7,6 +8,7 @@ module Sequel
     module MySQL
       # Database instance methods for MySQL databases accessed via Swift.
       module DatabaseMethods
+        extend Sequel::Database::ResetIdentifierMangling
         include Sequel::MySQL::DatabaseMethods
         
         private
@@ -18,7 +20,7 @@ module Sequel
 
         # Consider tinyint(1) columns as boolean.
         def schema_column_type(db_type)
-          db_type == 'tinyint(1)' ? :boolean : super
+          db_type =~ /\Atinyint\(1\)/ ? :boolean : super
         end
       
         # Apply the connectiong setting SQLs for every new connection.
@@ -37,7 +39,7 @@ module Sequel
         
         # Use Swift's escape method for quoting.
         def literal_string_append(sql, s)
-          sql << APOS << db.synchronize{|c| c.escape(s)} << APOS
+          sql << APOS << db.synchronize(@opts[:server]){|c| c.escape(s)} << APOS
         end
       end
     end
