@@ -14,23 +14,23 @@ class Sequel::ThreadedConnectionPool < Sequel::ConnectionPool
 
   # The following additional options are respected:
   # * :connection_handling - Set how to handle available connections.  By default,
-  #   uses a a stack for performance.  Can be set to :queue to use a queue, which reduces
-  #   the chances of connections becoming stale.
+  #   uses a a queue for fairness.  Can be set to :stack to use a stack, which may
+  #   offer better performance.
   # * :max_connections - The maximum number of connections the connection pool
   #   will open (default 4)
   # * :pool_sleep_time - The amount of time to sleep before attempting to acquire
   #   a connection again (default 0.001)
   # * :pool_timeout - The amount of seconds to wait to acquire a connection
   #   before raising a PoolTimeoutError (default 5)
-  def initialize(opts = {}, &block)
+  def initialize(db, opts = OPTS)
     super
     @max_size = Integer(opts[:max_connections] || 4)
     raise(Sequel::Error, ':max_connections must be positive') if @max_size < 1
     @mutex = Mutex.new  
-    @queue = opts[:connection_handling] == :queue
+    @connection_handling = opts[:connection_handling]
     @available_connections = []
     @allocated = {}
-    @timeout = Integer(opts[:pool_timeout] || 5)
+    @timeout = Float(opts[:pool_timeout] || 5)
     @sleep_time = Float(opts[:pool_sleep_time] || 0.001)
   end
   
@@ -49,7 +49,7 @@ class Sequel::ThreadedConnectionPool < Sequel::ConnectionPool
     hold do |c|
       sync do
         yield c
-        @available_connections.each{|c| yield c}
+        @available_connections.each{|conn| yield conn}
       end
     end
   end
@@ -64,10 +64,9 @@ class Sequel::ThreadedConnectionPool < Sequel::ConnectionPool
   # 
   # Once a connection is requested using #hold, the connection pool
   # creates new connections to the database.
-  def disconnect(opts={}, &block)
-    block ||= @disconnection_proc
+  def disconnect(opts=OPTS)
     sync do
-      @available_connections.each{|conn| block.call(conn)} if block
+      @available_connections.each{|conn| db.disconnect_connection(conn)}
       @available_connections.clear
     end
   end
@@ -106,13 +105,17 @@ class Sequel::ThreadedConnectionPool < Sequel::ConnectionPool
     rescue Sequel::DatabaseDisconnectError
       oconn = conn
       conn = nil
-      @disconnection_proc.call(oconn) if @disconnection_proc && oconn
+      db.disconnect_connection(oconn) if oconn
       @allocated.delete(t)
       raise
     ensure
       sync{release(t)} if conn
     end
   end
+
+  def pool_type
+    :threaded
+  end
   
   private
 
@@ -131,7 +134,14 @@ class Sequel::ThreadedConnectionPool < Sequel::ConnectionPool
   # available, tries to create a new connection. The calling code should already
   # have the mutex before calling this.
   def available
-    @available_connections.pop || make_new(DEFAULT_SERVER)
+    next_available || make_new(DEFAULT_SERVER)
+  end
+
+  # Return a connection to the pool of available connections, returns the connection.
+  # The calling code should already have the mutex before calling this.
+  def checkin_connection(conn)
+    @available_connections << conn
+    conn
   end
 
   # Alias the default make_new method, so subclasses can call it directly.
@@ -147,7 +157,19 @@ class Sequel::ThreadedConnectionPool < Sequel::ConnectionPool
     end
     super if (n || size) < @max_size
   end
-  
+
+  # Return the next available connection in the pool, or nil if there
+  # is not currently an available connection.  The calling code should already
+  # have the mutex before calling this.
+  def next_available
+    case @connection_handling
+    when :stack
+      @available_connections.pop
+    else
+      @available_connections.shift
+    end
+  end
+
   # Returns the connection owned by the supplied thread,
   # if any. The calling code should NOT already have the mutex before calling this.
   def owned_connection(thread)
@@ -157,10 +179,12 @@ class Sequel::ThreadedConnectionPool < Sequel::ConnectionPool
   # Releases the connection assigned to the supplied thread back to the pool.
   # The calling code should already have the mutex before calling this.
   def release(thread)
-    if @queue
-      @available_connections.unshift(@allocated.delete(thread))
+    conn = @allocated.delete(thread)
+
+    if @connection_handling == :disconnect
+      db.disconnect_connection(conn)
     else
-      @available_connections << @allocated.delete(thread)
+      checkin_connection(conn)
     end
   end
 
