@@ -6,10 +6,14 @@ module Sequel
     # ---------------------
 
     # Array of supported database adapters
-    ADAPTERS = %w'ado amalgalite db2 dbi do firebird ibmdb informix jdbc mock mysql mysql2 odbc openbase oracle postgres sqlite swift tinytds'.collect{|x| x.to_sym}
+    ADAPTERS = %w'ado amalgalite cubrid db2 dbi do firebird ibmdb informix jdbc mock mysql mysql2 odbc openbase oracle postgres sqlite swift tinytds'.collect{|x| x.to_sym}
 
-    # Whether to use the single threaded connection pool by default
-    @@single_threaded = false
+    @single_threaded = false
+
+    class << self
+      # Whether to use the single threaded connection pool by default
+      attr_accessor :single_threaded
+    end
 
     # The Database subclass for the given adapter scheme.
     # Raises Sequel::AdapterNotFound if the adapter
@@ -22,7 +26,7 @@ module Sequel
       unless klass = ADAPTER_MAP[scheme]
         # attempt to load the adapter file
         begin
-          Sequel.tsk_require "sequel/adapters/#{scheme}"
+          require "sequel/adapters/#{scheme}"
         rescue LoadError => e
           raise Sequel.convert_exception_class(e, AdapterNotFound)
         end
@@ -41,7 +45,7 @@ module Sequel
     end
     
     # Connects to a database.  See Sequel.connect.
-    def self.connect(conn_string, opts = {})
+    def self.connect(conn_string, opts = OPTS)
       case conn_string
       when String
         if match = /\A(jdbc|do):/o.match(conn_string)
@@ -55,11 +59,8 @@ module Sequel
           c = adapter_class(scheme)
           uri_options = c.send(:uri_to_options, uri)
           uri.query.split('&').collect{|s| s.split('=')}.each{|k,v| uri_options[k.to_sym] = v if k && !k.empty?} unless uri.query.to_s.strip.empty?
-          uri_options.to_a.each{|k,v| uri_options[k] = URI.unescape(v) if v.is_a?(String)}
-          opts = opts.merge(:orig_opts=>opts.dup)
-          opts[:uri] = conn_string
-          opts = uri_options.merge(opts)
-          opts[:adapter] = scheme
+          uri_options.to_a.each{|k,v| uri_options[k] = (defined?(URI::DEFAULT_PARSER) ? URI::DEFAULT_PARSER : URI).unescape(v) if v.is_a?(String)}
+          opts = uri_options.merge(opts).merge!(:orig_opts=>opts.dup, :uri=>conn_string, :adapter=>scheme)
         end
       when Hash
         opts = conn_string.merge(opts)
@@ -77,22 +78,18 @@ module Sequel
       begin
         db = c.new(opts)
         db.test_connection if opts[:test] && db.send(:typecast_value_boolean, opts[:test])
-        result = yield(db) if block_given?
+        if block_given?
+          return yield(db)
+        end
       ensure
         if block_given?
           db.disconnect if db
           Sequel.synchronize{::Sequel::DATABASES.delete(db)}
         end
       end
-      block_given? ? result : db
+      db
     end
     
-    # Sets the default single_threaded mode for new databases.
-    # See Sequel.single_threaded=.
-    def self.single_threaded=(value)
-      @@single_threaded = value
-    end
-
     # Sets the adapter scheme for the Database class. Call this method in
     # descendants of Database to allow connection using a URL. For example the
     # following:
@@ -107,7 +104,7 @@ module Sequel
     #   Sequel.connect('mydb://user:password@dbserver/mydb')
     def self.set_adapter_scheme(scheme) # :nodoc:
       @scheme = scheme
-      ADAPTER_MAP[scheme.to_sym] = self
+      ADAPTER_MAP[scheme] = self
     end
     private_class_method :set_adapter_scheme
     
@@ -140,12 +137,7 @@ module Sequel
         @pool.add_servers(servers.keys)
       end
     end
-    
-    # Connects to the database. This method should be overridden by descendants.
-    def connect(server)
-      raise NotImplemented, "#connect should be overridden by adapters"
-    end
-    
+
     # The database type for this database object, the same as the adapter scheme
     # by default.  Should be overridden in adapters (especially shared adapters)
     # to be the correct type, so that even if two separate Database objects are
@@ -169,16 +161,24 @@ module Sequel
     #   DB.disconnect # All servers
     #   DB.disconnect(:servers=>:server1) # Single server
     #   DB.disconnect(:servers=>[:server1, :server2]) # Multiple servers
-    def disconnect(opts = {})
+    def disconnect(opts = OPTS)
       pool.disconnect(opts)
     end
 
+    # Should only be called by the connection pool code to disconnect a connection.
+    # By default, calls the close method on the connection object, since most
+    # adapters use that, but should be overwritten on other adapters.
+    def disconnect_connection(conn)
+      conn.close
+    end
+
     # Yield a new Database instance for every server in the connection pool.
     # Intended for use in sharded environments where there is a need to make schema
     # modifications (DDL queries) on each shard.
     #
     #   DB.each_server{|db| db.create_table(:users){primary_key :id; String :name}}
     def each_server(&block)
+      raise(Error, "Database#each_server must be passed a block") unless block
       servers.each{|s| self.class.connect(server_opts(s), &block)}
     end
 
@@ -229,9 +229,11 @@ module Sequel
         @pool.hold(server || :default){|conn| yield conn}
       end
     else
+    # :nocov:
       def synchronize(server=nil, &block)
         @pool.hold(server || :default, &block)
       end
+    # :nocov:
     end
     
     # Attempts to acquire a database connection.  Returns true if successful.
@@ -243,6 +245,21 @@ module Sequel
       true
     end
 
+    # Check whether the given connection is currently valid, by
+    # running a query against it.  If the query fails, the
+    # connection should probably be removed from the connection
+    # pool.
+    def valid_connection?(conn)
+      sql = valid_connection_sql
+      begin
+        log_connection_execute(conn, sql)
+      rescue Sequel::DatabaseError, *database_error_classes
+        false
+      else
+        true
+      end
+    end
+
     private
     
     # The default options for the connection pool.
@@ -271,5 +288,10 @@ module Sequel
       opts.delete(:servers)
       opts
     end
+
+    # The SQL query to issue to check if a connection is valid.
+    def valid_connection_sql
+      @valid_connection_sql ||= select(nil).sql
+    end
   end
 end
