@@ -8,52 +8,41 @@ module Sequel
     # The default class to use for datasets
     DatasetClass = Sequel::Dataset
 
-    # The identifier input method to use by default
-    @@identifier_input_method = nil
+    @identifier_input_method = nil
+    @identifier_output_method = nil
+    @quote_identifiers = nil
 
-    # The identifier output method to use by default
-    @@identifier_output_method = nil
+    class << self
+      # The identifier input method to use by default for all databases (default: adapter default)
+      attr_reader :identifier_input_method
 
-    # Whether to quote identifiers (columns and tables) by default
-    @@quote_identifiers = nil
+      # The identifier output method to use by default for all databases (default: adapter default)
+      attr_reader :identifier_output_method
 
-    # The method to call on identifiers going into the database
-    def self.identifier_input_method
-      @@identifier_input_method
+      # Whether to quote identifiers (columns and tables) by default for all databases (default: adapter default)
+      attr_accessor :quote_identifiers
     end
-    
-    # Set the method to call on identifiers going into the database
-    # See Sequel.identifier_input_method=.
+
+    # Change the default identifier input method to use for all databases,
     def self.identifier_input_method=(v)
-      @@identifier_input_method = v || ""
-    end
-    
-    # The method to call on identifiers coming from the database
-    def self.identifier_output_method
-      @@identifier_output_method
-    end
-    
-    # Set the method to call on identifiers coming from the database
-    # See Sequel.identifier_output_method=.
-    def self.identifier_output_method=(v)
-      @@identifier_output_method = v || ""
+      @identifier_input_method = v.nil? ? false : v
     end
 
-    # Sets the default quote_identifiers mode for new databases.
-    # See Sequel.quote_identifiers=.
-    def self.quote_identifiers=(value)
-      @@quote_identifiers = value
-    end
+    # Change the default identifier output method to use for all databases,
+    def self.identifier_output_method=(v)
+      @identifier_output_method = v.nil? ? false : v
+     end
 
     # The class to use for creating datasets.  Should respond to
     # new with the Database argument as the first argument, and
     # an optional options hash.
     attr_reader :dataset_class
 
-    # The default schema to use, generally should be nil.  This sets
-    # the default schema used for some schema modification and
-    # introspection queries, but does not effect most dataset code.
-    attr_accessor :default_schema
+    # The identifier input method to use by default for this database (default: adapter default)
+    attr_reader :identifier_input_method
+
+    # The identifier output method to use by default for this database (default: adapter default)
+    attr_reader :identifier_output_method
 
     # If the database has any dataset modules associated with it,
     # use a subclass of the given class that includes the modules
@@ -64,6 +53,7 @@ module Sequel
         @dataset_modules.each{|m| c.send(:include, m)}
       end
       @dataset_class = c
+      reset_default_dataset
     end
 
     # Equivalent to extending all datasets produced by the database with a
@@ -90,7 +80,6 @@ module Sequel
     #   end
     def extend_datasets(mod=nil, &block)
       raise(Error, "must provide either mod or block, not both") if mod && block
-      reset_schema_utility_dataset
       mod = Module.new(&block) if block
       if @dataset_modules.empty?
        @dataset_modules = [mod]
@@ -99,42 +88,17 @@ module Sequel
        @dataset_modules << mod
       end
       @dataset_class.send(:include, mod)
+      reset_default_dataset
     end
 
-    # The method to call on identifiers going into the database
-    def identifier_input_method
-      case @identifier_input_method
-      when nil
-        @identifier_input_method = @opts.fetch(:identifier_input_method, (@@identifier_input_method.nil? ? identifier_input_method_default : @@identifier_input_method))
-        @identifier_input_method == "" ? nil : @identifier_input_method
-      when ""
-        nil
-      else
-        @identifier_input_method
-      end
-    end
-    
     # Set the method to call on identifiers going into the database:
     #
     #   DB[:items] # SELECT * FROM items
     #   DB.identifier_input_method = :upcase
     #   DB[:items] # SELECT * FROM ITEMS
     def identifier_input_method=(v)
-      reset_schema_utility_dataset
-      @identifier_input_method = v || ""
-    end
-    
-    # The method to call on identifiers coming from the database
-    def identifier_output_method
-      case @identifier_output_method
-      when nil
-        @identifier_output_method = @opts.fetch(:identifier_output_method, (@@identifier_output_method.nil? ? identifier_output_method_default : @@identifier_output_method))
-        @identifier_output_method == "" ? nil : @identifier_output_method
-      when ""
-        nil
-      else
-        @identifier_output_method
-      end
+      reset_default_dataset
+      @identifier_input_method = v
     end
     
     # Set the method to call on identifiers coming from the database:
@@ -143,8 +107,8 @@ module Sequel
     #   DB.identifier_output_method = :upcase
     #   DB[:items].first # {:ID=>1, :NAME=>'foo'}
     def identifier_output_method=(v)
-      reset_schema_utility_dataset
-      @identifier_output_method = v || ""
+      reset_default_dataset
+      @identifier_output_method = v
     end
 
     # Set whether to quote identifiers (columns and tables) for this database:
@@ -153,14 +117,13 @@ module Sequel
     #   DB.quote_identifiers = true
     #   DB[:items] # SELECT * FROM "items"
     def quote_identifiers=(v)
-      reset_schema_utility_dataset
+      reset_default_dataset
       @quote_identifiers = v
     end
     
     # Returns true if the database quotes identifiers.
     def quote_identifiers?
-      return @quote_identifiers unless @quote_identifiers.nil?
-      @quote_identifiers = @opts.fetch(:quote_identifiers, (@@quote_identifiers.nil? ? quote_identifiers_default : @@quote_identifiers))
+      @quote_identifiers
     end
     
     private
@@ -169,10 +132,12 @@ module Sequel
     def dataset_class_default
       self.class.const_get(:DatasetClass)
     end
-    
-    # The default value for default_schema.
-    def default_schema_default
-      nil
+
+    # Reset the default dataset used by most Database methods that
+    # create datasets.  Usually done after changes to the identifier
+    # mangling methods.
+    def reset_default_dataset
+      @default_dataset = dataset
     end
 
     # The method to apply to identifiers going into the database by default.
@@ -196,5 +161,14 @@ module Sequel
     def quote_identifiers_default
       true
     end
+
+    # Reset the identifier mangling options.  Overrides any already set on
+    # the instance.  Only for internal use by shared adapters.
+    def reset_identifier_mangling
+      @quote_identifiers = @opts.fetch(:quote_identifiers){(qi = Database.quote_identifiers).nil? ? quote_identifiers_default : qi}
+      @identifier_input_method = @opts.fetch(:identifier_input_method){(iim = Database.identifier_input_method).nil? ? identifier_input_method_default : (iim if iim)}
+      @identifier_output_method = @opts.fetch(:identifier_output_method){(iom = Database.identifier_output_method).nil? ? identifier_output_method_default : (iom if iom)}
+      reset_default_dataset
+    end
   end
 end
