@@ -21,6 +21,11 @@ module Sequel
     # The default options for join table columns.
     DEFAULT_JOIN_TABLE_COLUMN_OPTIONS = {:null=>false}
 
+    # The alter table operations that are combinable.
+    COMBINABLE_ALTER_TABLE_OPS = [:add_column, :drop_column, :rename_column,
+      :set_column_type, :set_column_default, :set_column_null,
+      :add_constraint, :drop_constraint]
+
     # Adds a column to the specified table. This method expects a column name,
     # a datatype and optionally a hash with additional constraints and options:
     #
@@ -41,7 +46,7 @@ module Sequel
     # :ignore_errors :: Ignore any DatabaseErrors that are raised
     #
     # See <tt>alter_table</tt>.
-    def add_index(table, columns, options={})
+    def add_index(table, columns, options=OPTS)
       e = options[:ignore_errors]
       begin
         alter_table(table){add_index(columns, options)}
@@ -70,7 +75,7 @@ module Sequel
     def alter_table(name, generator=nil, &block)
       generator ||= alter_table_generator(&block)
       remove_cached_schema(name)
-      apply_alter_table(name, generator.operations)
+      apply_alter_table_generator(name, generator)
       nil
     end
 
@@ -112,7 +117,7 @@ module Sequel
     # :name :: The name of the table to create
     # :no_index :: Set to true not to create the second index.
     # :no_primary_key :: Set to true to not create the primary key.
-    def create_join_table(hash, options={})
+    def create_join_table(hash, options=OPTS)
       keys = hash.keys.sort_by{|k| k.to_s}
       create_table(join_table_name(hash, options), options) do
         keys.each do |key|
@@ -149,8 +154,13 @@ module Sequel
     # :collate :: The collation to use for the table.
     # :engine :: The table engine to use for the table.
     #
+    # PostgreSQL specific options:
+    # :unlogged :: Create the table as an unlogged table.
+    # :inherits :: Inherit from a different tables.  An array can be
+    #              specified to inherit from multiple tables.
+    #
     # See <tt>Schema::Generator</tt> and the {"Schema Modification" guide}[link:files/doc/schema_modification_rdoc.html].
-    def create_table(name, options={}, &block)
+    def create_table(name, options=OPTS, &block)
       remove_cached_schema(name)
       options = {:generator=>options} if options.is_a?(Schema::CreateTableGenerator)
       if sql = options[:as]
@@ -170,7 +180,7 @@ module Sequel
     #   # SELECT NULL FROM a LIMIT 1 -- check existence
     #   # DROP TABLE a -- drop table if already exists
     #   # CREATE TABLE a (a integer)
-    def create_table!(name, options={}, &block)
+    def create_table!(name, options=OPTS, &block)
       drop_table?(name)
       create_table(name, options, &block)
     end
@@ -180,7 +190,7 @@ module Sequel
     #   DB.create_table?(:a){Integer :a} 
     #   # SELECT NULL FROM a LIMIT 1 -- check existence
     #   # CREATE TABLE a (a integer) -- if it doesn't already exist
-    def create_table?(name, options={}, &block)
+    def create_table?(name, options=OPTS, &block)
       if supports_create_table_if_not_exists?
         create_table(name, options.merge(:if_not_exists=>true), &block)
       elsif !table_exists?(name)
@@ -194,24 +204,48 @@ module Sequel
       create_table_generator_class.new(self, &block)
     end
     
-    # Creates a view, replacing it if it already exists:
+    # Creates a view, replacing a view with the same name if one already exists.
     #
-    #   DB.create_or_replace_view(:cheap_items, "SELECT * FROM items WHERE price < 100")
-    #   DB.create_or_replace_view(:ruby_items, DB[:items].filter(:category => 'ruby'))
-    def create_or_replace_view(name, source)
-      source = source.sql if source.is_a?(Dataset)
-      execute_ddl("CREATE OR REPLACE VIEW #{quote_schema_table(name)} AS #{source}")
-      remove_cached_schema(name)
-      nil
+    #   DB.create_or_replace_view(:some_items, "SELECT * FROM items WHERE price < 100")
+    #   DB.create_or_replace_view(:some_items, DB[:items].filter(:category => 'ruby'))
+    #
+    # For databases where replacing a view is not natively supported, support
+    # is emulated by dropping a view with the same name before creating the view.
+    def create_or_replace_view(name, source, options = OPTS)
+      if supports_create_or_replace_view?
+        options = options.merge(:replace=>true)
+      else
+        drop_view(name) rescue nil
+      end
+
+      create_view(name, source, options)
     end
     
     # Creates a view based on a dataset or an SQL string:
     #
     #   DB.create_view(:cheap_items, "SELECT * FROM items WHERE price < 100")
     #   DB.create_view(:ruby_items, DB[:items].filter(:category => 'ruby'))
-    def create_view(name, source)
-      source = source.sql if source.is_a?(Dataset)
-      execute_ddl("CREATE VIEW #{quote_schema_table(name)} AS #{source}")
+    #
+    # Options:
+    # :columns :: The column names to use for the view.  If not given,
+    #             automatically determined based on the input dataset.
+    #
+    # PostgreSQL/SQLite specific option:
+    # :temp :: Create a temporary view, automatically dropped on disconnect.
+    #
+    # PostgreSQL specific option:
+    # :materialized :: Creates a materialized view, similar to a regular view,
+    #                  but backed by a physical table.
+    # :recursive :: Creates a recursive view.  As columns must be specified for
+    #               recursive views, you can also set them as the value of this
+    #               option.  Since a recursive view requires a union that isn't
+    #               in a subquery, if you are providing a Dataset as the source
+    #               argument, if should probably call the union method with the
+    #               :all=>true and :from_self=>false options.
+    def create_view(name, source, options = OPTS)
+      execute_ddl(create_view_sql(name, source, options))
+      remove_cached_schema(name)
+      nil
     end
     
     # Removes a column from the specified table:
@@ -229,7 +263,7 @@ module Sequel
     #   DB.drop_index :posts, [:author, :title]
     #
     # See <tt>alter_table</tt>.
-    def drop_index(table, columns, options={})
+    def drop_index(table, columns, options=OPTS)
       alter_table(table){drop_index(columns, options)}
     end
 
@@ -238,7 +272,7 @@ module Sequel
     #
     #   drop_join_table(:cat_id=>:cats, :dog_id=>:dogs)
     #   # DROP TABLE cats_dogs
-    def drop_join_table(hash, options={})
+    def drop_join_table(hash, options=OPTS)
       drop_table(join_table_name(hash, options), options)
     end
     
@@ -281,6 +315,13 @@ module Sequel
     #   DB.drop_view(:cheap_items)
     #   DB.drop_view(:cheap_items, :pricey_items)
     #   DB.drop_view(:cheap_items, :pricey_items, :cascade=>true)
+    #
+    # Options:
+    # :cascade :: Also drop objects depending on this view.
+    #
+    # PostgreSQL specific options:
+    # :if_exists :: Do not raise an error if the view does not exist.
+    # :materialized :: Drop a materialized view.
     def drop_view(*names)
       options = names.last.is_a?(Hash) ? names.pop : {}
       names.each do |n|
@@ -333,19 +374,23 @@ module Sequel
 
     # Apply the changes in the given alter table ops to the table given by name.
     def apply_alter_table(name, ops)
-      alter_table_sql_list(name, ops).flatten.each{|sql| execute_ddl(sql)}
+      alter_table_sql_list(name, ops).each{|sql| execute_ddl(sql)}
     end
     
+    # Apply the operations in the given generator to the table given by name.
+    def apply_alter_table_generator(name, generator)
+      apply_alter_table(name, generator.operations)
+    end
+
     # The class used for alter_table generators.
     def alter_table_generator_class
       Schema::AlterTableGenerator
     end
     
-    # The SQL to execute to modify the DDL for the given table name.  op
-    # should be one of the operations returned by the AlterTableGenerator.
-    def alter_table_sql(table, op)
+    # SQL fragment for given alter table operation.
+    def alter_table_op_sql(table, op)
       quoted_name = quote_identifier(op[:name]) if op[:name]
-      alter_table_op = case op[:op]
+      case op[:op]
       when :add_column
         "ADD COLUMN #{column_definition_sql(op)}"
       when :drop_column
@@ -358,24 +403,59 @@ module Sequel
         "ALTER COLUMN #{quoted_name} SET DEFAULT #{literal(op[:default])}"
       when :set_column_null
         "ALTER COLUMN #{quoted_name} #{op[:null] ? 'DROP' : 'SET'} NOT NULL"
-      when :add_index
-        return index_definition_sql(table, op)
-      when :drop_index
-        return drop_index_sql(table, op)
       when :add_constraint
         "ADD #{constraint_definition_sql(op)}"
       when :drop_constraint
+        if op[:type] == :foreign_key
+          quoted_name ||= quote_identifier(foreign_key_name(table, op[:columns]))
+        end
         "DROP CONSTRAINT #{quoted_name}#{' CASCADE' if op[:cascade]}"
       else
-        raise Error, "Unsupported ALTER TABLE operation"
+        raise Error, "Unsupported ALTER TABLE operation: #{op[:op]}"
+      end
+    end
+
+    # The SQL to execute to modify the DDL for the given table name.  op
+    # should be one of the operations returned by the AlterTableGenerator.
+    def alter_table_sql(table, op)
+      case op[:op]
+      when :add_index
+        index_definition_sql(table, op)
+      when :drop_index
+        drop_index_sql(table, op)
+      else
+        "ALTER TABLE #{quote_schema_table(table)} #{alter_table_op_sql(table, op)}"
       end
-      "ALTER TABLE #{quote_schema_table(table)} #{alter_table_op}"
     end
 
     # Array of SQL DDL modification statements for the given table,
     # corresponding to the DDL changes specified by the operations.
     def alter_table_sql_list(table, operations)
-      operations.map{|op| alter_table_sql(table, op)}
+      if supports_combining_alter_table_ops?
+        grouped_ops = []
+        last_combinable = false
+        operations.each do |op|
+          if combinable_alter_table_op?(op)
+            if sql = alter_table_op_sql(table, op)
+              grouped_ops << [] unless last_combinable
+              grouped_ops.last << sql
+              last_combinable = true
+            end
+          elsif sql = alter_table_sql(table, op)
+            Array(sql).each{|s| grouped_ops << s}
+            last_combinable = false
+          end
+        end
+        grouped_ops.map do |gop|
+          if gop.is_a?(Array)
+            "ALTER TABLE #{quote_schema_table(table)} #{gop.join(', ')}"
+          else
+            gop
+          end
+        end
+      else
+        operations.map{|op| alter_table_sql(table, op)}.flatten.compact
+      end
     end
     
     # The SQL string specify the autoincrement property, generally used by
@@ -420,17 +500,32 @@ module Sequel
     
     # Add primary key SQL fragment to column creation SQL.
     def column_definition_primary_key_sql(sql, column)
-      sql << PRIMARY_KEY if column[:primary_key]
+      if column[:primary_key]
+        if name = column[:primary_key_constraint_name]
+          sql << " CONSTRAINT #{quote_identifier(name)}"
+        end
+        sql << PRIMARY_KEY
+      end
     end
     
     # Add foreign key reference SQL fragment to column creation SQL.
     def column_definition_references_sql(sql, column)
-      sql << column_references_column_constraint_sql(column) if column[:table]
+      if column[:table]
+        if name = column[:foreign_key_constraint_name]
+         sql << " CONSTRAINT #{quote_identifier(name)}"
+        end
+        sql << column_references_column_constraint_sql(column)
+      end
     end
     
     # Add unique constraint SQL fragment to column creation SQL.
     def column_definition_unique_sql(sql, column)
-      sql << UNIQUE if column[:unique]
+      if column[:unique]
+        if name = column[:unique_constraint_name]
+          sql << " CONSTRAINT #{quote_identifier(name)}"
+        end
+        sql << UNIQUE
+      end
     end
     
     # SQL for all given columns, used inside a CREATE TABLE block.
@@ -449,7 +544,7 @@ module Sequel
       sql << "(#{Array(column[:key]).map{|x| quote_identifier(x)}.join(COMMA_SEPARATOR)})" if column[:key]
       sql << " ON DELETE #{on_delete_clause(column[:on_delete])}" if column[:on_delete]
       sql << " ON UPDATE #{on_update_clause(column[:on_update])}" if column[:on_update]
-      sql << " DEFERRABLE INITIALLY DEFERRED" if column[:deferrable]
+      constraint_deferrable_sql_append(sql, column[:deferrable])
       sql
     end
   
@@ -458,25 +553,48 @@ module Sequel
       "FOREIGN KEY #{literal(constraint[:columns])}#{column_references_sql(constraint)}"
     end
 
+    # Whether the given alter table operation is combinable.
+    def combinable_alter_table_op?(op)
+      # Use a dynamic lookup for easier overriding in adapters
+      COMBINABLE_ALTER_TABLE_OPS.include?(op[:op])
+    end
+
     # SQL DDL fragment specifying a constraint on a table.
     def constraint_definition_sql(constraint)
       sql = constraint[:name] ? "CONSTRAINT #{quote_identifier(constraint[:name])} " : ""
       case constraint[:type]
       when :check
         check = constraint[:check]
-        sql << "CHECK #{filter_expr((check.is_a?(Array) && check.length == 1) ? check.first : check)}"
+        check = check.first if check.is_a?(Array) && check.length == 1
+        check = filter_expr(check)
+        check = "(#{check})" unless check[0..0] == '(' && check[-1..-1] == ')'
+        sql << "CHECK #{check}"
       when :primary_key
         sql << "PRIMARY KEY #{literal(constraint[:columns])}"
       when :foreign_key
-        sql << column_references_table_constraint_sql(constraint)
+        sql << column_references_table_constraint_sql(constraint.merge(:deferrable=>nil))
       when :unique
         sql << "UNIQUE #{literal(constraint[:columns])}"
       else
-        raise Error, "Invalid constriant type #{constraint[:type]}, should be :check, :primary_key, :foreign_key, or :unique"
+        raise Error, "Invalid constraint type #{constraint[:type]}, should be :check, :primary_key, :foreign_key, or :unique"
       end
+      constraint_deferrable_sql_append(sql, constraint[:deferrable])
       sql
     end
 
+    # SQL DDL fragment specifying the deferrable constraint attributes.
+    def constraint_deferrable_sql_append(sql, defer)
+      case defer
+      when nil
+      when false
+        sql << ' NOT DEFERRABLE'
+      when :immediate
+        sql << ' DEFERRABLE INITIALLY IMMEDIATE'
+      else
+        sql << ' DEFERRABLE INITIALLY DEFERRED'
+      end
+    end
+
     # Execute the create table statements using the generator.
     def create_table_from_generator(name, generator, options)
       execute_ddl(create_table_sql(name, generator, options))
@@ -501,6 +619,23 @@ module Sequel
 
     # DDL statement for creating a table with the given name, columns, and options
     def create_table_sql(name, generator, options)
+      unless supports_named_column_constraints?
+        # Split column constraints into table constraints if they have a name
+        generator.columns.each do |c|
+          if (constraint_name = c.delete(:foreign_key_constraint_name)) && (table = c.delete(:table))
+            opts = {}
+            opts[:name] = constraint_name
+            [:key, :on_delete, :on_update, :deferrable].each{|k| opts[k] = c[k]}
+            generator.foreign_key([c[:name]], table, opts)
+          end
+          if (constraint_name = c.delete(:unique_constraint_name)) && c.delete(:unique)
+            generator.unique(c[:name], :name=>constraint_name)
+          end
+          if (constraint_name = c.delete(:primary_key_constraint_name)) && c.delete(:primary_key)
+            generator.primary_key([c[:name]], :name=>constraint_name)
+          end
+        end
+      end
       "#{create_table_prefix_sql(name, options)} (#{column_list_sql(generator)})"
     end
 
@@ -517,18 +652,46 @@ module Sequel
       "#{create_table_prefix_sql(name, options)} AS #{sql}"
     end
 
-    # DDL statement for creating a table with the given name, columns, and options
+    # DDL fragment for initial part of CREATE TABLE statement
     def create_table_prefix_sql(name, options)
       "CREATE #{temporary_table_sql if options[:temp]}TABLE#{' IF NOT EXISTS' if options[:if_not_exists]} #{options[:temp] ? quote_identifier(name) : quote_schema_table(name)}"
     end
 
+    # DDL fragment for initial part of CREATE VIEW statement
+    def create_view_prefix_sql(name, options)
+      create_view_sql_append_columns("CREATE #{'OR REPLACE 'if options[:replace]}VIEW #{quote_schema_table(name)}", options[:columns])
+    end
+
+    # DDL statement for creating a view.
+    def create_view_sql(name, source, options)
+      source = source.sql if source.is_a?(Dataset)
+      "#{create_view_prefix_sql(name, options)} AS #{source}"
+    end
+
+    # Append the column list to the SQL, if a column list is given.
+    def create_view_sql_append_columns(sql, columns)
+      if columns
+        sql << ' ('
+        schema_utility_dataset.send(:identifier_list_append, sql, columns)
+        sql << ')'
+      end
+      sql
+    end
+
     # Default index name for the table and columns, may be too long
     # for certain databases.
     def default_index_name(table_name, columns)
       schema, table = schema_and_table(table_name)
-      "#{"#{schema}_" if schema and schema != default_schema}#{table}_#{columns.map{|c| [String, Symbol].any?{|cl| c.is_a?(cl)} ? c : literal(c).gsub(/\W/, '_')}.join(UNDERSCORE)}_index"
+      "#{"#{schema}_" if schema}#{table}_#{columns.map{|c| [String, Symbol].any?{|cl| c.is_a?(cl)} ? c : literal(c).gsub(/\W/, '_')}.join(UNDERSCORE)}_index"
     end
   
+    # Get foreign key name for given table and columns.
+    def foreign_key_name(table_name, columns)
+      keys = foreign_key_list(table_name).select{|key| key[:columns] == columns}
+      raise(Error, "#{keys.empty? ? 'Missing' : 'Ambiguous'} foreign key for #{columns.inspect}") unless keys.size == 1
+      keys.first[:name]
+    end
+    
     # The SQL to drop an index for the table.
     def drop_index_sql(table, op)
       "DROP INDEX #{quote_identifier(op[:name] || default_index_name(table, op[:columns]))}"
@@ -620,22 +783,11 @@ module Sequel
       schema_utility_dataset.quote_schema_table(table)
     end
     
-    # Proxy the quote_identifier method to the dataset, used for quoting tables and columns.
-    def quote_identifier(v)
-      schema_utility_dataset.quote_identifier(v)
-    end
-    
     # SQL DDL statement for renaming a table.
     def rename_table_sql(name, new_name)
       "ALTER TABLE #{quote_schema_table(name)} RENAME TO #{quote_schema_table(new_name)}"
     end
 
-    # Remove the cached schema_utility_dataset, because the identifier
-    # quoting has changed.
-    def reset_schema_utility_dataset
-      @schema_utility_dataset = nil
-    end
-    
     # Split the schema information from the table
     def schema_and_table(table_name)
       schema_utility_dataset.schema_and_table(table_name)
@@ -648,7 +800,12 @@ module Sequel
 
     # The dataset to use for proxying certain schema methods.
     def schema_utility_dataset
-      @schema_utility_dataset ||= dataset
+      @default_dataset
+    end
+
+    # Split the schema information from the table
+    def split_qualifiers(table_name)
+      schema_utility_dataset.split_qualifiers(table_name)
     end
 
     # SQL DDL fragment for temporary table
@@ -730,11 +887,11 @@ module Sequel
     # :text option is used, Sequel uses the :text type.
     def type_literal_generic_string(column)
       if column[:text]
-        :text
+        uses_clob_for_text? ? :clob : :text
       elsif column[:fixed]
-        "char(#{column[:size]||255})"
+        "char(#{column[:size]||default_string_column_size})"
       else
-        "varchar(#{column[:size]||255})"
+        "varchar(#{column[:size]||default_string_column_size})"
       end
     end
     
@@ -754,9 +911,14 @@ module Sequel
     def type_literal_specific(column)
       type = column[:type]
       type = "double precision" if type.to_s == 'double'
-      column[:size] ||= 255 if type.to_s == 'varchar'
+      column[:size] ||= default_string_column_size if type.to_s == 'varchar'
       elements = column[:size] || column[:elements]
       "#{type}#{literal(Array(elements)) if elements}#{UNSIGNED if column[:unsigned]}"
     end
+
+    # Whether clob should be used for String :text=>true columns.
+    def uses_clob_for_text?
+      false
+    end
   end
 end
