@@ -10,10 +10,11 @@ module Sequel
     # Action methods defined by Sequel that execute code on the database.
     ACTION_METHODS = (<<-METHS).split.map{|x| x.to_sym}
       << [] []= all avg count columns columns! delete each
-      empty? fetch_rows first get import insert insert_multiple interval last
-      map max min multi_insert range select_hash select_map select_order_map
-      set single_record single_value sum to_csv to_hash truncate update
+      empty? fetch_rows first first! get import insert insert_multiple interval last
+      map max min multi_insert paged_each range select_hash select_hash_groups select_map select_order_map
+      set single_record single_value sum to_csv to_hash to_hash_groups truncate update
     METHS
+    # REMOVE40 []= insert_multiple set to_csv
 
     # Inserts the given argument into the database.  Returns self so it
     # can be used safely when chaining:
@@ -33,15 +34,6 @@ module Sequel
       first(*conditions)
     end
 
-    # Update all records matching the conditions with the values specified.
-    # Returns the number of rows affected.
-    #
-    #   DB[:table][:id=>1] = {:id=>2} # UPDATE table SET id = 2 WHERE id = 1
-    #   # => 1 # number of rows affected
-    def []=(conditions, values)
-      filter(conditions).update(values)
-    end
-
     # Returns an array with all records in the dataset. If a block is given,
     # the array is iterated over after all items have been loaded.
     #
@@ -58,12 +50,15 @@ module Sequel
       a
     end
     
-    # Returns the average value for the given column.
+    # Returns the average value for the given column/expression.
+    # Uses a virtual row block if no argument is given.
     #
     #   DB[:table].avg(:number) # SELECT avg(number) FROM table LIMIT 1
     #   # => 3
-    def avg(column)
-      aggregate_dataset.get{avg(column)}
+    #   DB[:table].avg{function(column)} # SELECT avg(function(column)) FROM table LIMIT 1
+    #   # => 1
+    def avg(column=Sequel.virtual_row(&Proc.new))
+      aggregate_dataset.get{avg(column).as(:avg)}
     end
   
     # Returns the columns in the result set in order as an array of symbols.
@@ -77,7 +72,7 @@ module Sequel
     #   # => [:id, :name]
     def columns
       return @columns if @columns
-      ds = unfiltered.unordered.clone(:distinct => nil, :limit => 1, :offset=>nil)
+      ds = unfiltered.unordered.naked.clone(:distinct => nil, :limit => 1, :offset=>nil)
       ds.each{break}
       @columns = ds.instance_variable_get(:@columns)
       @columns || []
@@ -93,12 +88,30 @@ module Sequel
       columns
     end
     
-    # Returns the number of records in the dataset.
+    # Returns the number of records in the dataset. If an argument is provided,
+    # it is used as the argument to count.  If a block is provided, it is
+    # treated as a virtual row, and the result is used as the argument to
+    # count.
     #
-    #   DB[:table].count # SELECT COUNT(*) AS count FROM table LIMIT 1
+    #   DB[:table].count # SELECT count(*) AS count FROM table LIMIT 1
     #   # => 3
-    def count
-      aggregate_dataset.get{COUNT(:*){}.as(count)}.to_i
+    #   DB[:table].count(:column) # SELECT count(column) AS count FROM table LIMIT 1
+    #   # => 2
+    #   DB[:table].count{foo(column)} # SELECT count(foo(column)) AS count FROM table LIMIT 1
+    #   # => 1
+    def count(arg=(no_arg=true), &block)
+      if no_arg
+        if block
+          arg = Sequel.virtual_row(&block)
+          aggregate_dataset.get{count(arg).as(count)}
+        else
+          aggregate_dataset.get{count(:*){}.as(count)}.to_i
+        end
+      elsif block
+        raise Error, 'cannot provide both argument and block to Dataset#count'
+      else
+        aggregate_dataset.get{count(arg).as(count)}
+      end
     end
     
     # Deletes the records in the dataset.  The returned value should be 
@@ -123,14 +136,12 @@ module Sequel
     # Note that this method is not safe to use on many adapters if you are
     # running additional queries inside the provided block.  If you are
     # running queries inside the block, you should use +all+ instead of +each+
-    # for the outer queries, or use a separate thread or shard inside +each+:
-    def each(&block)
-      if @opts[:graph]
-        graph_each(&block)
-      elsif row_proc = @row_proc
+    # for the outer queries, or use a separate thread or shard inside +each+.
+    def each
+      if row_proc = @row_proc
         fetch_rows(select_sql){|r| yield row_proc.call(r)}
       else
-        fetch_rows(select_sql, &block)
+        fetch_rows(select_sql){|r| yield r}
       end
       self
     end
@@ -143,20 +154,17 @@ module Sequel
       get(Sequel::SQL::AliasedExpression.new(1, :one)).nil?
     end
 
-    # Executes a select query and fetches records, yielding each record to the
-    # supplied block.  The yielded records should be hashes with symbol keys.
-    # This method should probably should not be called by user code, use +each+
-    # instead.
-    def fetch_rows(sql)
-      raise NotImplemented, NOTIMPL_MSG
-    end
-    
     # If a integer argument is given, it is interpreted as a limit, and then returns all 
     # matching records up to that limit.  If no argument is passed,
     # it returns the first matching record.  If any other type of
     # argument(s) is passed, it is given to filter and the
-    # first matching record is returned. If a block is given, it is used
-    # to filter the dataset before returning anything.  Examples:
+    # first matching record is returned.  If a block is given, it is used
+    # to filter the dataset before returning anything.
+    #
+    # If there are no records in the dataset, returns nil (or an empty
+    # array if an integer argument is given).
+    #
+    # Examples:
     # 
     #   DB[:table].first # SELECT * FROM table LIMIT 1
     #   # => {:id=>7}
@@ -188,7 +196,7 @@ module Sequel
         ds.single_record
       else
         args = (args.size == 1) ? args.first : args
-        if Integer === args
+        if args.is_a?(Integer)
           ds.limit(args).all
         else
           ds.filter(args).single_record
@@ -196,20 +204,51 @@ module Sequel
       end
     end
 
+    # Calls first.  If first returns nil (signaling that no
+    # row matches), raise a Sequel::NoMatchingRow exception.
+    def first!(*args, &block)
+      first(*args, &block) || raise(Sequel::NoMatchingRow)
+    end
+
     # Return the column value for the first matching record in the dataset.
     # Raises an error if both an argument and block is given.
     #
     #   DB[:table].get(:id) # SELECT id FROM table LIMIT 1
     #   # => 3
     #
-    #   ds.get{sum(id)} # SELECT sum(id) FROM table LIMIT 1
+    #   ds.get{sum(id)} # SELECT sum(id) AS v FROM table LIMIT 1
     #   # => 6
-    def get(column=nil, &block)
-      if column
-        raise(Error, ARG_BLOCK_ERROR_MSG) if block
-        select(column).single_value
+    #
+    # You can pass an array of arguments to return multiple arguments,
+    # but you must make sure each element in the array has an alias that
+    # Sequel can determine:
+    #
+    #   DB[:table].get([:id, :name]) # SELECT id, name FROM table LIMIT 1
+    #   # => [3, 'foo']
+    #
+    #   DB[:table].get{[sum(id).as(sum), name]} # SELECT sum(id) AS sum, name FROM table LIMIT 1
+    #   # => [6, 'foo']
+    def get(column=(no_arg=true; nil), &block)
+      ds = naked
+      if block
+        raise(Error, ARG_BLOCK_ERROR_MSG) unless no_arg
+        ds = ds.select(&block)
+        column = ds.opts[:select]
+        column = nil if column.is_a?(Array) && column.length < 2
       else
-        select(&block).single_value
+        ds = if column.is_a?(Array)
+          ds.select(*column)
+        else
+          ds.select(auto_alias_expression(column))
+        end
+      end
+
+      if column.is_a?(Array)
+       if r = ds.single_record
+         r.values_at(*hash_key_symbols(column))
+       end
+      else
+        ds.single_value
       end
     end
     
@@ -236,7 +275,7 @@ module Sequel
     # :server :: Set the server/shard to use for the transaction and insert
     #            queries.
     # :slice :: Same as :commit_every, :commit_every takes precedence.
-    def import(columns, values, opts={})
+    def import(columns, values, opts=OPTS)
       return @db.transaction{insert(columns, values)} if values.is_a?(Dataset)
 
       return if values.empty?
@@ -300,36 +339,15 @@ module Sequel
       end
     end
     
-    # Inserts multiple values. If a block is given it is invoked for each
-    # item in the given array before inserting it.  See +multi_insert+ as
-    # a possibly faster version that may be able to insert multiple
-    # records in one SQL statement (if supported by the database).
-    # Returns an array of primary keys of inserted rows.
-    #
-    #   DB[:table].insert_multiple([{:x=>1}, {:x=>2}])
-    #   # => [4, 5]
-    #   # INSERT INTO table (x) VALUES (1)
-    #   # INSERT INTO table (x) VALUES (2)
-    #
-    #   DB[:table].insert_multiple([{:x=>1}, {:x=>2}]){|row| row[:y] = row[:x] * 2}
-    #   # => [6, 7]
-    #   # INSERT INTO table (x, y) VALUES (1, 2)
-    #   # INSERT INTO table (x, y) VALUES (2, 4)
-    def insert_multiple(array, &block)
-      if block
-        array.map{|i| insert(block.call(i))}
-      else
-        array.map{|i| insert(i)}
-      end
-    end
-    
     # Returns the interval between minimum and maximum values for the given 
-    # column.
+    # column/expression. Uses a virtual row block if no argument is given.
     #
     #   DB[:table].interval(:id) # SELECT (max(id) - min(id)) FROM table LIMIT 1
     #   # => 6
-    def interval(column)
-      aggregate_dataset.get{max(column) - min(column)}
+    #   DB[:table].interval{function(column)} # SELECT (max(function(column)) - min(function(column))) FROM table LIMIT 1
+    #   # => 7
+    def interval(column=Sequel.virtual_row(&Proc.new))
+      aggregate_dataset.get{(max(column) - min(column)).as(:interval)}
     end
 
     # Reverses the order and then runs #first with the given arguments and block.  Note that this
@@ -340,7 +358,7 @@ module Sequel
     #   DB[:table].order(:id).last # SELECT * FROM table ORDER BY id DESC LIMIT 1
     #   # => {:id=>10}
     #
-    #   DB[:table].order(:id.desc).last(2) # SELECT * FROM table ORDER BY id ASC LIMIT 2
+    #   DB[:table].order(Sequel.desc(:id)).last(2) # SELECT * FROM table ORDER BY id ASC LIMIT 2
     #   # => [{:id=>1}, {:id=>2}]
     def last(*args, &block)
       raise(Error, 'No order specified') unless @opts[:order]
@@ -375,20 +393,26 @@ module Sequel
       end
     end
 
-    # Returns the maximum value for the given column.
+    # Returns the maximum value for the given column/expression.
+    # Uses a virtual row block if no argument is given.
     #
     #   DB[:table].max(:id) # SELECT max(id) FROM table LIMIT 1
     #   # => 10
-    def max(column)
-      aggregate_dataset.get{max(column)}
+    #   DB[:table].max{function(column)} # SELECT max(function(column)) FROM table LIMIT 1
+    #   # => 7
+    def max(column=Sequel.virtual_row(&Proc.new))
+      aggregate_dataset.get{max(column).as(:max)}
     end
 
-    # Returns the minimum value for the given column.
+    # Returns the minimum value for the given column/expression.
+    # Uses a virtual row block if no argument is given.
     #
     #   DB[:table].min(:id) # SELECT min(id) FROM table LIMIT 1
     #   # => 1
-    def min(column)
-      aggregate_dataset.get{min(column)}
+    #   DB[:table].min{function(column)} # SELECT min(function(column)) FROM table LIMIT 1
+    #   # => 0
+    def min(column=Sequel.virtual_row(&Proc.new))
+      aggregate_dataset.get{min(column).as(:min)}
     end
 
     # This is a front end for import that allows you to submit an array of
@@ -403,25 +427,76 @@ module Sequel
     # values.
     #
     # This respects the same options as #import.
-    def multi_insert(hashes, opts={})
+    def multi_insert(hashes, opts=OPTS)
       return if hashes.empty?
       columns = hashes.first.keys
       import(columns, hashes.map{|h| columns.map{|c| h[c]}}, opts)
     end
 
+    # Yields each row in the dataset, but interally uses multiple queries as needed with
+    # limit and offset to process the entire result set without keeping all
+    # rows in the dataset in memory, even if the underlying driver buffers all
+    # query results in memory.
+    #
+    # Because this uses multiple queries internally, in order to remain consistent,
+    # it also uses a transaction internally.  Additionally, to make sure that all rows
+    # in the dataset are yielded and none are yielded twice, the dataset must have an
+    # unambiguous order.  Sequel requires that datasets using this method have an
+    # order, but it cannot ensure that the order is unambiguous.
+    #
+    # Options:
+    # :rows_per_fetch :: The number of rows to fetch per query.  Defaults to 1000.
+    def paged_each(opts=OPTS)
+      unless @opts[:order]
+        raise Sequel::Error, "Dataset#paged_each requires the dataset be ordered"
+      end
+
+      total_limit = @opts[:limit]
+      offset = @opts[:offset] || 0
+
+      if server = @opts[:server]
+        opts = opts.merge(:server=>server)
+      end
+
+      rows_per_fetch = opts[:rows_per_fetch] || 1000
+      num_rows_yielded = rows_per_fetch
+      total_rows = 0
+
+      db.transaction(opts) do
+        while num_rows_yielded == rows_per_fetch && (total_limit.nil? || total_rows < total_limit)
+          if total_limit && total_rows + rows_per_fetch > total_limit
+            rows_per_fetch = total_limit - total_rows
+          end
+
+          num_rows_yielded = 0
+          limit(rows_per_fetch, offset).each do |row|
+            num_rows_yielded += 1
+            total_rows += 1 if total_limit
+            yield row
+          end
+
+          offset += rows_per_fetch
+        end
+      end
+
+      self
+    end
+
     # Returns a +Range+ instance made from the minimum and maximum values for the
-    # given column.
+    # given column/expression.  Uses a virtual row block if no argument is given.
     #
     #   DB[:table].range(:id) # SELECT max(id) AS v1, min(id) AS v2 FROM table LIMIT 1
     #   # => 1..10
-    def range(column)
+    #   DB[:table].interval{function(column)} # SELECT max(function(column)) AS v1, min(function(column)) AS v2 FROM table LIMIT 1
+    #   # => 0..7
+    def range(column=Sequel.virtual_row(&Proc.new))
       if r = aggregate_dataset.select{[min(column).as(v1), max(column).as(v2)]}.first
         (r[:v1]..r[:v2])
       end
     end
     
     # Returns a hash with key_column values as keys and value_column values as
-    # values.  Similar to to_hash, but only selects the two columns.
+    # values.  Similar to to_hash, but only selects the columns given.
     #
     #   DB[:table].select_hash(:id, :name) # SELECT id, name FROM table
     #   # => {1=>'a', 2=>'b', ...}
@@ -436,19 +511,28 @@ module Sequel
     # that Sequel can determine.  Usually you can do this by calling the #as method
     # on the expression and providing an alias.
     def select_hash(key_column, value_column)
-      if key_column.is_a?(Array)
-        if value_column.is_a?(Array)
-          select(*(key_column + value_column)).to_hash(key_column.map{|c| hash_key_symbol(c)}, value_column.map{|c| hash_key_symbol(c)})
-        else
-          select(*(key_column + [value_column])).to_hash(key_column.map{|c| hash_key_symbol(c)}, hash_key_symbol(value_column))
-        end
-      elsif value_column.is_a?(Array)
-        select(key_column, *value_column).to_hash(hash_key_symbol(key_column), value_column.map{|c| hash_key_symbol(c)})
-      else
-        select(key_column, value_column).to_hash(hash_key_symbol(key_column), hash_key_symbol(value_column))
-      end
+      _select_hash(:to_hash, key_column, value_column)
     end
     
+    # Returns a hash with key_column values as keys and an array of value_column values.
+    # Similar to to_hash_groups, but only selects the columns given.
+    #
+    #   DB[:table].select_hash(:name, :id) # SELECT id, name FROM table
+    #   # => {'a'=>[1, 4, ...], 'b'=>[2, ...], ...}
+    #
+    # You can also provide an array of column names for either the key_column,
+    # the value column, or both:
+    #
+    #   DB[:table].select_hash([:first, :middle], [:last, :id]) # SELECT * FROM table
+    #   # {['a', 'b']=>[['c', 1], ['d', 2], ...], ...}
+    #
+    # When using this method, you must be sure that each expression has an alias
+    # that Sequel can determine.  Usually you can do this by calling the #as method
+    # on the expression and providing an alias.
+    def select_hash_groups(key_column, value_column)
+      _select_hash(:to_hash_groups, key_column, value_column)
+    end
+
     # Selects the column given (either as an argument or as a block), and
     # returns an array of all values of that column in the dataset.  If you
     # give a block argument that returns an array with multiple entries,
@@ -472,7 +556,6 @@ module Sequel
     def select_map(column=nil, &block)
       _select_map(column, false, &block)
     end
-
     
     # The same as select_map, but in addition orders the array by the column.
     #
@@ -494,12 +577,6 @@ module Sequel
       _select_map(column, true, &block)
     end
 
-    # Alias for update, but not aliased directly so subclasses
-    # don't have to override both methods.
-    def set(*args)
-      update(*args)
-    end
-    
     # Returns the first record in the dataset, or nil if the dataset
     # has no records. Users should probably use +first+ instead of
     # this method.
@@ -512,41 +589,22 @@ module Sequel
     # Returns nil if dataset is empty.  Users should generally use
     # +get+ instead of this method.
     def single_value
-      if r = naked.ungraphed.single_record
+      if r = ungraphed.naked.single_record
         r.values.first
       end
     end
     
-    # Returns the sum for the given column.
+    # Returns the sum for the given column/expression.
+    # Uses a virtual row block if no column is given.
     #
     #   DB[:table].sum(:id) # SELECT sum(id) FROM table LIMIT 1
     #   # => 55
-    def sum(column)
-      aggregate_dataset.get{sum(column)}
+    #   DB[:table].sum{function(column)} # SELECT sum(function(column)) FROM table LIMIT 1
+    #   # => 10
+    def sum(column=Sequel.virtual_row(&Proc.new))
+      aggregate_dataset.get{sum(column).as(:sum)}
     end
 
-    # Returns a string in CSV format containing the dataset records. By 
-    # default the CSV representation includes the column titles in the
-    # first line. You can turn that off by passing false as the 
-    # include_column_titles argument.
-    #
-    # This does not use a CSV library or handle quoting of values in
-    # any way.  If any values in any of the rows could include commas or line
-    # endings, you shouldn't use this.
-    #
-    #   puts DB[:table].to_csv # SELECT * FROM table
-    #   # id,name
-    #   # 1,Jim
-    #   # 2,Bob
-    def to_csv(include_column_titles = true)
-      n = naked
-      cols = n.columns
-      csv = ''
-      csv << "#{cols.join(COMMA_SEPARATOR)}\r\n" if include_column_titles
-      n.each{|r| csv << "#{cols.collect{|c| r[c]}.join(COMMA_SEPARATOR)}\r\n"}
-      csv
-    end
-    
     # Returns a hash with one column used as key and another used as value.
     # If rows have duplicate values for the key column, the latter row(s)
     # will overwrite the value of the previous row(s). If the value_column
@@ -591,6 +649,49 @@ module Sequel
       h
     end
 
+    # Returns a hash with one column used as key and the values being an
+    # array of column values. If the value_column is not given or nil, uses
+    # the entire hash as the value.
+    #
+    #   DB[:table].to_hash(:name, :id) # SELECT * FROM table
+    #   # {'Jim'=>[1, 4, 16, ...], 'Bob'=>[2], ...}
+    #
+    #   DB[:table].to_hash(:name) # SELECT * FROM table
+    #   # {'Jim'=>[{:id=>1, :name=>'Jim'}, {:id=>4, :name=>'Jim'}, ...], 'Bob'=>[{:id=>2, :name=>'Bob'}], ...}
+    #
+    # You can also provide an array of column names for either the key_column,
+    # the value column, or both:
+    #
+    #   DB[:table].to_hash([:first, :middle], [:last, :id]) # SELECT * FROM table
+    #   # {['Jim', 'Bob']=>[['Smith', 1], ['Jackson', 4], ...], ...}
+    #
+    #   DB[:table].to_hash([:first, :middle]) # SELECT * FROM table
+    #   # {['Jim', 'Bob']=>[{:id=>1, :first=>'Jim', :middle=>'Bob', :last=>'Smith'}, ...], ...}
+    def to_hash_groups(key_column, value_column = nil)
+      h = {}
+      if value_column
+        return naked.to_hash_groups(key_column, value_column) if row_proc
+        if value_column.is_a?(Array)
+          if key_column.is_a?(Array)
+            each{|r| (h[r.values_at(*key_column)] ||= []) << r.values_at(*value_column)}
+          else
+            each{|r| (h[r[key_column]] ||= []) << r.values_at(*value_column)}
+          end
+        else
+          if key_column.is_a?(Array)
+            each{|r| (h[r.values_at(*key_column)] ||= []) << r[value_column]}
+          else
+            each{|r| (h[r[key_column]] ||= []) << r[value_column]}
+          end
+        end
+      elsif key_column.is_a?(Array)
+        each{|r| (h[r.values_at(*key_column)] ||= []) << r}
+      else
+        each{|r| (h[r[key_column]] ||= []) << r}
+      end
+      h
+    end
+
     # Truncates the dataset.  Returns nil.
     #
     #   DB[:table].truncate # TRUNCATE table
@@ -609,7 +710,7 @@ module Sequel
     #
     #   DB[:table].update(:x=>:x+1, :y=>0) # UPDATE table SET x = (x + 1), y = 0
     #   # => 10
-    def update(values={}, &block)
+    def update(values=OPTS, &block)
       sql = update_sql(values)
       if uses_returning?(:update)
         returning_fetch_rows(sql, &block)
@@ -618,6 +719,13 @@ module Sequel
       end
     end
 
+    # Execute the given SQL and return the number of rows deleted.  This exists
+    # solely as an optimization, replacing with_sql(sql).delete.  It's significantly
+    # faster as it does not require cloning the current dataset.
+    def with_sql_delete(sql)
+      execute_dui(sql)
+    end
+
     protected
 
     # Internals of #import.  If primary key values are requested, use
@@ -645,18 +753,33 @@ module Sequel
   
     private
     
+    # Internals of +select_hash+ and +select_hash_groups+
+    def _select_hash(meth, key_column, value_column)
+      select(*(key_column.is_a?(Array) ? key_column : [key_column]) + (value_column.is_a?(Array) ? value_column : [value_column])).
+        send(meth, hash_key_symbols(key_column), hash_key_symbols(value_column))
+    end
+    
     # Internals of +select_map+ and +select_order_map+
     def _select_map(column, order, &block)
-      ds = naked.ungraphed
+      ds = ungraphed.naked
       columns = Array(column)
       virtual_row_columns(columns, block)
       select_cols = order ? columns.map{|c| c.is_a?(SQL::OrderedExpression) ? c.expression : c} : columns
-      ds = ds.select(*select_cols)
       ds = ds.order(*columns.map{|c| unaliased_identifier(c)}) if order
       if column.is_a?(Array) || (columns.length > 1)
-        ds._select_map_multiple(select_cols.map{|c| hash_key_symbol(c)})
+        ds.select(*select_cols)._select_map_multiple(hash_key_symbols(select_cols))
       else
-        ds._select_map_single
+        ds.select(auto_alias_expression(select_cols.first))._select_map_single
+      end
+    end
+
+    # Automatically alias the given expression if it does not have an identifiable alias.
+    def auto_alias_expression(v)
+      case v
+      when LiteralString, Symbol, SQL::Identifier, SQL::QualifiedIdentifier, SQL::AliasedExpression
+        v
+      else
+        SQL::AliasedExpression.new(v, :v)
       end
     end
 
@@ -667,46 +790,64 @@ module Sequel
 
     # Execute the given select SQL on the database using execute. Use the
     # :read_only server unless a specific server is set.
-    def execute(sql, opts={}, &block)
+    def execute(sql, opts=OPTS, &block)
       @db.execute(sql, {:server=>@opts[:server] || :read_only}.merge(opts), &block)
     end
     
     # Execute the given SQL on the database using execute_ddl.
-    def execute_ddl(sql, opts={}, &block)
+    def execute_ddl(sql, opts=OPTS, &block)
       @db.execute_ddl(sql, default_server_opts(opts), &block)
       nil
     end
     
     # Execute the given SQL on the database using execute_dui.
-    def execute_dui(sql, opts={}, &block)
+    def execute_dui(sql, opts=OPTS, &block)
       @db.execute_dui(sql, default_server_opts(opts), &block)
     end
     
     # Execute the given SQL on the database using execute_insert.
-    def execute_insert(sql, opts={}, &block)
+    def execute_insert(sql, opts=OPTS, &block)
       @db.execute_insert(sql, default_server_opts(opts), &block)
     end
     
     # Return a plain symbol given a potentially qualified or aliased symbol,
     # specifying the symbol that is likely to be used as the hash key
-    # for the column when records are returned.
-    def hash_key_symbol(s)
+    # for the column when records are returned.  Return nil if no hash key
+    # can be determined
+    def _hash_key_symbol(s, recursing=false)
       case s
       when Symbol
         _, c, a = split_symbol(s)
         (a || c).to_sym
-      when SQL::Identifier
-        hash_key_symbol(s.value)
+      when SQL::Identifier, SQL::Wrapper
+        _hash_key_symbol(s.value, true)
       when SQL::QualifiedIdentifier
-        hash_key_symbol(s.column)
+        _hash_key_symbol(s.column, true)
       when SQL::AliasedExpression
-        hash_key_symbol(s.aliaz)
+        _hash_key_symbol(s.aliaz, true)
       when String
-        s.to_sym
-      else
-        raise(Error, "#{s.inspect} is not supported, should be a Symbol, String, SQL::Identifier, SQL::QualifiedIdentifier, or SQL::AliasedExpression") 
+        s.to_sym if recursing
       end
     end
+
+    # Return a plain symbol given a potentially qualified or aliased symbol,
+    # specifying the symbol that is likely to be used as the hash key
+    # for the column when records are returned.  Raise Error if the hash key
+    # symbol cannot be returned.
+    def hash_key_symbol(s)
+      if v = _hash_key_symbol(s)
+        v
+      else
+        raise(Error, "#{s.inspect} is not supported, should be a Symbol, SQL::Identifier, SQL::QualifiedIdentifier, or SQL::AliasedExpression")
+      end
+    end
+
+    # If s is an array, return an array with the given hash key symbols.
+    # Otherwise, return a hash key symbol for the given expression 
+    # If a hash key symbol cannot be determined, raise an error.
+    def hash_key_symbols(s)
+      s.is_a?(Array) ? s.map{|c| hash_key_symbol(c)} : hash_key_symbol(s)
+    end
     
     # Modify the identifier returned from the database based on the
     # identifier_output_method.
@@ -747,10 +888,9 @@ module Sequel
       when SQL::AliasedExpression
         c.expression
       when SQL::OrderedExpression
-        expr = c.expression
-        if expr.is_a?(Symbol)
-          expr = unaliased_identifier(expr)
-          SQL::OrderedExpression.new(unaliased_identifier(c.expression), c.descending, :nulls=>c.nulls)
+        case expr = c.expression
+        when Symbol, SQL::AliasedExpression
+          SQL::OrderedExpression.new(unaliased_identifier(expr), c.descending, :nulls=>c.nulls)
         else
           c
         end
