@@ -10,10 +10,8 @@ module Sequel
     def quote_identifiers?
       if defined?(@quote_identifiers)
         @quote_identifiers
-      elsif db.respond_to?(:quote_identifiers?)
-        @quote_identifiers = db.quote_identifiers?
       else
-        @quote_identifiers = false
+        @quote_identifiers = db.quote_identifiers?
       end
     end
     
@@ -114,6 +112,16 @@ module Sequel
       supports_distinct_on?
     end
     
+    # Whether the dataset supports pattern matching by regular expressions.
+    def supports_regexp?
+      false
+    end
+
+    # Whether the dataset supports REPLACE syntax, false by default.
+    def supports_replace?
+      false
+    end
+
     # Whether the RETURNING clause is supported for the given type of query.
     # +type+ can be :insert, :update, or :delete.
     def supports_returning?(type)
@@ -154,12 +162,6 @@ module Sequel
       true
     end
 
-    # Whether using an offset returns an extra row number column that should be
-    # eliminated, false by default.
-    def offset_returns_row_number_column?
-      false
-    end
-
     # Whether the RETURNING clause is used for the given dataset.
     # +type+ can be :insert, :update, or :delete.
     def uses_returning?(type)
