@@ -17,7 +17,7 @@ module Sequel
       
       # The bind arguments to use for running this prepared statement
       attr_accessor :bind_arguments
-      
+
       # Set the bind arguments based on the hash and call super.
       def call(bind_vars={}, &block)
         ds = bind(bind_vars)
@@ -46,6 +46,10 @@ module Sequel
     module PreparedStatementMethods
       PLACEHOLDER_RE = /\A\$(.*)\z/
       
+      # Whether to log the full SQL query.  By default, just the prepared statement
+      # name is generally logged on adapters that support native prepared statements.
+      attr_accessor :log_sql
+      
       # The type of prepared statement, should be one of :select, :first,
       # :insert, :update, or :delete
       attr_accessor :prepared_type
@@ -76,7 +80,7 @@ module Sequel
       # the type of the statement and the prepared_modify_values.
       def prepared_sql
         case @prepared_type
-        when :select, :all
+        when :select, :all, :each
           # Most common scenario, so listed first.
           select_sql
         when :first
@@ -114,7 +118,9 @@ module Sequel
       # with the prepared SQL it represents (which in general won't have
       # substituted variables).
       def inspect
-        "<#{self.class.name}/PreparedStatement #{prepared_sql.inspect}>"
+        c = self.class
+        c = c.superclass while c.name.nil? || c.name == ''
+        "<#{c.name}/PreparedStatement #{prepared_sql.inspect}>"
       end
       
       protected
@@ -127,6 +133,8 @@ module Sequel
         when :select, :all
           # Most common scenario, so listed first
           all(&block)
+        when :each
+          each(&block)
         when :insert_select
           with_sql(prepared_sql).first
         when :first
@@ -139,7 +147,7 @@ module Sequel
           delete
         when Array
           case @prepared_type.at(0)
-          when :map, :to_hash
+          when :map, :to_hash, :to_hash_groups
             send(*@prepared_type, &block) 
           end
         else
@@ -243,7 +251,7 @@ module Sequel
     #   DB.call(:select_by_name, :name=>'Blah') # Same thing
     def prepare(type, name=nil, *values)
       ps = to_prepared_statement(type, values)
-      db.prepared_statements[name] = ps if name
+      db.set_prepared_statement(name, ps) if name
       ps
     end
     
