@@ -41,6 +41,7 @@ module Sequel
       set_defaults set_graph_aliases set_overrides unfiltered ungraphed ungrouped union
       unlimited unordered where with with_recursive with_sql
     METHS
+    # REMOVE40: query paginate set_defaults set_overrides
 
     # Register an extension callback for Dataset objects.  ext should be the
     # extension name symbol, and mod should either be a Module that the
@@ -63,24 +64,23 @@ module Sequel
       Sequel.synchronize{EXTENSIONS[ext] = block}
     end
 
-    # Adds an further filter to an existing filter using AND. If no filter 
-    # exists an error is raised. This method is identical to #filter except
-    # it expects an existing filter.
-    #
-    #   DB[:table].filter(:a).and(:b) # SELECT * FROM table WHERE a AND b
+    # Alias for where.
     def and(*cond, &block)
-      raise(InvalidOperation, "No existing filter found.") unless @opts[:having] || @opts[:where]
-      filter(*cond, &block)
+      where(*cond, &block)
     end
     
     # Returns a new clone of the dataset with with the given options merged.
     # If the options changed include options in COLUMN_CHANGE_OPTS, the cached
     # columns are deleted.  This method should generally not be called
     # directly by user code.
-    def clone(opts = {})
+    def clone(opts = nil)
       c = super()
-      c.opts = @opts.merge(opts)
-      c.instance_variable_set(:@columns, nil) if opts.keys.any?{|o| COLUMN_CHANGE_OPTS.include?(o)}
+      if opts
+        c.instance_variable_set(:@opts, @opts.merge(opts))
+        c.instance_variable_set(:@columns, nil) if @columns && !opts.each_key{|o| break if COLUMN_CHANGE_OPTS.include?(o)}
+      else
+        c.instance_variable_set(:@opts, @opts.dup)
+      end
       c
     end
 
@@ -115,14 +115,13 @@ module Sequel
     #
     #   DB[:items].except(DB[:other_items], :alias=>:i)
     #   # SELECT * FROM (SELECT * FROM items EXCEPT SELECT * FROM other_items) AS i
-    def except(dataset, opts={})
-      opts = {:all=>opts} unless opts.is_a?(Hash)
+    def except(dataset, opts=OPTS)
       raise(InvalidOperation, "EXCEPT not supported") unless supports_intersect_except?
       raise(InvalidOperation, "EXCEPT ALL not supported") if opts[:all] && !supports_intersect_except_all?
       compound_clone(:except, dataset, opts)
     end
 
-    # Performs the inverse of Dataset#filter.  Note that if you have multiple filter
+    # Performs the inverse of Dataset#where.  Note that if you have multiple filter
     # conditions, this is not the same as a negation of all conditions.
     #
     #   DB[:items].exclude(:category => 'software')
@@ -131,7 +130,7 @@ module Sequel
     #   DB[:items].exclude(:category => 'software', :id=>3)
     #   # SELECT * FROM items WHERE ((category != 'software') OR (id != 3))
     def exclude(*cond, &block)
-      _filter_or_exclude(true, @opts[:having] ? :having : :where, *cond, &block)
+      _filter_or_exclude(true, :where, *cond, &block)
     end
 
     # Inverts the given conditions and adds them to the HAVING clause.
@@ -142,18 +141,9 @@ module Sequel
       _filter_or_exclude(true, :having, *cond, &block)
     end
 
-    # Inverts the given conditions and adds them to the WHERE clause.
-    #
-    #   DB[:items].select_group(:name).exclude_where(:category => 'software')
-    #   # SELECT * FROM items WHERE (category != 'software')
-    #
-    #   DB[:items].select_group(:name).
-    #     exclude_having{count(name) < 2}.
-    #     exclude_where(:category => 'software')
-    #   # SELECT name FROM items WHERE (category != 'software')
-    #   # GROUP BY name HAVING (count(name) >= 2)
+    # Alias for exclude.
     def exclude_where(*cond, &block)
-      _filter_or_exclude(true, :where, *cond, &block)
+      exclude(*cond, &block)
     end
 
     # Return a clone of the dataset loaded with the extensions, see #extension!.
@@ -161,77 +151,9 @@ module Sequel
       clone.extension!(*exts)
     end
 
-    # Load an extension into the receiver.  In addition to requiring the extension file, this
-    # also modifies the dataset to work with the extension (usually extending it with a
-    # module defined in the extension file).  If no related extension file exists or the
-    # extension does not have specific support for Database objects, an Error will be raised.
-    # Returns self.
-    def extension!(*exts)
-      Sequel.extension(*exts)
-      exts.each do |ext|
-        if pr = Sequel.synchronize{EXTENSIONS[ext]}
-          pr.call(self)
-        else
-          raise(Error, "Extension #{ext} does not have specific support handling individual datasets")
-        end
-      end
-      self
-    end
-
-    # Returns a copy of the dataset with the given conditions imposed upon it.  
-    # If the query already has a HAVING clause, then the conditions are imposed in the 
-    # HAVING clause. If not, then they are imposed in the WHERE clause.
-    # 
-    # filter accepts the following argument types:
-    #
-    # * Hash - list of equality/inclusion expressions
-    # * Array - depends:
-    #   * If first member is a string, assumes the rest of the arguments
-    #     are parameters and interpolates them into the string.
-    #   * If all members are arrays of length two, treats the same way
-    #     as a hash, except it allows for duplicate keys to be
-    #     specified.
-    #   * Otherwise, treats each argument as a separate condition.
-    # * String - taken literally
-    # * Symbol - taken as a boolean column argument (e.g. WHERE active)
-    # * Sequel::SQL::BooleanExpression - an existing condition expression,
-    #   probably created using the Sequel expression filter DSL.
-    #
-    # filter also takes a block, which should return one of the above argument
-    # types, and is treated the same way.  This block yields a virtual row object,
-    # which is easy to use to create identifiers and functions.  For more details
-    # on the virtual row support, see the {"Virtual Rows" guide}[link:files/doc/virtual_rows_rdoc.html]
-    #
-    # If both a block and regular argument are provided, they get ANDed together.
-    #
-    # Examples:
-    #
-    #   DB[:items].filter(:id => 3)
-    #   # SELECT * FROM items WHERE (id = 3)
-    #
-    #   DB[:items].filter('price < ?', 100)
-    #   # SELECT * FROM items WHERE price < 100
-    #
-    #   DB[:items].filter([[:id, [1,2,3]], [:id, 0..10]])
-    #   # SELECT * FROM items WHERE ((id IN (1, 2, 3)) AND ((id >= 0) AND (id <= 10)))
-    #
-    #   DB[:items].filter('price < 100')
-    #   # SELECT * FROM items WHERE price < 100
-    #
-    #   DB[:items].filter(:active)
-    #   # SELECT * FROM items WHERE :active
-    #
-    #   DB[:items].filter{price < 100}
-    #   # SELECT * FROM items WHERE (price < 100)
-    # 
-    # Multiple filter calls can be chained for scoping:
-    #
-    #   software = dataset.filter(:category => 'software').filter{price < 100}
-    #   # SELECT * FROM items WHERE ((category = 'software') AND (price < 100))
-    #
-    # See the the {"Dataset Filtering" guide}[link:files/doc/dataset_filtering_rdoc.html] for more examples and details.
+    # Alias for where.
     def filter(*cond, &block)
-      _filter(@opts[:having] ? :having : :where, *cond, &block)
+      where(*cond, &block)
     end
     
     # Returns a cloned dataset with a :update lock style.
@@ -244,38 +166,38 @@ module Sequel
     # Returns a copy of the dataset with the source changed. If no
     # source is given, removes all tables.  If multiple sources
     # are given, it is the same as using a CROSS JOIN (cartesian product) between all tables.
+    # If a block is given, it is treated as a virtual row block, similar to +where+.
     #
     #   DB[:items].from # SQL: SELECT *
     #   DB[:items].from(:blah) # SQL: SELECT * FROM blah
     #   DB[:items].from(:blah, :foo) # SQL: SELECT * FROM blah, foo
-    def from(*source)
+    #   DB[:items].from{fun(arg)} # SQL: SELECT * FROM fun(arg)
+    def from(*source, &block)
+      virtual_row_columns(source, block)
       table_alias_num = 0
-      sources = []
       ctes = nil
-      source.each do |s|
+      source.map! do |s|
         case s
-        when Hash
-          s.each{|k,v| sources << SQL::AliasedExpression.new(k,v)}
         when Dataset
           if hoist_cte?(s)
             ctes ||= []
             ctes += s.opts[:with]
             s = s.clone(:with=>nil)
           end
-          sources << SQL::AliasedExpression.new(s, dataset_alias(table_alias_num+=1))
+          SQL::AliasedExpression.new(s, dataset_alias(table_alias_num+=1))
         when Symbol
           sch, table, aliaz = split_symbol(s)
           if aliaz
             s = sch ? SQL::QualifiedIdentifier.new(sch, table) : SQL::Identifier.new(table)
-            sources << SQL::AliasedExpression.new(s, aliaz.to_sym)
+            SQL::AliasedExpression.new(s, aliaz.to_sym)
           else
-            sources << s
+            s
           end
         else
-          sources << s
+          s
         end
       end
-      o = {:from=>sources.empty? ? nil : sources}
+      o = {:from=>source.empty? ? nil : source}
       o[:with] = (opts[:with] || []) + ctes if ctes
       o[:num_dataset_sources] = table_alias_num if table_alias_num > 0
       clone(o)
@@ -292,7 +214,7 @@ module Sequel
     #
     #   ds.from_self(:alias=>:foo)
     #   # SELECT * FROM (SELECT id, name FROM items ORDER BY name) AS foo
-    def from_self(opts={})
+    def from_self(opts=OPTS)
       fs = {}
       @opts.keys.each{|k| fs[k] = nil unless NON_SQL_OPTIONS.include?(k)}
       clone(fs).from(opts[:alias] ? as(opts[:alias]) : self)
@@ -329,23 +251,23 @@ module Sequel
     #
     #   dataset.grep([:a, :b], %w'%foo% %bar%', :all_patterns=>true, :all_columns=>true)
     #   # SELECT * FROM a WHERE ((a LIKE '%foo%') AND (b LIKE '%foo%') AND (a LIKE '%bar%') AND (b LIKE '%bar%'))
-    def grep(columns, patterns, opts={})
+    def grep(columns, patterns, opts=OPTS)
       if opts[:all_patterns]
         conds = Array(patterns).map do |pat|
           SQL::BooleanExpression.new(opts[:all_columns] ? :AND : :OR, *Array(columns).map{|c| SQL::StringExpression.like(c, pat, opts)})
         end
-        filter(SQL::BooleanExpression.new(opts[:all_patterns] ? :AND : :OR, *conds))
+        where(SQL::BooleanExpression.new(opts[:all_patterns] ? :AND : :OR, *conds))
       else
         conds = Array(columns).map do |c|
           SQL::BooleanExpression.new(:OR, *Array(patterns).map{|pat| SQL::StringExpression.like(c, pat, opts)})
         end
-        filter(SQL::BooleanExpression.new(opts[:all_columns] ? :AND : :OR, *conds))
+        where(SQL::BooleanExpression.new(opts[:all_columns] ? :AND : :OR, *conds))
       end
     end
 
     # Returns a copy of the dataset with the results grouped by the value of 
     # the given columns.  If a block is given, it is treated
-    # as a virtual row block, similar to +filter+.
+    # as a virtual row block, similar to +where+.
     #
     #   DB[:items].group(:id) # SELECT * FROM items GROUP BY id
     #   DB[:items].group(:id, :name) # SELECT * FROM items GROUP BY id, name
@@ -362,7 +284,7 @@ module Sequel
     
     # Returns a dataset grouped by the given column with count by group.
     # Column aliases may be supplied, and will be included in the select clause.
-    # If a block is given, it is treated as a virtual row block, similar to +filter+.
+    # If a block is given, it is treated as a virtual row block, similar to +where+.
     #
     # Examples:
     #
@@ -397,7 +319,7 @@ module Sequel
       clone(:group_options=>:rollup)
     end
 
-    # Returns a copy of the dataset with the HAVING conditions changed. See #filter for argument types.
+    # Returns a copy of the dataset with the HAVING conditions changed. See #where for argument types.
     #
     #   DB[:items].group(:sum).having(:sum=>10)
     #   # SELECT * FROM items GROUP BY sum HAVING (sum = 10)
@@ -422,27 +344,30 @@ module Sequel
     #
     #   DB[:items].intersect(DB[:other_items], :alias=>:i)
     #   # SELECT * FROM (SELECT * FROM items INTERSECT SELECT * FROM other_items) AS i
-    def intersect(dataset, opts={})
-      opts = {:all=>opts} unless opts.is_a?(Hash)
+    def intersect(dataset, opts=OPTS)
       raise(InvalidOperation, "INTERSECT not supported") unless supports_intersect_except?
       raise(InvalidOperation, "INTERSECT ALL not supported") if opts[:all] && !supports_intersect_except_all?
       compound_clone(:intersect, dataset, opts)
     end
 
-    # Inverts the current filter.
+    # Inverts the current WHERE and HAVING clauses.  If there is neither a
+    # WHERE or HAVING clause, adds a WHERE clause that is always false.
     #
-    #   DB[:items].filter(:category => 'software').invert
+    #   DB[:items].where(:category => 'software').invert
     #   # SELECT * FROM items WHERE (category != 'software')
     #
-    #   DB[:items].filter(:category => 'software', :id=>3).invert
+    #   DB[:items].where(:category => 'software', :id=>3).invert
     #   # SELECT * FROM items WHERE ((category != 'software') OR (id != 3))
     def invert
-      having, where = @opts[:having], @opts[:where]
-      raise(Error, "No current filter") unless having || where
-      o = {}
-      o[:having] = SQL::BooleanExpression.invert(having) if having
-      o[:where] = SQL::BooleanExpression.invert(where) if where
-      clone(o)
+      having, where = @opts.values_at(:having, :where)
+      if having.nil? && where.nil?
+        where(false)
+      else
+        o = {}
+        o[:having] = SQL::BooleanExpression.invert(having) if having
+        o[:where] = SQL::BooleanExpression.invert(where) if where
+        clone(o)
+      end
     end
 
     # Alias of +inner_join+
@@ -459,7 +384,6 @@ module Sequel
     # * type - The type of join to do (e.g. :inner)
     # * table - Depends on type:
     #   * Dataset - a subselect is performed with an alias of tN for some value of N
-    #   * Model (or anything responding to :table_name) - table.table_name
     #   * String, Symbol: table
     # * expr - specifies conditions, depends on type:
     #   * Hash, Array of two element arrays - Assumes key (1st arg) is column of joined table (unless already
@@ -472,7 +396,7 @@ module Sequel
     #     the result set if this is used.
     #   * nil - If a block is not given, doesn't use ON or USING, so the JOIN should be a NATURAL
     #     or CROSS join. If a block is given, uses an ON clause based on the block, see below.
-    #   * Everything else - pretty much the same as a using the argument in a call to filter,
+    #   * Everything else - pretty much the same as a using the argument in a call to where,
     #     so strings are considered literal, symbols specify boolean columns, and Sequel
     #     expressions can be used. Uses a JOIN with an ON clause.
     # * options - a hash of options, with any of the following keys:
@@ -482,11 +406,12 @@ module Sequel
     #     the last joined or primary table is used.
     #   * :qualify - Can be set to false to not do any implicit qualification.  Can be set
     #     to :deep to use the Qualifier AST Transformer, which will attempt to qualify
-    #     subexpressions of the expression tree.
+    #     subexpressions of the expression tree.  Can be set to :symbol to only qualify
+    #     symbols. Defaults to the value of default_join_table_qualification.
     # * block - The block argument should only be given if a JOIN with an ON clause is used,
     #   in which case it yields the table alias/name for the table currently being joined,
     #   the table alias/name for the last joined (or first table), and an array of previous
-    #   SQL::JoinClause. Unlike +filter+, this block is not treated as a virtual row block.
+    #   SQL::JoinClause. Unlike +where+, this block is not treated as a virtual row block.
     #
     # Examples:
     #
@@ -500,11 +425,11 @@ module Sequel
     #   # SELECT * FROM a LEFT JOIN b AS c USING (d)
     #
     #   DB[:a].natural_join(:b).join_table(:inner, :c) do |ta, jta, js|
-    #     (:d.qualify(ta) > :e.qualify(jta)) & {:f.qualify(ta)=>DB.from(js.first.table).select(:g)}
+    #     (Sequel.qualify(ta, :d) > Sequel.qualify(jta, :e)) & {Sequel.qualify(ta, :f)=>DB.from(js.first.table).select(:g)}
     #   end
     #   # SELECT * FROM a NATURAL JOIN b INNER JOIN c
     #   #   ON ((c.d > b.e) AND (c.f IN (SELECT g FROM b)))
-    def join_table(type, table, expr=nil, options={}, &block)
+    def join_table(type, table, expr=nil, options=OPTS, &block)
       if hoist_cte?(table)
         s, ds = hoist_cte(table)
         return s.join_table(type, ds, expr, options, &block)
@@ -513,30 +438,21 @@ module Sequel
       using_join = expr.is_a?(Array) && !expr.empty? && expr.all?{|x| x.is_a?(Symbol)}
       if using_join && !supports_join_using?
         h = {}
-        expr.each{|s| h[s] = s}
+        expr.each{|e| h[e] = e}
         return join_table(type, table, h, options)
       end
 
-      case options
-      when Hash
-        table_alias = options[:table_alias]
-        last_alias = options[:implicit_qualifier]
-        qualify_type = options[:qualify]
-      when Symbol, String, SQL::Identifier
-        table_alias = options
-        last_alias = nil 
-      else
-        raise Error, "invalid options format for join_table: #{options.inspect}"
-      end
+      table_alias = options[:table_alias]
+      last_alias = options[:implicit_qualifier]
+      qualify_type = options[:qualify]
 
-      if Dataset === table
+      if table.is_a?(Dataset)
         if table_alias.nil?
           table_alias_num = (@opts[:num_dataset_sources] || 0) + 1
           table_alias = dataset_alias(table_alias_num)
         end
         table_name = table_alias
       else
-        table = table.table_name if table.respond_to?(:table_name)
         table, implicit_table_alias = split_alias(table)
         table_alias ||= implicit_table_alias
         table_name = table_alias || table
@@ -551,6 +467,7 @@ module Sequel
         last_alias ||= @opts[:last_joined_table] || first_source_alias
         if Sequel.condition_specifier?(expr)
           expr = expr.collect do |k, v|
+            qualify_type = default_join_table_qualification if qualify_type.nil?
             case qualify_type
             when false
               nil # Do no qualification
@@ -594,10 +511,10 @@ module Sequel
     #   DB[:items].limit(10...20) # SELECT * FROM items LIMIT 10 OFFSET 10
     #   DB[:items].limit(10..20) # SELECT * FROM items LIMIT 11 OFFSET 10
     #   DB[:items].limit(nil, 20) # SELECT * FROM items OFFSET 20
-    def limit(l, o = nil)
+    def limit(l, o = (no_offset = true; nil))
       return from_self.limit(l, o) if @opts[:sql]
 
-      if Range === l
+      if l.is_a?(Range)
         o = l.first
         l = l.last - l.first + (l.exclude_end? ? 0 : 1)
       end
@@ -612,16 +529,21 @@ module Sequel
           raise(Error, 'Offsets must be greater than or equal to 0') unless o >= 0
         end
         opts[:offset] = o
+      elsif !no_offset
+        opts[:offset] = nil
       end
       clone(opts)
     end
     
     # Returns a cloned dataset with the given lock style.  If style is a
-    # string, it will be used directly.  Otherwise, a symbol may be used
-    # for database independent locking.  Currently :update is respected
-    # by most databases, and :share is supported by some.
+    # string, it will be used directly. You should never pass a string
+    # to this method that is derived from user input, as that can lead to
+    # SQL injection.
     #
-    #   DB[:items].lock_style('FOR SHARE') # SELECT * FROM items FOR SHARE
+    # A symbol may be used for database independent locking behavior, but
+    # all supported symbols have separate methods (e.g. for_update).
+    #
+    #   DB[:items].lock_style('FOR SHARE NOWAIT') # SELECT * FROM items FOR SHARE NOWAIT
     def lock_style(style)
       clone(:lock => style)
     end
@@ -641,26 +563,29 @@ module Sequel
     # Adds an alternate filter to an existing filter using OR. If no filter 
     # exists an +Error+ is raised.
     #
-    #   DB[:items].filter(:a).or(:b) # SELECT * FROM items WHERE a OR b
+    #   DB[:items].where(:a).or(:b) # SELECT * FROM items WHERE a OR b
     def or(*cond, &block)
-      clause = (@opts[:having] ? :having : :where)
-      raise(InvalidOperation, "No existing filter found.") unless @opts[clause]
       cond = cond.first if cond.size == 1
-      clone(clause => SQL::BooleanExpression.new(:OR, @opts[clause], filter_expr(cond, &block)))
+      v = @opts[:where]
+      if v.nil? || (cond.respond_to?(:empty?) && cond.empty? && !block)
+        clone
+      else
+        clone(:where => SQL::BooleanExpression.new(:OR, v, filter_expr(cond, &block)))
+      end
     end
 
     # Returns a copy of the dataset with the order changed. If the dataset has an
     # existing order, it is ignored and overwritten with this order. If a nil is given
     # the returned dataset has no order. This can accept multiple arguments
     # of varying kinds, such as SQL functions.  If a block is given, it is treated
-    # as a virtual row block, similar to +filter+.
+    # as a virtual row block, similar to +where+.
     #
     #   DB[:items].order(:name) # SELECT * FROM items ORDER BY name
     #   DB[:items].order(:a, :b) # SELECT * FROM items ORDER BY a, b
-    #   DB[:items].order('a + b'.lit) # SELECT * FROM items ORDER BY a + b
+    #   DB[:items].order(Sequel.lit('a + b')) # SELECT * FROM items ORDER BY a + b
     #   DB[:items].order(:a + :b) # SELECT * FROM items ORDER BY (a + b)
-    #   DB[:items].order(:name.desc) # SELECT * FROM items ORDER BY name DESC
-    #   DB[:items].order(:name.asc(:nulls=>:last)) # SELECT * FROM items ORDER BY name ASC NULLS LAST
+    #   DB[:items].order(Sequel.desc(:name)) # SELECT * FROM items ORDER BY name DESC
+    #   DB[:items].order(Sequel.asc(:name, :nulls=>:last)) # SELECT * FROM items ORDER BY name ASC NULLS LAST
     #   DB[:items].order{sum(name).desc} # SELECT * FROM items ORDER BY sum(name) DESC
     #   DB[:items].order(nil) # SELECT * FROM items
     def order(*columns, &block)
@@ -700,23 +625,12 @@ module Sequel
     
     # Qualify to the given table, or first source if no table is given.
     #
-    #   DB[:items].filter(:id=>1).qualify
+    #   DB[:items].where(:id=>1).qualify
     #   # SELECT items.* FROM items WHERE (items.id = 1)
     #
-    #   DB[:items].filter(:id=>1).qualify(:i)
+    #   DB[:items].where(:id=>1).qualify(:i)
     #   # SELECT i.* FROM items WHERE (i.id = 1)
     def qualify(table=first_source)
-      qualify_to(table)
-    end
-
-    # Return a copy of the dataset with unqualified identifiers in the
-    # SELECT, WHERE, GROUP, HAVING, and ORDER clauses qualified by the
-    # given table. If no columns are currently selected, select all
-    # columns of the given table.
-    #
-    #   DB[:items].filter(:id=>1).qualify_to(:i)
-    #   # SELECT i.* FROM items WHERE (i.id = 1)
-    def qualify_to(table)
       o = @opts
       return clone if o[:sql]
       h = {}
@@ -726,19 +640,7 @@ module Sequel
       h[:select] = [SQL::ColumnAll.new(table)] if !o[:select] || o[:select].empty?
       clone(h)
     end
-    
-    # Qualify the dataset to its current first source.  This is useful
-    # if you have unqualified identifiers in the query that all refer to
-    # the first source, and you want to join to another table which
-    # has columns with the same name as columns in the current dataset.
-    # See +qualify_to+.
-    #
-    #   DB[:items].filter(:id=>1).qualify_to_first_source
-    #   # SELECT items.* FROM items WHERE (items.id = 1)
-    def qualify_to_first_source
-      qualify_to(first_source)
-    end
-    
+
     # Modify the RETURNING clause, only supported on a few databases.  If returning
     # is used, instead of insert returning the autogenerated primary key or
     # update/delete returning the number of modified rows, results are
@@ -755,31 +657,29 @@ module Sequel
     # given, the existing order is inverted.
     #
     #   DB[:items].reverse(:id) # SELECT * FROM items ORDER BY id DESC
+    #   DB[:items].reverse{foo(bar)} # SELECT * FROM items ORDER BY foo(bar) DESC
     #   DB[:items].order(:id).reverse # SELECT * FROM items ORDER BY id DESC
-    #   DB[:items].order(:id).reverse(:name.asc) # SELECT * FROM items ORDER BY name ASC
-    def reverse(*order)
+    #   DB[:items].order(:id).reverse(Sequel.desc(:name)) # SELECT * FROM items ORDER BY name ASC
+    def reverse(*order, &block)
+      virtual_row_columns(order, block)
       order(*invert_order(order.empty? ? @opts[:order] : order))
     end
 
     # Alias of +reverse+
-    def reverse_order(*order)
-      reverse(*order)
+    def reverse_order(*order, &block)
+      reverse(*order, &block)
     end
 
     # Returns a copy of the dataset with the columns selected changed
     # to the given columns. This also takes a virtual row block,
-    # similar to +filter+.
+    # similar to +where+.
     #
     #   DB[:items].select(:a) # SELECT a FROM items
     #   DB[:items].select(:a, :b) # SELECT a, b FROM items
     #   DB[:items].select{[a, sum(b)]} # SELECT a, sum(b) FROM items
     def select(*columns, &block)
       virtual_row_columns(columns, block)
-      m = []
-      columns.each do |i|
-        i.is_a?(Hash) ? m.concat(i.map{|k, v| SQL::AliasedExpression.new(k,v)}) : m << i
-      end
-      clone(:select => m)
+      clone(:select => columns)
     end
     
     # Returns a copy of the dataset selecting the wildcard if no arguments
@@ -817,7 +717,7 @@ module Sequel
 
     # Set both the select and group clauses with the given +columns+.
     # Column aliases may be supplied, and will be included in the select clause.
-    # This also takes a virtual row block similar to +filter+.
+    # This also takes a virtual row block similar to +where+.
     #
     #   DB[:items].select_group(:a, :b)
     #   # SELECT a, b FROM items GROUP BY a, b
@@ -829,16 +729,9 @@ module Sequel
       select(*columns).group(*columns.map{|c| unaliased_identifier(c)})
     end
 
-    # Returns a copy of the dataset with the given columns added
-    # to the existing selected columns. If no columns are currently selected
-    # it will just select the columns given. 
-    #
-    #   DB[:items].select(:a).select(:b) # SELECT b FROM items
-    #   DB[:items].select(:a).select_more(:b) # SELECT a, b FROM items
-    #   DB[:items].select_more(:b) # SELECT b FROM items
+    # Alias for select_append. 
     def select_more(*columns, &block)
-      columns = @opts[:select] + columns if @opts[:select]
-      select(*columns, &block)
+      select_append(*columns, &block)
     end
     
     # Set the server for this dataset to use.  Used to pick a specific database
@@ -854,33 +747,13 @@ module Sequel
       clone(:server=>servr)
     end
 
-    # Set the default values for insert and update statements.  The values hash passed
-    # to insert or update are merged into this hash, so any values in the hash passed
-    # to insert or update will override values passed to this method.  
-    #
-    #   DB[:items].set_defaults(:a=>'a', :c=>'c').insert(:a=>'d', :b=>'b')
-    #   # INSERT INTO items (a, c, b) VALUES ('d', 'c', 'b')
-    def set_defaults(hash)
-      clone(:defaults=>(@opts[:defaults]||{}).merge(hash))
-    end
-
-    # Set values that override hash arguments given to insert and update statements.
-    # This hash is merged into the hash provided to insert or update, so values
-    # will override any values given in the insert/update hashes.
-    #
-    #   DB[:items].set_overrides(:a=>'a', :c=>'c').insert(:a=>'d', :b=>'b')
-    #   # INSERT INTO items (a, c, b) VALUES ('a', 'c', 'b')
-    def set_overrides(hash)
-      clone(:overrides=>hash.merge(@opts[:overrides]||{}))
-    end
-    
     # Unbind bound variables from this dataset's filter and return an array of two
     # objects.  The first object is a modified dataset where the filter has been
     # replaced with one that uses bound variable placeholders.  The second object
     # is the hash of unbound variables.  You can then prepare and execute (or just
     # call) the dataset with the bound variables to get results.
     #
-    #   ds, bv = DB[:items].filter(:a=>1).unbind
+    #   ds, bv = DB[:items].where(:a=>1).unbind
     #   ds # SELECT * FROM items WHERE (a = $a)
     #   bv #  {:a => 1}
     #   ds.call(:select, bv)
@@ -922,8 +795,7 @@ module Sequel
     #
     #   DB[:items].union(DB[:other_items], :alias=>:i)
     #   # SELECT * FROM (SELECT * FROM items UNION SELECT * FROM other_items) AS i
-    def union(dataset, opts={})
-      opts = {:all=>opts} unless opts.is_a?(Hash)
+    def union(dataset, opts=OPTS)
       compound_clone(:union, dataset, opts)
     end
     
@@ -941,13 +813,56 @@ module Sequel
       order(nil)
     end
     
-    # Add a condition to the WHERE clause.  See +filter+ for argument types.
+    # Returns a copy of the dataset with the given WHERE conditions imposed upon it.  
+    # 
+    # Accepts the following argument types:
     #
-    #   DB[:items].group(:a).having(:a).filter(:b)
-    #   # SELECT * FROM items GROUP BY a HAVING a AND b
+    # * Hash - list of equality/inclusion expressions
+    # * Array - depends:
+    #   * If first member is a string, assumes the rest of the arguments
+    #     are parameters and interpolates them into the string.
+    #   * If all members are arrays of length two, treats the same way
+    #     as a hash, except it allows for duplicate keys to be
+    #     specified.
+    #   * Otherwise, treats each argument as a separate condition.
+    # * String - taken literally
+    # * Symbol - taken as a boolean column argument (e.g. WHERE active)
+    # * Sequel::SQL::BooleanExpression - an existing condition expression,
+    #   probably created using the Sequel expression filter DSL.
     #
-    #   DB[:items].group(:a).having(:a).where(:b)
-    #   # SELECT * FROM items WHERE b GROUP BY a HAVING a
+    # where also accepts a block, which should return one of the above argument
+    # types, and is treated the same way.  This block yields a virtual row object,
+    # which is easy to use to create identifiers and functions.  For more details
+    # on the virtual row support, see the {"Virtual Rows" guide}[link:files/doc/virtual_rows_rdoc.html]
+    #
+    # If both a block and regular argument are provided, they get ANDed together.
+    #
+    # Examples:
+    #
+    #   DB[:items].where(:id => 3)
+    #   # SELECT * FROM items WHERE (id = 3)
+    #
+    #   DB[:items].where('price < ?', 100)
+    #   # SELECT * FROM items WHERE price < 100
+    #
+    #   DB[:items].where([[:id, [1,2,3]], [:id, 0..10]])
+    #   # SELECT * FROM items WHERE ((id IN (1, 2, 3)) AND ((id >= 0) AND (id <= 10)))
+    #
+    #   DB[:items].where('price < 100')
+    #   # SELECT * FROM items WHERE price < 100
+    #
+    #   DB[:items].where(:active)
+    #   # SELECT * FROM items WHERE :active
+    #
+    #   DB[:items].where{price < 100}
+    #   # SELECT * FROM items WHERE (price < 100)
+    # 
+    # Multiple where calls can be chained for scoping:
+    #
+    #   software = dataset.where(:category => 'software').where{price < 100}
+    #   # SELECT * FROM items WHERE ((category = 'software') AND (price < 100))
+    #
+    # See the the {"Dataset Filtering" guide}[link:files/doc/dataset_filtering_rdoc.html] for more examples and details.
     def where(*cond, &block)
       _filter(:where, *cond, &block)
     end
@@ -958,9 +873,9 @@ module Sequel
     # :args :: Specify the arguments/columns for the CTE, should be an array of symbols.
     # :recursive :: Specify that this is a recursive CTE
     #
-    #   DB[:items].with(:items, DB[:syx].filter(:name.like('A%')))
+    #   DB[:items].with(:items, DB[:syx].where(:name.like('A%')))
     #   # WITH items AS (SELECT * FROM syx WHERE (name LIKE 'A%')) SELECT * FROM items
-    def with(name, dataset, opts={})
+    def with(name, dataset, opts=OPTS)
       raise(Error, 'This datatset does not support common table expressions') unless supports_cte?
       if hoist_cte?(dataset)
         s, ds = hoist_cte(dataset)
@@ -976,18 +891,17 @@ module Sequel
     # :args :: Specify the arguments/columns for the CTE, should be an array of symbols.
     # :union_all :: Set to false to use UNION instead of UNION ALL combining the nonrecursive and recursive parts.
     #
-    #   DB[:t].select(:i___id, :pi___parent_id).
-    #    with_recursive(:t,
-    #                   DB[:i1].filter(:parent_id=>nil),
-    #                   DB[:t].join(:t, :i=>:parent_id).select(:i1__id, :i1__parent_id),
-    #                   :args=>[:i, :pi])
-    #   # WITH RECURSIVE t(i, pi) AS (
-    #   #   SELECT * FROM i1 WHERE (parent_id IS NULL)
+    #   DB[:t].with_recursive(:t,
+    #     DB[:i1].select(:id, :parent_id).where(:parent_id=>nil),
+    #     DB[:i1].join(:t, :id=>:parent_id).select(:i1__id, :i1__parent_id),
+    #     :args=>[:id, :parent_id])
+    #   
+    #   # WITH RECURSIVE "t"("id", "parent_id") AS (
+    #   #   SELECT "id", "parent_id" FROM "i1" WHERE ("parent_id" IS NULL)
     #   #   UNION ALL
-    #   #   SELECT i1.id, i1.parent_id FROM t INNER JOIN t ON (t.i = t.parent_id)
-    #   # )
-    #   # SELECT i AS id, pi AS parent_id FROM t
-    def with_recursive(name, nonrecursive, recursive, opts={})
+    #   #   SELECT "i1"."id", "i1"."parent_id" FROM "i1" INNER JOIN "t" ON ("t"."id" = "i1"."parent_id")
+    #   # ) SELECT * FROM "t"
+    def with_recursive(name, nonrecursive, recursive, opts=OPTS)
       raise(Error, 'This datatset does not support common table expressions') unless supports_cte?
       if hoist_cte?(nonrecursive)
         s, ds = hoist_cte(nonrecursive)
@@ -1046,7 +960,8 @@ module Sequel
 
     private
 
-    # Internal filter/exclude method so it works on either the having or where clauses.
+    # Internal filtering method so it works on either the WHERE or HAVING clauses, with or
+    # without inversion.
     def _filter_or_exclude(invert, clause, *cond, &block)
       cond = cond.first if cond.size == 1
       if cond.respond_to?(:empty?) && cond.empty? && !block
@@ -1064,7 +979,12 @@ module Sequel
       _filter_or_exclude(false, clause, *cond, &block)
     end
 
-    # SQL expression object based on the expr type.  See +filter+.
+    # The default :qualify option to use for join tables if one is not specified.
+    def default_join_table_qualification
+      :symbol
+    end
+    
+    # SQL expression object based on the expr type.  See +where+.
     def filter_expr(expr = nil, &block)
       expr = nil if expr == []
       if expr && block
@@ -1100,7 +1020,7 @@ module Sequel
       when String
         LiteralString.new("(#{expr})")
       else
-        raise(Error, 'Invalid filter argument')
+        raise(Error, "Invalid filter argument: #{expr.inspect}")
       end
     end
     
@@ -1119,11 +1039,10 @@ module Sequel
     # Inverts the given order by breaking it into a list of column references
     # and inverting them.
     #
-    #   DB[:items].invert_order([:id.desc]]) #=> [:id]
-    #   DB[:items].invert_order(:category, :price.desc]) #=> [:category.desc, :price]
+    #   DB[:items].invert_order([Sequel.desc(:id)]]) #=> [Sequel.asc(:id)]
+    #   DB[:items].invert_order([:category, Sequel.desc(:price)]) #=> [Sequel.desc(:category), Sequel.asc(:price)]
     def invert_order(order)
       return nil unless order
-      new_order = []
       order.map do |f|
         case f
         when SQL::OrderedExpression
@@ -1143,7 +1062,14 @@ module Sequel
     # Treat the +block+ as a virtual_row block if not +nil+ and
     # add the resulting columns to the +columns+ array (modifies +columns+).
     def virtual_row_columns(columns, block)
-      columns.concat(Array(Sequel.virtual_row(&block))) if block
+      if block
+        v = Sequel.virtual_row(&block)
+        if v.is_a?(Array)
+          columns.concat(v)
+        else
+          columns << v
+        end
+      end
     end
   end
 end
