@@ -1,8 +1,11 @@
 # Adds the Sequel::Migration and Sequel::Migrator classes, which allow
 # the user to easily group schema changes and migrate the database
 # to a newer version or revert to a previous version.
-
 #
+# To load the extension:
+#
+#   Sequel.extension :migration
+
 module Sequel
   # Sequel's older migration class, available for backward compatibility.
   # Uses subclasses with up and down instance methods for each migration:
@@ -58,6 +61,11 @@ module Sequel
       @db.send(method_sym, *args, &block)
     end
 
+    # This object responds to all methods the database responds to.
+    def respond_to_missing?(meth, include_private)
+      @db.respond_to?(meth, include_private)
+    end
+
     # The default up action does nothing
     def up
     end
@@ -77,7 +85,7 @@ module Sequel
     # Whether to use transactions for this migration, default depends on the
     # database.
     attr_accessor :use_transactions
-
+    
     # Don't set transaction use by default.
     def initialize
       @use_transactions = nil
@@ -233,11 +241,14 @@ module Sequel
       @actions << [:drop_constraint, args.first]
     end
 
-    def add_foreign_key(*args)
+    def add_foreign_key(key, table, *args)
+      @actions << [:drop_foreign_key, key, *args]
+    end
+
+    def add_primary_key(*args)
       raise if args.first.is_a?(Array)
       @actions << [:drop_column, args.first]
     end
-    alias add_primary_key add_foreign_key
 
     def add_index(*args)
       @actions << [:drop_index, *args]
@@ -358,23 +369,24 @@ module Sequel
 
     # Return whether the migrator is current (i.e. it does not need to make
     # any changes).  Takes the same arguments as #run.
-    def self.is_current?(db, directory, opts={})
+    def self.is_current?(db, directory, opts=OPTS)
       migrator_class(directory).new(db, directory, opts).is_current?
     end
 
     # Migrates the supplied database using the migration files in the the specified directory. Options:
-    # * :column :: The column in the :table argument storing the migration version (default: :version).
-    # * :current :: The current version of the database.  If not given, it is retrieved from the database
-    #               using the :table and :column options.
-    # * :table :: The table containing the schema version (default: :schema_info).
-    # * :target :: The target version to which to migrate.  If not given, migrates to the maximum version.
+    # :allow_missing_migration_files :: Don't raise an error if there are missing migration files.
+    # :column :: The column in the :table argument storing the migration version (default: :version).
+    # :current :: The current version of the database.  If not given, it is retrieved from the database
+    #             using the :table and :column options.
+    # :table :: The table containing the schema version (default: :schema_info).
+    # :target :: The target version to which to migrate.  If not given, migrates to the maximum version.
     #
     # Examples: 
     #   Sequel::Migrator.run(DB, "migrations")
     #   Sequel::Migrator.run(DB, "migrations", :target=>15, :current=>10)
     #   Sequel::Migrator.run(DB, "app1/migrations", :column=> :app2_version)
     #   Sequel::Migrator.run(DB, "app2/migrations", :column => :app2_version, :table=>:schema_info2)
-    def self.run(db, directory, opts={})
+    def self.run(db, directory, opts=OPTS)
       migrator_class(directory).new(db, directory, opts).run
     end
 
@@ -420,10 +432,11 @@ module Sequel
     attr_reader :target
 
     # Setup the state for the migrator
-    def initialize(db, directory, opts={})
+    def initialize(db, directory, opts=OPTS)
       raise(Error, "Must supply a valid migration path") unless File.directory?(directory)
       @db = db
       @directory = directory
+      @allow_missing_migration_files = opts[:allow_missing_migration_files]
       @files = get_migration_files
       schema, table = @db.send(:schema_and_table, opts[:table]  || self.class.const_get(:DEFAULT_SCHEMA_TABLE))
       @table = schema ? Sequel::SQL::QualifiedIdentifier.new(schema, table) : table
@@ -489,7 +502,7 @@ module Sequel
     attr_reader :migrations
 
     # Set up all state for the migrator instance
-    def initialize(db, directory, opts={})
+    def initialize(db, directory, opts=OPTS)
       super
       @target = opts[:target] || latest_migration_version
       @current = opts[:current] || current_migration_version
@@ -542,7 +555,7 @@ module Sequel
         raise(Error, "Duplicate migration version: #{version}") if files[version]
         files[version] = File.join(directory, file)
       end
-      1.upto(files.length - 1){|i| raise(Error, "Missing migration version: #{i}") unless files[i]}
+      1.upto(files.length - 1){|i| raise(Error, "Missing migration version: #{i}") unless files[i]} unless @allow_missing_migration_files
       files
     end
     
@@ -570,9 +583,8 @@ module Sequel
     def schema_dataset
       c = column
       ds = db.from(table)
-      if !db.table_exists?(table)
-        db.create_table(table){Integer c, :default=>0, :null=>false}
-      elsif !ds.columns.include?(c)
+      db.create_table?(table){Integer c, :default=>0, :null=>false}
+      unless ds.columns.include?(c)
         db.alter_table(table){add_column c, Integer, :default=>0, :null=>false}
       end
       ds.insert(c=>0) if ds.empty?
@@ -616,7 +628,7 @@ module Sequel
     attr_reader :migration_tuples
 
     # Set up all state for the migrator instance
-    def initialize(db, directory, opts={})
+    def initialize(db, directory, opts=OPTS)
       super
       @target = opts[:target]
       @applied_migrations = get_applied_migrations
@@ -663,7 +675,7 @@ module Sequel
     def get_applied_migrations
       am = ds.select_order_map(column)
       missing_migration_files = am - files.map{|f| File.basename(f).downcase}
-      raise(Error, "Applied migration files not in file system: #{missing_migration_files.join(', ')}") if missing_migration_files.length > 0
+      raise(Error, "Applied migration files not in file system: #{missing_migration_files.join(', ')}") if missing_migration_files.length > 0 && !@allow_missing_migration_files
       am
     end
     
