@@ -15,25 +15,31 @@
 #
 # To turn an existing Array into a PGArray:
 #
+#   Sequel.pg_array(array)
+#
+# If you have loaded the {core_extensions extension}[link:files/doc/core_extensions_rdoc.html],
+# or you have loaded the {core_refinements extension}[link:files/doc/core_refinements_rdoc.html]
+# and have activated refinements for the file, you can also use Array#pg_array:
+#
 #   array.pg_array
 #
 # You can also provide a type, though it many cases it isn't necessary:
 #
+#   Sequel.pg_array(array, :varchar) # or :integer, :"double precision", etc.
 #   array.pg_array(:varchar) # or :integer, :"double precision", etc.
 #
 # So if you want to insert an array into an integer[] database column:
 #
-#   DB[:table].insert(:column=>[1, 2, 3].pg_array)
+#   DB[:table].insert(:column=>Sequel.pg_array([1, 2, 3]))
 #
-# If you would like to use PostgreSQL arrays in your model objects, you
-# probably want to modify the schema parsing/typecasting so that it
-# recognizes and correctly handles the arrays, which you can do by:
+# To use this extension, first load it into your Sequel::Database instance:
 #
 #   DB.extension :pg_array
 #
-# If you are not using the native postgres adapter, you probably
-# also want to use the typecast_on_load plugin in the model, and
-# set it to typecast the array column(s) on load.
+# If you are not using the native postgres adapter and are using array
+# types as model column values you probably should use the
+# pg_typecast_on_load plugin in the model, and set it to typecast the
+# array column(s) on load.
 #
 # This extension by default includes handlers for array types for
 # all scalar types that the native postgres adapter handles. It
@@ -41,18 +47,24 @@
 # general, you just need to make sure that the scalar type is
 # handled and has the appropriate converter installed in
 # Sequel::Postgres::PG_TYPES under the appropriate type OID.
-# Then you can call Sequel::Postgres::PGArray.register with
-# the appropriate arguments to automatically set up a handler
-# for the array type.
+# Then you can call
+# Sequel::Postgres::PGArray::DatabaseMethods#register_array_type
+# to automatically set up a handler for the array type.  So if you
+# want to support the foo[] type (assuming the foo type is already
+# supported):
 #
-# For example, if you add support for a scalar custom type named
-# foo which uses OID 1234, and you want to add support for the
-# foo[] type, which uses type OID 4321, you need to do:
+#   DB.register_array_type('foo')
+#
+# You can also register array types on a global basis using
+# Sequel::Postgres::PGArray.register.  In this case, you'll have
+# to specify the type oids:
 #
 #   Sequel::Postgres::PGArray.register('foo', :oid=>4321, :scalar_oid=>1234)
 #
-# Sequel::Postgres::PGArray.register has many additional options
-# and should be able to handle most PostgreSQL array types.
+# Both Sequel::Postgres::PGArray::DatabaseMethods#register_array_type
+# and Sequel::Postgres::PGArray.register support many options to
+# customize the array type handling.  See the Sequel::Postgres::PGArray.register
+# method documentation.
 #
 # If you want an easy way to call PostgreSQL array functions and
 # operators, look into the pg_array_ops extension.
@@ -93,6 +105,8 @@ module Sequel
   module Postgres
     # Represents a PostgreSQL array column value.
     class PGArray < DelegateClass(Array)
+      include Sequel::SQL::AliasMethods
+
       ARRAY = "ARRAY".freeze
       DOUBLE_COLON = '::'.freeze
       EMPTY_BRACKET = '[]'.freeze
@@ -106,8 +120,8 @@ module Sequel
       NULL = 'NULL'.freeze
       QUOTE = '"'.freeze
 
-      # Hash of database array type name strings to symbols (e.g. 'double precision' => :float),
-      # used by the schema parsing.
+      # Global hash of database array type name strings to symbols (e.g. 'double precision' => :float),
+      # used by the schema parsing for array types registered globally.
       ARRAY_TYPES = {}
 
       # Registers an array type that the extension should handle.  Makes a Database instance that
@@ -135,6 +149,8 @@ module Sequel
       #                     typecast method.  For example, for an array of integers, this could be set to
       #                     :integer, so that the typecast_value_integer method is called on all of the
       #                     array elements.  Defaults to :type_symbol option.
+      # :type_procs :: A hash mapping oids to conversion procs, used for looking up the :scalar_oid and
+      #                value and setting the :oid value.  Defaults to the global Sequel::Postgres::PG_TYPES.
       # :type_symbol :: The base of the schema type symbol for this type.  For example, if you provide
       #                 :integer, Sequel will recognize this type as :integer_array during schema parsing.
       #                 Defaults to the db_type argument.
@@ -142,12 +158,19 @@ module Sequel
       #                     typecasting method to be created in the database.  This should only be used
       #                     to alias existing array types.  For example, if there is an array type that can be
       #                     treated just like an integer array, you can do :typecast_method=>:integer.
+      # :typecast_method_map :: The map in which to place the database type string to type symbol mapping.
+      #                         Defaults to ARRAY_TYPES.
+      # :typecast_methods_module :: If given, a module object to add the typecasting method to.  Defaults
+      #                             to DatabaseMethods.
       #
       # If a block is given, it is treated as the :converter option.
-      def self.register(db_type, opts={}, &block)
+      def self.register(db_type, opts=OPTS, &block)
         db_type = db_type.to_s
         typecast_method = opts[:typecast_method]
         type = (typecast_method || opts[:type_symbol] || db_type).to_sym
+        type_procs = opts[:type_procs] || PG_TYPES
+        mod = opts[:typecast_methods_module] || DatabaseMethods
+        typecast_method_map = opts[:typecast_method_map] || ARRAY_TYPES
 
         if converter = opts[:converter]
           raise Error, "can't provide both a block and :converter option to register" if block
@@ -157,23 +180,35 @@ module Sequel
 
         if soid = opts[:scalar_oid]
           raise Error, "can't provide both a converter and :scalar_oid option to register" if converter 
-          raise Error, "no conversion proc for :scalar_oid=>#{soid.inspect} in PG_TYPES" unless converter = PG_TYPES[soid]
+          raise Error, "no conversion proc for :scalar_oid=>#{soid.inspect}" unless converter = type_procs[soid]
         end
 
         array_type = (opts[:array_type] || db_type).to_s.dup.freeze
         creator = (opts[:parser] == :json ? JSONCreator : Creator).new(array_type, converter)
 
-        ARRAY_TYPES[db_type] = :"#{type}_array"
+        typecast_method_map[db_type] = :"#{type}_array"
 
-        DatabaseMethods.define_array_typecast_method(type, creator, opts.fetch(:scalar_typecast, type)) unless typecast_method
+        define_array_typecast_method(mod, type, creator, opts.fetch(:scalar_typecast, type)) unless typecast_method
 
         if oid = opts[:oid]
-          Sequel::Postgres::PG_TYPES[oid] = creator
+          type_procs[oid] = creator
         end
 
         nil
       end
 
+      # Define a private array typecasting method in the given module for the given type that uses
+      # the creator argument to do the type conversion.
+      def self.define_array_typecast_method(mod, type, creator, scalar_typecast)
+        mod.class_eval do
+          meth = :"typecast_value_#{type}_array"
+          scalar_typecast_method = :"typecast_value_#{scalar_typecast}"
+          define_method(meth){|v| typecast_value_pg_array(v, creator, scalar_typecast_method)}
+          private meth
+        end
+      end
+      private_class_method :define_array_typecast_method
+
       module DatabaseMethods
         APOS = "'".freeze
         DOUBLE_APOS = "''".freeze
@@ -181,20 +216,20 @@ module Sequel
         ESCAPE_REPLACEMENT = '\\\\\1'.freeze
         BLOB_RANGE = 1...-1
 
-        # Reset the conversion procs when extending the Database object, so
-        # it will pick up the array convertors.  This is only done for the native
-        # postgres adapter.
+        # Create the local hash of database type strings to schema type symbols,
+        # used for array types local to this database.
         def self.extended(db)
-          db.reset_conversion_procs if db.respond_to?(:reset_conversion_procs)
-        end
+          db.instance_eval do
+            @pg_array_schema_types ||= {}
+            copy_conversion_procs([1009, 1007, 1016, 1231, 1022, 1000, 1001, 1182, 1183, 1270, 1005, 1028, 1021, 1014, 1015])
+            [:string_array, :integer_array, :decimal_array, :float_array, :boolean_array, :blob_array, :date_array, :time_array, :datetime_array].each do |v|
+              @schema_type_classes[v] = PGArray
+            end
+          end
 
-        # Define a private array typecasting method for the given type that uses
-        # the creator argument to do the type conversion.
-        def self.define_array_typecast_method(type, creator, scalar_typecast)
-          meth = :"typecast_value_#{type}_array"
-          scalar_typecast_method = :"typecast_value_#{scalar_typecast}"
-          define_method(meth){|v| typecast_value_pg_array(v, creator, scalar_typecast_method)}
-          private meth
+          procs = db.conversion_procs
+          procs[1115] = Creator.new("timestamp without time zone", procs[1114])
+          procs[1185] = Creator.new("timestamp with time zone", procs[1184])
         end
 
         # Handle arrays in bound variables
@@ -209,13 +244,24 @@ module Sequel
           end
         end
 
-        # Make the column type detection handle registered array types.
-        def schema_column_type(db_type)
-          if (db_type =~ /\A([^(]+)(?:\([^(]+\))?\[\]\z/io) && (type = ARRAY_TYPES[$1])
-            type
-          else
-            super
+        # Register a database specific array type.  This can be used to support
+        # different array types per Database.  Use of this method does not
+        # affect global state, unlike PGArray.register.  See PGArray.register for
+        # possible options.
+        def register_array_type(db_type, opts=OPTS, &block)
+          opts = {:type_procs=>conversion_procs, :typecast_method_map=>@pg_array_schema_types, :typecast_methods_module=>(class << self; self; end)}.merge(opts)
+          unless (opts.has_key?(:scalar_oid) || block) && opts.has_key?(:oid)
+            array_oid, scalar_oid = from(:pg_type).where(:typname=>db_type.to_s).get([:typarray, :oid])
+            opts[:scalar_oid] = scalar_oid unless opts.has_key?(:scalar_oid) || block
+            opts[:oid] = array_oid unless opts.has_key?(:oid)
           end
+          PGArray.register(db_type, opts, &block)
+          @schema_type_classes[:"#{opts[:typecast_method] || opts[:type_symbol] || db_type}_array"] = PGArray
+        end
+
+        # Return PGArray if this type matches any supported array type.
+        def schema_type_class(type)
+          super || (ARRAY_TYPES.each_value{|v| return PGArray if type == v}; nil)
         end
 
         private
@@ -236,19 +282,45 @@ module Sequel
           end
         end
 
+        # Automatically handle array types for the given named types. 
+        def convert_named_procs_to_procs(named_procs)
+          h = super
+          unless h.empty?
+            from(:pg_type).where(:oid=>h.keys).select_map([:typname, :oid, :typarray]).each do |name, scalar_oid, array_oid|
+              register_array_type(name, :type_procs=>h, :oid=>array_oid.to_i, :scalar_oid=>scalar_oid.to_i)
+            end
+          end
+          h
+        end
+
         # Manually override the typecasting for timestamp array types so that
         # they use the database's timezone instead of the global Sequel
         # timezone.
-        def get_conversion_procs(conn)
+        def get_conversion_procs
           procs = super
 
-          converter = method(:to_application_timestamp)
-          procs[1115] = Creator.new("timestamp without time zone", converter)
-          procs[1185] = Creator.new("timestamp with time zone", converter)
+          procs[1115] = Creator.new("timestamp without time zone", procs[1114])
+          procs[1185] = Creator.new("timestamp with time zone", procs[1184])
 
           procs
         end
 
+        # Look into both the current database's array schema types and the global
+        # array schema types to get the type symbol for the given database type
+        # string.
+        def pg_array_schema_type(type)
+          @pg_array_schema_types[type] || ARRAY_TYPES[type]
+        end
+
+        # Make the column type detection handle registered array types.
+        def schema_column_type(db_type)
+          if (db_type =~ /\A([^(]+)(?:\([^(]+\))?\[\]\z/io) && (type = pg_array_schema_type($1))
+            type
+          else
+            super
+          end
+        end
+
         # Given a value to typecast and the type of PGArray subclass:
         # * If given a PGArray with a matching array_type, use it directly.
         # * If given a PGArray with a different array_type, return a PGArray
@@ -271,8 +343,6 @@ module Sequel
               value = Sequel.recursive_map(value, method(scalar_typecast_method))
             end
             PGArray.new(value, creator.type)
-          when String
-            creator.call(value)
           else
             raise Sequel::InvalidValue, "invalid value for array type: #{value.inspect}"
           end
@@ -394,10 +464,12 @@ module Sequel
         end
 
         if Sequel::Postgres.respond_to?(:parse_pg_array)
+        # :nocov:
           # Use sequel_pg's C-based parser if it has already been defined.
           def call(string)
             PGArray.new(Sequel::Postgres.parse_pg_array(string, @converter), @type)
           end
+        # :nocov:
         else
           # Parse the string using Parser with the appropriate
           # converter, and return a PGArray with the appropriate database
@@ -423,7 +495,7 @@ module Sequel
         # recursive map of the output is done to make sure that the entires in the
         # correct type.
         def call(string)
-          array = JSON.parse(string.gsub(SUBST_RE){|m| SUBST[m]})
+          array = Sequel.parse_json(string.gsub(SUBST_RE){|m| SUBST[m]})
           array = Sequel.recursive_map(array, @converter) if @converter
           PGArray.new(array, @type)
         end
@@ -497,15 +569,48 @@ module Sequel
     end
   end
 
+  module SQL::Builders
+    # Return a Postgres::PGArray proxy for the given array and database array type.
+    def pg_array(v, array_type=nil)
+      case v
+      when Postgres::PGArray
+        if array_type.nil? || v.array_type == array_type
+          v
+        else
+          Postgres::PGArray.new(v.to_a, array_type)
+        end
+      when Array
+        Postgres::PGArray.new(v, array_type)
+      else
+        # May not be defined unless the pg_array_ops extension is used
+        pg_array_op(v)
+      end
+    end
+  end
+
   Database.register_extension(:pg_array, Postgres::PGArray::DatabaseMethods)
 end
 
-class Array
-  # Return a PGArray proxy to the receiver, using a
-  # specific database type if given.  This is mostly useful
-  # as a short cut for creating PGArray objects that didn't
-  # come from the database.
-  def pg_array(type=nil)
-    Sequel::Postgres::PGArray.new(self, type)
+# :nocov:
+if Sequel.core_extensions?
+  class Array
+    # Return a PGArray proxy to the receiver, using a
+    # specific database type if given.  This is mostly useful
+    # as a short cut for creating PGArray objects that didn't
+    # come from the database.
+    def pg_array(type=nil)
+      Sequel::Postgres::PGArray.new(self, type)
+    end
   end
 end
+
+if defined?(Sequel::CoreRefinements)
+  module Sequel::CoreRefinements
+    refine Array do
+      def pg_array(type=nil)
+        Sequel::Postgres::PGArray.new(self, type)
+      end
+    end
+  end
+end
+# :nocov:
