@@ -1,6 +1,6 @@
-module Sequel
-  tsk_require 'json'
+require 'json'
 
+module Sequel
   module Plugins
     # The json_serializer plugin handles serializing entire Sequel::Model
     # objects to JSON, as well as support for deserializing JSON directly
@@ -36,20 +36,6 @@ module Sequel
     #   album.to_json(:root => true)
     #   # => '{"album":{"id":1,"name":"RF","artist_id":2}}'
     #
-    # In addition to creating JSON, this plugin also enables Sequel::Model
-    # objects to be automatically created when JSON is parsed:
-    #
-    #   json = album.to_json
-    #   album = JSON.parse(json)
-    #
-    # In addition, you can update existing model objects directly from JSON
-    # using +from_json+:
-    #
-    #   album.from_json(json)
-    #
-    # This works by parsing the JSON (which should return a hash), and then
-    # calling +set+ with the returned hash.
-    #
     # Additionally, +to_json+ also exists as a class and dataset method, both
     # of which return all objects in the dataset:
     #
@@ -61,6 +47,60 @@ module Sequel
     #
     #   Album.to_json(:array=>[Album[1], Album[2]])
     #
+    # In addition to creating JSON, this plugin also enables Sequel::Model
+    # classes to create instances directly from JSON using the from_json class
+    # method:
+    #
+    #   json = album.to_json
+    #   album = Album.from_json(json)
+    #
+    # The array_from_json class method exists to parse arrays of model instances
+    # from json:
+    #
+    #   json = Album.filter(:artist_id=>1).to_json
+    #   albums = Album.array_from_json(json)
+    #
+    # These does not necessarily round trip, since doing so would let users
+    # create model objects with arbitrary values.  By default, from_json will
+    # call set with the values in the hash.  If you want to specify the allowed
+    # fields, you can use the :fields option, which will call set_fields with
+    # the given fields:
+    #
+    #   Album.from_json(album.to_json, :fields=>%w'id name')
+    #
+    # If you want to update an existing instance, you can use the from_json
+    # instance method:
+    #
+    #   album.from_json(json)
+    #
+    # Both of these allow creation of cached associated objects, if you provide
+    # the :associations option:
+    #
+    #   album.from_json(json, :associations=>:artist)
+    #
+    # You can even provide options when setting up the associated objects:
+    #
+    #   album.from_json(json, :associations=>{:artist=>{:fields=>%w'id name', :associations=>:tags}})
+    #
+    # Note that active_support/json makes incompatible changes to the to_json API,
+    # and breaks some aspects of the json_serializer plugin.  You can undo the damage
+    # done by active_support/json by doing:
+    #
+    #   class Array
+    #     def to_json(options = {})
+    #       JSON.generate(self)
+    #     end
+    #   end
+    #
+    #   class Hash
+    #     def to_json(options = {})
+    #       JSON.generate(self)
+    #     end
+    #   end
+    #
+    # Note that this will probably cause active_support/json to no longer work
+    # correctly in some cases.
+    #
     # Usage:
     #
     #   # Add JSON output capability to all model subclass instances (called before loading subclasses)
@@ -97,55 +137,106 @@ module Sequel
         # The default opts to use when serializing model objects to JSON.
         attr_reader :json_serializer_opts
 
-        # Create a new model object from the hash provided by parsing
-        # JSON.  Handles column values (stored in +values+), associations
-        # (stored in +associations+), and other values (by calling a
-        # setter method).  If an entry in the hash is not a column or
-        # an association, and no setter method exists, raises an Error.
-        def json_create(hash)
-          obj = new
-          cols = columns.map{|x| x.to_s}
-          assocs = associations.map{|x| x.to_s}
-          meths = obj.send(:setter_methods, nil, nil)
-          hash.delete(JSON.create_id)
-          hash.each do |k, v|
-            if assocs.include?(k)
-              obj.associations[k.to_sym] = v
-            elsif meths.include?("#{k}=")
-              obj.send("#{k}=", v)
-            elsif cols.include?(k)
-              obj.values[k.to_sym] = v
-            else
-              raise Error, "Entry in JSON hash not an association or column and no setter method exists: #{k}"
-            end
+        # Attempt to parse a single instance from the given JSON string,
+        # with options passed to InstanceMethods#from_json_node.
+        def from_json(json, opts=OPTS)
+          v = Sequel.parse_json(json)
+          case v
+          when self
+            v
+          when Hash
+            new.from_json_node(v, opts)
+          else
+            raise Error, "parsed json doesn't return a hash or instance of #{self}"
           end
-          obj
         end
 
-        # Call the dataset +to_json+ method.
-        def to_json(*a)
-          dataset.to_json(*a)
+        # Attempt to parse an array of instances from the given JSON string,
+        # with options passed to InstanceMethods#from_json_node.
+        def array_from_json(json, opts=OPTS)
+          v = Sequel.parse_json(json)
+          if v.is_a?(Array)
+            raise(Error, 'parsed json returned an array containing non-hashes') unless v.all?{|ve| ve.is_a?(Hash) || ve.is_a?(self)}
+            v.map{|ve| ve.is_a?(self) ? ve : new.from_json_node(ve, opts)}
+          else
+            raise(Error, 'parsed json did not return an array')
+          end
         end
-        
-        # Copy the current model object's default json options into the subclass.
-        def inherited(subclass)
-          super
+
+        Plugins.inherited_instance_variables(self, :@json_serializer_opts=>lambda do |json_serializer_opts|
           opts = {}
           json_serializer_opts.each{|k, v| opts[k] = (v.is_a?(Array) || v.is_a?(Hash)) ? v.dup : v}
-          subclass.instance_variable_set(:@json_serializer_opts, opts)
-        end
+          opts
+        end)
+
+        Plugins.def_dataset_methods(self, :to_json)
       end
 
       module InstanceMethods
         # Parse the provided JSON, which should return a hash,
-        # and call +set+ with that hash.
-        def from_json(json, opts={})
-          h = JSON.parse(json)
-          if fields = opts[:fields]
-            set_fields(h, fields, opts)
-          else
-            set(h)
+        # and process the hash with from_json_node.
+        def from_json(json, opts=OPTS)
+          from_json_node(Sequel.parse_json(json), opts)
+        end
+
+        # Using the provided hash, update the instance with data contained in the hash. By default, just
+        # calls set with the hash values.
+        # 
+        # Options:
+        # :associations :: Indicates that the associations cache should be updated by creating
+        #                  a new associated object using data from the hash.  Should be a Symbol
+        #                  for a single association, an array of symbols for multiple associations,
+        #                  or a hash with symbol keys and dependent association option hash values.
+        # :fields :: Changes the behavior to call set_fields using the provided fields, instead of calling set.
+        def from_json_node(hash, opts=OPTS)
+          unless hash.is_a?(Hash)
+            raise Error, "parsed json doesn't return a hash"
           end
+
+          populate_associations = {}
+
+          if assocs = opts[:associations]
+            assocs = case assocs
+            when Symbol
+              {assocs=>{}}
+            when Array
+              assocs_tmp = {}
+              assocs.each{|v| assocs_tmp[v] = {}}
+              assocs_tmp
+            when Hash
+              assocs
+            else
+              raise Error, ":associations should be Symbol, Array, or Hash if present"
+            end
+
+            assocs.each do |assoc, assoc_opts|
+              if assoc_values = hash.delete(assoc.to_s)
+                unless r = model.association_reflection(assoc)
+                  raise Error, "Association #{assoc} is not defined for #{model}"
+                end
+
+                populate_associations[assoc] = if r.returns_array?
+                  raise Error, "Attempt to populate array association with a non-array" unless assoc_values.is_a?(Array)
+                  assoc_values.map{|v| v.is_a?(r.associated_class) ? v : r.associated_class.new.from_json_node(v, assoc_opts)}
+                else
+                  raise Error, "Attempt to populate non-array association with an array" if assoc_values.is_a?(Array)
+                  assoc_values.is_a?(r.associated_class) ? assoc_values : r.associated_class.new.from_json_node(assoc_values, assoc_opts)
+                end
+              end
+            end
+          end
+
+          if fields = opts[:fields]
+            set_fields(hash, fields, opts)
+          else
+            set(hash)
+          end
+
+          populate_associations.each do |assoc, values|
+            associations[assoc] = values
+          end
+
+          self
         end
 
         # Return a string in JSON format.  Accepts the following
@@ -159,14 +250,10 @@ module Sequel
         #             to include in the JSON output.  Using a nested
         #             hash, you can pass options to associations
         #             to affect the JSON used for associated objects.
-        # :naked :: Not to add the JSON.create_id (json_class) key to the JSON
-        #           output hash, so when the JSON is parsed, it
-        #           will yield a hash instead of a model object.
         # :only :: Symbol or Array of Symbols of columns to only
         #          include in the JSON output, ignoring all other
         #          columns.
         # :root :: Qualify the JSON with the name of the object.
-        #          Implies :naked since the object name is explicit.
         def to_json(*a)
           if opts = a.first.is_a?(Hash)
             opts = model.json_serializer_opts.merge(a.first)
@@ -180,7 +267,9 @@ module Sequel
           else
             vals.keys - Array(opts[:except])
           end
-          h = (JSON.create_id && !opts[:naked] && !opts[:root]) ? {JSON.create_id=>model.name} : {}
+
+          h = {}
+
           cols.each{|c| h[c.to_s] = send(c)}
           if inc = opts[:include]
             if inc.is_a?(Hash)
@@ -188,9 +277,9 @@ module Sequel
                 v = v.empty? ? [] : [v]
                 h[k.to_s] = case objs = send(k)
                 when Array
-                  objs.map{|obj| Literal.new(obj.to_json(*v))}
+                  objs.map{|obj| Literal.new(Sequel.object_to_json(obj, *v))}
                 else
-                  Literal.new(objs.to_json(*v))
+                  Literal.new(Sequel.object_to_json(objs, *v))
                 end
               end
             else
@@ -198,7 +287,7 @@ module Sequel
             end
           end
           h = {model.send(:underscore, model.to_s) => h} if opts[:root]
-          h.to_json(*a)
+          Sequel.object_to_json(h, *a)
         end
       end
 
@@ -231,12 +320,11 @@ module Sequel
           collection_root = case opts[:root]
           when nil, false, :instance
             false
-          when :collection
-            opts = opts.dup
-            opts.delete(:root)
-            opts[:naked] = true unless opts.has_key?(:naked)
+          when :both
             true
           else
+            opts = opts.dup
+            opts.delete(:root)
             true
           end
 
@@ -247,15 +335,15 @@ module Sequel
             else
               all
             end
-            array.map{|obj| Literal.new(obj.to_json(opts))}
+            array.map{|obj| Literal.new(Sequel.object_to_json(obj, opts))}
            else
             all
           end
 
           if collection_root
-            {model.send(:pluralize, model.send(:underscore, model.to_s)) => res}.to_json(*a)
+            Sequel.object_to_json({model.send(:pluralize, model.send(:underscore, model.to_s)) => res}, *a)
           else
-            res.to_json(*a)
+            Sequel.object_to_json(res, *a)
           end
         end
       end
