@@ -13,6 +13,29 @@ module Sequel
     #
     # Which will give you the tags for all of the artist's albums.
     #
+    # Let's break down the 2nd argument of the many_through_many call:
+    #
+    #   [[:albums_artists, :artist_id, :album_id],
+    #    [:albums, :id, :id],
+    #    [:albums_tags, :album_id, :tag_id]]
+    #
+    # This argument is an array of arrays with three elements.  Each entry in the main array represents a JOIN in SQL:
+    #
+    # * The first element in each array represents the name of the table to join.
+    # * The second element in each array represents the column used to join to the previous table.
+    # * The third element in each array represents the column used to join to the next table.
+    #
+    # So the "Artist.many_through_many :tags" is translated into something similar to:
+    #
+    #   FROM artists
+    #   JOIN albums_artists ON (artists.id = albums_artists.artist_id)
+    #   JOIN albums ON (albums_artists.album_id = albums.id)
+    #   JOIN albums_tags ON (albums.id = albums_tag.album_id)
+    #   JOIN tags ON (albums_tags.tag_id = tags.id)
+    #
+    # The "artists.id" and "tags.id" criteria come from other association options (defaulting to the primary keys of the current and
+    # associated tables), but hopefully you can see how each argument in the array is used in the JOIN clauses.
+    #
     # Here are some more examples:
     #
     #   # Same as Artist.many_to_many :albums
@@ -53,7 +76,7 @@ module Sequel
           self[:uses_left_composite_keys] ? (0...self[:through].first[:left].length).map{|i| :"x_foreign_key_#{i}_x"} : :x_foreign_key_x
         end
 
-        %w'associated_key_table eager_loading_predicate_key edges final_edge final_reverse_edge reverse_edges'.each do |meth|
+        %w'associated_key_table predicate_key edges final_edge final_reverse_edge reverse_edges'.each do |meth|
           class_eval(<<-END, __FILE__, __LINE__+1)
             def #{meth}
               cached_fetch(:#{meth}){calculate_edges[:#{meth}]}
@@ -87,7 +110,7 @@ module Sequel
 
         # Transform the :through option into a list of edges and reverse edges to use to join tables when loading the association.
         def calculate_edges
-          es = [{:left_table=>self[:model].table_name, :left_key=>self[:left_primary_key]}]
+          es = [{:left_table=>self[:model].table_name, :left_key=>self[:left_primary_key_column]}]
           self[:through].each do |t|
             es.last.merge!(:right_key=>t[:left], :right_table=>t[:table], :join_type=>t[:join_type]||self[:graph_join_type], :conditions=>(t[:conditions]||[]).to_a, :block=>t[:block])
             es.last[:only_conditions] = t[:only_conditions] if t.include?(:only_conditions)
@@ -109,7 +132,7 @@ module Sequel
                :final_reverse_edge=>final_reverse_edge,
                :edges=>edges,
                :reverse_edges=>reverse_edges,
-               :eager_loading_predicate_key=>qualify(final_reverse_alias, edges.first[:right]),
+               :predicate_key=>qualify(final_reverse_alias, edges.first[:right]),
                :associated_key_table=>final_reverse_edge[:alias],
           }
           h.each{|k, v| cached_set(k, v)}
@@ -123,25 +146,18 @@ module Sequel
         # * through - The tables and keys to join between the current table and the associated table.
         #   Must be an array, with elements that are either 3 element arrays, or hashes with keys :table, :left, and :right.
         #   The required entries in the array/hash are:
-        #   * :table (first array element) - The name of the table to join.
-        #   * :left (middle array element) - The key joining the table to the previous table. Can use an
-        #     array of symbols for a composite key association.
-        #   * :right (last array element) - The key joining the table to the next table. Can use an
-        #     array of symbols for a composite key association.
+        #   :table (first array element) :: The name of the table to join.
+        #   :left (middle array element) :: The key joining the table to the previous table. Can use an
+        #                                   array of symbols for a composite key association.
+        #   :right (last array element) :: The key joining the table to the next table. Can use an
+        #                                  array of symbols for a composite key association.
         #   If a hash is provided, the following keys are respected when using eager_graph:
-        #   * :block - A proc to use as the block argument to join.
-        #   * :conditions - Extra conditions to add to the JOIN ON clause.  Must be a hash or array of two pairs.
-        #   * :join_type - The join type to use for the join, defaults to :left_outer.
-        #   * :only_conditions - Conditions to use for the join instead of the ones specified by the keys.
-        # * opts - The options for the associaion.  Takes the same options as associate, and supports these additional options:
-        #   * :left_primary_key - column in current table that the first :left option in
-        #     through points to, as a symbol. Defaults to primary key of current table. Can use an
-        #     array of symbols for a composite key association.
-        #   * :right_primary_key - column in associated table that the final :right option in 
-        #     through points to, as a symbol. Defaults to primary key of the associated table.  Can use an
-        #     array of symbols for a composite key association.
-        #   * :uniq - Adds a after_load callback that makes the array of objects unique.
-        def many_through_many(name, through, opts={}, &block)
+        #   :block :: A proc to use as the block argument to join.
+        #   :conditions :: Extra conditions to add to the JOIN ON clause.  Must be a hash or array of two pairs.
+        #   :join_type :: The join type to use for the join, defaults to :left_outer.
+        #   :only_conditions :: Conditions to use for the join instead of the ones specified by the keys.
+        # * opts - The options for the associaion.  Takes the same options as many_to_many.
+        def many_through_many(name, through, opts=OPTS, &block)
           associate(:many_through_many, name, opts.merge(through.is_a?(Hash) ? through : {:through=>through}), &block)
         end
 
@@ -171,35 +187,32 @@ module Sequel
           uses_lcks = opts[:uses_left_composite_keys] = left_key.is_a?(Array)
           left_keys = Array(left_key)
           left_pk = (opts[:left_primary_key] ||= self.primary_key)
+          opts[:eager_loader_key] = left_pk unless opts.has_key?(:eager_loader_key)
           left_pks = opts[:left_primary_keys] = Array(left_pk)
+          lpkc = opts[:left_primary_key_column] ||= left_pk
+          opts[:left_primary_key_columns] ||= Array(lpkc)
           opts[:dataset] ||= lambda do
-            ds = opts.associated_class
-            opts.reverse_edges.each{|t| ds = ds.join(t[:table], Array(t[:left]).zip(Array(t[:right])), :table_alias=>t[:alias])}
+            ds = opts.associated_dataset
+            opts.reverse_edges.each{|t| ds = ds.join(t[:table], Array(t[:left]).zip(Array(t[:right])), :table_alias=>t[:alias], :qualify=>:deep)}
             ft = opts.final_reverse_edge
-            ds.join(ft[:table],  Array(ft[:left]).zip(Array(ft[:right])) + left_keys.zip(left_pks.map{|k| send(k)}), :table_alias=>ft[:alias])
+            ds.join(ft[:table],  Array(ft[:left]).zip(Array(ft[:right])) + opts.predicate_keys.zip(left_pks.map{|k| send(k)}), :table_alias=>ft[:alias], :qualify=>:deep)
           end
 
+          slice_range = opts.slice_range
           left_key_alias = opts[:left_key_alias] ||= opts.default_associated_key_alias
           opts[:eager_loader] ||= lambda do |eo|
-            h = eo[:key_hash][left_pk]
+            h = eo[:id_map]
             rows = eo[:rows]
             rows.each{|object| object.associations[name] = []}
             ds = opts.associated_class 
-            opts.reverse_edges.each{|t| ds = ds.join(t[:table], Array(t[:left]).zip(Array(t[:right])), :table_alias=>t[:alias])}
+            opts.reverse_edges.each{|t| ds = ds.join(t[:table], Array(t[:left]).zip(Array(t[:right])), :table_alias=>t[:alias], :qualify=>:deep)}
             ft = opts.final_reverse_edge
-            ds = ds.join(ft[:table], Array(ft[:left]).zip(Array(ft[:right])) + [[opts.eager_loading_predicate_key, h.keys]], :table_alias=>ft[:alias])
+            ds = ds.join(ft[:table], Array(ft[:left]).zip(Array(ft[:right])) + [[opts.predicate_key, h.keys]], :table_alias=>ft[:alias], :qualify=>:deep)
             ds = model.eager_loading_dataset(opts, ds, nil, eo[:associations], eo)
-            case opts.eager_limit_strategy
-            when :window_function
+            if opts.eager_limit_strategy == :window_function
               delete_rn = true
               rn = ds.row_number_column
               ds = apply_window_function_eager_limit_strategy(ds, opts)
-            when :correlated_subquery
-              ds = apply_correlated_subquery_eager_limit_strategy(ds, opts) do |xds|
-                dsa = ds.send(:dataset_alias, 2)
-                opts.reverse_edges.each{|t| xds = xds.join(t[:table], Array(t[:left]).zip(Array(t[:right])), :table_alias=>t[:alias])}
-                xds.join(ft[:table], Array(ft[:left]).zip(Array(ft[:right])) + left_keys.map{|k| [k, SQL::QualifiedIdentifier.new(ft[:table], k)]}, :table_alias=>dsa)
-              end
             end
             ds.all do |assoc_record|
               assoc_record.values.delete(rn) if delete_rn
@@ -212,8 +225,7 @@ module Sequel
               objects.each{|object| object.associations[name].push(assoc_record)}
             end
             if opts.eager_limit_strategy == :ruby
-              limit, offset = opts.limit_and_offset
-              rows.each{|o| o.associations[name] = o.associations[name].slice(offset||0, limit) || []}
+              rows.each{|o| o.associations[name] = o.associations[name][slice_range] || []}
             end
           end
 
@@ -227,11 +239,11 @@ module Sequel
             ds = eo[:self]
             iq = eo[:implicit_qualifier]
             opts.edges.each do |t|
-              ds = ds.graph(t[:table], t.fetch(:only_conditions, (Array(t[:right]).zip(Array(t[:left])) + t[:conditions])), :select=>false, :table_alias=>ds.unused_table_alias(t[:table]), :join_type=>t[:join_type], :implicit_qualifier=>iq, &t[:block])
+              ds = ds.graph(t[:table], t.fetch(:only_conditions, (Array(t[:right]).zip(Array(t[:left])) + t[:conditions])), :select=>false, :table_alias=>ds.unused_table_alias(t[:table]), :join_type=>t[:join_type], :qualify=>:deep, :implicit_qualifier=>iq, &t[:block])
               iq = nil
             end
             fe = opts.final_edge
-            ds.graph(opts.associated_class, use_only_conditions ? only_conditions : (Array(opts.right_primary_key).zip(Array(fe[:left])) + conditions), :select=>select, :table_alias=>eo[:table_alias], :join_type=>join_type, &graph_block)
+            ds.graph(opts.associated_class, use_only_conditions ? only_conditions : (Array(opts.right_primary_key).zip(Array(fe[:left])) + conditions), :select=>select, :table_alias=>eo[:table_alias], :qualify=>:deep, :join_type=>join_type, &graph_block)
           end
 
           def_association_dataset_methods(opts)
@@ -243,22 +255,26 @@ module Sequel
 
         # Use a subquery to filter rows to those related to the given associated object
         def many_through_many_association_filter_expression(op, ref, obj)
-          lpks = ref[:left_primary_keys]
+          lpks = ref[:left_primary_key_columns]
           lpks = lpks.first if lpks.length == 1
           lpks = ref.qualify(model.table_name, lpks)
           edges = ref.edges
           first, rest = edges.first, edges[1..-1]
-          last = edges.last
           ds = model.db[first[:table]].select(*Array(ref.qualify(first[:table], first[:right])))
-          rest.each{|e| ds = ds.join(e[:table], e.fetch(:only_conditions, (Array(e[:right]).zip(Array(e[:left])) + e[:conditions])), :table_alias=>ds.unused_table_alias(e[:table]), &e[:block])}
+          rest.each{|e| ds = ds.join(e[:table], e.fetch(:only_conditions, (Array(e[:right]).zip(Array(e[:left])) + e[:conditions])), :table_alias=>ds.unused_table_alias(e[:table]), :qualify=>:deep, &e[:block])}
           last_alias = if rest.empty?
             first[:table]
           else
             last_join = ds.opts[:join].last
             last_join.table_alias || last_join.table
           end
-          meths = ref.right_primary_keys
-          meths = ref.qualify(obj.model.table_name, meths) if obj.is_a?(Sequel::Dataset)
+
+          meths = if obj.is_a?(Sequel::Dataset)
+            ref.qualify(obj.model.table_name, ref.right_primary_keys)
+          else
+            ref.right_primary_key_methods
+          end
+
           exp = association_filter_key_expression(ref.qualify(last_alias, Array(ref.final_edge[:left])), meths, obj)
           if exp == SQL::Constants::FALSE
             association_filter_handle_inversion(op, exp, Array(lpks))
