@@ -83,8 +83,11 @@ module Sequel
         
         # Allow nested attributes to be set for the given associations.  Options:
         # * :destroy - Allow destruction of nested records.
-        # * :fields - If provided, should be an Array.  Restricts the fields allowed to be
-        #   modified through the association_attributes= method to the specific fields given.
+        # * :fields - If provided, should be an Array or proc. If it is an array,
+        #   restricts the fields allowed to be modified through the
+        #   association_attributes= method to the specific fields given. If it is
+        #   a proc, it will be called with the associated object and should return an
+        #   array of the allowable fields.
         # * :limit - For *_to_many associations, a limit on the number of records
         #   that will be processed, to prevent denial of service attacks.
         # * :reject_if - A proc that is given each attribute hash before it is
@@ -168,7 +171,11 @@ module Sequel
 
             # Don't need to validate the object twice if :validate association option is not false
             # and don't want to validate it at all if it is false.
-            send(reflection[:type] == :many_to_one ? :before_save_hook : :after_save_hook){send(reflection.setter_method, obj.save(:validate=>false))}
+            if reflection[:type] == :many_to_one 
+              before_save_hook{send(reflection.setter_method, obj.save(:validate=>false))}
+            else
+              after_save_hook{send(reflection.setter_method, obj)}
+            end
           end
           obj
         end
@@ -189,7 +196,7 @@ module Sequel
         # :destroy option is given, destroy the object after disassociating it
         # (unless destroying the object would automatically disassociate it).
         # Returns the object removed.
-        def nested_attributes_remove(reflection, obj, opts={})
+        def nested_attributes_remove(reflection, obj, opts=OPTS)
           if !opts[:destroy] || reflection.remove_before_destroy?
             before_save_hook do
               if reflection.returns_array?
@@ -207,6 +214,7 @@ module Sequel
         # specific :fields if configured.
         def nested_attributes_set_attributes(reflection, obj, attributes)
           if fields = reflection[:nested_attributes][:fields]
+            fields = fields.call(obj) if fields.respond_to?(:call)
             obj.set_only(attributes, fields)
           else
             obj.set(attributes)
