@@ -30,11 +30,11 @@ module Sequel
     #
     # == Example
     #
-    #   require 'sequel'
-    #   # Require json, as the plugin doesn't require it for you.
+    #   # Require json if you plan to use it, as the plugin doesn't require it for you.
     #   require 'json'
     #
-    #   # Register custom serializer/deserializer pair
+    #   # Register custom serializer/deserializer pair, if desired
+    #   require 'sequel/plugins/serialization'
     #   Sequel::Plugins::Serialization.register_format(:reverse,
     #     lambda{|v| v.reverse},
     #     lambda{|v| v.reverse})
@@ -82,9 +82,21 @@ module Sequel
       def self.register_format(format, serializer, deserializer)
         REGISTERED_FORMATS[format] = [serializer, deserializer]
       end
-      register_format(:marshal, lambda{|v| [Marshal.dump(v)].pack('m')}, lambda{|v| Marshal.load(v.unpack('m')[0]) rescue Marshal.load(v)})
+      register_format(:marshal, lambda{|v| [Marshal.dump(v)].pack('m')},
+        lambda do |v|
+          begin
+            Marshal.load(v.unpack('m')[0])
+          rescue => e
+            begin
+              # Backwards compatibility for unpacked marshal output.
+              Marshal.load(v)
+            rescue
+              raise e
+            end
+          end
+        end)
       register_format(:yaml, lambda{|v| v.to_yaml}, lambda{|v| YAML.load(v)})
-      register_format(:json, lambda{|v| v.to_json}, lambda{|v| JSON.parse(v)})
+      register_format(:json, lambda{|v| Sequel.object_to_json(v)}, lambda{|v| Sequel.parse_json(v)})
 
       module ClassMethods
         # A hash with column name symbols and callable values, with the value
@@ -99,16 +111,7 @@ module Sequel
         # call be overridden and call super to get the serialization behavior
         attr_accessor :serialization_module
 
-        # Copy the serialization_map and deserialization map into the subclass.
-        def inherited(subclass)
-          super
-          sm = serialization_map.dup
-          dsm = deserialization_map.dup
-          subclass.instance_eval do
-            @deserialization_map = dsm
-            @serialization_map = sm
-          end
-        end
+        Plugins.inherited_instance_variables(self, :@deserialization_map=>:dup, :@serialization_map=>:dup)
         
         # Create instance level reader that deserializes column values on request,
         # and instance level writer that stores new deserialized values.
@@ -142,6 +145,8 @@ module Sequel
               define_method(column) do 
                 if deserialized_values.has_key?(column)
                   deserialized_values[column]
+                elsif frozen?
+                  deserialize_value(column, super())
                 else
                   deserialized_values[column] = deserialize_value(column, super())
                 end
@@ -156,32 +161,28 @@ module Sequel
       end
 
       module InstanceMethods
-        # Hash of deserialized values, used as a cache.
-        attr_reader :deserialized_values
-
-        # Set @deserialized_values to the empty hash
-        def initialize_set(values)
-          @deserialized_values = {}
-          super
-        end
-
         # Serialize deserialized values before saving
         def before_save
           serialize_deserialized_values
           super
         end
         
-        # Initialization the deserialized values for objects retrieved from the database.
-        def set_values(*)
+        # Hash of deserialized values, used as a cache.
+        def deserialized_values
           @deserialized_values ||= {}
+        end
+
+        # Freeze the deserialized values
+        def freeze
+          deserialized_values.freeze
           super
         end
 
         private
 
-        # Empty the deserialized values when refreshing.
-        def _refresh(*)
-          @deserialized_values = {}
+        # Clear any cached deserialized values when doing a manual refresh.
+        def _refresh_set_values(hash)
+          @deserialized_values.clear if @deserialized_values
           super
         end
 
