@@ -21,7 +21,6 @@ module Sequel
       # Setup the validations hash for the given model.
       def self.apply(model)
         model.class_eval do
-          @validation_mutex = Mutex.new
           @validations = {}
           @validation_reflections = {}
         end
@@ -51,6 +50,11 @@ module Sequel
           def method_missing(m, *args, &block)
             @receiver.send(:"validates_#{m}", *args, &block)
           end
+
+          # This object responds to all validates_* methods the model responds to.
+          def respond_to_missing?(meth, include_private)
+            @receiver.respond_to?(:"validates_#{meth}", include_private)
+          end
         end
     
         # Returns true if validations are defined.
@@ -58,27 +62,21 @@ module Sequel
           !validations.empty?
         end
 
-        # Setup the validations and validation_reflections hash in the subclass.
-        def inherited(subclass)
-          vr = @validation_reflections
-          subclass.class_eval do
-            @validation_mutex = Mutex.new
-            @validations = {}
-            h = {}
-            vr.each{|k,v| h[k] = v.dup}
-            @validation_reflections = h
-          end
-          super
-        end
-    
+        Plugins.inherited_instance_variables(self, :@validations=>:hash_dup, :@validation_reflections=>:hash_dup)
+
         # Instructs the model to skip validations defined in superclasses
         def skip_superclass_validations
+          superclass.validations.each do |att, procs|
+            if ps = @validations[att]
+              @validations[att] -= procs
+            end
+          end
           @skip_superclass_validations = true
         end
         
         # Instructs the model to skip validations defined in superclasses
         def skip_superclass_validations?
-          defined?(@skip_superclass_validations) && @skip_superclass_validations
+          @skip_superclass_validations
         end
 
         # Defines validations by converting a longhand block into a series of 
@@ -102,7 +100,6 @@ module Sequel
     
         # Validates the given instance.
         def validate(o)
-          superclass.validate(o) if superclass.respond_to?(:validate) && !skip_superclass_validations?
           validations.each do |att, procs|
             v = case att
             when Array
@@ -195,7 +192,7 @@ module Sequel
           end
           tag = opts[:tag]
           atts.each do |a| 
-            a_vals = @validation_mutex.synchronize{validations[a] ||= []}
+            a_vals = Sequel.synchronize{validations[a] ||= []}
             if tag && (old = a_vals.find{|x| x[0] == tag})
               old[1] = blk
             else
@@ -267,35 +264,6 @@ module Sequel
           end
         end
     
-        # Validates whether an attribute is not a string.  This is generally useful
-        # in conjunction with raise_on_typecast_failure = false, where you are
-        # passing in string values for non-string attributes (such as numbers and dates).
-        # If typecasting fails (invalid number or date), the value of the attribute will
-        # be a string in an invalid format, and if typecasting succeeds, the value will
-        # not be a string.
-        #
-        # Possible Options:
-        # * :message - The message to use (default: 'is a string' or 'is not a valid (integer|datetime|etc.)' if the type is known)
-        def validates_not_string(*atts)
-          opts = {
-            :tag => :not_string,
-          }.merge!(extract_options!(atts))
-          reflect_validation(:not_string, opts, atts)
-          atts << opts
-          validates_each(*atts) do |o, a, v|
-            if v.is_a?(String)
-              unless message = opts[:message]
-                message = if sch = o.db_schema[a] and typ = sch[:type]
-                  "is not a valid #{typ}"
-                else
-                  "is a string"
-                end
-              end
-              o.errors.add(a, message)
-            end
-          end
-        end
-    
         # Validates whether an attribute is a number.
         #
         # Possible Options:
@@ -357,6 +325,28 @@ module Sequel
           end
         end
     
+        # Validates whether an attribute has the correct ruby type for the associated
+        # database type.  This is generally useful in conjunction with
+        # raise_on_typecast_failure = false, to handle typecasting errors at validation
+        # time instead of at setter time. 
+        #
+        # Possible Options:
+        # * :message - The message to use (default: 'is not a valid (integer|datetime|etc.)')
+        def validates_schema_type(*atts)
+          opts = {
+            :tag => :schema_type,
+          }.merge!(extract_options!(atts))
+          reflect_validation(:schema_type, opts, atts)
+          atts << opts
+          validates_each(*atts) do |o, a, v|
+            next if v.nil? || (klass = o.send(:schema_type_class, a)).nil?
+            if klass.is_a?(Array) ? !klass.any?{|kls| v.is_a?(kls)} : !v.is_a?(klass)
+              message = opts[:message] || "is not a valid #{Array(klass).join(" or ").downcase}"
+              o.errors.add(a, message)
+            end
+          end
+        end
+    
         # Validates only if the fields in the model (specified by atts) are
         # unique in the database.  Pass an array of fields instead of multiple
         # fields to specify that the combination of fields must be unique,
@@ -385,6 +375,7 @@ module Sequel
             error_field = a
             a = Array(a)
             v = Array(v)
+            next if v.empty? || !v.all?
             ds = o.class.filter(a.zip(v))
             num_dups = ds.count
             allow = if num_dups == 0
@@ -426,10 +417,12 @@ module Sequel
         # Handle the :if option for validations
         def validation_if_proc(o, i)
           case i
-          when Symbol then o.send(i)
-          when Proc then o.instance_eval(&i)
-          when nil then true
-          else raise(::Sequel::Error, "invalid value for :if validation option")
+          when Symbol
+            o.send(i)
+          when Proc
+            o.instance_eval(&i)
+          else
+            raise(::Sequel::Error, "invalid value for :if validation option")
           end
         end
       end
@@ -438,6 +431,7 @@ module Sequel
         # Validates the object.
         def validate
           model.validate(self)
+          super
         end
       end
     end
