@@ -1,12 +1,13 @@
 module Sequel
   if RUBY_VERSION < '1.9.0'
+  # :nocov:
     # If on Ruby 1.8, create a <tt>Sequel::BasicObject</tt> class that is similar to the
     # the Ruby 1.9 +BasicObject+ class.  This is used in a few places where proxy
     # objects are needed that respond to any method call.
     class BasicObject
       # The instance methods to not remove from the class when removing
       # other methods.
-      KEEP_METHODS = %w"__id__ __send__ __metaclass__ instance_eval == equal? initialize method_missing"
+      KEEP_METHODS = %w"__id__ __send__ __metaclass__ instance_eval instance_exec == equal? initialize method_missing"
 
       # Remove all but the most basic instance methods from the class.  A separate
       # method so that it can be called again if necessary if you load libraries
@@ -16,6 +17,7 @@ module Sequel
       end
       remove_methods!
     end
+  # :nocov:
   else
     # If on 1.9, create a <tt>Sequel::BasicObject</tt> class that is just like the
     # default +BasicObject+ class, except that missing constants are resolved in
@@ -60,29 +62,40 @@ module Sequel
 
     # Base class for all SQL expression objects.
     class Expression
-      # Expression objects are assumed to be value objects, where their
-      # attribute values can't change after assignment.  In order to make
-      # it easy to define equality and hash methods, subclass
-      # instances assume that the only values that affect the results of
-      # such methods are the values of the object's attributes.
-      def self.attr_reader(*args)
-        super
-        comparison_attrs.concat args
-      end
+      @comparison_attrs = []
 
-      # All attributes used for equality and hash methods.
-      def self.comparison_attrs
-        @comparison_attrs ||= self == Expression ? [] : superclass.comparison_attrs.clone
-      end
+      class << self
+        # All attributes used for equality and hash methods.
+        attr_reader :comparison_attrs
 
-      # Create a to_s instance method that takes a dataset, and calls
-      # the method provided on the dataset with args as the argument (self by default).
-      # Used to DRY up some code.
-      def self.to_s_method(meth, args=:self) # :nodoc:
-        class_eval("def to_s(ds) ds.#{meth}(#{args}) end", __FILE__, __LINE__)
-        class_eval("def to_s_append(ds, sql) ds.#{meth}_append(sql, #{args}) end", __FILE__, __LINE__)
+        # Expression objects are assumed to be value objects, where their
+        # attribute values can't change after assignment.  In order to make
+        # it easy to define equality and hash methods, subclass
+        # instances assume that the only values that affect the results of
+        # such methods are the values of the object's attributes.
+        def attr_reader(*args)
+          super
+          comparison_attrs.concat(args)
+        end
+
+        # Copy the comparison_attrs into the subclass.
+        def inherited(subclass)
+          super
+          subclass.instance_variable_set(:@comparison_attrs, comparison_attrs.dup)
+        end
+
+          private
+
+        # Create a to_s instance method that takes a dataset, and calls
+        # the method provided on the dataset with args as the argument (self by default).
+        # Used to DRY up some code.
+        #
+        # Do not call this method with untrusted input, as that can result in
+        # arbitrary code execution.
+        def to_s_method(meth, args=:self) # :nodoc:
+          class_eval("def to_s_append(ds, sql) ds.#{meth}_append(sql, #{args}) end", __FILE__, __LINE__)
+        end
       end
-      private_class_method :to_s_method
 
       # Alias of <tt>eql?</tt>
       def ==(other)
@@ -113,7 +126,9 @@ module Sequel
       
       # Alias of +to_s+
       def sql_literal(ds)
-        to_s(ds)
+        s = ''
+        to_s_append(ds, s)
+        s
       end
     end
 
@@ -140,6 +155,9 @@ module Sequel
       # Bitwise mathematical operators used in +NumericMethods+
       BITWISE_OPERATORS = [:&, :|, :^, :<<, :>>, :%]
 
+      # Operators that check for equality
+      EQUALITY_OPERATORS = [:'=', :'!=']
+
       # Inequality operators used in +InequalityMethods+
       INEQUALITY_OPERATORS = [:<, :>, :<=, :>=]
 
@@ -152,10 +170,14 @@ module Sequel
       # Operators that use IS, used for special casing to override literal true/false values
       IS_OPERATORS = [:IS, :'IS NOT']
 
+      # Operators that do pattern matching via regular expressions
+      REGEXP_OPERATORS = [:~, :'!~', :'~*', :'!~*']
+      
+      # Operators that do pattern matching via LIKE
+      LIKE_OPERATORS = [:LIKE, :'NOT LIKE', :ILIKE, :'NOT ILIKE']
+
       # Operator symbols that take exactly two arguments
-      TWO_ARITY_OPERATORS = [:'=', :'!=', :LIKE, :'NOT LIKE', \
-        :~, :'!~', :'~*', :'!~*', :ILIKE, :'NOT ILIKE'] + \
-        INEQUALITY_OPERATORS + IS_OPERATORS + IN_OPERATORS
+      TWO_ARITY_OPERATORS = EQUALITY_OPERATORS + INEQUALITY_OPERATORS + IS_OPERATORS + IN_OPERATORS + REGEXP_OPERATORS + LIKE_OPERATORS
 
       # Operator symbols that take one or more arguments
       N_ARITY_OPERATORS = [:AND, :OR, :'||'] + MATHEMATICAL_OPERATORS + BITWISE_OPERATORS
@@ -166,12 +188,12 @@ module Sequel
       # Custom expressions that may have different syntax on different databases
       CUSTOM_EXPRESSIONS = [:extract]
 
-      # An array of args for this object
-      attr_reader :args
-
       # The operator symbol for this object
       attr_reader :op
       
+      # An array of args for this object
+      attr_reader :args
+
       # Set the operator symbol and arguments for this object to the ones given.
       # Convert all args that are hashes or arrays of two element arrays to +BooleanExpressions+,
       # other than the second arg for an IN/NOT IN operator.
@@ -217,7 +239,7 @@ module Sequel
     module AliasMethods
       # Create an SQL alias (+AliasedExpression+) of the receiving column or expression to the given alias.
       #
-      #   :column.as(:alias) # "column" AS "alias"
+      #   Sequel.function(:func).as(:alias) # func() AS "alias"
       def as(aliaz)
         AliasedExpression.new(self, aliaz)
       end
@@ -300,7 +322,7 @@ module Sequel
       #
       #   Sequel.asc(:a) # a ASC
       #   Sequel.asc(:b, :nulls=>:last) # b ASC NULLS LAST
-      def asc(arg, opts={})
+      def asc(arg, opts=OPTS)
         SQL::OrderedExpression.new(arg, false, opts)
       end
 
@@ -352,6 +374,46 @@ module Sequel
         cast(arg, sql_type || String).sql_string
       end
 
+      # Return an emulated function call for getting the number of characters
+      # in the argument:
+      #
+      #   Sequel.char_length(:a) # char_length(a) -- Most databases
+      #   Sequel.char_length(:a) # length(a) -- SQLite
+      def char_length(arg)
+        SQL::EmulatedFunction.new(:char_length, arg)
+      end
+
+      # Do a deep qualification of the argument using the qualifier.  This recurses into
+      # nested structures.
+      #
+      #   Sequel.deep_qualify(:table, :column) # "table"."column"
+      #   Sequel.deep_qualify(:table, Sequel.+(:column, 1)) # "table"."column" + 1
+      #   Sequel.deep_qualify(:table, Sequel.like(:a, 'b')) # "table"."a" LIKE 'b' ESCAPE '\'
+      def deep_qualify(qualifier, expr)
+        Sequel::Qualifier.new(Sequel, qualifier).transform(expr)
+      end
+
+      # Return a delayed evaluation that uses the passed block. This is used
+      # to delay evaluations of the code to runtime.  For example, with
+      # the following code:
+      #
+      #   ds = DB[:table].where{column > Time.now}
+      #
+      # The filter is fixed to the time that where was called. Unless you are
+      # only using the dataset once immediately after creating it, that's
+      # probably not desired.  If you just want to set it to the time when the
+      # query is sent to the database, you can wrap it in Sequel.delay:
+      #
+      #   ds = DB[:table].where{column > Sequel.delay{Time.now}}
+      #
+      # Note that for dates and timestamps, you are probably better off using
+      # Sequel::CURRENT_DATE and Sequel::CURRENT_TIMESTAMP instead of this
+      # generic delayed evaluation facility.
+      def delay(&block)
+        raise(Error, "Sequel.delay requires a block") unless block
+        SQL::DelayedEvaluation.new(block)
+      end
+
       # Order the given argument descending.
       # Options:
       #
@@ -361,7 +423,7 @@ module Sequel
       #
       #   Sequel.desc(:a) # b DESC
       #   Sequel.desc(:b, :nulls=>:first) # b DESC NULLS FIRST
-      def desc(arg, opts={})
+      def desc(arg, opts=OPTS)
         SQL::OrderedExpression.new(arg, true, opts)
       end
 
@@ -389,6 +451,20 @@ module Sequel
         end
 
         case arg
+        when Symbol
+          t, c, a = Sequel.split_symbol(arg)
+
+          arg = if t
+            SQL::QualifiedIdentifier.new(t, c)
+          else
+            SQL::Identifier.new(c)
+          end
+
+          if a
+            arg = SQL::AliasedExpression.new(arg, a)
+          end
+
+          arg
         when SQL::Expression, LiteralString, SQL::Blob
           arg
         when Hash
@@ -549,10 +625,21 @@ module Sequel
       #   Sequel.subscript(:array, 1) # array[1]
       #   Sequel.subscript(:array, 1, 2) # array[1, 2]
       #   Sequel.subscript(:array, [1, 2]) # array[1, 2]
+      #   Sequel.subscript(:array, 1..2) # array[1:2]
+      #   Sequel.subscript(:array, 1...3) # array[1:2]
       def subscript(exp, *subs)
         SQL::Subscript.new(exp, subs.flatten)
       end
 
+      # Return an emulated function call for trimming a string of spaces from
+      # both sides (similar to ruby's String#strip).
+      #
+      #   Sequel.trim(:a) # trim(a) -- Most databases
+      #   Sequel.trim(:a) # ltrim(rtrim(a)) -- Microsoft SQL Server
+      def trim(arg)
+        SQL::EmulatedFunction.new(:trim, arg)
+      end
+
       # Return a <tt>SQL::ValueList</tt> created from the given array.  Used if the array contains
       # all two element arrays and you want it treated as an SQL value list (IN predicate) 
       # instead of as a conditions specifier (similar to a hash).  This is not necessary if you are using
@@ -573,8 +660,8 @@ module Sequel
       # Cast the reciever to the given SQL type.  You can specify a ruby class as a type,
       # and it is handled similarly to using a database independent type in the schema methods.
       #
-      #   :a.cast(:integer) # CAST(a AS integer)
-      #   :a.cast(String) # CAST(a AS varchar(255))
+      #   Sequel.function(:func).cast(:integer) # CAST(func() AS integer)
+      #   Sequel.function(:func).cast(String) # CAST(func() AS varchar(255))
       def cast(sql_type)
         Cast.new(self, sql_type)
       end
@@ -583,23 +670,23 @@ module Sequel
       # and return the result as a +NumericExpression+, so you can use the bitwise operators
       # on the result. 
       #
-      #   :a.cast_numeric # CAST(a AS integer)
-      #   :a.cast_numeric(Float) # CAST(a AS double precision)
+      #   Sequel.function(:func).cast_numeric # CAST(func() AS integer)
+      #   Sequel.function(:func).cast_numeric(Float) # CAST(func() AS double precision)
       def cast_numeric(sql_type = nil)
-        cast(sql_type || Integer).sql_number
+        Cast.new(self, sql_type || Integer).sql_number
       end
 
       # Cast the reciever to the given SQL type (or the database's default String type if none given),
       # and return the result as a +StringExpression+, so you can use +
       # directly on the result for SQL string concatenation.
       #
-      #   :a.cast_string # CAST(a AS varchar(255))
-      #   :a.cast_string(:text) # CAST(a AS text)
+      #   Sequel.function(:func).cast_string # CAST(func() AS varchar(255))
+      #   Sequel.function(:func).cast_string(:text) # CAST(func() AS text)
       def cast_string(sql_type = nil)
-        cast(sql_type || String).sql_string
+        Cast.new(self, sql_type || String).sql_string
       end
     end
-    
+
     # Adds methods that allow you to treat an object as an instance of a specific
     # +ComplexExpression+ subclass.  This is useful if another library
     # overrides the methods defined by Sequel.
@@ -614,7 +701,7 @@ module Sequel
     #
     #   :price.sql_number/10 > 100
     module ComplexExpressionMethods
-      # Extract a datetime_part (e.g. year, month) from self:
+      # Extract a datetime part (e.g. year, month) from self:
       #
       #   :date.extract(:year) # extract(year FROM "date")
       #
@@ -646,35 +733,19 @@ module Sequel
       end
     end
 
-    # Includes an +identifier+ method that returns <tt>Identifier</tt>s.
-    module IdentifierMethods
-      # Return self wrapped as an <tt>SQL::Identifier</tt>.
-      #
-      #   :a__b # "a"."b"
-      #   :a__b.identifier # "a__b"
-      def identifier
-        Identifier.new(self)
-      end
-    end
-
     # This module includes the inequality methods (>, <, >=, <=) that are defined on objects that can be 
-    # used in a numeric or string context in SQL (+Symbol+ (except on ruby 1.9), +LiteralString+, 
-    # <tt>SQL::GenericExpression</tt>).
+    # used in a numeric or string context in SQL.
     #
-    #   'a'.lit > :b # a > "b"
-    #   'a'.lit < :b # a > "b"
-    #   'a'.lit >= :b # a >= "b"
-    #   'a'.lit <= :b # a <= "b"
+    #   Sequel.lit('a') > :b # a > "b"
+    #   Sequel.lit('a') < :b # a > "b"
+    #   Sequel.lit('a') >= :b # a >= "b"
+    #   Sequel.lit('a') <= :b # a <= "b"
     module InequalityMethods
       ComplexExpression::INEQUALITY_OPERATORS.each do |o|
         module_eval("def #{o}(o) BooleanExpression.new(#{o.inspect}, self, o) end", __FILE__, __LINE__)
       end
     end
 
-    # Only exists for backwards compatibility, ignore it.
-    module NoBooleanInputMethods
-    end
-
     # This module includes the standard mathematical methods (+, -, *, and /)
     # that are defined on objects that can be used in a numeric context in SQL
     # (+Symbol+, +LiteralString+, and +SQL::GenericExpression+).
@@ -756,7 +827,7 @@ module Sequel
       # :nulls :: Set to :first to use NULLS FIRST (so NULL values are ordered
       #           before other values), or :last to use NULLS LAST (so NULL values
       #           are ordered after other values).
-      def asc(opts={})
+      def asc(opts=OPTS)
         OrderedExpression.new(self, false, opts)
       end
       
@@ -766,19 +837,31 @@ module Sequel
       # :nulls :: Set to :first to use NULLS FIRST (so NULL values are ordered
       #           before other values), or :last to use NULLS LAST (so NULL values
       #           are ordered after other values).
-      def desc(opts={})
+      def desc(opts=OPTS)
         OrderedExpression.new(self, true, opts)
       end
     end
 
     # Includes a +qualify+ method that created <tt>QualifiedIdentifier</tt>s, used for qualifying column
-    # names with a table or table names with a schema.
+    # names with a table or table names with a schema, and the * method for returning all columns in
+    # the identifier if no arguments are given.
     module QualifyingMethods
+      # If no arguments are given, return an SQL::ColumnAll:
+      #
+      #   Sequel.expr(:a__b).*  # a.b.*
+      def *(ce=(arg=false;nil))
+        if arg == false
+          Sequel::SQL::ColumnAll.new(self)
+        else
+          super(ce)
+        end
+      end
+
       # Qualify the receiver with the given +qualifier+ (table for column/schema for table).
       #
-      #   :column.qualify(:table) # "table"."column"
-      #   :table.qualify(:schema) # "schema"."table"
-      #   :column.qualify(:table).qualify(:schema) # "schema"."table"."column"
+      #   Sequel.expr(:column).qualify(:table) # "table"."column"
+      #   Sequel.expr(:table).qualify(:schema) # "schema"."table"
+      #   Sequel.qualify(:table, :column).qualify(:schema) # "schema"."table"."column"
       def qualify(qualifier)
         QualifiedIdentifier.new(qualifier, self)
       end
@@ -824,6 +907,8 @@ module Sequel
       #   :array.sql_subscript(1) # array[1]
       #   :array.sql_subscript(1, 2) # array[1, 2]
       #   :array.sql_subscript([1, 2]) # array[1, 2]
+      #   :array.sql_subscript(:array, 1..2) # array[1:2]
+      #   :array.sql_subscript(:array, 1...3) # array[1:2]
       def sql_subscript(*sub)
         Subscript.new(self, sub.flatten)
       end
@@ -852,8 +937,16 @@ module Sequel
     # stored as a blob type in the database. Sequel represents binary data as a Blob object because 
     # most database engines require binary data to be escaped differently than regular strings.
     class Blob < ::String
-      # Returns +self+, used so that Blobs don't get wrapped in multiple
-      # levels.
+      include SQL::AliasMethods
+      include SQL::CastMethods
+
+      # Return a LiteralString with the same content if no args are given, otherwise
+      # return a SQL::PlaceholderLiteralString with the current string and the given args.
+      def lit(*args)
+        args.empty? ? LiteralString.new(self) : SQL::PlaceholderLiteralString.new(self, args)
+      end
+      
+      # Returns +self+, since it is already a blob.
       def to_sequel_blob
         self
       end
@@ -885,27 +978,33 @@ module Sequel
       #   ~from_value_pairs(hash)
       #   from_value_pairs(hash, :OR, true)
       def self.from_value_pairs(pairs, op=:AND, negate=false)
-        pairs = pairs.collect do |l,r|
-          ce = case r
-          when Range
-            new(:AND, new(:>=, l, r.begin), new(r.exclude_end? ? :< : :<=, l, r.end))
-          when ::Array, ::Sequel::Dataset
-            new(:IN, l, r)
-          when NegativeBooleanConstant
-            new(:"IS NOT", l, r.constant)
-          when BooleanConstant
-            new(:IS, l, r.constant)
-          when NilClass, TrueClass, FalseClass
-            new(:IS, l, r)
-          when Regexp
-            StringExpression.like(l, r)
-          else
-            new(:'=', l, r)
-          end
-          negate ? invert(ce) : ce
-        end
+        pairs = pairs.map{|l,r| from_value_pair(l, r)}
+        pairs.map!{|ce| invert(ce)} if negate
         pairs.length == 1 ? pairs.at(0) : new(op, *pairs)
       end
+
+      # Return a BooleanExpression based on the right side of the pair.
+      def self.from_value_pair(l, r)
+        case r
+        when Range
+          new(:AND, new(:>=, l, r.begin), new(r.exclude_end? ? :< : :<=, l, r.end))
+        when ::Array, ::Sequel::Dataset
+          new(:IN, l, r)
+        when NegativeBooleanConstant
+          new(:"IS NOT", l, r.constant)
+        when BooleanConstant
+          new(:IS, l, r.constant)
+        when NilClass, TrueClass, FalseClass
+          new(:IS, l, r)
+        when Regexp
+          StringExpression.like(l, r)
+        when DelayedEvaluation
+          Sequel.delay{from_value_pair(l, r.callable.call)}
+        else
+          new(:'=', l, r)
+        end
+      end
+      private_class_method :from_value_pair
       
       # Invert the expression, if possible.  If the expression cannot
       # be inverted, raise an error.  An inverted expression should match everything that the
@@ -974,6 +1073,17 @@ module Sequel
         !@no_expression
       end
 
+      # Merge the CASE expression into the conditions, useful for databases that
+      # don't support CASE expressions.
+      def with_merged_expression
+        if expression?
+          e = expression
+          CaseExpression.new(conditions.map{|c, r| [::Sequel::SQL::BooleanExpression.new(:'=', e, c), r]}, default)
+        else
+          self
+        end
+      end
+
       to_s_method :case_expression_sql
     end
 
@@ -1049,7 +1159,7 @@ module Sequel
     
     # Represents inverse boolean constants (currently only +NOTNULL+). A
     # special class to allow for special behavior.
-    class NegativeBooleanConstant < BooleanConstant
+    class NegativeBooleanConstant < Constant
       to_s_method :negative_boolean_constant_sql, '@constant'
     end
     
@@ -1073,14 +1183,29 @@ module Sequel
                              Constants::NULL=>Constants::NOTNULL, Constants::NOTNULL=>Constants::NULL}
     end
 
+    # Represents a delayed evaluation, encapsulating a callable
+    # object which returns the value to use when called.
+    class DelayedEvaluation < GenericExpression
+      # A callable object that returns the value of the evaluation
+      # when called. 
+      attr_reader :callable
+
+      # Set the callable object
+      def initialize(callable)
+        @callable = callable
+      end
+
+      to_s_method :delayed_evaluation_sql, '@callable'
+    end
+
     # Represents an SQL function call.
     class Function < GenericExpression
-      # The array of arguments to pass to the function (may be blank)
-      attr_reader :args
-
       # The SQL function to call
       attr_reader :f
       
+      # The array of arguments to pass to the function (may be blank)
+      attr_reader :args
+
       # Set the functions and args to the given arguments
       def initialize(f, *args)
         @f, @args = f, args
@@ -1088,6 +1213,12 @@ module Sequel
 
       to_s_method :function_sql
     end
+
+    # Represents an SQL function call that is translated/emulated
+    # on databases that lack support for such a function.
+    class EmulatedFunction < Function
+      to_s_method :emulated_function_sql
+    end
     
     class GenericExpression
       include AliasMethods
@@ -1173,16 +1304,16 @@ module Sequel
     # required for the prepared statement support and for database-specific
     # literalization.
     class PlaceholderLiteralString < GenericExpression
+      # The literal string containing placeholders.  This can also be an array
+      # of strings, where each arg in args goes between the string elements. 
+      attr_reader :str
+
       # The arguments that will be subsituted into the placeholders.
       # Either an array of unnamed placeholders (which will be substituted in
       # order for ? characters), or a hash of named placeholders (which will be
       # substituted for :key phrases).
       attr_reader :args
 
-      # The literal string containing placeholders.  This can also be an array
-      # of strings, where each arg in args goes between the string elements. 
-      attr_reader :str
-
       # Whether to surround the expression with parantheses
       attr_reader :parens
 
@@ -1231,7 +1362,7 @@ module Sequel
       # Options:
       #
       # :nulls :: Can be :first/:last for NULLS FIRST/LAST.
-      def initialize(expression, descending = true, opts={})
+      def initialize(expression, descending = true, opts=OPTS)
         @expression, @descending, @nulls = expression, descending, opts[:nulls]
       end
 
@@ -1257,12 +1388,12 @@ module Sequel
     class QualifiedIdentifier < GenericExpression
       include QualifyingMethods
 
-      # The column/table referenced
-      attr_reader :column
-
       # The table/schema qualifying the reference
       attr_reader :table
 
+      # The column/table referenced
+      attr_reader :column
+
       # Set the table and column to the given arguments
       def initialize(table, column)
         @table, @column = table, column
@@ -1372,9 +1503,6 @@ module Sequel
     class ValueList < ::Array
     end
 
-    # Deprecated name for +ValueList+, used for backwards compatibility
-    SQLArray = ValueList
-
     # The purpose of the +VirtualRow+ class is to allow the easy creation of SQL identifiers and functions
     # without relying on methods defined on +Symbol+.  This is useful if another library defines
     # the methods defined by Sequel, if you are running on ruby 1.9, or if you are not using the
@@ -1382,8 +1510,8 @@ module Sequel
     #
     # An instance of this class is yielded to the block supplied to <tt>Dataset#filter</tt>, <tt>Dataset#order</tt>, and <tt>Dataset#select</tt>
     # (and the other methods that accept a block and pass it to one of those methods).
-    # If the block doesn't take an argument, the block is instance_evaled in the context of
-    # a new instance of this class.
+    # If the block doesn't take an argument, the block is instance_execed in the context of
+    # an instance of this class.
     #
     # +VirtualRow+ uses +method_missing+ to return either an +Identifier+, +QualifiedIdentifier+, +Function+, or +WindowFunction+, 
     # depending on how it is called.
@@ -1486,7 +1614,7 @@ module Sequel
           if args.empty?
             Function.new(m)
           else
-            case arg = args.shift
+            case args.shift
             when :*
               Function.new(m, WILDCARD)
             when :distinct
@@ -1496,7 +1624,7 @@ module Sequel
               fun_args = ::Kernel.Array(opts[:*] ? WILDCARD : opts[:args])
               WindowFunction.new(Function.new(m, *fun_args), Window.new(opts))
             else
-              raise Error, 'unsupported VirtualRow method argument used with block'
+              Kernel.raise(Error, 'unsupported VirtualRow method argument used with block')
             end
           end
         elsif args.empty?
@@ -1506,6 +1634,8 @@ module Sequel
           Function.new(m, *args)
         end
       end
+
+      Sequel::VIRTUAL_ROW = new
     end
 
     # A +Window+ is part of a window function specifying the window over which the function operates.
@@ -1522,7 +1652,7 @@ module Sequel
       attr_reader :opts
 
       # Set the options to the options given
-      def initialize(opts={})
+      def initialize(opts=OPTS)
         @opts = opts
       end
 
@@ -1570,20 +1700,18 @@ module Sequel
     include SQL::NumericMethods
     include SQL::StringMethods
     include SQL::InequalityMethods
-
-    # If the core extensions are enabled, these will already be included
-    # in String, so we don't need to include/define them here.
-    unless Sequel.core_extensions?
-      include Sequel::SQL::AliasMethods
-      include Sequel::SQL::CastMethods
+    include SQL::AliasMethods
+    include SQL::CastMethods
       
-      def lit(*args)
-        args.empty? ? self : SQL::PlaceholderLiteralString.new(self, args)
-      end
+    # Return self if no args are given, otherwise return a SQL::PlaceholderLiteralString
+    # with the current string and the given args.
+    def lit(*args)
+      args.empty? ? self : SQL::PlaceholderLiteralString.new(self, args)
+    end
       
-      def to_sequel_blob
-        SQL::Blob.new(self)
-      end
+    # Convert a literal string to a SQL::Blob.
+    def to_sequel_blob
+      SQL::Blob.new(self)
     end
   end
   
