@@ -1,41 +1,37 @@
+SEQUEL_ADAPTER_TEST = :firebird
+
 require File.join(File.dirname(File.expand_path(__FILE__)), 'spec_helper.rb')
 
-unless defined?(FIREBIRD_DB)
-  FIREBIRD_URL = 'firebird://sysdba:masterkey@localhost/reality_spec' unless defined? FIREBIRD_URL
-  FIREBIRD_DB = Sequel.connect(ENV['SEQUEL_FB_SPEC_DB']||FIREBIRD_URL)
-end
-INTEGRATION_DB = FIREBIRD_DB unless defined?(INTEGRATION_DB)
-
-def FIREBIRD_DB.sqls
+def DB.sqls
   (@sqls ||= [])
 end
 logger = Object.new
 def logger.method_missing(m, msg)
-  FIREBIRD_DB.sqls.push(msg)
+  DB.sqls.push(msg)
 end
-FIREBIRD_DB.loggers = [logger]
+DB.loggers = [logger]
 
-FIREBIRD_DB.create_table! :test do
+DB.create_table! :test do
   varchar :name,  :size => 50
   integer :val,   :index => true
 end
 
-FIREBIRD_DB.create_table! :test2 do
+DB.create_table! :test2 do
   integer :val
   timestamp :time_stamp
 end
 
-FIREBIRD_DB.create_table! :test3 do
+DB.create_table! :test3 do
   integer :val
   timestamp :time_stamp
 end
 
-FIREBIRD_DB.create_table! :test5 do
+DB.create_table! :test5 do
   primary_key :xid
   integer :val
 end
 
-FIREBIRD_DB.create_table! :test6 do
+DB.create_table! :test6 do
   primary_key :xid
   blob :val
   String :val2
@@ -45,7 +41,7 @@ end
 
 describe "A Firebird database" do
   before do
-    @db = FIREBIRD_DB
+    @db = DB
   end
 
   specify "should provide disconnect functionality" do
@@ -62,7 +58,7 @@ end
 
 describe "A Firebird dataset" do
   before do
-    @d = FIREBIRD_DB[:test]
+    @d = DB[:test]
     @d.delete # remove all records
     @d.quote_identifiers = true
   end
@@ -176,7 +172,7 @@ describe "A Firebird dataset" do
   end
 
   specify "should support transactions" do
-    FIREBIRD_DB.transaction do
+    DB.transaction do
       @d << {:name => 'abc', :val => 1}
     end
 
@@ -184,14 +180,14 @@ describe "A Firebird dataset" do
   end
 
   specify "should have #transaction yield the connection" do
-    FIREBIRD_DB.transaction do |conn|
+    DB.transaction do |conn|
       conn.should_not == nil
     end
   end
 
   specify "should correctly rollback transactions" do
     proc do
-      FIREBIRD_DB.transaction do
+      DB.transaction do
         @d << {:name => 'abc', :val => 1}
         raise RuntimeError, 'asdf'
       end
@@ -201,7 +197,7 @@ describe "A Firebird dataset" do
   end
 
   specify "should handle returning inside of the block by committing" do
-    def FIREBIRD_DB.ret_commit
+    def DB.ret_commit
       transaction do
         self[:test] << {:name => 'abc'}
         return
@@ -209,12 +205,12 @@ describe "A Firebird dataset" do
       end
     end
     @d.count.should == 0
-    FIREBIRD_DB.ret_commit
+    DB.ret_commit
     @d.count.should == 1
-    FIREBIRD_DB.ret_commit
+    DB.ret_commit
     @d.count.should == 2
     proc do
-      FIREBIRD_DB.transaction do
+      DB.transaction do
         raise RuntimeError, 'asdf'
       end
     end.should raise_error(RuntimeError)
@@ -223,7 +219,7 @@ describe "A Firebird dataset" do
   end
 
   specify "should quote and upcase reserved keywords" do
-    @d = FIREBIRD_DB[:testing]
+    @d = DB[:testing]
     @d.quote_identifiers = true
     @d.select(:select).sql.should == \
       'SELECT "SELECT" FROM "TESTING"'
@@ -232,7 +228,7 @@ end
 
 describe "A Firebird dataset with a timestamp field" do
   before do
-    @d = FIREBIRD_DB[:test3]
+    @d = DB[:test3]
     @d.delete
   end
 
@@ -246,75 +242,89 @@ end
 
 describe "A Firebird database" do
   before do
-    @db = FIREBIRD_DB
+    @db = DB
     @db.drop_table?(:posts)
     @db.sqls.clear
   end
 
   specify "should allow us to name the sequences" do
     @db.create_table(:posts){primary_key :id, :sequence_name => "seq_test"}
-    @db.sqls.should == [
-      "DROP SEQUENCE SEQ_TEST",
-      "CREATE TABLE POSTS (ID integer PRIMARY KEY )",
-      "CREATE SEQUENCE SEQ_TEST",
-      "          CREATE TRIGGER BI_POSTS_ID for POSTS\n          ACTIVE BEFORE INSERT position 0\n          as               begin\n                if ((new.ID is null) or (new.ID = 0)) then\n                begin\n                  new.ID = next value for seq_test;\n                end\n              end\n\n"
-    ]
+    check_sqls do
+      @db.sqls.should == [
+        "DROP SEQUENCE SEQ_TEST",
+        "CREATE TABLE POSTS (ID integer PRIMARY KEY )",
+        "CREATE SEQUENCE SEQ_TEST",
+        "          CREATE TRIGGER BI_POSTS_ID for POSTS\n          ACTIVE BEFORE INSERT position 0\n          as               begin\n                if ((new.ID is null) or (new.ID = 0)) then\n                begin\n                  new.ID = next value for seq_test;\n                end\n              end\n\n"
+      ]
+    end
   end
 
   specify "should allow us to set the starting position for the sequences" do
     @db.create_table(:posts){primary_key :id, :sequence_start_position => 999}
-    @db.sqls.should == [
-      "DROP SEQUENCE SEQ_POSTS_ID",
-      "CREATE TABLE POSTS (ID integer PRIMARY KEY )",
-      "CREATE SEQUENCE SEQ_POSTS_ID",
-      "ALTER SEQUENCE SEQ_POSTS_ID RESTART WITH 999",
-      "          CREATE TRIGGER BI_POSTS_ID for POSTS\n          ACTIVE BEFORE INSERT position 0\n          as               begin\n                if ((new.ID is null) or (new.ID = 0)) then\n                begin\n                  new.ID = next value for seq_posts_id;\n                end\n              end\n\n"
-    ]
+    check_sqls do
+      @db.sqls.should == [
+        "DROP SEQUENCE SEQ_POSTS_ID",
+        "CREATE TABLE POSTS (ID integer PRIMARY KEY )",
+        "CREATE SEQUENCE SEQ_POSTS_ID",
+        "ALTER SEQUENCE SEQ_POSTS_ID RESTART WITH 999",
+        "          CREATE TRIGGER BI_POSTS_ID for POSTS\n          ACTIVE BEFORE INSERT position 0\n          as               begin\n                if ((new.ID is null) or (new.ID = 0)) then\n                begin\n                  new.ID = next value for seq_posts_id;\n                end\n              end\n\n"
+      ]
+    end
   end
 
   specify "should allow us to name and set the starting position for the sequences" do
     @db.create_table(:posts){primary_key :id, :sequence_name => "seq_test", :sequence_start_position => 999}
-    @db.sqls.should == [
-      "DROP SEQUENCE SEQ_TEST",
-      "CREATE TABLE POSTS (ID integer PRIMARY KEY )",
-      "CREATE SEQUENCE SEQ_TEST",
-      "ALTER SEQUENCE SEQ_TEST RESTART WITH 999",
-      "          CREATE TRIGGER BI_POSTS_ID for POSTS\n          ACTIVE BEFORE INSERT position 0\n          as               begin\n                if ((new.ID is null) or (new.ID = 0)) then\n                begin\n                  new.ID = next value for seq_test;\n                end\n              end\n\n"
-    ]
+    check_sqls do
+      @db.sqls.should == [
+        "DROP SEQUENCE SEQ_TEST",
+        "CREATE TABLE POSTS (ID integer PRIMARY KEY )",
+        "CREATE SEQUENCE SEQ_TEST",
+        "ALTER SEQUENCE SEQ_TEST RESTART WITH 999",
+        "          CREATE TRIGGER BI_POSTS_ID for POSTS\n          ACTIVE BEFORE INSERT position 0\n          as               begin\n                if ((new.ID is null) or (new.ID = 0)) then\n                begin\n                  new.ID = next value for seq_test;\n                end\n              end\n\n"
+      ]
+    end
   end
 
   specify "should allow us to name the triggers" do
     @db.create_table(:posts){primary_key :id, :trigger_name => "trig_test"}
-    @db.sqls.should == [
-      "DROP SEQUENCE SEQ_POSTS_ID",
-      "CREATE TABLE POSTS (ID integer PRIMARY KEY )",
-      "CREATE SEQUENCE SEQ_POSTS_ID",
-      "          CREATE TRIGGER TRIG_TEST for POSTS\n          ACTIVE BEFORE INSERT position 0\n          as               begin\n                if ((new.ID is null) or (new.ID = 0)) then\n                begin\n                  new.ID = next value for seq_posts_id;\n                end\n              end\n\n"
-    ]
+    check_sqls do
+      @db.sqls.should == [
+        "DROP SEQUENCE SEQ_POSTS_ID",
+        "CREATE TABLE POSTS (ID integer PRIMARY KEY )",
+        "CREATE SEQUENCE SEQ_POSTS_ID",
+        "          CREATE TRIGGER TRIG_TEST for POSTS\n          ACTIVE BEFORE INSERT position 0\n          as               begin\n                if ((new.ID is null) or (new.ID = 0)) then\n                begin\n                  new.ID = next value for seq_posts_id;\n                end\n              end\n\n"
+      ]
+    end
   end
 
   specify "should allow us to not create the sequence" do
     @db.create_table(:posts){primary_key :id, :create_sequence => false}
-    @db.sqls.should == [
-      "CREATE TABLE POSTS (ID integer PRIMARY KEY )",
-      "          CREATE TRIGGER BI_POSTS_ID for POSTS\n          ACTIVE BEFORE INSERT position 0\n          as               begin\n                if ((new.ID is null) or (new.ID = 0)) then\n                begin\n                  new.ID = next value for seq_posts_id;\n                end\n              end\n\n"
-    ]
+    check_sqls do
+      @db.sqls.should == [
+        "CREATE TABLE POSTS (ID integer PRIMARY KEY )",
+        "          CREATE TRIGGER BI_POSTS_ID for POSTS\n          ACTIVE BEFORE INSERT position 0\n          as               begin\n                if ((new.ID is null) or (new.ID = 0)) then\n                begin\n                  new.ID = next value for seq_posts_id;\n                end\n              end\n\n"
+      ]
+    end
   end
 
   specify "should allow us to not create the trigger" do
     @db.create_table(:posts){primary_key :id, :create_trigger => false}
-    @db.sqls.should == [
-      "DROP SEQUENCE SEQ_POSTS_ID",
-      "CREATE TABLE POSTS (ID integer PRIMARY KEY )",
-      "CREATE SEQUENCE SEQ_POSTS_ID",
-    ]
+    check_sqls do
+      @db.sqls.should == [
+        "DROP SEQUENCE SEQ_POSTS_ID",
+        "CREATE TABLE POSTS (ID integer PRIMARY KEY )",
+        "CREATE SEQUENCE SEQ_POSTS_ID",
+      ]
+    end
   end
 
   specify "should allow us to not create either the sequence nor the trigger" do
     @db.create_table(:posts){primary_key :id, :create_sequence => false, :create_trigger => false}
-    @db.sqls.should == [
-      "CREATE TABLE POSTS (ID integer PRIMARY KEY )"
-    ]
+    check_sqls do
+      @db.sqls.should == [
+        "CREATE TABLE POSTS (ID integer PRIMARY KEY )"
+      ]
+    end
   end
 
   specify "should support column operations" do
@@ -355,7 +365,7 @@ end
 
 describe "Postgres::Dataset#insert" do
   before do
-    @ds = FIREBIRD_DB[:test5]
+    @ds = DB[:test5]
     @ds.delete
   end
 
@@ -381,7 +391,7 @@ describe "Postgres::Dataset#insert" do
   end
 
   specify "should return nil if the table has no primary key" do
-    ds = FIREBIRD_DB[:test]
+    ds = DB[:test]
     ds.delete
     ds.insert(:name=>'a').should == nil
   end
@@ -389,7 +399,7 @@ end
 
 describe "Postgres::Dataset#insert" do
   before do
-    @ds = FIREBIRD_DB[:test6]
+    @ds = DB[:test6]
     @ds.delete
   end
 
