@@ -1,43 +1,19 @@
-# encoding: utf-8
+SEQUEL_ADAPTER_TEST = :mssql
+
 require File.join(File.dirname(File.expand_path(__FILE__)), 'spec_helper.rb')
 
-require ENV['SEQUEL_MSSQL_SPEC_REQUIRE'] if ENV['SEQUEL_MSSQL_SPEC_REQUIRE']
-
-unless defined?(MSSQL_DB)
-  MSSQL_URL = 'jdbc:sqlserver://localhost;integratedSecurity=true;database=sandbox' unless defined? MSSQL_URL
-  MSSQL_DB = Sequel.connect(ENV['SEQUEL_MSSQL_SPEC_DB']||MSSQL_URL)
-end
-INTEGRATION_DB = MSSQL_DB unless defined?(INTEGRATION_DB)
-
-def MSSQL_DB.sqls
+def DB.sqls
   (@sqls ||= [])
 end
 logger = Object.new
 def logger.method_missing(m, msg)
-  MSSQL_DB.sqls << msg
-end
-MSSQL_DB.loggers = [logger]
-
-MSSQL_DB.create_table! :test do
-  text :name
-  integer :value, :index => true
-end
-MSSQL_DB.create_table! :test2 do
-  text :name
-  integer :value
-end
-MSSQL_DB.create_table! :test3 do
-  integer :value
-  timestamp :time
-end
-MSSQL_DB.create_table! :test4 do
-  varchar :name, :size => 20
-  varbinary :value
+  DB.sqls << msg
 end
+DB.loggers = [logger]
 
 describe "A MSSQL database" do
   before do
-    @db = MSSQL_DB
+    @db = DB
   end
 
   cspecify "should be able to read fractional part of timestamp", :odbc do
@@ -54,23 +30,57 @@ describe "A MSSQL database" do
     proc{@db.server_version}.should_not raise_error
     proc{@db.dataset.server_version}.should_not raise_error
   end
+end
   
+describe "A MSSQL database" do
+  before do
+    @db = DB
+    @db.create_table! :test3 do
+      Integer :value
+      Time :time
+    end
+  end
+  after do
+    @db.drop_table?(:test3)
+  end
+
   specify "should work with NOLOCK" do
     @db.transaction{@db[:test3].nolock.all.should == []}
   end
 end
 
+describe "MSSQL" do
+  before(:all) do
+    @db = DB
+    @db.create_table!(:test3){Integer :v3}
+    @db.create_table!(:test4){Integer :v4}
+    @db[:test3].import([:v3], [[1], [2]])
+    @db[:test4].import([:v4], [[1], [3]])
+  end
+  after(:all) do
+    @db.drop_table?(:test3, :test4)
+  end
+
+  specify "should should support CROSS APPLY" do
+    @db[:test3].cross_apply(@db[:test4].where(:test3__v3=>:test4__v4)).select_order_map([:v3, :v4]).should == [[1,1]]
+  end
+
+  specify "should should support OUTER APPLY" do
+    @db[:test3].outer_apply(@db[:test4].where(:test3__v3=>:test4__v4)).select_order_map([:v3, :v4]).should == [[1,1], [2, nil]]
+  end
+end
+
 # This spec is currently disabled as the SQL Server 2008 R2 Express doesn't support
 # full text searching.  Even if full text searching is supported,
 # you may need to create a full text catalog on the database first via:
 #   CREATE FULLTEXT CATALOG ftscd AS DEFAULT
 describe "MSSQL full_text_search" do
   before do
-    @db = MSSQL_DB
-    @db.drop_table(:posts) rescue nil
+    @db = DB
+    @db.drop_table?(:posts)
   end
   after do
-    @db.drop_table(:posts) rescue nil
+    @db.drop_table?(:posts)
   end
   
   specify "should support fulltext indexes and full_text_search" do
@@ -91,32 +101,31 @@ end if false
 
 describe "MSSQL Dataset#join_table" do
   specify "should emulate the USING clause with ON" do
-    MSSQL_DB[:items].join(:categories, [:id]).sql.should ==
+    DB[:items].join(:categories, [:id]).sql.should ==
       'SELECT * FROM [ITEMS] INNER JOIN [CATEGORIES] ON ([CATEGORIES].[ID] = [ITEMS].[ID])'
     ['SELECT * FROM [ITEMS] INNER JOIN [CATEGORIES] ON (([CATEGORIES].[ID1] = [ITEMS].[ID1]) AND ([CATEGORIES].[ID2] = [ITEMS].[ID2]))',
       'SELECT * FROM [ITEMS] INNER JOIN [CATEGORIES] ON (([CATEGORIES].[ID2] = [ITEMS].[ID2]) AND ([CATEGORIES].[ID1] = [ITEMS].[ID1]))'].
-      should include(MSSQL_DB[:items].join(:categories, [:id1, :id2]).sql)
-    MSSQL_DB[:items___i].join(:categories___c, [:id]).sql.should ==
+      should include(DB[:items].join(:categories, [:id1, :id2]).sql)
+    DB[:items___i].join(:categories___c, [:id]).sql.should ==
       'SELECT * FROM [ITEMS] AS [I] INNER JOIN [CATEGORIES] AS [C] ON ([C].[ID] = [I].[ID])'
   end
 end
 
 describe "MSSQL Dataset#output" do
   before do
-    @db = MSSQL_DB
+    @db = DB
     @db.create_table!(:items){String :name; Integer :value}
     @db.create_table!(:out){String :name; Integer :value}
     @ds = @db[:items]
   end
   after do
-    @db.drop_table(:items)
-    @db.drop_table(:out)
+    @db.drop_table?(:items, :out)
   end
 
   specify "should format OUTPUT clauses without INTO for DELETE statements" do
     @ds.output(nil, [:deleted__name, :deleted__value]).delete_sql.should =~
       /DELETE FROM \[ITEMS\] OUTPUT \[DELETED\].\[(NAME|VALUE)\], \[DELETED\].\[(NAME|VALUE)\]/
-    @ds.output(nil, [:deleted.*]).delete_sql.should =~
+    @ds.output(nil, [Sequel::SQL::ColumnAll.new(:deleted)]).delete_sql.should =~
       /DELETE FROM \[ITEMS\] OUTPUT \[DELETED\].*/
   end
   
@@ -130,7 +139,7 @@ describe "MSSQL Dataset#output" do
   specify "should format OUTPUT clauses without INTO for INSERT statements" do
     @ds.output(nil, [:inserted__name, :inserted__value]).insert_sql(:name => "name", :value => 1).should =~
       /INSERT INTO \[ITEMS\] \(\[(NAME|VALUE)\], \[(NAME|VALUE)\]\) OUTPUT \[INSERTED\].\[(NAME|VALUE)\], \[INSERTED\].\[(NAME|VALUE)\] VALUES \((N'name'|1), (N'name'|1)\)/
-    @ds.output(nil, [:inserted.*]).insert_sql(:name => "name", :value => 1).should =~
+    @ds.output(nil, [Sequel::SQL::ColumnAll.new(:inserted)]).insert_sql(:name => "name", :value => 1).should =~
       /INSERT INTO \[ITEMS\] \(\[(NAME|VALUE)\], \[(NAME|VALUE)\]\) OUTPUT \[INSERTED\].* VALUES \((N'name'|1), (N'name'|1)\)/
   end
 
@@ -144,7 +153,7 @@ describe "MSSQL Dataset#output" do
   specify "should format OUTPUT clauses without INTO for UPDATE statements" do
     @ds.output(nil, [:inserted__name, :deleted__value]).update_sql(:value => 2).should =~
       /UPDATE \[ITEMS\] SET \[VALUE\] = 2 OUTPUT \[(INSERTED\].\[NAME|DELETED\].\[VALUE)\], \[(INSERTED\].\[NAME|DELETED\].\[VALUE)\]/
-    @ds.output(nil, [:inserted.*]).update_sql(:value => 2).should =~
+    @ds.output(nil, [Sequel::SQL::ColumnAll.new(:inserted)]).update_sql(:value => 2).should =~
       /UPDATE \[ITEMS\] SET \[VALUE\] = 2 OUTPUT \[INSERTED\].*/
   end
 
@@ -180,94 +189,61 @@ describe "MSSQL Dataset#output" do
   end
 end
 
-describe "MSSQL dataset" do
+describe "MSSQL dataset using #with and #with_recursive" do
   before do
-    @db = MSSQL_DB
-    @ds = MSSQL_DB[:t]
-  end
-
-  describe "using #with and #with_recursive" do
-    before do
+    @db = DB
+    @ds = DB[:t]
       @ds1 = @ds.with(:t, @db[:x])
       @ds2 = @ds.with_recursive(:t, @db[:x], @db[:t])
-    end
+  end
 
-    specify "should prepend UPDATE statements with WITH clause" do
-      @ds1.update_sql(:x => :y).should == 'WITH [T] AS (SELECT * FROM [X]) UPDATE [T] SET [X] = [Y]'
-      @ds2.update_sql(:x => :y).should == 'WITH [T] AS (SELECT * FROM [X] UNION ALL SELECT * FROM [T]) UPDATE [T] SET [X] = [Y]'
-    end
+  specify "should prepend UPDATE statements with WITH clause" do
+    @ds1.update_sql(:x => :y).should == 'WITH [T] AS (SELECT * FROM [X]) UPDATE [T] SET [X] = [Y]'
+    @ds2.update_sql(:x => :y).should == 'WITH [T] AS (SELECT * FROM [X] UNION ALL SELECT * FROM [T]) UPDATE [T] SET [X] = [Y]'
+  end
 
-    specify "should prepend DELETE statements with WITH clause" do
-      @ds1.filter(:y => 1).delete_sql.should == 'WITH [T] AS (SELECT * FROM [X]) DELETE FROM [T] WHERE ([Y] = 1)'
-      @ds2.filter(:y => 1).delete_sql.should == 'WITH [T] AS (SELECT * FROM [X] UNION ALL SELECT * FROM [T]) DELETE FROM [T] WHERE ([Y] = 1)'
-    end
+  specify "should prepend DELETE statements with WITH clause" do
+    @ds1.filter(:y => 1).delete_sql.should == 'WITH [T] AS (SELECT * FROM [X]) DELETE FROM [T] WHERE ([Y] = 1)'
+    @ds2.filter(:y => 1).delete_sql.should == 'WITH [T] AS (SELECT * FROM [X] UNION ALL SELECT * FROM [T]) DELETE FROM [T] WHERE ([Y] = 1)'
+  end
 
-    specify "should prepend INSERT statements with WITH clause" do
-      @ds1.insert_sql(@db[:t]).should == 'WITH [T] AS (SELECT * FROM [X]) INSERT INTO [T] SELECT * FROM [T]'
-      @ds2.insert_sql(@db[:t]).should == 'WITH [T] AS (SELECT * FROM [X] UNION ALL SELECT * FROM [T]) INSERT INTO [T] SELECT * FROM [T]'
-    end
+  specify "should prepend INSERT statements with WITH clause" do
+    @ds1.insert_sql(@db[:t]).should == 'WITH [T] AS (SELECT * FROM [X]) INSERT INTO [T] SELECT * FROM [T]'
+    @ds2.insert_sql(@db[:t]).should == 'WITH [T] AS (SELECT * FROM [X] UNION ALL SELECT * FROM [T]) INSERT INTO [T] SELECT * FROM [T]'
+  end
 
-    specify "should move WITH clause on joined dataset to top level" do
-      @db[:s].inner_join(@ds1).sql.should == "WITH [T] AS (SELECT * FROM [X]) SELECT * FROM [S] INNER JOIN (SELECT * FROM [T]) AS [T1]"
-      @ds1.inner_join(@db[:s].with(:s, @db[:y])).sql.should == "WITH [T] AS (SELECT * FROM [X]), [S] AS (SELECT * FROM [Y]) SELECT * FROM [T] INNER JOIN (SELECT * FROM [S]) AS [T1]"
-    end
-
-    describe "on #import" do
-      before do
-        @db = @db.clone
-        class << @db
-          attr_reader :import_sqls
-
-          def execute(sql, opts={})
-            @import_sqls ||= []
-            @import_sqls << sql
-          end
-          alias execute_dui execute
-
-          def transaction(opts={})
-            @import_sqls ||= []
-            @import_sqls << 'BEGIN'
-            yield
-            @import_sqls << 'COMMIT'
-          end
-        end
-      end
-
-      specify "should prepend INSERT statements with WITH clause" do
-        @db[:items].with(:items, @db[:inventory].group(:type)).import([:x, :y], [[1, 2], [3, 4], [5, 6]], :slice => 2)
-        @db.import_sqls.should == [
-          'BEGIN',
-          "WITH [ITEMS] AS (SELECT * FROM [INVENTORY] GROUP BY [TYPE]) INSERT INTO [ITEMS] ([X], [Y]) SELECT 1, 2 UNION ALL SELECT 3, 4",
-          'COMMIT',
-          'BEGIN',
-          "WITH [ITEMS] AS (SELECT * FROM [INVENTORY] GROUP BY [TYPE]) INSERT INTO [ITEMS] ([X], [Y]) SELECT 5, 6",
-          'COMMIT'
-        ]
-      end
-    end
+  specify "should move WITH clause on joined dataset to top level" do
+    @db[:s].inner_join(@ds1).sql.should == "WITH [T] AS (SELECT * FROM [X]) SELECT * FROM [S] INNER JOIN (SELECT * FROM [T]) AS [T1]"
+    @ds1.inner_join(@db[:s].with(:s, @db[:y])).sql.should == "WITH [T] AS (SELECT * FROM [X]), [S] AS (SELECT * FROM [Y]) SELECT * FROM [T] INNER JOIN (SELECT * FROM [S]) AS [T1]"
   end
 end
 
 describe "MSSQL::Dataset#import" do
   before do
-    @db = MSSQL_DB
-    @db.create_table!(:test){primary_key :x; Integer :y}
+    @db = DB
     @db.sqls.clear
     @ds = @db[:test]
   end
   after do
-    @db.drop_table(:test) rescue nil
+    @db.drop_table?(:test)
   end
   
   specify "#import should work correctly with an arbitrary output value" do
+    @db.create_table!(:test){primary_key :x; Integer :y}
     @ds.output(nil, [:inserted__y, :inserted__x]).import([:y], [[3], [4]]).should == [{:y=>3, :x=>1}, {:y=>4, :x=>2}]
     @ds.all.should == [{:x=>1, :y=>3}, {:x=>2, :y=>4}]
   end
+
+  specify "should handle WITH statements" do
+    @db.create_table!(:test){Integer :x; Integer :y}
+    @db[:testx].with(:testx, @db[:test]).import([:x, :y], [[1, 2], [3, 4], [5, 6]], :slice => 2)
+    @ds.select_order_map([:x, :y]).should == [[1, 2], [3, 4], [5, 6]]
+  end
 end
 
 describe "MSSQL joined datasets" do
   before do
-    @db = MSSQL_DB
+    @db = DB
   end
 
   specify "should format DELETE statements" do
@@ -283,7 +259,7 @@ end
 
 describe "Offset support" do
   before do
-    @db = MSSQL_DB
+    @db = DB
     @db.create_table!(:i){Integer :id; Integer :parent_id}
     @ds = @db[:i].order(:id)
     @hs = []
@@ -291,7 +267,7 @@ describe "Offset support" do
     @ds.import [:id, :parent_id], [[1,nil],[2,nil],[3,1],[4,1],[5,3],[6,5]]
   end
   after do
-    @db.drop_table(:i)
+    @db.drop_table?(:i)
   end
   
   specify "should return correct rows" do
@@ -306,7 +282,7 @@ end
 
 describe "Common Table Expressions" do
   before do
-    @db = MSSQL_DB
+    @db = DB
     @db.create_table!(:i1){Integer :id; Integer :parent_id}
     @db.create_table!(:i2){Integer :id; Integer :parent_id}
     @ds = @db[:i1]
@@ -314,8 +290,7 @@ describe "Common Table Expressions" do
     @ds.import [:id, :parent_id], [[1,nil],[2,nil],[3,1],[4,1],[5,3],[6,5]]
   end
   after do
-    @db.drop_table(:i1)
-    @db.drop_table(:i2)
+    @db.drop_table?(:i1, :i2)
   end
 
   specify "using #with should be able to update" do
@@ -331,7 +306,7 @@ describe "Common Table Expressions" do
 
   specify "using #with_recursive should be able to update" do
     ds = @ds.with_recursive(:t, @ds.filter(:parent_id=>1).or(:id => 1), @ds.join(:t, :i=>:parent_id).select(:i1__id, :i1__parent_id), :args=>[:i, :pi])
-    ds.filter(~{:id => @db[:t].select(:i)}).update(:parent_id => 1)
+    ds.exclude(:id => @db[:t].select(:i)).update(:parent_id => 1)
     @ds[:id => 1].should == {:id => 1, :parent_id => nil}
     @ds[:id => 2].should == {:id => 2, :parent_id => 1}
     @ds[:id => 5].should == {:id => 5, :parent_id => 3}
@@ -379,13 +354,17 @@ end
 
 describe "MSSSQL::Dataset#insert" do
   before do
-    @db = MSSQL_DB
+    @db = DB
     @db.create_table!(:test5){primary_key :xid; Integer :value}
+    @db.create_table! :test4 do
+      String :name, :size => 20
+      column :value, 'varbinary(max)'
+    end
     @db.sqls.clear
     @ds = @db[:test5]
   end
   after do
-    @db.drop_table(:test5) rescue nil
+    @db.drop_table?(:test5, :test4)
   end
 
   specify "should have insert_select return nil if disable_insert_output is used" do
@@ -393,7 +372,7 @@ describe "MSSSQL::Dataset#insert" do
   end
   
   specify "should have insert_select return nil if the server version is not 2005+" do
-    @ds.meta_def(:server_version){8000760}
+    def @ds.server_version() 8000760 end
     @ds.insert_select(:value=>10).should == nil
   end
 
@@ -402,17 +381,23 @@ describe "MSSSQL::Dataset#insert" do
     h[:value].should == 10
     @ds.first(:xid=>h[:xid])[:value].should == 10
   end
-end
 
-describe "MSSSQL::Dataset#disable_insert_output" do
+  cspecify "should allow large text and binary values", :odbc do
+    blob = Sequel::SQL::Blob.new("0" * (65*1024))
+    @db[:test4].insert(:name => 'max varbinary test', :value => blob)
+    b = @db[:test4].where(:name => 'max varbinary test').get(:value)
+    b.length.should == blob.length
+    b.should == blob
+  end
+
   specify "should play nicely with simple_select_all?" do
-    MSSQL_DB[:test].disable_insert_output.send(:simple_select_all?).should == true
+    DB[:test4].disable_insert_output.send(:simple_select_all?).should == true
   end
 end
 
 describe "MSSSQL::Dataset#into" do
   before do
-    @db = MSSQL_DB
+    @db = DB
   end
 
   specify "should format SELECT statement" do
@@ -424,17 +409,16 @@ describe "MSSSQL::Dataset#into" do
     @db[:t].insert(:id => 1, :value => "test")
     @db << @db[:t].into(:new).select_sql
     @db[:new].all.should == [{:id => 1, :value => "test"}]
-    @db.drop_table(:t)
-    @db.drop_table(:new)
+    @db.drop_table?(:t, :new)
   end
 end
 
 describe "A MSSQL database" do
   before do
-    @db = MSSQL_DB
+    @db = DB
   end
   after do
-    @db.drop_table(:a)
+    @db.drop_table?(:a)
   end
   
   specify "should handle many existing types for set_column_allow_null" do
@@ -456,65 +440,69 @@ describe "A MSSQL database" do
 end
 
 describe "MSSQL::Database#rename_table" do
+  after do
+    DB.drop_table?(:foo)
+  end
+
   specify "should work on non-schema bound tables which need escaping" do
-    MSSQL_DB.quote_identifiers = true
-    MSSQL_DB.create_table! :'foo bar' do
+    DB.quote_identifiers = true
+    DB.create_table! :'foo bar' do
       text :name
     end
-    MSSQL_DB.drop_table :baz rescue nil
-    proc { MSSQL_DB.rename_table 'foo bar', 'baz' }.should_not raise_error
+    DB.drop_table? :foo
+    proc { DB.rename_table 'foo bar', 'foo' }.should_not raise_error
   end
   
-  specify "should workd on schema bound tables" do
-    MSSQL_DB.execute(<<-SQL)
+  specify "should work on schema bound tables" do
+    DB.execute(<<-SQL)
       IF NOT EXISTS (SELECT * FROM sys.schemas WHERE name = 'MY')
         EXECUTE sp_executesql N'create schema MY'
     SQL
-    MSSQL_DB.create_table! :MY__foo do
+    DB.create_table! :MY__foo do
       text :name
     end
-    proc { MSSQL_DB.rename_table :MY__foo, :MY__bar }.should_not raise_error
-    proc { MSSQL_DB.rename_table :MY__bar, :foo }.should_not raise_error
+    proc { DB.rename_table :MY__foo, :MY__bar }.should_not raise_error
+    proc { DB.rename_table :MY__bar, :foo }.should_not raise_error
   end
 end
 
 describe "MSSQL::Dataset#count" do
   specify "should work with a distinct query with an order clause" do
-    MSSQL_DB.create_table!(:items){String :name; Integer :value}
-    MSSQL_DB[:items].insert(:name => "name", :value => 1)
-    MSSQL_DB[:items].insert(:name => "name", :value => 1)
-    MSSQL_DB[:items].select(:name, :value).distinct.order(:name).count.should == 1
-    MSSQL_DB[:items].select(:name, :value).group(:name, :value).order(:name).count.should == 1
+    DB.create_table!(:items){String :name; Integer :value}
+    DB[:items].insert(:name => "name", :value => 1)
+    DB[:items].insert(:name => "name", :value => 1)
+    DB[:items].select(:name, :value).distinct.order(:name).count.should == 1
+    DB[:items].select(:name, :value).group(:name, :value).order(:name).count.should == 1
   end
 end
 
 describe "MSSQL::Database#create_table" do
   specify "should support collate with various other column options" do
-    MSSQL_DB.create_table!(:items){ String :name, :size => 128, :collate => :sql_latin1_general_cp1_ci_as, :default => 'foo', :null => false, :unique => true}
-    MSSQL_DB[:items].insert
-    MSSQL_DB[:items].select_map(:name).should == ["foo"]
+    DB.create_table!(:items){ String :name, :size => 128, :collate => :sql_latin1_general_cp1_ci_as, :default => 'foo', :null => false, :unique => true}
+    DB[:items].insert
+    DB[:items].select_map(:name).should == ["foo"]
   end
 end
 
 describe "MSSQL::Database#mssql_unicode_strings = false" do
   before do
-    MSSQL_DB.mssql_unicode_strings = false
+    DB.mssql_unicode_strings = false
   end
   after do
-    MSSQL_DB.drop_table(:items)
-    MSSQL_DB.mssql_unicode_strings = true
+    DB.drop_table?(:items)
+    DB.mssql_unicode_strings = true
   end
 
   specify "should work correctly" do
-    MSSQL_DB.create_table!(:items){String :name}
-    MSSQL_DB[:items].mssql_unicode_strings.should == false
-    MSSQL_DB[:items].insert(:name=>'foo')
-    MSSQL_DB[:items].select_map(:name).should == ['foo']
+    DB.create_table!(:items){String :name}
+    DB[:items].mssql_unicode_strings.should == false
+    DB[:items].insert(:name=>'foo')
+    DB[:items].select_map(:name).should == ['foo']
   end
 
   specify "should be overridable at the dataset level" do
-    MSSQL_DB.create_table!(:items){String :name}
-    ds = MSSQL_DB[:items]
+    DB.create_table!(:items){String :name}
+    ds = DB[:items]
     ds.mssql_unicode_strings.should == false
     ds.mssql_unicode_strings = true
     ds.mssql_unicode_strings.should == true
@@ -526,7 +514,7 @@ end
 describe "A MSSQL database adds index with include" do
   before :all do
     @table_name = :test_index_include
-    @db = MSSQL_DB
+    @db = DB
     @db.create_table! @table_name do
       integer :col1
       integer :col2
@@ -535,7 +523,7 @@ describe "A MSSQL database adds index with include" do
   end
 
   after :all do
-    @db.drop_table @table_name
+    @db.drop_table? @table_name
   end
 
   cspecify "should be able add index with include" do
@@ -545,3 +533,86 @@ describe "A MSSQL database adds index with include" do
     @db.indexes(@table_name).should have_key("#{@table_name}_col1_index".to_sym)
   end
 end
+
+describe "MSSQL::Database#drop_column with a schema" do
+  before do
+    DB.run "create schema test" rescue nil
+  end
+  after do
+    DB.drop_table(:test__items)
+    DB.run "drop schema test" rescue nil
+  end
+
+  specify "drops columns with a default value" do
+    DB.create_table!(:test__items){ Integer :id; String :name, :default => 'widget' }
+    DB.drop_column(:test__items, :name)
+    DB[:test__items].columns.should == [:id]
+  end
+end
+
+describe "Database#foreign_key_list" do
+  before(:all) do
+    DB.create_table! :items do
+      primary_key :id
+      integer     :sku
+    end
+    DB.create_table! :prices do
+      integer     :item_id
+      datetime    :valid_from
+      float       :price
+      primary_key [:item_id, :valid_from]
+      foreign_key [:item_id], :items, :key => :id, :name => :fk_prices_items
+    end
+    DB.create_table! :sales do
+      integer  :id
+      integer  :price_item_id
+      datetime :price_valid_from
+      foreign_key [:price_item_id, :price_valid_from], :prices, :key => [:item_id, :valid_from], :name => :fk_sales_prices, :on_delete => :cascade
+    end
+  end
+  after(:all) do
+    DB.drop_table :sales
+    DB.drop_table :prices
+    DB.drop_table :items
+  end
+  it "should support typical foreign keys" do
+    DB.foreign_key_list(:prices).should == [{:name      => :fk_prices_items, 
+                                                   :table     => :items, 
+                                                   :columns   => [:item_id], 
+                                                   :key       => [:id], 
+                                                   :on_update => :no_action, 
+                                                   :on_delete => :no_action }]
+  end
+  it "should support a foreign key with multiple columns" do
+    DB.foreign_key_list(:sales).should == [{:name      => :fk_sales_prices, 
+                                                  :table     => :prices, 
+                                                  :columns   => [:price_item_id, :price_valid_from], 
+                                                  :key       => [:item_id, :valid_from], 
+                                                  :on_update => :no_action, 
+                                                  :on_delete => :cascade }]
+  end
+
+  context "with multiple schemas" do
+    before(:all) do
+      DB.execute_ddl "create schema vendor"
+      DB.create_table! :vendor__vendors do
+        primary_key :id
+        varchar     :name
+      end
+      DB.create_table! :vendor__mapping do
+        integer :vendor_id
+        integer :item_id
+        foreign_key [:vendor_id], :vendor__vendors, :name => :fk_mapping_vendor
+        foreign_key [:item_id], :items, :name => :fk_mapping_item
+      end
+    end
+    after(:all) do
+      DB.drop_table :vendor__mapping
+      DB.drop_table :vendor__vendors
+      DB.execute_ddl "drop schema vendor"
+    end
+    it "should support mixed schema bound tables" do
+ DB.foreign_key_list(:vendor__mapping).sort_by{|h| h[:name].to_s}.should == [{:name => :fk_mapping_item, :table => :items, :columns => [:item_id], :key => [:id], :on_update => :no_action, :on_delete => :no_action }, {:name => :fk_mapping_vendor, :table => :vendor__vendors, :columns => [:vendor_id], :key => [:id], :on_update => :no_action, :on_delete => :no_action }]
+    end
+  end
+end
