@@ -1,5 +1,11 @@
 require 'rubygems'
 require 'logger'
+
+if ENV['COVERAGE']
+  require File.join(File.dirname(File.expand_path(__FILE__)), "../sequel_coverage")
+  SimpleCov.sequel_coverage(:group=>%r{lib/sequel/adapters})
+end
+
 unless Object.const_defined?('Sequel')
   $:.unshift(File.join(File.dirname(File.expand_path(__FILE__)), "../../lib/"))
   require 'sequel'
@@ -9,12 +15,8 @@ begin
 rescue LoadError
 end
 
-if ENV['SEQUEL_COLUMNS_INTROSPECTION']
-  Sequel.extension :columns_introspection
-  Sequel::Dataset.introspect_all_columns
-end
-
-Sequel::Model.cache_anonymous_models = false
+Sequel::Database.extension :columns_introspection if ENV['SEQUEL_COLUMNS_INTROSPECTION']
+Sequel.cache_anonymous_models = false
 
 class Sequel::Database
   def log_duration(duration, message)
@@ -25,10 +27,10 @@ end
 (defined?(RSpec) ? RSpec::Core::ExampleGroup : Spec::Example::ExampleGroup).class_eval do
   def log 
     begin
-      INTEGRATION_DB.loggers << Logger.new(STDOUT)
+      DB.loggers << Logger.new(STDOUT)
       yield
     ensure
-     INTEGRATION_DB.loggers.pop
+     DB.loggers.pop
     end 
   end 
 
@@ -37,12 +39,12 @@ end
     pending = false
     checked.each do |c|
       case c
-      when INTEGRATION_DB.adapter_scheme
+      when DB.adapter_scheme
         pending = c
       when Proc
-        pending = c if c.first.call(INTEGRATION_DB)
+        pending = c if c.first.call(DB)
       when Array
-        pending = c if c.first == INTEGRATION_DB.adapter_scheme && c.last == INTEGRATION_DB.call(INTEGRATION_DB)
+        pending = c if c.first == DB.adapter_scheme && c.last == DB.call(DB)
       end
     end
     if pending
@@ -51,4 +53,17 @@ end
       specify(message, &block)
     end
   end
+
+  def check_sqls
+    yield unless ENV['SEQUEL_NO_CHECK_SQLS']
+  end
+  def self.check_sqls
+    yield unless ENV['SEQUEL_NO_CHECK_SQLS']
+  end
+end
+
+unless defined?(DB)
+  env_var = "SEQUEL_#{SEQUEL_ADAPTER_TEST.to_s.upcase}_URL"
+  env_var = ENV.has_key?(env_var) ? env_var : 'SEQUEL_INTEGRATION_URL'
+  DB = Sequel.connect(ENV[env_var])
 end
