@@ -3,13 +3,14 @@ require File.join(File.dirname(File.expand_path(__FILE__)), "spec_helper")
 describe "class_table_inheritance plugin" do
   before do
     @db = Sequel.mock(:autoid=>proc{|sql| 1})
+    def @db.supports_schema_parsing?() true end
     def @db.schema(table, opts={})
       {:employees=>[[:id, {:primary_key=>true, :type=>:integer}], [:name, {:type=>:string}], [:kind, {:type=>:string}]],
        :managers=>[[:id, {:type=>:integer}], [:num_staff, {:type=>:integer}]],
        :executives=>[[:id, {:type=>:integer}], [:num_managers, {:type=>:integer}]],
        :staff=>[[:id, {:type=>:integer}], [:manager_id, {:type=>:integer}]],
        }[table.is_a?(Sequel::Dataset) ? table.first_source_table : table]
-     end
+    end
     @db.extend_datasets do
       def columns
         {[:employees]=>[:id, :name, :kind],
@@ -23,7 +24,7 @@ describe "class_table_inheritance plugin" do
       end
     end
     class ::Employee < Sequel::Model(@db)
-      def _refresh(x); @values[:id] = 1 end
+      def _save_refresh; @values[:id] = 1 end
       def self.columns
         dataset.columns
       end
@@ -54,6 +55,12 @@ describe "class_table_inheritance plugin" do
     Staff.simple_table.should == nil
   end
   
+    specify "should have working row_proc if using set_dataset in subclass to remove columns" do
+      Manager.set_dataset(Manager.dataset.select(*(Manager.columns - [:blah])))
+      Manager.dataset._fetch = {:id=>1, :kind=>'Executive'}
+      Manager[1].should == Executive.load(:id=>1, :kind=>'Executive')
+    end
+
   specify "should use a joined dataset in subclasses" do
     Employee.dataset.sql.should == 'SELECT * FROM employees'
     Manager.dataset.sql.should == 'SELECT * FROM employees INNER JOIN managers USING (id)'
@@ -162,6 +169,13 @@ describe "class_table_inheritance plugin" do
     @db.sqls.should == ["DELETE FROM executives WHERE (id = 1)", "DELETE FROM managers WHERE (id = 1)", "DELETE FROM employees WHERE (id = 1)"]
   end
 
+  it "should not allow deletion of frozen object" do
+    o = Executive.load(:id=>1)
+    o.freeze
+    proc{o.delete}.should raise_error(Sequel::Error)
+    @db.sqls.should == []
+  end
+
   it "should insert the correct rows into all tables when inserting" do
     Executive.create(:num_managers=>3, :num_staff=>2, :name=>'E')
     sqls = @db.sqls
@@ -190,7 +204,7 @@ describe "class_table_inheritance plugin" do
   it "should handle many_to_one relationships correctly" do
     Manager.dataset._fetch = {:id=>3, :name=>'E', :kind=>'Executive', :num_managers=>3}
     Staff.load(:manager_id=>3).manager.should == Executive.load(:id=>3, :name=>'E', :kind=>'Executive', :num_managers=>3)
-    @db.sqls.should == ['SELECT * FROM employees INNER JOIN managers USING (id) WHERE (managers.id = 3) LIMIT 1']
+    @db.sqls.should == ['SELECT * FROM employees INNER JOIN managers USING (id) WHERE (id = 3) LIMIT 1']
   end
   
   it "should handle one_to_many relationships correctly" do
