@@ -25,6 +25,15 @@ describe "force_encoding plugin" do
     o.x.encoding.should == @e1
   end
   
+  specify "should work correctly when given a frozen string" do
+    s = 'blah'
+    s.force_encoding('US-ASCII')
+    s.freeze
+    o = @c.new(:x=>s)
+    o.x.should == 'blah'
+    o.x.encoding.should == @e1
+  end
+  
   specify "should have a forced_encoding class accessor" do
     s = 'blah'
     s.force_encoding('US-ASCII')
@@ -74,7 +83,7 @@ describe "force_encoding plugin" do
   
   specify "should work when saving new model instances" do
     o = @c.new
-    ds = MODEL_DB[:a]
+    ds = DB[:a]
     def ds.first
       s = 'blah'
       s.force_encoding('US-ASCII')
@@ -88,7 +97,7 @@ describe "force_encoding plugin" do
   
   specify "should work when refreshing model instances" do
     o = @c.load(:id=>1, :x=>'as')
-    ds = MODEL_DB[:a]
+    ds = DB[:a]
     def ds.first
       s = 'blah'
       s.force_encoding('US-ASCII')
@@ -99,23 +108,7 @@ describe "force_encoding plugin" do
     o.x.should == 'blah'
     o.x.encoding.should == @e1
   end
-  
-  specify "should work when used with the identity_map plugin if the identity_map plugin is setup first" do
-    @c = Class.new(Sequel::Model) do
-    end
-    @c.columns :id, :x
-    @c.plugin :identity_map
-    @c.plugin :force_encoding, 'UTF-8'
-    @c.with_identity_map do
-      o = @c.load(:id=>1)
-      s = 'blah'
-      s.force_encoding('US-ASCII')
-      @c.load(:id=>1, :x=>s)
-      o.x.should == 'blah'
-      o.x.encoding.should == @e1
-    end
-  end
 end 
 else
-  skip_warn "force_encoding plugin: only works on ruby 1.9"
+  skip_warn "force_encoding plugin: only works on ruby 1.9+"
 end
