@@ -1,5 +1,7 @@
 require File.join(File.dirname(File.expand_path(__FILE__)), 'spec_helper')
 
+Sequel.extension :migration
+
 describe "Migration.descendants" do
   before do
     Sequel::Migration.descendants.clear
@@ -55,6 +57,12 @@ describe "Migration.apply" do
     m.apply(@db, :up).should == nil
     m.apply(@db, :down).should == nil
   end
+
+  specify "should respond to the methods the database responds to" do
+    m = Sequel::Migration.new(Sequel.mock)
+    m.respond_to?(:foo).should be_false
+    m.respond_to?(:execute).should be_true
+  end if RUBY_VERSION >= '1.9'
 end
 
 describe "SimpleMigration#apply" do
@@ -122,6 +130,8 @@ describe "Reversible Migrations with Sequel.migration{change{}}" do
         add_column :d, String
         add_constraint :blah, 'd IS NOT NULL'
         add_foreign_key :e, :b
+        add_foreign_key [:e], :b, :name=>'e_fk'
+        add_foreign_key [:e, :a], :b
         add_primary_key :f, :b
         add_index :e, :name=>'e_n'
         add_full_text_index :e, :name=>'e_ft'
@@ -145,6 +155,8 @@ describe "Reversible Migrations with Sequel.migration{change{}}" do
         [:add_column, :d, String],
         [:add_constraint, :blah, "d IS NOT NULL"],
         [:add_foreign_key, :e, :b],
+        [:add_foreign_key, [:e], :b, {:name=>"e_fk"}],
+        [:add_foreign_key, [:e, :a], :b],
         [:add_primary_key, :f, :b],
         [:add_index, :e, {:name=>"e_n"}],
         [:add_full_text_index, :e, {:name=>"e_ft"}],
@@ -167,7 +179,9 @@ describe "Reversible Migrations with Sequel.migration{change{}}" do
         [:drop_index, :e, {:name=>"e_ft"}],
         [:drop_index, :e, {:name=>"e_n"}],
         [:drop_column, :f],
-        [:drop_column, :e],
+        [:drop_foreign_key, [:e, :a]],
+        [:drop_foreign_key, [:e], {:name=>"e_fk"}],
+        [:drop_foreign_key, :e],
         [:drop_constraint, :blah],
         [:drop_column, :d]]
       ],
@@ -180,19 +194,19 @@ describe "Reversible Migrations with Sequel.migration{change{}}" do
   
   specify "should raise in the down direction if migration uses unsupported method" do
     m = Sequel.migration{change{run 'SQL'}}
-    proc{m.apply(@db, :up)}.should_not raise_error(Sequel::Error)
+    proc{m.apply(@db, :up)}.should_not raise_error
     proc{m.apply(@db, :down)}.should raise_error(Sequel::Error)
   end
   
   specify "should raise in the down direction if migration uses add_primary_key with an array" do
     m = Sequel.migration{change{alter_table(:a){add_primary_key [:b]}}}
-    proc{m.apply(@db, :up)}.should_not raise_error(Sequel::Error)
+    proc{m.apply(@db, :up)}.should_not raise_error
     proc{m.apply(@db, :down)}.should raise_error(Sequel::Error)
   end
   
   specify "should raise in the down direction if migration uses add_foreign_key with an array" do
     m = Sequel.migration{change{alter_table(:a){add_foreign_key [:b]}}}
-    proc{m.apply(@db, :up)}.should_not raise_error(Sequel::Error)
+    proc{m.apply(@db, :up)}.should_not raise_error
     proc{m.apply(@db, :down)}.should raise_error(Sequel::Error)
   end
 end
@@ -219,7 +233,7 @@ describe "Sequel::IntegerMigrator" do
         @tables_created << name.to_sym
       end
       
-      def dataset(opts={})
+      def dataset
         ds = super
         ds.extend(Module.new do
           def count; 1; end
@@ -247,6 +261,10 @@ describe "Sequel::IntegerMigrator" do
   specify "should raise and error if there is a missing integer migration version" do
     proc{Sequel::Migrator.apply(@db, "spec/files/missing_integer_migrations")}.should raise_error(Sequel::Migrator::Error)
   end
+  
+  specify "should not raise and error if there is a missing integer migration version and allow_missing_migration_files is true" do
+    proc{Sequel::Migrator.run(@db, "spec/files/missing_integer_migrations", :allow_missing_migration_files => true)}.should_not raise_error
+  end
 
   specify "should raise and error if there is a duplicate integer migration version" do
     proc{Sequel::Migrator.apply(@db, "spec/files/duplicate_integer_migrations")}.should raise_error(Sequel::Migrator::Error)
@@ -597,6 +615,18 @@ describe "Sequel::TimestampMigrator" do
     @db[:schema_migrations].select_order_map(:filename).should == %w'1273253849_create_sessions.rb 1273253851_create_nodes.rb 1273253853_3_create_users.rb'
   end
   
+  specify "should not raise error for applied migrations not in file system if :allow_missing_migration_files is true" do
+    @dir = 'spec/files/timestamped_migrations'
+    @m.apply(@db, @dir)
+    [:schema_migrations, :sm1111, :sm2222, :sm3333].each{|n| @db.table_exists?(n).should be_true}
+    @db[:schema_migrations].select_order_map(:filename).should == %w'1273253849_create_sessions.rb 1273253851_create_nodes.rb 1273253853_3_create_users.rb'
+
+    @dir = 'spec/files/missing_timestamped_migrations'
+    proc{@m.run(@db, @dir, :allow_missing_migration_files => true)}.should_not raise_error
+    [:schema_migrations, :sm1111, :sm2222, :sm3333].each{|n| @db.table_exists?(n).should be_true}
+    @db[:schema_migrations].select_order_map(:filename).should == %w'1273253849_create_sessions.rb 1273253851_create_nodes.rb 1273253853_3_create_users.rb'
+  end
+  
   specify "should raise error missing column name in existing schema_migrations table" do
     @dir = 'spec/files/timestamped_migrations'
     @m.apply(@db, @dir)
@@ -633,32 +663,32 @@ describe "Sequel::TimestampMigrator" do
 
   specify "should use TimestampMigrator if TimestampMigrator.apply is called even for integer migrations directory" do
     Sequel::TimestampMigrator.apply(@db, "spec/files/integer_migrations")
-    @db.sqls.should == ["SELECT NULL FROM schema_migrations LIMIT 1", "CREATE TABLE schema_migrations (filename varchar(255) PRIMARY KEY)", "SELECT NULL FROM schema_info LIMIT 1", "SELECT filename FROM schema_migrations ORDER BY filename", "CREATE TABLE sm1111 (smc1 integer)", "INSERT INTO schema_migrations (filename) VALUES ('001_create_sessions.rb')", "CREATE TABLE sm2222 (smc2 integer)", "INSERT INTO schema_migrations (filename) VALUES ('002_create_nodes.rb')", "CREATE TABLE sm3333 (smc3 integer)", "INSERT INTO schema_migrations (filename) VALUES ('003_3_create_users.rb')"]
+    @db.sqls.should == ["SELECT NULL AS nil FROM schema_migrations LIMIT 1", "CREATE TABLE schema_migrations (filename varchar(255) PRIMARY KEY)", "SELECT NULL AS nil FROM schema_info LIMIT 1", "SELECT filename FROM schema_migrations ORDER BY filename", "CREATE TABLE sm1111 (smc1 integer)", "INSERT INTO schema_migrations (filename) VALUES ('001_create_sessions.rb')", "CREATE TABLE sm2222 (smc2 integer)", "INSERT INTO schema_migrations (filename) VALUES ('002_create_nodes.rb')", "CREATE TABLE sm3333 (smc3 integer)", "INSERT INTO schema_migrations (filename) VALUES ('003_3_create_users.rb')"]
   end
 
   specify "should not use transactions by default" do
     Sequel::TimestampMigrator.apply(@db, "spec/files/transaction_unspecified_migrations")
-    @db.sqls.should == ["SELECT NULL FROM schema_migrations LIMIT 1", "CREATE TABLE schema_migrations (filename varchar(255) PRIMARY KEY)", "SELECT NULL FROM schema_info LIMIT 1", "SELECT filename FROM schema_migrations ORDER BY filename", "CREATE TABLE sm11111 (smc1 integer)", "INSERT INTO schema_migrations (filename) VALUES ('001_create_alt_basic.rb')", "CREATE TABLE sm (smc1 integer)", "INSERT INTO schema_migrations (filename) VALUES ('002_create_basic.rb')"]
+    @db.sqls.should == ["SELECT NULL AS nil FROM schema_migrations LIMIT 1", "CREATE TABLE schema_migrations (filename varchar(255) PRIMARY KEY)", "SELECT NULL AS nil FROM schema_info LIMIT 1", "SELECT filename FROM schema_migrations ORDER BY filename", "CREATE TABLE sm11111 (smc1 integer)", "INSERT INTO schema_migrations (filename) VALUES ('001_create_alt_basic.rb')", "CREATE TABLE sm (smc1 integer)", "INSERT INTO schema_migrations (filename) VALUES ('002_create_basic.rb')"]
   end
 
   specify "should use transactions by default if database supports transactional ddl" do
     @db.meta_def(:supports_transactional_ddl?){true}
     Sequel::TimestampMigrator.apply(@db, "spec/files/transaction_unspecified_migrations")
-    @db.sqls.should == ["SELECT NULL FROM schema_migrations LIMIT 1", "CREATE TABLE schema_migrations (filename varchar(255) PRIMARY KEY)", "SELECT NULL FROM schema_info LIMIT 1", "SELECT filename FROM schema_migrations ORDER BY filename", "BEGIN", "CREATE TABLE sm11111 (smc1 integer)", "INSERT INTO schema_migrations (filename) VALUES ('001_create_alt_basic.rb')", "COMMIT", "BEGIN", "CREATE TABLE sm (smc1 integer)", "INSERT INTO schema_migrations (filename) VALUES ('002_create_basic.rb')", "COMMIT"]
+    @db.sqls.should == ["SELECT NULL AS nil FROM schema_migrations LIMIT 1", "CREATE TABLE schema_migrations (filename varchar(255) PRIMARY KEY)", "SELECT NULL AS nil FROM schema_info LIMIT 1", "SELECT filename FROM schema_migrations ORDER BY filename", "BEGIN", "CREATE TABLE sm11111 (smc1 integer)", "INSERT INTO schema_migrations (filename) VALUES ('001_create_alt_basic.rb')", "COMMIT", "BEGIN", "CREATE TABLE sm (smc1 integer)", "INSERT INTO schema_migrations (filename) VALUES ('002_create_basic.rb')", "COMMIT"]
   end
 
   specify "should support transaction use on a per migration basis" do
     Sequel::TimestampMigrator.apply(@db, "spec/files/transaction_specified_migrations")
-    @db.sqls.should == ["SELECT NULL FROM schema_migrations LIMIT 1", "CREATE TABLE schema_migrations (filename varchar(255) PRIMARY KEY)", "SELECT NULL FROM schema_info LIMIT 1", "SELECT filename FROM schema_migrations ORDER BY filename", "BEGIN", "CREATE TABLE sm11111 (smc1 integer)", "INSERT INTO schema_migrations (filename) VALUES ('001_create_alt_basic.rb')", "COMMIT", "CREATE TABLE sm (smc1 integer)", "INSERT INTO schema_migrations (filename) VALUES ('002_create_basic.rb')"]
+    @db.sqls.should == ["SELECT NULL AS nil FROM schema_migrations LIMIT 1", "CREATE TABLE schema_migrations (filename varchar(255) PRIMARY KEY)", "SELECT NULL AS nil FROM schema_info LIMIT 1", "SELECT filename FROM schema_migrations ORDER BY filename", "BEGIN", "CREATE TABLE sm11111 (smc1 integer)", "INSERT INTO schema_migrations (filename) VALUES ('001_create_alt_basic.rb')", "COMMIT", "CREATE TABLE sm (smc1 integer)", "INSERT INTO schema_migrations (filename) VALUES ('002_create_basic.rb')"]
   end
 
   specify "should force transactions if enabled by the migrator" do
     Sequel::TimestampMigrator.run(@db, "spec/files/transaction_specified_migrations", :use_transactions=>true)
-    @db.sqls.should == ["SELECT NULL FROM schema_migrations LIMIT 1", "CREATE TABLE schema_migrations (filename varchar(255) PRIMARY KEY)", "SELECT NULL FROM schema_info LIMIT 1", "SELECT filename FROM schema_migrations ORDER BY filename", "BEGIN", "CREATE TABLE sm11111 (smc1 integer)", "INSERT INTO schema_migrations (filename) VALUES ('001_create_alt_basic.rb')", "COMMIT", "BEGIN", "CREATE TABLE sm (smc1 integer)", "INSERT INTO schema_migrations (filename) VALUES ('002_create_basic.rb')", "COMMIT"]
+    @db.sqls.should == ["SELECT NULL AS nil FROM schema_migrations LIMIT 1", "CREATE TABLE schema_migrations (filename varchar(255) PRIMARY KEY)", "SELECT NULL AS nil FROM schema_info LIMIT 1", "SELECT filename FROM schema_migrations ORDER BY filename", "BEGIN", "CREATE TABLE sm11111 (smc1 integer)", "INSERT INTO schema_migrations (filename) VALUES ('001_create_alt_basic.rb')", "COMMIT", "BEGIN", "CREATE TABLE sm (smc1 integer)", "INSERT INTO schema_migrations (filename) VALUES ('002_create_basic.rb')", "COMMIT"]
   end
 
   specify "should not use transactions if disabled in the migrator" do
     Sequel::TimestampMigrator.run(@db, "spec/files/transaction_unspecified_migrations", :use_transactions=>false)
-    @db.sqls.should == ["SELECT NULL FROM schema_migrations LIMIT 1", "CREATE TABLE schema_migrations (filename varchar(255) PRIMARY KEY)", "SELECT NULL FROM schema_info LIMIT 1", "SELECT filename FROM schema_migrations ORDER BY filename", "CREATE TABLE sm11111 (smc1 integer)", "INSERT INTO schema_migrations (filename) VALUES ('001_create_alt_basic.rb')", "CREATE TABLE sm (smc1 integer)", "INSERT INTO schema_migrations (filename) VALUES ('002_create_basic.rb')"]
+    @db.sqls.should == ["SELECT NULL AS nil FROM schema_migrations LIMIT 1", "CREATE TABLE schema_migrations (filename varchar(255) PRIMARY KEY)", "SELECT NULL AS nil FROM schema_info LIMIT 1", "SELECT filename FROM schema_migrations ORDER BY filename", "CREATE TABLE sm11111 (smc1 integer)", "INSERT INTO schema_migrations (filename) VALUES ('001_create_alt_basic.rb')", "CREATE TABLE sm (smc1 integer)", "INSERT INTO schema_migrations (filename) VALUES ('002_create_basic.rb')"]
   end
 end
