@@ -11,7 +11,7 @@ describe "NestedAttributes plugin" do
 
   def check_sql_array(*shoulds)
     sqls = @db.sqls
-    shoulds.length.should == sqls.length
+    sqls.length.should == shoulds.length
     shoulds.zip(sqls){|s, i| check_sqls(s, i)}
   end
 
@@ -36,7 +36,7 @@ describe "NestedAttributes plugin" do
     @Artist.one_to_many :albums, :class=>@Album, :key=>:artist_id
     @Artist.one_to_many :concerts, :class=>@Concert, :key=>:artist_id
     @Artist.one_to_one :first_album, :class=>@Album, :key=>:artist_id
-    @Album.many_to_one :artist, :class=>@Artist
+    @Album.many_to_one :artist, :class=>@Artist, :reciprocal=>:albums
     @Album.many_to_many :tags, :class=>@Tag, :left_key=>:album_id, :right_key=>:tag_id, :join_table=>:at
     @Artist.nested_attributes :albums, :first_album, :destroy=>true, :remove=>true
     @Artist.nested_attributes :concerts, :destroy=>true, :remove=>true
@@ -59,9 +59,8 @@ describe "NestedAttributes plugin" do
     @db.sqls.should == []
     a.save
     check_sql_array(["INSERT INTO artists (name, id) VALUES ('Ar', 1)", "INSERT INTO artists (id, name) VALUES (1, 'Ar')"],
-      "INSERT INTO albums (name) VALUES ('Al')",
-      "UPDATE albums SET artist_id = NULL WHERE ((artist_id = 1) AND (id != 2))",
-      ["UPDATE albums SET artist_id = 1, name = 'Al' WHERE (id = 2)", "UPDATE albums SET name = 'Al', artist_id = 1 WHERE (id = 2)"])
+      "UPDATE albums SET artist_id = NULL WHERE (artist_id = 1)",
+      ["INSERT INTO albums (artist_id, name) VALUES (1, 'Al')", "INSERT INTO albums (name, artist_id) VALUES ('Al', 1)"])
   end
   
   it "should support creating new one_to_many objects" do
@@ -341,7 +340,7 @@ describe "NestedAttributes plugin" do
     @Album.nested_attributes :artist
     proc{a.set(:artist_attributes=>{:id=>'20', :_delete=>'1'})}.should raise_error(Sequel::Error)
     @Album.nested_attributes :artist, :destroy=>true
-    proc{a.set(:artist_attributes=>{:id=>'20', :_delete=>'1'})}.should_not raise_error(Sequel::Error)
+    proc{a.set(:artist_attributes=>{:id=>'20', :_delete=>'1'})}.should_not raise_error
   end
   
   it "should only allow removing associated objects if :remove option is used in the nested_attributes call" do
@@ -351,7 +350,7 @@ describe "NestedAttributes plugin" do
     @Album.nested_attributes :artist
     proc{a.set(:artist_attributes=>{:id=>'20', :_remove=>'1'})}.should raise_error(Sequel::Error)
     @Album.nested_attributes :artist, :remove=>true
-    proc{a.set(:artist_attributes=>{:id=>'20', :_remove=>'1'})}.should_not raise_error(Sequel::Error)
+    proc{a.set(:artist_attributes=>{:id=>'20', :_remove=>'1'})}.should_not raise_error
   end
   
   it "should raise an Error if a primary key is given in a nested attribute hash, but no matching associated object exists" do
@@ -359,7 +358,7 @@ describe "NestedAttributes plugin" do
     ar = @Artist.load(:id=>20, :name=>'Ar')
     ar.associations[:albums] = [al]
     proc{ar.set(:albums_attributes=>[{:id=>30, :_delete=>'t'}])}.should raise_error(Sequel::Error)
-    proc{ar.set(:albums_attributes=>[{:id=>10, :_delete=>'t'}])}.should_not raise_error(Sequel::Error)
+    proc{ar.set(:albums_attributes=>[{:id=>10, :_delete=>'t'}])}.should_not raise_error
   end
   
   it "should not raise an Error if an unmatched primary key is given, if the :strict=>false option is used" do
@@ -389,7 +388,7 @@ describe "NestedAttributes plugin" do
     co = @Concert.load(:tour=>'To', :date=>'2004-04-05', :playlist=>'Pl')
     ar.associations[:concerts] = [co]
     proc{ar.set(:concerts_attributes=>[{:tour=>'To', :date=>'2004-04-04', :_delete=>'t'}])}.should raise_error(Sequel::Error)
-    proc{ar.set(:concerts_attributes=>[{:tour=>'To', :date=>'2004-04-05', :_delete=>'t'}])}.should_not raise_error(Sequel::Error)
+    proc{ar.set(:concerts_attributes=>[{:tour=>'To', :date=>'2004-04-05', :_delete=>'t'}])}.should_not raise_error
   end
 
   it "should not raise an Error if an unmatched composite primary key is given, if the :strict=>false option is used" do
@@ -420,7 +419,7 @@ describe "NestedAttributes plugin" do
   end
   
   it "should not attempt to validate nested attributes if the :validate=>false association option is used" do
-    @Album.many_to_one :artist, :class=>@Artist, :validate=>false
+    @Album.many_to_one :artist, :class=>@Artist, :validate=>false, :reciprocal=>nil
     @Album.nested_attributes :artist, :tags, :destroy=>true, :remove=>true
     @Artist.class_eval do
       def validate
@@ -555,4 +554,22 @@ describe "NestedAttributes plugin" do
     proc{al.set(:tags_attributes=>[{:id=>30, :name=>'T2', :number=>3}])}.should raise_error(Sequel::Error)
     proc{al.set(:tags_attributes=>[{:name=>'T2', :number=>3}])}.should raise_error(Sequel::Error)
   end
+
+  it "should accept a proc for the :fields option that accepts the associated object and returns an array of fields" do
+    @Tag.columns :id, :name, :number
+    @Album.nested_attributes :tags, :destroy=>true, :remove=>true, :fields=>proc{|object| object.is_a?(@Tag) ? [:name] : []}
+
+    al = @Album.load(:id=>10, :name=>'Al')
+    t = @Tag.load(:id=>30, :name=>'T', :number=>10)
+    al.associations[:tags] = [t]
+    al.set(:tags_attributes=>[{:id=>30, :name=>'T2'}, {:name=>'T3'}])
+    @db.sqls.should == []
+    al.save
+    check_sql_array("UPDATE albums SET name = 'Al' WHERE (id = 10)",
+      "UPDATE tags SET name = 'T2' WHERE (id = 30)",
+      "INSERT INTO tags (name) VALUES ('T3')",
+      ["INSERT INTO at (album_id, tag_id) VALUES (10, 1)", "INSERT INTO at (tag_id, album_id) VALUES (1, 10)"])
+    proc{al.set(:tags_attributes=>[{:id=>30, :name=>'T2', :number=>3}])}.should raise_error(Sequel::Error)
+    proc{al.set(:tags_attributes=>[{:name=>'T2', :number=>3}])}.should raise_error(Sequel::Error)
+  end
 end
