@@ -1,17 +1,27 @@
 require File.join(File.dirname(File.expand_path(__FILE__)), "spec_helper")
 
 describe "pg_array extension" do
+  before(:all) do
+    Sequel.extension :pg_array
+    @pg_types = Sequel::Postgres::PG_TYPES.dup
+    @pg_named_types = Sequel::Postgres::PG_NAMED_TYPES.dup
+  end
+  after(:all) do
+    Sequel::Postgres::PG_TYPES.replace(@pg_types)
+    Sequel::Postgres::PG_NAMED_TYPES.replace(@pg_named_types)
+  end
+
   before do
     @db = Sequel.connect('mock://postgres', :quote_identifiers=>false)
-    @db.extend(Module.new{def bound_variable_arg(arg, conn) arg end; def get_conversion_procs(conn) {} end})
     @db.extend_datasets(Module.new{def supports_timestamp_timezones?; false; end; def supports_timestamp_usecs?; false; end})
     @db.extension(:pg_array)
     @m = Sequel::Postgres
-    @convertor = @m::PG_TYPES
+    @converter = @m::PG_TYPES
+    @db.sqls
   end
 
   it "should parse single dimensional text arrays" do
-    c = @convertor[1009]
+    c = @converter[1009]
     c.call("{a}").to_a.first.should be_a_kind_of(String)
     c.call("{}").to_a.should == []
     c.call("{a}").to_a.should == ['a']
@@ -20,7 +30,7 @@ describe "pg_array extension" do
   end
 
   it "should parse multi-dimensional text arrays" do
-    c = @convertor[1009]
+    c = @converter[1009]
     c.call("{{}}").to_a.should == [[]]
     c.call("{{a},{b}}").to_a.should == [['a'], ['b']]
     c.call('{{"a b"},{c}}').to_a.should == [['a b'], ['c']]
@@ -29,12 +39,12 @@ describe "pg_array extension" do
   end
 
   it "should parse text arrays with embedded deliminaters" do
-    c = @convertor[1009]
+    c = @converter[1009]
     c.call('{{"{},","\\",\\,\\\\\\"\\""}}').to_a.should == [['{},', '",,\\""']]
   end
 
   it "should parse single dimensional integer arrays" do
-    c = @convertor[1007]
+    c = @converter[1007]
     c.call("{1}").to_a.first.should be_a_kind_of(Integer)
     c.call("{}").to_a.should == []
     c.call("{1}").to_a.should == [1]
@@ -43,7 +53,7 @@ describe "pg_array extension" do
   end
 
   it "should parse multiple dimensional integer arrays" do
-    c = @convertor[1007]
+    c = @converter[1007]
     c.call("{{}}").to_a.should == [[]]
     c.call("{{1}}").to_a.should == [[1]]
     c.call('{{2},{3}}').to_a.should == [[2], [3]]
@@ -51,7 +61,7 @@ describe "pg_array extension" do
   end
 
   it "should parse single dimensional float arrays" do
-    c = @convertor[1022]
+    c = @converter[1022]
     c.call("{}").to_a.should == []
     c.call("{1.5}").to_a.should == [1.5]
     c.call('{2.5,3.5}').to_a.should == [2.5, 3.5]
@@ -59,7 +69,7 @@ describe "pg_array extension" do
   end
 
   it "should parse multiple dimensional float arrays" do
-    c = @convertor[1022]
+    c = @converter[1022]
     c.call("{{}}").to_a.should == [[]]
     c.call("{{1.5}}").to_a.should == [[1.5]]
     c.call('{{2.5},{3.5}}').to_a.should == [[2.5], [3.5]]
@@ -67,14 +77,14 @@ describe "pg_array extension" do
   end
 
   it "should parse integers in float arrays as floats" do
-    c = @convertor[1022]
+    c = @converter[1022]
     c.call("{1}").to_a.first.should be_a_kind_of(Float)
     c.call("{1}").to_a.should == [1.0]
     c.call('{{{1,2},{3,4}},{{5,6},{7,8}}}').to_a.should == [[[1.0, 2.0], [3.0, 4.0]], [[5.0, 6.0], [7.0, 8.0]]]
   end
 
   it "should parse single dimensional decimal arrays" do
-    c = @convertor[1231]
+    c = @converter[1231]
     c.call("{}").to_a.should == []
     c.call("{1.5}").to_a.should == [BigDecimal.new('1.5')]
     c.call('{2.5,3.5}').to_a.should == [BigDecimal.new('2.5'), BigDecimal.new('3.5')]
@@ -82,7 +92,7 @@ describe "pg_array extension" do
   end
 
   it "should parse multiple dimensional decimal arrays" do
-    c = @convertor[1231]
+    c = @converter[1231]
     c.call("{{}}").to_a.should == [[]]
     c.call("{{1.5}}").to_a.should == [[BigDecimal.new('1.5')]]
     c.call('{{2.5},{3.5}}').to_a.should == [[BigDecimal.new('2.5')], [BigDecimal.new('3.5')]]
@@ -90,20 +100,20 @@ describe "pg_array extension" do
   end
 
   it "should parse decimal values with arbitrary precision" do
-    c = @convertor[1231]
+    c = @converter[1231]
     c.call("{1.000000000000000000005}").to_a.should == [BigDecimal.new('1.000000000000000000005')]
     c.call("{{1.000000000000000000005,2.000000000000000000005},{3.000000000000000000005,4.000000000000000000005}}").to_a.should == [[BigDecimal.new('1.000000000000000000005'), BigDecimal.new('2.000000000000000000005')], [BigDecimal.new('3.000000000000000000005'), BigDecimal.new('4.000000000000000000005')]]
   end
 
   it "should parse integers in decimal arrays as BigDecimals" do
-    c = @convertor[1231]
+    c = @converter[1231]
     c.call("{1}").to_a.first.should be_a_kind_of(BigDecimal)
     c.call("{1}").to_a.should == [BigDecimal.new('1')]
     c.call('{{{1,2},{3,4}},{{5,6},{7,8}}}').to_a.should == [[[BigDecimal.new('1'), BigDecimal.new('2')], [BigDecimal.new('3'), BigDecimal.new('4')]], [[BigDecimal.new('5'), BigDecimal.new('6')], [BigDecimal.new('7'), BigDecimal.new('8')]]]
   end
 
   it "should parse arrays with NULL values" do
-    @convertor.values_at(1007, 1009, 1022, 1231).each do |c|
+    @converter.values_at(1007, 1009, 1022, 1231).each do |c|
       c.call("{NULL}").should == [nil]
       c.call("{NULL,NULL}").should == [nil,nil]
       c.call("{{NULL,NULL},{NULL,NULL}}").should == [[nil,nil],[nil,nil]]
@@ -111,7 +121,7 @@ describe "pg_array extension" do
   end
 
   it 'should parse arrays with "NULL" values' do
-    c = @convertor[1009]
+    c = @converter[1009]
     c.call('{NULL,"NULL",NULL}').to_a.should == [nil, "NULL", nil]
     c.call('{NULLA,"NULL",NULL}').to_a.should == ["NULLA", "NULL", nil]
   end
@@ -146,15 +156,29 @@ describe "pg_array extension" do
     @db.literal(@m::PGArray.new([nil, "{},[]'\""], :"varchar(255)")).should == "ARRAY[NULL,'{},[]''\"']::varchar(255)[]"
   end
 
-  it "should have Array#pg_array method for easy PGArray creation" do
-    @db.literal([1].pg_array).should == 'ARRAY[1]'
-    @db.literal([1, 2].pg_array(:int4)).should == 'ARRAY[1,2]::int4[]'
-    @db.literal([[[1], [2]], [[3], [4]]].pg_array(:real)).should == 'ARRAY[[[1],[2]],[[3],[4]]]::real[]'
+  it "should have Sequel.pg_array method for easy PGArray creation" do
+    @db.literal(Sequel.pg_array([1])).should == 'ARRAY[1]'
+    @db.literal(Sequel.pg_array([1, 2], :int4)).should == 'ARRAY[1,2]::int4[]'
+    @db.literal(Sequel.pg_array([[[1], [2]], [[3], [4]]], :real)).should == 'ARRAY[[[1],[2]],[[3],[4]]]::real[]'
+  end
+
+  it "should have Sequel.pg_array return existing PGArrays as-is" do
+    a = Sequel.pg_array([1])
+    Sequel.pg_array(a).should equal(a)
+  end
+
+  it "should have Sequel.pg_array create a new PGArrays if type of existing does not match" do
+    a = Sequel.pg_array([1], :int4)
+    b = Sequel.pg_array(a, :int8)
+    a.should == b
+    a.should_not equal(b)
+    a.array_type.should == :int4
+    b.array_type.should == :int8
   end
 
   it "should support using arrays as bound variables" do
     @db.bound_variable_arg(1, nil).should == 1
-    @db.bound_variable_arg([1,2].pg_array, nil).should == '{1,2}'
+    @db.bound_variable_arg(Sequel.pg_array([1,2]), nil).should == '{1,2}'
     @db.bound_variable_arg([1,2], nil).should == '{1,2}'
     @db.bound_variable_arg([[1,2]], nil).should == '{{1,2}}'
     @db.bound_variable_arg([1.0,2.0], nil).should == '{1.0,2.0}'
@@ -169,49 +193,52 @@ describe "pg_array extension" do
 
   it "should support typecasting of the various array types" do
     {
-      :integer=>{:class=>Integer, :convert=>['1', '1', 1, '1']},
-      :float=>{:db_type=>'double precision',  :class=>Float, :convert=>['1.1', '1.1', 1.1, '1.1']},
-      :decimal=>{:db_type=>'numeric', :class=>BigDecimal, :convert=>['1.00000000000000000000000001', '1.00000000000000000000000001', BigDecimal.new('1.00000000000000000000000001'), '1.00000000000000000000000001']},
-      :string=>{:db_type=>'text', :class=>String, :convert=>['1', 1, '1', "'1'"]},
-      :bigint=>{:class=>Integer, :convert=>['1', '1', 1, '1']},
-      :boolean=>{:class=>TrueClass, :convert=>['t', 't', true, 'true']},
-      :blob=>{:db_type=>'bytea', :class=>Sequel::SQL::Blob, :convert=>['1', '1', '1', "'1'"]},
-      :date=>{:class=>Date, :convert=>['2011-10-12', '2011-10-12', Date.new(2011, 10, 12), "'2011-10-12'"]},
-      :time=>{:db_type=>'time without time zone', :class=>Sequel::SQLTime, :convert=>['01:02:03', '01:02:03', Sequel::SQLTime.create(1, 2, 3), "'01:02:03'"]},
-      :datetime=>{:db_type=>'timestamp without time zone', :class=>Time, :convert=>['2011-10-12 01:02:03', '2011-10-12 01:02:03', Time.local(2011, 10, 12, 1, 2, 3), "'2011-10-12 01:02:03'"]},
-      :time_timezone=>{:db_type=>'time with time zone', :class=>Sequel::SQLTime, :convert=>['01:02:03', '01:02:03', Sequel::SQLTime.create(1, 2, 3), "'01:02:03'"]},
-      :datetime_timezone=>{:db_type=>'timestamp with time zone', :class=>Time, :convert=>['2011-10-12 01:02:03', '2011-10-12 01:02:03', Time.local(2011, 10, 12, 1, 2, 3), "'2011-10-12 01:02:03'"]},
+      :integer=>{:class=>Integer, :convert=>['1', 1, '1']},
+      :float=>{:db_type=>'double precision',  :class=>Float, :convert=>['1.1', 1.1, '1.1']},
+      :decimal=>{:db_type=>'numeric', :class=>BigDecimal, :convert=>['1.00000000000000000000000001', BigDecimal.new('1.00000000000000000000000001'), '1.00000000000000000000000001']},
+      :string=>{:db_type=>'text', :class=>String, :convert=>[1, '1', "'1'"]},
+      :bigint=>{:class=>Integer, :convert=>['1', 1, '1']},
+      :boolean=>{:class=>TrueClass, :convert=>['t', true, 'true']},
+      :blob=>{:db_type=>'bytea', :class=>Sequel::SQL::Blob, :convert=>['1', '1', "'1'"]},
+      :date=>{:class=>Date, :convert=>['2011-10-12', Date.new(2011, 10, 12), "'2011-10-12'"]},
+      :time=>{:db_type=>'time without time zone', :class=>Sequel::SQLTime, :convert=>['01:02:03', Sequel::SQLTime.create(1, 2, 3), "'01:02:03'"]},
+      :datetime=>{:db_type=>'timestamp without time zone', :class=>Time, :convert=>['2011-10-12 01:02:03', Time.local(2011, 10, 12, 1, 2, 3), "'2011-10-12 01:02:03'"]},
+      :time_timezone=>{:db_type=>'time with time zone', :class=>Sequel::SQLTime, :convert=>['01:02:03', Sequel::SQLTime.create(1, 2, 3), "'01:02:03'"]},
+      :datetime_timezone=>{:db_type=>'timestamp with time zone', :class=>Time, :convert=>['2011-10-12 01:02:03', Time.local(2011, 10, 12, 1, 2, 3), "'2011-10-12 01:02:03'"]},
     }.each do |type, h|
       meth = :"#{type}_array"
       db_type = h[:db_type]||type
       klass = h[:class]
-      text_in, array_in, value, output = h[:convert]
+      array_in, value, output = h[:convert]
 
-      ["{#{text_in}}", [array_in]].each do |input|
+      [[array_in]].each do |input|
         v = @db.typecast_value(meth, input)
         v.should == [value]
         v.first.should be_a_kind_of(klass)
         v.array_type.should_not be_nil
-        @db.typecast_value(meth, [value].pg_array).should == v
+        @db.typecast_value(meth, Sequel.pg_array([value])).should == v
         @db.typecast_value(meth, v).should equal(v)
       end
 
-      ["{{#{text_in}}}", [[array_in]]].each do |input|
+      [[[array_in]]].each do |input|
         v = @db.typecast_value(meth, input)
         v.should == [[value]]
         v.first.first.should be_a_kind_of(klass)
         v.array_type.should_not be_nil
-        @db.typecast_value(meth, [[value]].pg_array).should == v
+        @db.typecast_value(meth, Sequel.pg_array([[value]])).should == v
         @db.typecast_value(meth, v).should equal(v)
       end
 
       @db.literal(@db.typecast_value(meth, [array_in])).should == "ARRAY[#{output}]::#{db_type}[]"
-      @db.typecast_value(meth, '{}').should == []
-      @db.literal(@db.typecast_value(meth, '{}')).should == "ARRAY[]::#{db_type}[]"
+      @db.literal(@db.typecast_value(meth, [])).should == "ARRAY[]::#{db_type}[]"
     end
     proc{@db.typecast_value(:integer_array, {})}.should raise_error(Sequel::InvalidValue)
   end
 
+  it "should support SQL::AliasMethods" do
+    @db.select(Sequel.pg_array([1], :integer).as(:col1)).sql.should == 'SELECT ARRAY[1]::integer[] AS col1'
+  end
+
   it "should support registering custom array types" do
     Sequel::Postgres::PGArray.register('foo')
     @db.typecast_value(:foo_array, []).should be_a_kind_of(Sequel::Postgres::PGArray)
@@ -227,18 +254,32 @@ describe "pg_array extension" do
   end
 
   it "should support using a block as a custom conversion proc given as block" do
-    Sequel::Postgres::PGArray.register('foo'){|s| (s*2).to_i}
-    @db.typecast_value(:foo_array, '{1}').should == [11]
+    Sequel::Postgres::PGArray.register('foo', :oid=>1234){|s| (s*2).to_i}
+    @converter[1234].call('{1}').should == [11]
   end
 
   it "should support using a block as a custom conversion proc given as :converter option" do
-    Sequel::Postgres::PGArray.register('foo', :converter=>proc{|s| (s*2).to_i})
-    @db.typecast_value(:foo_array, '{1}').should == [11]
+    Sequel::Postgres::PGArray.register('foo', :oid=>1234, :converter=>proc{|s| (s*2).to_i})
+    @converter[1234].call('{1}').should == [11]
   end
 
   it "should support using an existing scaler conversion proc via the :scalar_oid option" do
-    Sequel::Postgres::PGArray.register('foo', :scalar_oid=>16)
-    @db.typecast_value(:foo_array, '{"t"}').should == [true]
+    Sequel::Postgres::PGArray.register('foo', :oid=>1234, :scalar_oid=>16)
+    @converter[1234].call('{t}').should == [true]
+  end
+
+  it "should support using a given conversion procs hash via the :type_procs option" do
+    h = {16=>proc{|s| "!#{s}"}}
+    Sequel::Postgres::PGArray.register('foo', :oid=>1234, :scalar_oid=>16, :type_procs=>h)
+    h[1234].call('{t}').should == ["!t"]
+  end
+
+  it "should support adding methods to the given module via the :typecast_methods_module option" do
+    m = Module.new
+    Sequel::Postgres::PGArray.register('foo15', :scalar_typecast=>:boolean, :typecast_methods_module=>m)
+    @db.typecast_value(:foo15_array, ['t']).should == ['t']
+    @db.extend(m)
+    @db.typecast_value(:foo15_array, ['t']).should == [true]
   end
 
   it "should raise an error if using :scalar_oid option with unexisting scalar conversion proc" do
@@ -263,7 +304,7 @@ describe "pg_array extension" do
     Sequel::Postgres::PG_TYPES[2].should be_a_kind_of(Sequel::Postgres::PGArray::JSONCreator)
   end
 
-  it "should support registering convertors with :parser=>:json option" do
+  it "should support registering converters with :parser=>:json option and blocks" do
     Sequel::Postgres::PGArray.register('foo', :oid=>4, :parser=>:json){|s| s * 2}
     Sequel::Postgres::PG_TYPES[4].call('{{1, 2}, {3, 4}}').should == [[2, 4], [6, 8]]
   end
@@ -279,9 +320,60 @@ describe "pg_array extension" do
     @db.schema(:items).map{|e| e[1][:type]}.should == [:float_array]
   end
 
-  it "should set appropriate timestamp conversion procs when getting conversion procs" do
-    procs = @db.send(:get_conversion_procs, nil)
+  it "should support registering custom array types on a per-Database basis" do
+    @db.register_array_type('banana', :oid=>7865){|s| s}
+    @db.typecast_value(:banana_array, []).should be_a_kind_of(Sequel::Postgres::PGArray)
+    @db.fetch = [{:name=>'id', :db_type=>'banana[]'}]
+    @db.schema(:items).map{|e| e[1][:type]}.should == [:banana_array]
+    @db.conversion_procs.should have_key(7865)
+    @db.respond_to?(:typecast_value_banana_array, true).should be_true
+
+    db = Sequel.connect('mock://postgres', :quote_identifiers=>false)
+    db.extend_datasets(Module.new{def supports_timestamp_timezones?; false; end; def supports_timestamp_usecs?; false; end})
+    db.extension(:pg_array)
+    db.fetch = [{:name=>'id', :db_type=>'banana[]'}]
+    db.schema(:items).map{|e| e[1][:type]}.should == [nil]
+    db.conversion_procs.should_not have_key(7865)
+    db.respond_to?(:typecast_value_banana_array, true).should be_false
+  end
+
+  it "should automatically look up the array and scalar oids when registering per-Database types" do
+    @db.fetch = [[{:oid=>21, :typarray=>7866}], [{:name=>'id', :db_type=>'banana[]'}]]
+    @db.register_array_type('banana', :scalar_typecast=>:integer)
+    @db.sqls.should == ["SELECT typarray, oid FROM pg_type WHERE (typname = 'banana') LIMIT 1"]
+    @db.schema(:items).map{|e| e[1][:type]}.should == [:banana_array]
+    @db.conversion_procs[7866].call("{1,2}").should == [1,2]
+    @db.typecast_value(:banana_array, %w'1 2').should == [1,2]
+  end
+
+  it "should not automatically look up oids if given both scalar and array oids" do
+    @db.register_array_type('banana', :oid=>7866, :scalar_oid=>21, :scalar_typecast=>:integer)
+    @db.sqls.should == []
+    @db.conversion_procs[7866].call("{1,2}").should == [1,2]
+    @db.typecast_value(:banana_array, %w'1 2').should == [1,2]
+  end
+
+  it "should not automatically look up oids if given array oid and block" do
+    @db.register_array_type('banana', :oid=>7866, :scalar_typecast=>:integer){|s| s.to_i}
+    @db.sqls.should == []
+    @db.conversion_procs[7866].call("{1,2}").should == [1,2]
+    @db.typecast_value(:banana_array, %w'1 2').should == [1,2]
+  end
+
+  it "should set appropriate timestamp conversion procs when resetting conversion procs" do
+    Sequel::Postgres::PG_NAMED_TYPES[:foo] = proc{|v| v*2}
+    @db.fetch = [[{:oid=>2222, :typname=>'foo'}], [{:oid=>2222, :typarray=>2223, :typname=>'foo'}]]
+    @db.reset_conversion_procs
+    procs = @db.conversion_procs
     procs[1185].call('{"2011-10-20 11:12:13"}').should == [Time.local(2011, 10, 20, 11, 12, 13)]
     procs[1115].call('{"2011-10-20 11:12:13"}').should == [Time.local(2011, 10, 20, 11, 12, 13)]
+    procs[2222].call('1').should == '11'
+    procs[2223].call('{"2"}').should == ['22']
+  end
+
+  it "should return correct results for Database#schema_type_class" do
+    @db.register_array_type('banana', :oid=>7866, :scalar_typecast=>:integer){|s| s.to_i}
+    @db.schema_type_class(:banana_array).should == Sequel::Postgres::PGArray
+    @db.schema_type_class(:integer).should == Integer
   end
 end
