@@ -1,23 +1,27 @@
 require File.join(File.dirname(File.expand_path(__FILE__)), 'spec_helper.rb')
 
-shared_examples_for "eager limit strategies" do
+shared_examples_for "one_to_one eager limit strategies" do
   specify "eager loading one_to_one associations should work correctly" do
     Artist.one_to_one :first_album, {:clone=>:first_album}.merge(@els) if @els
     Artist.one_to_one  :last_album, {:clone=>:last_album}.merge(@els) if @els
+    Artist.one_to_one  :second_album, {:clone=>:second_album}.merge(@els) if @els
     @album.update(:artist => @artist)
     diff_album = @diff_album.call
-    al, ar, t = @pr.call
+    ar = @pr.call[1]
     
-    a = Artist.eager(:first_album, :last_album).order(:name).all
+    a = Artist.eager(:first_album, :last_album, :second_album).order(:name).all
     a.should == [@artist, ar]
     a.first.first_album.should == @album
     a.first.last_album.should == diff_album
+    a.first.second_album.should == diff_album
     a.last.first_album.should == nil
     a.last.last_album.should == nil
+    a.last.second_album.should == nil
 
     # Check that no extra columns got added by the eager loading
     a.first.first_album.values.should == @album.values
     a.first.last_album.values.should == diff_album.values
+    a.first.second_album.values.should == diff_album.values
 
     same_album = @same_album.call
     a = Artist.eager(:first_album).order(:name).all
@@ -25,45 +29,52 @@ shared_examples_for "eager limit strategies" do
     [@album, same_album].should include(a.first.first_album)
     a.last.first_album.should == nil
   end
+end
 
+shared_examples_for "one_to_many eager limit strategies" do
   specify "should correctly handle limits and offsets when eager loading one_to_many associations" do
     Artist.one_to_many :first_two_albums, {:clone=>:first_two_albums}.merge(@els) if @els
     Artist.one_to_many :second_two_albums, {:clone=>:second_two_albums}.merge(@els) if @els
+    Artist.one_to_many :not_first_albums, {:clone=>:not_first_albums}.merge(@els) if @els
     Artist.one_to_many :last_two_albums, {:clone=>:last_two_albums}.merge(@els) if @els
     @album.update(:artist => @artist)
     middle_album = @middle_album.call
     diff_album = @diff_album.call
-    al, ar, t = @pr.call
+    ar = @pr.call[1]
     
-    ars = Artist.eager(:first_two_albums, :second_two_albums, :last_two_albums).order(:name).all
+    ars = Artist.eager(:first_two_albums, :second_two_albums, :not_first_albums, :last_two_albums).order(:name).all
     ars.should == [@artist, ar]
     ars.first.first_two_albums.should == [@album, middle_album]
     ars.first.second_two_albums.should == [middle_album, diff_album]
+    ars.first.not_first_albums.should == [middle_album, diff_album]
     ars.first.last_two_albums.should == [diff_album, middle_album]
     ars.last.first_two_albums.should == []
     ars.last.second_two_albums.should == []
+    ars.last.not_first_albums.should == []
     ars.last.last_two_albums.should == []
     
     # Check that no extra columns got added by the eager loading
     ars.first.first_two_albums.map{|x| x.values}.should == [@album, middle_album].map{|x| x.values}
     ars.first.second_two_albums.map{|x| x.values}.should == [middle_album, diff_album].map{|x| x.values}
+    ars.first.not_first_albums.map{|x| x.values}.should == [middle_album, diff_album].map{|x| x.values}
     ars.first.last_two_albums.map{|x| x.values}.should == [diff_album, middle_album].map{|x| x.values}
   end
-  
+end
+
+shared_examples_for "many_to_many eager limit strategies" do
   specify "should correctly handle limits and offsets when eager loading many_to_many associations" do
-    if @els == {:eager_limit_strategy=>:correlated_subquery} && Sequel.guarded?(:derby)
-      pending("Derby errors with correlated subqueries on many_to_many associations")
-    end
-    Album.many_to_many :first_two_tags, {:clone=>:first_two_tags}.merge(@els) if @els
-    Album.many_to_many :second_two_tags, {:clone=>:second_two_tags}.merge(@els) if @els
-    Album.many_to_many :last_two_tags, {:clone=>:last_two_tags}.merge(@els) if @els
+    Album.send @many_to_many_method||:many_to_many, :first_two_tags, {:clone=>:first_two_tags}.merge(@els) if @els
+    Album.send @many_to_many_method||:many_to_many, :second_two_tags, {:clone=>:second_two_tags}.merge(@els) if @els
+    Album.send @many_to_many_method||:many_to_many, :not_first_tags, {:clone=>:not_first_tags}.merge(@els) if @els
+    Album.send @many_to_many_method||:many_to_many, :last_two_tags, {:clone=>:last_two_tags}.merge(@els) if @els
     tu, tv = @other_tags.call
-    al, ar, t = @pr.call
+    al = @pr.call.first
     
-    als = Album.eager(:first_two_tags, :second_two_tags, :last_two_tags).order(:name).all
+    als = Album.eager(:first_two_tags, :second_two_tags, :not_first_tags, :last_two_tags).order(:name).all
     als.should == [@album, al]
     als.first.first_two_tags.should == [@tag, tu]
     als.first.second_two_tags.should == [tu, tv]
+    als.first.not_first_tags.should == [tu, tv]
     als.first.last_two_tags.should == [tv, tu]
     als.last.first_two_tags.should == []
     als.last.second_two_tags.should == []
@@ -72,24 +83,26 @@ shared_examples_for "eager limit strategies" do
     # Check that no extra columns got added by the eager loading
     als.first.first_two_tags.map{|x| x.values}.should == [@tag, tu].map{|x| x.values}
     als.first.second_two_tags.map{|x| x.values}.should == [tu, tv].map{|x| x.values}
+    als.first.not_first_tags.map{|x| x.values}.should == [tu, tv].map{|x| x.values}
     als.first.last_two_tags.map{|x| x.values}.should == [tv, tu].map{|x| x.values}
   end
-  
+end
+
+shared_examples_for "many_through_many eager limit strategies" do
   specify "should correctly handle limits and offsets when eager loading many_through_many associations" do
-    if @els == {:eager_limit_strategy=>:correlated_subquery} && Sequel.guarded?(:derby)
-      pending("Derby errors with correlated subqueries on many_through_many associations")
-    end
     Artist.many_through_many :first_two_tags, {:clone=>:first_two_tags}.merge(@els) if @els
     Artist.many_through_many :second_two_tags, {:clone=>:second_two_tags}.merge(@els) if @els
+    Artist.many_through_many :not_first_tags, {:clone=>:not_first_tags}.merge(@els) if @els
     Artist.many_through_many :last_two_tags, {:clone=>:last_two_tags}.merge(@els) if @els
     @album.update(:artist => @artist)
     tu, tv = @other_tags.call
-    al, ar, t = @pr.call
+    ar = @pr.call[1]
     
-    ars = Artist.eager(:first_two_tags, :second_two_tags, :last_two_tags).order(:name).all
+    ars = Artist.eager(:first_two_tags, :second_two_tags, :not_first_tags, :last_two_tags).order(:name).all
     ars.should == [@artist, ar]
     ars.first.first_two_tags.should == [@tag, tu]
     ars.first.second_two_tags.should == [tu, tv]
+    ars.first.not_first_tags.should == [tu, tv]
     ars.first.last_two_tags.should == [tv, tu]
     ars.last.first_two_tags.should == []
     ars.last.second_two_tags.should == []
@@ -98,10 +111,18 @@ shared_examples_for "eager limit strategies" do
     # Check that no extra columns got added by the eager loading
     ars.first.first_two_tags.map{|x| x.values}.should == [@tag, tu].map{|x| x.values}
     ars.first.second_two_tags.map{|x| x.values}.should == [tu, tv].map{|x| x.values}
+    ars.first.not_first_tags.map{|x| x.values}.should == [tu, tv].map{|x| x.values}
     ars.first.last_two_tags.map{|x| x.values}.should == [tv, tu].map{|x| x.values}
   end
 end
 
+shared_examples_for "eager limit strategies" do
+  it_should_behave_like "one_to_one eager limit strategies"
+  it_should_behave_like "one_to_many eager limit strategies"
+  it_should_behave_like "many_to_many eager limit strategies"
+  it_should_behave_like "many_through_many eager limit strategies"
+end
+
 shared_examples_for "filtering/excluding by associations" do
   specify "should work correctly when filtering by associations" do
     @album.update(:artist => @artist)
@@ -229,7 +250,7 @@ shared_examples_for "filtering/excluding by associations" do
     @album.add_tag(@tag)
     @Album.filter(:tags=>@tag).all.should == [@album]
     @Album.filter(:alias_tags=>@tag).all.should == [@album]
-    album, artist, tag = @pr.call
+    album, tag = @pr.call.values_at(0, 2)
     @Album.exclude(:tags=>@tag).all.should == [album]
     @Album.exclude(:alias_tags=>@tag).all.should == [album]
     @Album.exclude(:tags=>tag).all.sort_by{|x| x.pk}.should == [@album, album]
@@ -244,22 +265,22 @@ shared_examples_for "filtering/excluding by associations" do
     album.update(:artist => artist)
 
     @Artist.filter(:albums=>@Album).all.sort_by{|x| x.pk}.should == [@artist, artist]
-    @Artist.filter(:albums=>@Album.filter(Array(Album.primary_key).map{|k| k.qualify(Album.table_name)}.zip(Array(album.pk)))).all.sort_by{|x| x.pk}.should == [artist]
+    @Artist.filter(:albums=>@Album.filter(Array(Album.primary_key).map{|k| Sequel.qualify(Album.table_name, k)}.zip(Array(album.pk)))).all.sort_by{|x| x.pk}.should == [artist]
     @Artist.filter(:albums=>@Album.filter(1=>0)).all.sort_by{|x| x.pk}.should == []
     @Artist.filter(:first_album=>@Album).all.sort_by{|x| x.pk}.should == [@artist, artist]
-    @Artist.filter(:first_album=>@Album.filter(Array(Album.primary_key).map{|k| k.qualify(Album.table_name)}.zip(Array(album.pk)))).all.sort_by{|x| x.pk}.should == [artist]
+    @Artist.filter(:first_album=>@Album.filter(Array(Album.primary_key).map{|k| Sequel.qualify(Album.table_name, k)}.zip(Array(album.pk)))).all.sort_by{|x| x.pk}.should == [artist]
     @Artist.filter(:first_album=>@Album.filter(1=>0)).all.sort_by{|x| x.pk}.should == []
     @Album.filter(:artist=>@Artist).all.sort_by{|x| x.pk}.should == [@album, album]
-    @Album.filter(:artist=>@Artist.filter(Array(Artist.primary_key).map{|k| k.qualify(Artist.table_name)}.zip(Array(artist.pk)))).all.sort_by{|x| x.pk}.should == [album]
+    @Album.filter(:artist=>@Artist.filter(Array(Artist.primary_key).map{|k| Sequel.qualify(Artist.table_name, k)}.zip(Array(artist.pk)))).all.sort_by{|x| x.pk}.should == [album]
     @Album.filter(:artist=>@Artist.filter(1=>0)).all.sort_by{|x| x.pk}.should == []
     @Album.filter(:tags=>@Tag).all.sort_by{|x| x.pk}.should == [@album, album]
-    @Album.filter(:tags=>@Tag.filter(Array(Tag.primary_key).map{|k| k.qualify(Tag.table_name)}.zip(Array(tag.pk)))).all.sort_by{|x| x.pk}.should == [album]
+    @Album.filter(:tags=>@Tag.filter(Array(Tag.primary_key).map{|k| Sequel.qualify(Tag.table_name, k)}.zip(Array(tag.pk)))).all.sort_by{|x| x.pk}.should == [album]
     @Album.filter(:tags=>@Tag.filter(1=>0)).all.sort_by{|x| x.pk}.should == []
     @Album.filter(:alias_tags=>@Tag).all.sort_by{|x| x.pk}.should == [@album, album]
-    @Album.filter(:alias_tags=>@Tag.filter(Array(Tag.primary_key).map{|k| k.qualify(Tag.table_name)}.zip(Array(tag.pk)))).all.sort_by{|x| x.pk}.should == [album]
+    @Album.filter(:alias_tags=>@Tag.filter(Array(Tag.primary_key).map{|k| Sequel.qualify(Tag.table_name, k)}.zip(Array(tag.pk)))).all.sort_by{|x| x.pk}.should == [album]
     @Album.filter(:alias_tags=>@Tag.filter(1=>0)).all.sort_by{|x| x.pk}.should == []
     @Tag.filter(:albums=>@Album).all.sort_by{|x| x.pk}.should == [@tag, tag]
-    @Tag.filter(:albums=>@Album.filter(Array(Album.primary_key).map{|k| k.qualify(Album.table_name)}.zip(Array(album.pk)))).all.sort_by{|x| x.pk}.should == [tag]
+    @Tag.filter(:albums=>@Album.filter(Array(Album.primary_key).map{|k| Sequel.qualify(Album.table_name, k)}.zip(Array(album.pk)))).all.sort_by{|x| x.pk}.should == [tag]
     @Tag.filter(:albums=>@Album.filter(1=>0)).all.sort_by{|x| x.pk}.should == []
   end
 
@@ -271,24 +292,24 @@ shared_examples_for "filtering/excluding by associations" do
     album.update(:artist => artist)
 
     @Artist.exclude(:albums=>@Album).all.sort_by{|x| x.pk}.should == []
-    @Artist.exclude(:albums=>@Album.filter(Array(Album.primary_key).map{|k| k.qualify(Album.table_name)}.zip(Array(album.pk)))).all.sort_by{|x| x.pk}.should == [@artist]
+    @Artist.exclude(:albums=>@Album.filter(Array(Album.primary_key).map{|k| Sequel.qualify(Album.table_name, k)}.zip(Array(album.pk)))).all.sort_by{|x| x.pk}.should == [@artist]
     @Artist.exclude(:albums=>@Album.filter(1=>0)).all.sort_by{|x| x.pk}.should == [@artist, artist]
     @Album.exclude(:artist=>@Artist).all.sort_by{|x| x.pk}.should == []
-    @Album.exclude(:artist=>@Artist.filter(Array(Artist.primary_key).map{|k| k.qualify(Artist.table_name)}.zip(Array(artist.pk)))).all.sort_by{|x| x.pk}.should == [@album]
+    @Album.exclude(:artist=>@Artist.filter(Array(Artist.primary_key).map{|k| Sequel.qualify(Artist.table_name, k)}.zip(Array(artist.pk)))).all.sort_by{|x| x.pk}.should == [@album]
     @Album.exclude(:artist=>@Artist.filter(1=>0)).all.sort_by{|x| x.pk}.should == [@album, album]
     @Album.exclude(:tags=>@Tag).all.sort_by{|x| x.pk}.should == []
-    @Album.exclude(:tags=>@Tag.filter(Array(Tag.primary_key).map{|k| k.qualify(Tag.table_name)}.zip(Array(tag.pk)))).all.sort_by{|x| x.pk}.should == [@album]
+    @Album.exclude(:tags=>@Tag.filter(Array(Tag.primary_key).map{|k| Sequel.qualify(Tag.table_name, k)}.zip(Array(tag.pk)))).all.sort_by{|x| x.pk}.should == [@album]
     @Album.exclude(:tags=>@Tag.filter(1=>0)).all.sort_by{|x| x.pk}.should == [@album, album]
     @Album.exclude(:alias_tags=>@Tag).all.sort_by{|x| x.pk}.should == []
-    @Album.exclude(:alias_tags=>@Tag.filter(Array(Tag.primary_key).map{|k| k.qualify(Tag.table_name)}.zip(Array(tag.pk)))).all.sort_by{|x| x.pk}.should == [@album]
+    @Album.exclude(:alias_tags=>@Tag.filter(Array(Tag.primary_key).map{|k| Sequel.qualify(Tag.table_name, k)}.zip(Array(tag.pk)))).all.sort_by{|x| x.pk}.should == [@album]
     @Album.exclude(:alias_tags=>@Tag.filter(1=>0)).all.sort_by{|x| x.pk}.should == [@album, album]
     @Tag.exclude(:albums=>@Album).all.sort_by{|x| x.pk}.should == []
-    @Tag.exclude(:albums=>@Album.filter(Array(Album.primary_key).map{|k| k.qualify(Album.table_name)}.zip(Array(album.pk)))).all.sort_by{|x| x.pk}.should == [@tag]
+    @Tag.exclude(:albums=>@Album.filter(Array(Album.primary_key).map{|k| Sequel.qualify(Album.table_name, k)}.zip(Array(album.pk)))).all.sort_by{|x| x.pk}.should == [@tag]
     @Tag.exclude(:albums=>@Album.filter(1=>0)).all.sort_by{|x| x.pk}.should == [@tag, tag]
   end
 end
 
-shared_examples_for "regular and composite key associations" do  
+shared_examples_for "basic regular and composite key associations" do  
   specify "should return no objects if none are associated" do
     @album.artist.should == nil
     @artist.first_album.should == nil
@@ -339,7 +360,7 @@ shared_examples_for "regular and composite key associations" do
     Album.tags.all.should == []
     Album.alias_tags.all.should == []
     Artist.albums.all.should == []
-    Artist.tags.all.should == []
+    Artist.tags.all.should == [] unless @no_many_through_many
     Artist.albums.tags.all.should == []
 
     @album.update(:artist => @artist)
@@ -350,7 +371,7 @@ shared_examples_for "regular and composite key associations" do
     Album.tags.all.should == [@tag]
     Album.alias_tags.all.should == [@tag]
     Artist.albums.all.should == [@album]
-    Artist.tags.all.should == [@tag]
+    Artist.tags.all.should == [@tag] unless @no_many_through_many
     Artist.albums.tags.all.should == [@tag]
 
     album.add_tag(tag)
@@ -361,7 +382,7 @@ shared_examples_for "regular and composite key associations" do
     Album.tags.order(:name).all.should == [@tag, tag]
     Album.alias_tags.order(:name).all.should == [@tag, tag]
     Artist.albums.order(:name).all.should == [@album, album]
-    Artist.tags.order(:name).all.should == [@tag, tag]
+    Artist.tags.order(:name).all.should == [@tag, tag] unless @no_many_through_many
     Artist.albums.tags.order(:name).all.should == [@tag, tag]
 
     Tag.filter(Tag.qualified_primary_key_hash(tag.pk)).albums.all.should == [album]
@@ -369,7 +390,7 @@ shared_examples_for "regular and composite key associations" do
     Album.filter(Album.qualified_primary_key_hash(album.pk)).tags.all.should == [tag]
     Album.filter(Album.qualified_primary_key_hash(album.pk)).alias_tags.all.should == [tag]
     Artist.filter(Artist.qualified_primary_key_hash(artist.pk)).albums.all.should == [album]
-    Artist.filter(Artist.qualified_primary_key_hash(artist.pk)).tags.all.should == [tag]
+    Artist.filter(Artist.qualified_primary_key_hash(artist.pk)).tags.all.should == [tag] unless @no_many_through_many
     Artist.filter(Artist.qualified_primary_key_hash(artist.pk)).albums.tags.all.should == [tag]
 
     Artist.filter(Artist.qualified_primary_key_hash(artist.pk)).albums.filter(Album.qualified_primary_key_hash(album.pk)).tags.all.should == [tag]
@@ -408,8 +429,8 @@ shared_examples_for "regular and composite key associations" do
     @artist.add_album(@album)
     @album.add_tag(@tag)
     
-    @artist.remove_all_albums
     @album.remove_all_tags
+    @artist.remove_all_albums
     
     @album.reload
     @artist.reload
@@ -444,48 +465,6 @@ shared_examples_for "regular and composite key associations" do
     a.first.albums.first.artist.should == @artist
   end
   
-  describe "when filtering/excluding by associations" do
-    before do
-      @Artist = Artist.dataset
-      @Album = Album.dataset
-      @Tag = Tag.dataset
-    end
-
-    it_should_behave_like "filtering/excluding by associations"
-  end
-
-  describe "when filtering/excluding by associations when joining" do
-    def self_join(c)
-      c.join(c.table_name.as(:b), Array(c.primary_key).zip(Array(c.primary_key))).select_all(c.table_name)
-    end
-
-    before do
-      @Artist = self_join(Artist)
-      @Album = self_join(Album)
-      @Tag = self_join(Tag)
-    end
-
-    it_should_behave_like "filtering/excluding by associations"
-  end
-
-  describe "with no :eager_limit_strategy" do
-    it_should_behave_like "eager limit strategies"
-  end
-
-  describe "with :eager_limit_strategy=>true" do
-    before do
-      @els = {:eager_limit_strategy=>true}
-    end
-    it_should_behave_like "eager limit strategies"
-  end
-
-  describe "with :eager_limit_strategy=>:window_function" do
-    before do
-      @els = {:eager_limit_strategy=>:window_function}
-    end
-    it_should_behave_like "eager limit strategies"
-  end if INTEGRATION_DB.dataset.supports_window_functions?
-
   specify "should eager load via eager_graph correctly" do
     @album.update(:artist => @artist)
     @album.add_tag(@tag)
@@ -503,6 +482,55 @@ shared_examples_for "regular and composite key associations" do
     a.first.albums.first.artist.should == @artist
   end
   
+  describe "when filtering/excluding by associations" do
+    before do
+      @Artist = Artist.dataset
+      @Album = Album.dataset
+      @Tag = Tag.dataset
+    end
+
+    it_should_behave_like "filtering/excluding by associations"
+  end
+end
+
+shared_examples_for "regular and composite key associations" do  
+  it_should_behave_like "basic regular and composite key associations"
+
+  describe "when filtering/excluding by associations when joining" do
+    def self_join(c)
+      c.join(Sequel.as(c.table_name, :b), Array(c.primary_key).zip(Array(c.primary_key))).select_all(c.table_name)
+    end
+
+    before do
+      @Artist = self_join(Artist)
+      @Album = self_join(Album)
+      @Tag = self_join(Tag)
+    end
+
+    it_should_behave_like "filtering/excluding by associations"
+  end
+
+  describe "with :eager_limit_strategy=>:ruby" do
+    before do
+      @els = {:eager_limit_strategy=>:ruby}
+    end
+    it_should_behave_like "eager limit strategies"
+  end
+
+  describe "with :eager_limit_strategy=>true" do
+    before do
+      @els = {:eager_limit_strategy=>true}
+    end
+    it_should_behave_like "one_to_one eager limit strategies"
+  end if DB.dataset.supports_ordered_distinct_on?
+
+  describe "with :eager_limit_strategy=>:window_function" do
+    before do
+      @els = {:eager_limit_strategy=>:window_function}
+    end
+    it_should_behave_like "eager limit strategies"
+  end if DB.dataset.supports_window_functions?
+
   specify "should work with a many_through_many association" do
     @album.update(:artist => @artist)
     @album.add_tag(@tag)
@@ -535,7 +563,7 @@ end
 
 describe "Sequel::Model Simple Associations" do
   before(:all) do
-    @db = INTEGRATION_DB
+    @db = DB
     @db.drop_table?(:albums_tags, :tags, :albums, :artists)
     @db.create_table(:artists) do
       primary_key :id
@@ -561,24 +589,28 @@ describe "Sequel::Model Simple Associations" do
       plugin :dataset_associations
       one_to_many :albums, :order=>:name
       one_to_one :first_album, :class=>:Album, :order=>:name
-      one_to_one :last_album, :class=>:Album, :order=>:name.desc
+      one_to_one :second_album, :class=>:Album, :order=>:name, :limit=>[nil, 1]
+      one_to_one :last_album, :class=>:Album, :order=>Sequel.desc(:name)
       one_to_many :first_two_albums, :class=>:Album, :order=>:name, :limit=>2
       one_to_many :second_two_albums, :class=>:Album, :order=>:name, :limit=>[2, 1]
-      one_to_many :last_two_albums, :class=>:Album, :order=>:name.desc, :limit=>2
+      one_to_many :not_first_albums, :class=>:Album, :order=>:name, :limit=>[nil, 1]
+      one_to_many :last_two_albums, :class=>:Album, :order=>Sequel.desc(:name), :limit=>2
       plugin :many_through_many
       many_through_many :tags, [[:albums, :artist_id, :id], [:albums_tags, :album_id, :tag_id]]
       many_through_many :first_two_tags, :clone=>:tags, :order=>:tags__name, :limit=>2
       many_through_many :second_two_tags, :clone=>:tags, :order=>:tags__name, :limit=>[2, 1]
-      many_through_many :last_two_tags, :clone=>:tags, :order=>:tags__name.desc, :limit=>2
+      many_through_many :not_first_tags, :clone=>:tags, :order=>:tags__name, :limit=>[nil, 1]
+      many_through_many :last_two_tags, :clone=>:tags, :order=>Sequel.desc(:tags__name), :limit=>2
     end
     class ::Album < Sequel::Model(@db)
       plugin :dataset_associations
-      many_to_one :artist
+      many_to_one :artist, :reciprocal=>nil
       many_to_many :tags, :right_key=>:tag_id
       many_to_many :alias_tags, :clone=>:tags, :join_table=>:albums_tags___at
       many_to_many :first_two_tags, :clone=>:tags, :order=>:name, :limit=>2
       many_to_many :second_two_tags, :clone=>:tags, :order=>:name, :limit=>[2, 1]
-      many_to_many :last_two_tags, :clone=>:tags, :order=>:name.desc, :limit=>2
+      many_to_many :not_first_tags, :clone=>:tags, :order=>:name, :limit=>[nil, 1]
+      many_to_many :last_two_tags, :clone=>:tags, :order=>Sequel.desc(:name), :limit=>2
     end
     class ::Tag < Sequel::Model(@db)
       plugin :dataset_associations
@@ -603,13 +635,6 @@ describe "Sequel::Model Simple Associations" do
   
   it_should_behave_like "regular and composite key associations"
 
-  describe "with :eager_limit_strategy=>:correlated_subquery" do
-    before do
-      @els = {:eager_limit_strategy=>:correlated_subquery}
-    end
-    it_should_behave_like "eager limit strategies"
-  end unless Sequel.guarded?(:mysql, :db2, :oracle, :h2)
-
   specify "should handle many_to_one associations with same name as :key" do
     Album.def_column_alias(:artist_id_id, :artist_id)
     Album.many_to_one :artist_id, :key_column =>:artist_id, :class=>Artist
@@ -632,7 +657,7 @@ describe "Sequel::Model Simple Associations" do
     Artist.set_dataset(:artists___ar)
     Album.set_dataset(:albums___a)
     Tag.set_dataset(:tags___t)
-    Artist.one_to_many :balbums, :class=>Album, :key=>:artist_id
+    Artist.one_to_many :balbums, :class=>Album, :key=>:artist_id, :reciprocal=>nil
     Album.many_to_many :btags, :class=>Tag, :join_table=>:albums_tags, :right_key=>:tag_id
     Album.many_to_one :bartist, :class=>Artist, :key=>:artist_id
     Tag.many_to_many :balbums, :class=>Album, :join_table=>:albums_tags, :right_key=>:album_id
@@ -650,13 +675,13 @@ describe "Sequel::Model Simple Associations" do
   
   specify "should have add method accept hashes and create new records" do
     @artist.remove_all_albums
-    Album.delete
+    Album.dataset.delete
     @album = @artist.add_album(:name=>'Al2')
     Album.first[:name].should == 'Al2'
     @artist.albums_dataset.first[:name].should == 'Al2'
     
     @album.remove_all_tags
-    Tag.delete
+    Tag.dataset.delete
     @album.add_tag(:name=>'T2')
     Tag.first[:name].should == 'T2'
     @album.tags_dataset.first[:name].should == 'T2'
@@ -730,7 +755,7 @@ end
 
 describe "Sequel::Model Composite Key Associations" do
   before(:all) do
-    @db = INTEGRATION_DB
+    @db = DB
     @db.drop_table?(:albums_tags, :tags, :albums, :artists)
     @db.create_table(:artists) do
       Integer :id1
@@ -766,34 +791,38 @@ describe "Sequel::Model Composite Key Associations" do
     [:albums_tags, :tags, :albums, :artists].each{|t| @db[t].delete}
     class ::Artist < Sequel::Model(@db)
       plugin :dataset_associations
-      set_primary_key :id1, :id2
+      set_primary_key [:id1, :id2]
       unrestrict_primary_key
       one_to_many :albums, :key=>[:artist_id1, :artist_id2], :order=>:name
       one_to_one :first_album, :clone=>:albums, :order=>:name
-      one_to_one :last_album, :clone=>:albums, :order=>:name.desc
+      one_to_one :last_album, :clone=>:albums, :order=>Sequel.desc(:name)
+      one_to_one :second_album, :clone=>:albums, :limit=>[nil, 1]
       one_to_many :first_two_albums, :clone=>:albums, :order=>:name, :limit=>2
       one_to_many :second_two_albums, :clone=>:albums, :order=>:name, :limit=>[2, 1]
-      one_to_many :last_two_albums, :clone=>:albums, :order=>:name.desc, :limit=>2
+      one_to_many :not_first_albums, :clone=>:albums, :order=>:name, :limit=>[nil, 1]
+      one_to_many :last_two_albums, :clone=>:albums, :order=>Sequel.desc(:name), :limit=>2
       plugin :many_through_many
       many_through_many :tags, [[:albums, [:artist_id1, :artist_id2], [:id1, :id2]], [:albums_tags, [:album_id1, :album_id2], [:tag_id1, :tag_id2]]]
       many_through_many :first_two_tags, :clone=>:tags, :order=>:tags__name, :limit=>2
       many_through_many :second_two_tags, :clone=>:tags, :order=>:tags__name, :limit=>[2, 1]
-      many_through_many :last_two_tags, :clone=>:tags, :order=>:tags__name.desc, :limit=>2
+      many_through_many :not_first_tags, :clone=>:tags, :order=>:tags__name, :limit=>[nil, 1]
+      many_through_many :last_two_tags, :clone=>:tags, :order=>Sequel.desc(:tags__name), :limit=>2
     end
     class ::Album < Sequel::Model(@db)
       plugin :dataset_associations
-      set_primary_key :id1, :id2
+      set_primary_key [:id1, :id2]
       unrestrict_primary_key
-      many_to_one :artist, :key=>[:artist_id1, :artist_id2]
+      many_to_one :artist, :key=>[:artist_id1, :artist_id2], :reciprocal=>nil
       many_to_many :tags, :left_key=>[:album_id1, :album_id2], :right_key=>[:tag_id1, :tag_id2]
       many_to_many :alias_tags, :clone=>:tags, :join_table=>:albums_tags___at
       many_to_many :first_two_tags, :clone=>:tags, :order=>:name, :limit=>2
       many_to_many :second_two_tags, :clone=>:tags, :order=>:name, :limit=>[2, 1]
-      many_to_many :last_two_tags, :clone=>:tags, :order=>:name.desc, :limit=>2
+      many_to_many :not_first_tags, :clone=>:tags, :order=>:name, :limit=>[nil, 1]
+      many_to_many :last_two_tags, :clone=>:tags, :order=>Sequel.desc(:name), :limit=>2
     end
     class ::Tag < Sequel::Model(@db)
       plugin :dataset_associations
-      set_primary_key :id1, :id2
+      set_primary_key [:id1, :id2]
       unrestrict_primary_key
       many_to_many :albums, :right_key=>[:album_id1, :album_id2], :left_key=>[:tag_id1, :tag_id2]
     end
@@ -816,22 +845,15 @@ describe "Sequel::Model Composite Key Associations" do
 
   it_should_behave_like "regular and composite key associations"
 
-  describe "with :eager_limit_strategy=>:correlated_subquery" do
-    before do
-      @els = {:eager_limit_strategy=>:correlated_subquery}
-    end
-    it_should_behave_like "eager limit strategies"
-  end if INTEGRATION_DB.dataset.supports_multiple_column_in? && !Sequel.guarded?(:mysql, :db2, :oracle)
-
   specify "should have add method accept hashes and create new records" do
     @artist.remove_all_albums
-    Album.delete
+    Album.dataset.delete
     @artist.add_album(:id1=>1, :id2=>2, :name=>'Al2')
     Album.first[:name].should == 'Al2'
     @artist.albums_dataset.first[:name].should == 'Al2'
     
     @album.remove_all_tags
-    Tag.delete
+    Tag.dataset.delete
     @album.add_tag(:id1=>1, :id2=>2, :name=>'T2')
     Tag.first[:name].should == 'T2'
     @album.tags_dataset.first[:name].should == 'T2'
@@ -863,9 +885,158 @@ describe "Sequel::Model Composite Key Associations" do
   end
 end
 
+describe "Sequel::Model pg_array_to_many" do
+  before(:all) do
+    @db = DB
+    @db.extension :pg_array
+    Sequel.extension :pg_array_ops
+    @db.drop_table?(:tags, :albums, :artists)
+    @db.create_table(:artists) do
+      primary_key :id
+      String :name
+    end
+    @db.create_table(:albums) do
+      primary_key :id
+      String :name
+      foreign_key :artist_id, :artists
+      column :tag_ids, 'int4[]'
+    end
+    @db.create_table(:tags) do
+      primary_key :id
+      String :name
+    end
+  end
+  before do
+    [:tags, :albums, :artists].each{|t| @db[t].delete}
+    class ::Artist < Sequel::Model(@db)
+      plugin :dataset_associations
+      one_to_many :albums, :order=>:name
+      one_to_one :first_album, :class=>:Album, :order=>:name
+    end
+    class ::Album < Sequel::Model(@db)
+      plugin :dataset_associations
+      plugin :pg_array_associations
+      many_to_one :artist, :reciprocal=>nil
+      pg_array_to_many :tags, :key=>:tag_ids, :save_after_modify=>true
+      pg_array_to_many :alias_tags, :clone=>:tags
+      pg_array_to_many :first_two_tags, :clone=>:tags, :order=>:name, :limit=>2
+      pg_array_to_many :second_two_tags, :clone=>:tags, :order=>:name, :limit=>[2, 1]
+      pg_array_to_many :not_first_tags, :clone=>:tags, :order=>:name, :limit=>[nil, 1]
+      pg_array_to_many :last_two_tags, :clone=>:tags, :order=>Sequel.desc(:name), :limit=>2
+    end
+    class ::Tag < Sequel::Model(@db)
+      plugin :dataset_associations
+      plugin :pg_array_associations
+      many_to_pg_array :albums
+    end
+    @album = Album.create(:name=>'Al')
+    @artist = Artist.create(:name=>'Ar')
+    @tag = Tag.create(:name=>'T')
+    @many_to_many_method = :pg_array_to_many
+    @no_many_through_many = true
+    @same_album = lambda{Album.create(:name=>'Al', :artist_id=>@artist.id)}
+    @diff_album = lambda{Album.create(:name=>'lA', :artist_id=>@artist.id)}
+    @middle_album = lambda{Album.create(:name=>'Bl', :artist_id=>@artist.id)}
+    @other_tags = lambda{t = [Tag.create(:name=>'U'), Tag.create(:name=>'V')]; Tag.all{|x| @album.add_tag(x)}; t}
+    @pr = lambda{[Album.create(:name=>'Al2'),Artist.create(:name=>'Ar2'),Tag.create(:name=>'T2')]}
+    @ins = lambda{}
+  end
+  after do
+    [:Tag, :Album, :Artist].each{|x| Object.send(:remove_const, x)}
+  end
+  after(:all) do
+    @db.drop_table?(:tags, :albums, :artists)
+  end
+  
+  it_should_behave_like "basic regular and composite key associations"
+  it_should_behave_like "many_to_many eager limit strategies"
+
+  it "should handle adding and removing entries in array" do
+    a = Album.create
+    a.typecast_on_assignment = false
+    a.add_tag(@tag)
+    a.remove_tag(@tag)
+    a.save
+  end
+end if DB.database_type == :postgres && DB.adapter_scheme == :postgres && DB.server_version >= 90300
+
+describe "Sequel::Model many_to_pg_array" do
+  before(:all) do
+    @db = DB
+    @db.extension :pg_array
+    Sequel.extension :pg_array_ops
+    @db.drop_table?(:tags, :albums, :artists)
+    @db.create_table(:artists) do
+      primary_key :id
+      String :name
+    end
+    @db.create_table(:albums) do
+      primary_key :id
+      String :name
+      foreign_key :artist_id, :artists
+    end
+    @db.create_table(:tags) do
+      primary_key :id
+      String :name
+      column :album_ids, 'int4[]'
+    end
+  end
+  before do
+    [:tags, :albums, :artists].each{|t| @db[t].delete}
+    class ::Artist < Sequel::Model(@db)
+      plugin :dataset_associations
+      one_to_many :albums, :order=>:name
+      one_to_one :first_album, :class=>:Album, :order=>:name
+    end
+    class ::Album < Sequel::Model(@db)
+      plugin :dataset_associations
+      plugin :pg_array_associations
+      many_to_one :artist, :reciprocal=>nil
+      many_to_pg_array :tags
+      many_to_pg_array :alias_tags, :clone=>:tags
+      many_to_pg_array :first_two_tags, :clone=>:tags, :order=>:name, :limit=>2
+      many_to_pg_array :second_two_tags, :clone=>:tags, :order=>:name, :limit=>[2, 1]
+      many_to_pg_array :not_first_tags, :clone=>:tags, :order=>:name, :limit=>[nil, 1]
+      many_to_pg_array :last_two_tags, :clone=>:tags, :order=>Sequel.desc(:name), :limit=>2
+    end
+    class ::Tag < Sequel::Model(@db)
+      plugin :dataset_associations
+      plugin :pg_array_associations
+      pg_array_to_many :albums
+    end
+    @album = Album.create(:name=>'Al')
+    @artist = Artist.create(:name=>'Ar')
+    @tag = Tag.create(:name=>'T')
+    @many_to_many_method = :pg_array_to_many
+    @no_many_through_many = true
+    @same_album = lambda{Album.create(:name=>'Al', :artist_id=>@artist.id)}
+    @diff_album = lambda{Album.create(:name=>'lA', :artist_id=>@artist.id)}
+    @middle_album = lambda{Album.create(:name=>'Bl', :artist_id=>@artist.id)}
+    @other_tags = lambda{t = [Tag.create(:name=>'U'), Tag.create(:name=>'V')]; Tag.all{|x| @album.add_tag(x)}; @tag.refresh; t.each{|x| x.refresh}; t}
+    @pr = lambda{[Album.create(:name=>'Al2'),Artist.create(:name=>'Ar2'),Tag.create(:name=>'T2')]}
+    @ins = lambda{}
+  end
+  after do
+    [:Tag, :Album, :Artist].each{|x| Object.send(:remove_const, x)}
+  end
+  after(:all) do
+    @db.drop_table?(:tags, :albums, :artists)
+  end
+  
+  it_should_behave_like "basic regular and composite key associations"
+  it_should_behave_like "many_to_many eager limit strategies"
+
+  it "should handle adding and removing entries in array" do
+    a = Album.create
+    @tag.typecast_on_assignment = false
+    a.add_tag(@tag)
+    a.remove_tag(@tag)
+  end
+end if DB.database_type == :postgres && DB.adapter_scheme == :postgres && DB.server_version >= 90300
+
 describe "Sequel::Model Associations with clashing column names" do
   before(:all) do
-    @db = INTEGRATION_DB
+    @db = DB
     @db.drop_table?(:bars_foos, :bars, :foos)
     @db.create_table(:foos) do
       primary_key :id
