@@ -2,8 +2,8 @@ require File.join(File.dirname(File.expand_path(__FILE__)), 'spec_helper.rb')
 
 describe "Eagerly loading a tree structure" do
   before(:all) do
-    INTEGRATION_DB.instance_variable_set(:@schemas, {})
-    INTEGRATION_DB.create_table!(:nodes) do
+    DB.instance_variable_set(:@schemas, {})
+    DB.create_table!(:nodes) do
       primary_key :id
       foreign_key :parent_id, :nodes
     end
@@ -12,7 +12,7 @@ describe "Eagerly loading a tree structure" do
       one_to_many :children, :key=>:parent_id
     
       # Only useful when eager loading
-      many_to_one :ancestors, :eager_loader=>(proc do |eo|
+      many_to_one :ancestors, :eager_loader_key=>nil, :eager_loader=>(proc do |eo|
         # Handle cases where the root node has the same parent_id as primary_key
         # and also when it is NULL
         non_root_nodes = eo[:rows].reject do |n| 
@@ -36,7 +36,7 @@ describe "Eagerly loading a tree structure" do
           end
         end
       end)
-      many_to_one :descendants, :eager_loader=>(proc do |eo|
+      many_to_one :descendants, :eager_loader_key=>nil, :eager_loader=>(proc do |eo|
         id_map = {}
         eo[:rows].each do |n|
           # Initialize an empty array of child associations for each parent node
@@ -68,7 +68,7 @@ describe "Eagerly loading a tree structure" do
     Node.insert(:parent_id=>6)
   end
   after(:all) do
-    INTEGRATION_DB.drop_table :nodes
+    DB.drop_table :nodes
     Object.send(:remove_const, :Node)
   end
 
@@ -128,14 +128,14 @@ describe "Association Extensions" do
         first(:name=>name) || model.create(:name=>name, :author_id=>model_object.pk)
       end
     end
-    INTEGRATION_DB.instance_variable_set(:@schemas, {})
-    INTEGRATION_DB.create_table!(:authors) do
+    DB.instance_variable_set(:@schemas, {})
+    DB.create_table!(:authors) do
       primary_key :id
     end
     class ::Author < Sequel::Model
       one_to_many :authorships, :extend=>FindOrCreate
     end
-    INTEGRATION_DB.create_table!(:authorships) do
+    DB.create_table!(:authorships) do
       primary_key :id
       foreign_key :author_id, :authors
       String :name
@@ -146,7 +146,7 @@ describe "Association Extensions" do
     @author = Author.create
   end
   after do
-    INTEGRATION_DB.drop_table :authorships, :authors
+    DB.drop_table :authorships, :authors
     Object.send(:remove_const, :Author)
     Object.send(:remove_const, :Authorship)
   end
@@ -171,8 +171,8 @@ end
 
 describe "has_many :through has_many and has_one :through belongs_to" do
   before(:all) do
-    INTEGRATION_DB.instance_variable_set(:@schemas, {})
-    INTEGRATION_DB.create_table!(:firms) do
+    DB.instance_variable_set(:@schemas, {})
+    DB.create_table!(:firms) do
       primary_key :id
     end
     class ::Firm < Sequel::Model
@@ -185,7 +185,7 @@ describe "has_many :through has_many and has_one :through belongs_to" do
           end
         end), \
         :eager_loader=>(proc do |eo|
-          id_map = eo[:key_hash][Firm.primary_key]
+          id_map = eo[:id_map]
           eo[:rows].each{|firm| firm.associations[:invoices] = []}
           Invoice.eager_graph(:client).filter(:client__firm_id=>id_map.keys).all do |inv|
             id_map[inv.client.firm_id].each do |firm|
@@ -195,7 +195,7 @@ describe "has_many :through has_many and has_one :through belongs_to" do
         end)
     end
 
-    INTEGRATION_DB.create_table!(:clients) do
+    DB.create_table!(:clients) do
       primary_key :id
       foreign_key :firm_id, :firms
     end
@@ -204,7 +204,7 @@ describe "has_many :through has_many and has_one :through belongs_to" do
       one_to_many :invoices
     end
 
-    INTEGRATION_DB.create_table!(:invoices) do
+    DB.create_table!(:invoices) do
       primary_key :id
       foreign_key :client_id, :clients
     end
@@ -251,7 +251,7 @@ describe "has_many :through has_many and has_one :through belongs_to" do
     @invoice5 = Invoice.create(:client => @client3)
   end
   after(:all) do
-    INTEGRATION_DB.drop_table :invoices, :clients, :firms
+    DB.drop_table :invoices, :clients, :firms
     Object.send(:remove_const, :Firm)
     Object.send(:remove_const, :Client)
     Object.send(:remove_const, :Invoice)
@@ -322,97 +322,62 @@ end
 
 describe "Polymorphic Associations" do
   before(:all) do
-    INTEGRATION_DB.instance_variable_set(:@schemas, {})
-    INTEGRATION_DB.create_table!(:assets) do
+    DB.instance_variable_set(:@schemas, {})
+    DB.create_table!(:assets) do
       primary_key :id
       Integer :attachable_id
       String :attachable_type
     end
     class ::Asset < Sequel::Model
       m = method(:constantize)
-      many_to_one :attachable, :reciprocal=>:assets, \
+      many_to_one :attachable, :reciprocal=>:assets,
+        :setter=>(proc do |attachable|
+          self[:attachable_id] = (attachable.pk if attachable)
+          self[:attachable_type] = (attachable.class.name if attachable)
+        end),
         :dataset=>(proc do
           klass = m.call(attachable_type)
-          klass.filter(klass.primary_key=>attachable_id)
-        end), \
+          klass.where(klass.primary_key=>attachable_id)
+        end),
         :eager_loader=>(proc do |eo|
           id_map = {}
           eo[:rows].each do |asset|
             asset.associations[:attachable] = nil 
             ((id_map[asset.attachable_type] ||= {})[asset.attachable_id] ||= []) << asset
           end 
-          id_map.each do |klass_name, id_map|
+          id_map.each do |klass_name, idmap|
             klass = m.call(klass_name)
-            klass.filter(klass.primary_key=>id_map.keys).all do |attach|
-              id_map[attach.pk].each do |asset|
+            klass.where(klass.primary_key=>idmap.keys).all do |attach|
+              idmap[attach.pk].each do |asset|
                 asset.associations[:attachable] = attach
               end 
             end 
           end 
         end)
-            
-      private
-
-      def _attachable=(attachable)
-        self[:attachable_id] = (attachable.pk if attachable)
-        self[:attachable_type] = (attachable.class.name if attachable)
-      end 
     end 
   
-    INTEGRATION_DB.create_table!(:posts) do
+    DB.create_table!(:posts) do
       primary_key :id
     end
     class ::Post < Sequel::Model
-      one_to_many :assets, :key=>:attachable_id, :reciprocal=>:attachable do |ds|
-        ds.filter(:attachable_type=>'Post')
-      end 
-      
-      private
-
-      def _add_asset(asset)
-        asset.attachable_id = pk
-        asset.attachable_type = 'Post'
-        asset.save
-      end
-      def _remove_asset(asset)
-        asset.attachable_id = nil
-        asset.attachable_type = nil
-        asset.save
-      end
-      def _remove_all_assets
-        Asset.filter(:attachable_id=>pk, :attachable_type=>'Post')\
-          .update(:attachable_id=>nil, :attachable_type=>nil)
-      end
+      one_to_many :assets, :key=>:attachable_id, :reciprocal=>:attachable, :conditions=>{:attachable_type=>'Post'},
+        :adder=>proc{|asset| asset.update(:attachable_id=>pk, :attachable_type=>'Post')},
+        :remover=>proc{|asset| asset.update(:attachable_id=>nil, :attachable_type=>nil)},
+        :clearer=>proc{assets_dataset.update(:attachable_id=>nil, :attachable_type=>nil)}
     end 
   
-    INTEGRATION_DB.create_table!(:notes) do
+    DB.create_table!(:notes) do
       primary_key :id
     end
     class ::Note < Sequel::Model
-      one_to_many :assets, :key=>:attachable_id, :reciprocal=>:attachable do |ds|
-        ds.filter(:attachable_type=>'Note')
-      end 
-      
-      private
-
-      def _add_asset(asset)
-        asset.attachable_id = pk
-        asset.attachable_type = 'Note'
-        asset.save
-      end
-      def _remove_asset(asset)
-        asset.attachable_id = nil
-        asset.attachable_type = nil
-        asset.save
-      end
-      def _remove_all_assets
-        Asset.filter(:attachable_id=>pk, :attachable_type=>'Note')\
-          .update(:attachable_id=>nil, :attachable_type=>nil)
-      end
+      one_to_many :assets, :key=>:attachable_id, :reciprocal=>:attachable, :conditions=>{:attachable_type=>'Note'},     
+        :adder=>proc{|asset| asset.update(:attachable_id=>pk, :attachable_type=>'Note')},
+        :remover=>proc{|asset| asset.update(:attachable_id=>nil, :attachable_type=>nil)},
+        :clearer=>proc{assets_dataset.update(:attachable_id=>nil, :attachable_type=>nil)}
     end
   end
   before do
-    [:assets, :posts, :notes].each{|t| INTEGRATION_DB[t].delete}
+    [:assets, :posts, :notes].each{|t| DB[t].delete}
     @post = Post.create
     Note.create
     @note = Note.create
@@ -422,7 +387,7 @@ describe "Polymorphic Associations" do
     @asset2.associations.clear
   end
   after(:all) do
-    INTEGRATION_DB.drop_table :assets, :posts, :notes
+    DB.drop_table :assets, :posts, :notes
     Object.send(:remove_const, :Asset)
     Object.send(:remove_const, :Post)
     Object.send(:remove_const, :Note)
@@ -483,14 +448,23 @@ end
 
 describe "many_to_one/one_to_many not referencing primary key" do
   before(:all) do
-    INTEGRATION_DB.instance_variable_set(:@schemas, {})
-    INTEGRATION_DB.create_table!(:clients) do
+    DB.instance_variable_set(:@schemas, {})
+    DB.create_table!(:clients) do
       primary_key :id
       String :name
     end
     class ::Client < Sequel::Model
-      one_to_many :invoices, :reciprocal=>:client, \
-        :dataset=>proc{Invoice.filter(:client_name=>name)}, \
+      one_to_many :invoices, :reciprocal=>:client,
+        :adder=>(proc do |invoice|
+          invoice.client_name = name
+          invoice.save
+        end),
+        :remover=>(proc do |invoice|
+          invoice.client_name = nil
+          invoice.save
+        end),
+        :clearer=>proc{invoices_dataset.update(:client_name=>nil)},
+        :dataset=>proc{Invoice.filter(:client_name=>name)},
         :eager_loader=>(proc do |eo|
           id_map = {}
           eo[:rows].each do |client|
@@ -502,54 +476,35 @@ describe "many_to_one/one_to_many not referencing primary key" do
             client.associations[:invoices] << inv 
           end 
         end)
-
-      private
-
-      def _add_invoice(invoice)
-        invoice.client_name = name
-        invoice.save
-      end
-      def _remove_invoice(invoice)
-        invoice.client_name = nil
-        invoice.save
-      end
-      def _remove_all_invoices
-        Invoice.filter(:client_name=>name).update(:client_name=>nil)
-      end
     end 
   
-    INTEGRATION_DB.create_table!(:invoices) do
+    DB.create_table!(:invoices) do
       primary_key :id
       String :client_name
     end
     class ::Invoice < Sequel::Model
-      many_to_one :client, :key=>:client_name, \
-        :dataset=>proc{Client.filter(:name=>client_name)}, \
+      many_to_one :client, :key=>:client_name,
+        :setter=>proc{|client| self.client_name = (client.name if client)},
+        :dataset=>proc{Client.filter(:name=>client_name)},
         :eager_loader=>(proc do |eo|
-          id_map = eo[:key_hash][:client_name]
+          id_map = eo[:id_map]
           eo[:rows].each{|inv| inv.associations[:client] = nil}
           Client.filter(:name=>id_map.keys).all do |client|
             id_map[client.name].each{|inv| inv.associations[:client] = client}
           end 
         end)
-      
-      private
-
-      def _client=(client)
-        self.client_name = (client.name if client)
-      end
     end
   end
   before do
-    Client.delete
-    Invoice.delete
+    Client.dataset.delete
+    Invoice.dataset.delete
     @client1 = Client.create(:name=>'X')
     @client2 = Client.create(:name=>'Y')
     @invoice1 = Invoice.create(:client_name=>'X')
     @invoice2 = Invoice.create(:client_name=>'X')
   end
   after(:all) do
-    INTEGRATION_DB.drop_table :invoices, :clients
+    DB.drop_table :invoices, :clients
     Object.send(:remove_const, :Client)
     Object.send(:remove_const, :Invoice)
   end
@@ -610,7 +565,7 @@ describe "many_to_one/one_to_many not referencing primary key" do
   end
 
   it "should remove all associated objects correctly" do
-    invs = @client1.remove_all_invoices
+    @client1.remove_all_invoices
     @invoice1.refresh.client.should == nil
     @invoice1.client_name.should == nil
     @invoice2.refresh.client.should == nil
@@ -620,7 +575,7 @@ end
 
 describe "statistics associations" do
   before(:all) do
-    INTEGRATION_DB.create_table!(:projects) do
+    DB.create_table!(:projects) do
       primary_key :id
       String :name
     end
@@ -629,11 +584,11 @@ describe "statistics associations" do
        :dataset=>proc{Ticket.filter(:project_id=>id).select{sum(hours).as(hours)}},
        :eager_loader=>(proc do |eo|
         eo[:rows].each{|p| p.associations[:ticket_hours] = nil}
-        Ticket.filter(:project_id=>eo[:key_hash][:id].keys).
-         group(:project_id).
-         select{[project_id.as(project_id), sum(hours).as(hours)]}.
+        Ticket.filter(:project_id=>eo[:id_map].keys).
+         select_group(:project_id).
+         select_append{sum(hours).as(hours)}.
          all do |t|
-          p = eo[:key_hash][:id][t.values.delete(:project_id)].first
+          p = eo[:id_map][t.values.delete(:project_id)].first
           p.associations[:ticket_hours] = t
          end
        end)  
@@ -644,7 +599,7 @@ describe "statistics associations" do
       end 
     end 
 
-    INTEGRATION_DB.create_table!(:tickets) do
+    DB.create_table!(:tickets) do
       primary_key :id
       foreign_key :project_id, :projects
       Integer :hours
@@ -661,7 +616,7 @@ describe "statistics associations" do
     @ticket4 = Ticket.create(:project=>@project2, :hours=>20)
   end
   after(:all) do
-    INTEGRATION_DB.drop_table :tickets, :projects
+    DB.drop_table :tickets, :projects
     Object.send(:remove_const, :Project)
     Object.send(:remove_const, :Ticket)
   end
@@ -680,21 +635,21 @@ end
 
 describe "one to one associations" do
   before(:all) do
-    INTEGRATION_DB.create_table!(:books) do
+    DB.create_table!(:books) do
       primary_key :id
     end
     class ::Book < Sequel::Model
-      one_to_one :first_page, :class=>:Page, :conditions=>{:page_number=>1}
-      one_to_one :second_page, :class=>:Page, :conditions=>{:page_number=>2}
+      one_to_one :first_page, :class=>:Page, :conditions=>{:page_number=>1}, :reciprocal=>nil
+      one_to_one :second_page, :class=>:Page, :conditions=>{:page_number=>2}, :reciprocal=>nil
     end
 
-    INTEGRATION_DB.create_table!(:pages) do
+    DB.create_table!(:pages) do
       primary_key :id
       foreign_key :book_id, :books
       Integer :page_number
     end
     class ::Page < Sequel::Model
-      many_to_one :book
+      many_to_one :book, :reciprocal=>nil
     end
 
     @book1 = Book.create
@@ -705,7 +660,7 @@ describe "one to one associations" do
     @page4 = Page.create(:book=>@book2, :page_number=>2)
   end
   after(:all) do
-    INTEGRATION_DB.drop_table :pages, :books
+    DB.drop_table :pages, :books
     Object.send(:remove_const, :Book)
     Object.send(:remove_const, :Page)
   end
