@@ -1,12 +1,11 @@
 require File.join(File.dirname(File.expand_path(__FILE__)), 'spec_helper.rb')
 
-# H2 and MSSQL don't support USING joins
 # DB2 does not seem to support USING joins in every version; it seems to be
 # valid expression in DB2 iSeries UDB though.
-unless Sequel.guarded?(:h2, :mssql, :db2)
+unless !DB.dataset.supports_join_using? || Sequel.guarded?(:db2)
 describe "Class Table Inheritance Plugin" do
   before(:all) do
-    @db = INTEGRATION_DB
+    @db = DB
     @db.instance_variable_set(:@schemas, {})
     @db.drop_table?(:staff, :executives, :managers, :employees)
     @db.create_table(:employees) do
@@ -149,7 +148,7 @@ end
 
 describe "Many Through Many Plugin" do
   before(:all) do
-    @db = INTEGRATION_DB
+    @db = DB
     @db.instance_variable_set(:@schemas, {})
     @db.drop_table?(:albums_artists, :albums, :artists)
     @db.create_table(:albums) do
@@ -199,7 +198,7 @@ describe "Many Through Many Plugin" do
   end
   
   def self_join(c)
-    c.join(c.table_name.as(:b), Array(c.primary_key).zip(Array(c.primary_key))).select_all(c.table_name)
+    c.join(Sequel.as(c.table_name, :b), Array(c.primary_key).zip(Array(c.primary_key))).select_all(c.table_name)
   end
 
   specify "should handle super simple case with 1 join table" do
@@ -397,7 +396,7 @@ end
 
 describe "Lazy Attributes plugin" do 
   before(:all) do
-    @db = INTEGRATION_DB
+    @db = DB
     @db.create_table!(:items) do
       primary_key :id
       String :name
@@ -430,20 +429,18 @@ describe "Lazy Attributes plugin" do
   
   specify "should load lazy attribute for all items returned when accessing any item if using identity map " do
     Item.create(:name=>'K', :num=>2)
-    Item.with_identity_map do
-      a = Item.order(:name).all
-      a.should == [Item.load(:id=>1, :name=>'J'), Item.load(:id=>2, :name=>'K')]
-      a.map{|x| x[:num]}.should == [nil, nil]
-      a.first.num.should == 1
-      a.map{|x| x[:num]}.should == [1, 2]
-      a.last.num.should == 2
-    end
+    a = Item.order(:name).all
+    a.should == [Item.load(:id=>1, :name=>'J'), Item.load(:id=>2, :name=>'K')]
+    a.map{|x| x[:num]}.should == [nil, nil]
+    a.first.num.should == 1
+    a.map{|x| x[:num]}.should == [1, 2]
+    a.last.num.should == 2
   end
 end
 
 describe "Tactical Eager Loading Plugin" do
   before(:all) do
-    @db = INTEGRATION_DB
+    @db = DB
     @db.instance_variable_set(:@schemas, {})
     @db.create_table!(:artists) do
       primary_key :id
@@ -496,102 +493,99 @@ describe "Tactical Eager Loading Plugin" do
   end
 end
 
-describe "Identity Map plugin" do 
-  before do
-    @db = INTEGRATION_DB
-    @db.create_table!(:items) do
-      primary_key :id
-      String :name
-      Integer :num
-    end
-    class ::Item < Sequel::Model(@db)
-      plugin :identity_map
-    end
-    Item.create(:name=>'J', :num=>3)
-  end
-  after do
-    @db.drop_table?(:items)
-    Object.send(:remove_const, :Item)
-  end
-
-  specify "should return the same instance if retrieved more than once" do
-    Item.with_identity_map{Item.first.object_id.should == Item.first.object_id}
-  end
-  
-  specify "should merge attributes that don't exist in the model" do
-    Item.with_identity_map do 
-      i = Item.select(:id, :name).first
-      i.values.should == {:id=>1, :name=>'J'}
-      Item.first
-      i.values.should == {:id=>1, :name=>'J', :num=>3}
-    end
-  end
-end
-
 describe "Touch plugin" do
   before(:all) do
-    @db = INTEGRATION_DB
-    @db.instance_variable_set(:@schemas, {})
-    @db.create_table!(:artists) do
+    @db = DB
+    @db.drop_table? :albums_artists, :albums, :artists
+    @db.create_table(:artists) do
       primary_key :id
       String :name
       DateTime :updated_at
     end
-    @db.create_table!(:albums) do
+    @db.create_table(:albums) do
       primary_key :id
       String :name
       foreign_key :artist_id, :artists
       DateTime :updated_at
     end
+    @db.create_join_table({:album_id=>:albums, :artist_id=>:artists}, :no_index=>true)
   end
   before do
     @db[:albums].delete
     @db[:artists].delete
     class ::Album < Sequel::Model(@db)
-      many_to_one :artist
-      plugin :touch, :associations=>:artist
     end
     class ::Artist < Sequel::Model(@db)
     end 
     
     @artist = Artist.create(:name=>'1')
-    @album = Album.create(:name=>'A', :artist=>@artist)
+    @album = Album.create(:name=>'A', :artist_id=>@artist.id)
   end
   after do
     [:Album, :Artist].each{|s| Object.send(:remove_const, s)}
   end
   after(:all) do
-    @db.drop_table? :albums, :artists
+    @db.drop_table? :albums_artists, :albums, :artists
   end
 
   specify "should update the timestamp column when touching the record" do
+    Album.plugin :touch
     @album.updated_at.should == nil
     @album.touch
     @album.updated_at.to_i.should be_within(2).of(Time.now.to_i)
   end
   
-  cspecify "should update the timestamp column for associated records when the record is updated or destroyed", [:do, :sqlite], [:jdbc, :sqlite], [:swift] do
+  cspecify "should update the timestamp column for many_to_one associated records when the record is updated or destroyed", [:do, :sqlite], [:jdbc, :sqlite], [:swift] do
+    Album.many_to_one :artist
+    Album.plugin :touch, :associations=>:artist
     @artist.updated_at.should == nil
     @album.update(:name=>'B')
     ua = @artist.reload.updated_at
     if ua.is_a?(Time)
-      ua.to_i.should be_within(2).of(Time.now.to_i)
+      ua.to_i.should be_within(60).of(Time.now.to_i)
     else
-      (DateTime.now - ua).should be_within(2.0/86400).of(0)
+      (DateTime.now - ua).should be_within(60.0/86400).of(0)
     end
     @artist.update(:updated_at=>nil)
     @album.destroy
     if ua.is_a?(Time)
-      ua.to_i.should be_within(2).of(Time.now.to_i)
+      ua.to_i.should be_within(60).of(Time.now.to_i)
     else
-      (DateTime.now - ua).should be_within(2.0/86400).of(0)
+      (DateTime.now - ua).should be_within(60.0/86400).of(0)
+    end
+  end
+
+  cspecify "should update the timestamp column for one_to_many associated records when the record is updated", [:do, :sqlite], [:jdbc, :sqlite], [:swift] do
+    Artist.one_to_many :albums
+    Artist.plugin :touch, :associations=>:albums
+    @album.updated_at.should == nil
+    @artist.update(:name=>'B')
+    ua = @album.reload.updated_at
+    if ua.is_a?(Time)
+      ua.to_i.should be_within(60).of(Time.now.to_i)
+    else
+      (DateTime.now - ua).should be_within(60.0/86400).of(0)
+    end
+  end
+
+  cspecify "should update the timestamp column for many_to_many associated records when the record is updated", [:do, :sqlite], [:jdbc, :sqlite], [:swift] do
+    Artist.many_to_many :albums
+    Artist.plugin :touch, :associations=>:albums
+    @artist.add_album(@album)
+    @album.updated_at.should == nil
+    @artist.update(:name=>'B')
+    ua = @album.reload.updated_at
+    if ua.is_a?(Time)
+      ua.to_i.should be_within(60).of(Time.now.to_i)
+    else
+      (DateTime.now - ua).should be_within(60.0/86400).of(0)
     end
   end
 end
 
 describe "Serialization plugin" do 
   before do
-    @db = INTEGRATION_DB
+    @db = DB
     @db.create_table!(:items) do
       primary_key :id
       String :stuff
@@ -618,7 +612,7 @@ end
 
 describe "OptimisticLocking plugin" do 
   before(:all) do
-    @db = INTEGRATION_DB
+    @db = DB
     @db.create_table!(:people) do
       primary_key :id
       String :name
@@ -660,7 +654,7 @@ end
 
 describe "Composition plugin" do 
   before do
-    @db = INTEGRATION_DB
+    @db = DB
     @db.create_table!(:events) do
       primary_key :id
       Integer :year
@@ -712,10 +706,10 @@ describe "Composition plugin" do
 end
 
 # DB2's implemention of CTE is too limited to use this plugin
-if INTEGRATION_DB.dataset.supports_cte? and !Sequel.guarded?(:db2)
+if DB.dataset.supports_cte? and !Sequel.guarded?(:db2)
   describe "RcteTree Plugin" do
     before(:all) do
-      @db = INTEGRATION_DB
+      @db = DB
       @db.create_table!(:nodes) do
         primary_key :id
         Integer :parent_id
@@ -947,7 +941,7 @@ end
 
 describe "Instance Filters plugin" do 
   before(:all) do
-    @db = INTEGRATION_DB
+    @db = DB
     @db.create_table!(:items) do
       primary_key :id
       String :name
@@ -1010,7 +1004,7 @@ end
 
 describe "UpdatePrimaryKey plugin" do 
   before(:all) do
-    @db = INTEGRATION_DB
+    @db = DB
     @db.create_table!(:t) do
       Integer :a, :primary_key=>true
       Integer :b
@@ -1034,7 +1028,7 @@ describe "UpdatePrimaryKey plugin" do
     @db[:t].all.should == [{:a=>1, :b=>4}]
     @c.first.set(:b=>5).save
     @db[:t].all.should == [{:a=>1, :b=>5}]
-    @c.first.set(:b=>6).save(:b)
+    @c.first.set(:b=>6).save(:columns=>:b)
     @db[:t].all.should == [{:a=>1, :b=>6}]
   end
 
@@ -1046,7 +1040,7 @@ describe "UpdatePrimaryKey plugin" do
   specify "should handle updating just the primary key field when saving changes" do
     @c.first.update(:a=>2)
     @db[:t].all.should == [{:a=>2, :b=>3}]
-    @c.first.set(:a=>3).save(:a)
+    @c.first.set(:a=>3).save(:columns=>:a)
     @db[:t].all.should == [{:a=>3, :b=>3}]
   end
 
@@ -1068,8 +1062,8 @@ end
 
 describe "AssociationPks plugin" do 
   before(:all) do
-    @db = INTEGRATION_DB
-    @db.drop_table?(:albums_tags, :tags, :albums, :artists)
+    @db = DB
+    @db.drop_table?(:albums_tags, :albums_vocalists, :vocalists_instruments, :vocalists_hits, :hits, :instruments, :vocalists, :tags, :albums, :artists)
     @db.create_table(:artists) do
       primary_key :id
       String :name
@@ -1087,6 +1081,46 @@ describe "AssociationPks plugin" do
       foreign_key :album_id, :albums
       foreign_key :tag_id, :tags
     end
+    @db.create_table(:vocalists) do
+      String :first
+      String :last
+      primary_key [:first, :last]
+      foreign_key :album_id, :albums
+    end
+    @db.create_table(:albums_vocalists) do
+      foreign_key :album_id, :albums
+      String :first
+      String :last
+      foreign_key [:first, :last], :vocalists
+    end
+    @db.create_table(:instruments) do
+      primary_key :id
+      String :first
+      String :last
+      foreign_key [:first, :last], :vocalists
+    end
+    @db.create_table(:vocalists_instruments) do
+      String :first
+      String :last
+      foreign_key [:first, :last], :vocalists
+      foreign_key :instrument_id, :instruments
+    end
+    @db.create_table(:hits) do
+      Integer :year
+      Integer :week
+      primary_key [:year, :week]
+      String :first
+      String :last
+      foreign_key [:first, :last], :vocalists
+    end
+    @db.create_table(:vocalists_hits) do
+      String :first
+      String :last
+      foreign_key [:first, :last], :vocalists
+      Integer :year
+      Integer :week
+      foreign_key [:year, :week], :hits
+    end
     class ::Artist < Sequel::Model
       plugin :association_pks
       one_to_many :albums, :order=>:id
@@ -1097,9 +1131,19 @@ describe "AssociationPks plugin" do
     end 
     class ::Tag < Sequel::Model
     end 
+    class ::Vocalist < Sequel::Model
+      set_primary_key [:first, :last]
+      plugin :association_pks
+    end
+    class ::Instrument < Sequel::Model
+      plugin :association_pks
+    end
+    class ::Hit < Sequel::Model
+      set_primary_key [:year, :week]
+    end
   end
   before do
-    [:albums_tags, :tags, :albums, :artists].each{|t| @db[t].delete}
+    [:albums_tags, :albums_vocalists, :vocalists_instruments, :vocalists_hits, :hits, :instruments, :vocalists, :tags, :albums, :artists].each{|t| @db[t].delete}
     @ar1 =@db[:artists].insert(:name=>'YJM')
     @ar2 =@db[:artists].insert(:name=>'AS')
     @al1 =@db[:albums].insert(:name=>'RF', :artist_id=>@ar1)
@@ -1111,10 +1155,34 @@ describe "AssociationPks plugin" do
     {@al1=>[@t1, @t2, @t3], @al2=>[@t2]}.each do |aid, tids|
       tids.each{|tid| @db[:albums_tags].insert([aid, tid])}
     end
+    @v1 = ['F1', 'L1']
+    @v2 = ['F2', 'L2']
+    @v3 = ['F3', 'L3']
+    @db[:vocalists].insert(@v1 + [@al1])
+    @db[:vocalists].insert(@v2 + [@al1])
+    @db[:vocalists].insert(@v3 + [@al1])
+    @i1 = @db[:instruments].insert([:first, :last], @v1)
+    @i2 = @db[:instruments].insert([:first, :last], @v1)
+    @i3 = @db[:instruments].insert([:first, :last], @v1)
+    @h1 = [1997, 1]
+    @h2 = [1997, 2]
+    @h3 = [1997, 3]
+    @db[:hits].insert(@h1 + @v1)
+    @db[:hits].insert(@h2 + @v1)
+    @db[:hits].insert(@h3 + @v1)
+    {@al1=>[@v1, @v2, @v3], @al2=>[@v2]}.each do |aid, vids|
+      vids.each{|vid| @db[:albums_vocalists].insert([aid] + vid)}
+    end
+    {@v1=>[@i1, @i2, @i3], @v2=>[@i2]}.each do |vid, iids|
+      iids.each{|iid| @db[:vocalists_instruments].insert(vid + [iid])}
+    end
+    {@v1=>[@h1, @h2, @h3], @v2=>[@h2]}.each do |vid, hids|
+      hids.each{|hid| @db[:vocalists_hits].insert(vid + hid)}
+    end
   end
   after(:all) do
-    @db.drop_table? :albums_tags, :tags, :albums, :artists
-    [:Artist, :Album, :Tag].each{|s| Object.send(:remove_const, s)}
+    @db.drop_table? :albums_tags, :albums_vocalists, :vocalists_instruments, :vocalists_hits, :hits, :instruments, :vocalists, :tags, :albums, :artists
+    [:Artist, :Album, :Tag, :Vocalist, :Instrument, :Hit].each{|s| Object.send(:remove_const, s)}
   end
 
   specify "should return correct associated pks for one_to_many associations" do
@@ -1125,6 +1193,36 @@ describe "AssociationPks plugin" do
     Album.order(:id).all.map{|a| a.tag_pks.sort}.should == [[@t1, @t2, @t3], [@t2], []]
   end
 
+  specify "should return correct associated right-side cpks for one_to_many associations" do
+    Album.one_to_many :vocalists, :order=>:first
+    Album.order(:id).all.map{|a| a.vocalist_pks.sort}.should == [[@v1, @v2, @v3], [], []]
+  end
+
+  specify "should return correct associated right-side cpks for many_to_many associations" do
+    Album.many_to_many :vocalists, :join_table=>:albums_vocalists, :right_key=>[:first, :last], :order=>:first
+    Album.order(:id).all.map{|a| a.vocalist_pks.sort}.should == [[@v1, @v2, @v3], [@v2], []]
+  end
+
+  specify "should return correct associated pks for left-side cpks for one_to_many associations" do
+    Vocalist.one_to_many :instruments, :key=>[:first, :last], :order=>:id
+    Vocalist.order(:first, :last).all.map{|a| a.instrument_pks.sort}.should == [[@i1, @i2, @i3], [], []]
+  end
+
+  specify "should return correct associated pks for left-side cpks for many_to_many associations" do
+    Vocalist.many_to_many :instruments, :join_table=>:vocalists_instruments, :left_key=>[:first, :last], :order=>:id
+    Vocalist.order(:first, :last).all.map{|a| a.instrument_pks.sort}.should == [[@i1, @i2, @i3], [@i2], []]
+  end
+
+  specify "should return correct associated right-side cpks for left-side cpks for one_to_many associations" do
+    Vocalist.one_to_many :hits, :key=>[:first, :last], :order=>:week
+    Vocalist.order(:first, :last).all.map{|a| a.hit_pks.sort}.should == [[@h1, @h2, @h3], [], []]
+  end
+
+  specify "should return correct associated right-side cpks for left-side cpks for many_to_many associations" do
+    Vocalist.many_to_many :hits, :join_table=>:vocalists_hits, :left_key=>[:first, :last], :right_key=>[:year, :week], :order=>:week
+    Vocalist.order(:first, :last).all.map{|a| a.hit_pks.sort}.should == [[@h1, @h2, @h3], [@h2], []]
+  end
+
   specify "should set associated pks correctly for a one_to_many association" do
     Artist.use_transactions = true
     Album.order(:id).select_map(:artist_id).should == [@ar1, @ar1, @ar1]
@@ -1162,12 +1260,134 @@ describe "AssociationPks plugin" do
     Album[@al3].tag_pks = []
     @db[:albums_tags].filter(:album_id=>@al1).select_order_map(:tag_id).should == []
   end
-end
 
+  specify "should set associated right-side cpks correctly for a one_to_many association" do
+    Album.use_transactions = true
+    Album.one_to_many :vocalists, :order=>:first
+    Album.order(:id).all.map{|a| a.vocalist_pks.sort}.should == [[@v1, @v2, @v3], [], []]
+
+    Album[@al2].vocalist_pks = [@v1, @v3]
+    Album[@al1].vocalist_pks.should == [@v2]
+    Vocalist.order(:first, :last).select_map(:album_id).should == [@al2, @al1, @al2]
+
+    Album[@al1].vocalist_pks = [@v1]
+    Album[@al2].vocalist_pks.should == [@v3]
+    Vocalist.order(:first, :last).select_map(:album_id).should == [@al1, nil, @al2]
+
+    Album[@al1].vocalist_pks = [@v1, @v2]
+    Album[@al2].vocalist_pks.should == [@v3]
+    Vocalist.order(:first, :last).select_map(:album_id).should == [@al1, @al1, @al2]
+  end
+
+  specify "should set associated right-side cpks correctly for a many_to_many association" do
+    Album.use_transactions = true
+    Album.many_to_many :vocalists, :join_table=>:albums_vocalists, :right_key=>[:first, :last], :order=>:first
+
+    @db[:albums_vocalists].filter(:album_id=>@al1).select_order_map([:first, :last]).should == [@v1, @v2, @v3]
+    Album[@al1].vocalist_pks = [@v1, @v3]
+    @db[:albums_vocalists].filter(:album_id=>@al1).select_order_map([:first, :last]).should == [@v1, @v3]
+    Album[@al1].vocalist_pks = []
+    @db[:albums_vocalists].filter(:album_id=>@al1).select_order_map([:first, :last]).should == []
+
+    @db[:albums_vocalists].filter(:album_id=>@al2).select_order_map([:first, :last]).should == [@v2]
+    Album[@al2].vocalist_pks = [@v1, @v2]
+    @db[:albums_vocalists].filter(:album_id=>@al2).select_order_map([:first, :last]).should == [@v1, @v2]
+    Album[@al2].vocalist_pks = []
+    @db[:albums_vocalists].filter(:album_id=>@al1).select_order_map([:first, :last]).should == []
+
+    @db[:albums_vocalists].filter(:album_id=>@al3).select_order_map([:first, :last]).should == []
+    Album[@al3].vocalist_pks = [@v1, @v3]
+    @db[:albums_vocalists].filter(:album_id=>@al3).select_order_map([:first, :last]).should == [@v1, @v3]
+    Album[@al3].vocalist_pks = []
+    @db[:albums_vocalists].filter(:album_id=>@al1).select_order_map([:first, :last]).should == []
+  end
+
+  specify "should set associated pks correctly with left-side cpks for a one_to_many association" do
+    Vocalist.use_transactions = true
+    Vocalist.one_to_many :instruments, :key=>[:first, :last], :order=>:id
+    Vocalist.order(:first, :last).all.map{|a| a.instrument_pks.sort}.should == [[@i1, @i2, @i3], [], []]
+
+    Vocalist[@v2].instrument_pks = [@i1, @i3]
+    Vocalist[@v1].instrument_pks.should == [@i2]
+    Instrument.order(:id).select_map([:first, :last]).should == [@v2, @v1, @v2]
+
+    Vocalist[@v1].instrument_pks = [@i1]
+    Vocalist[@v2].instrument_pks.should == [@i3]
+    Instrument.order(:id).select_map([:first, :last]).should == [@v1, [nil, nil], @v2]
+
+    Vocalist[@v1].instrument_pks = [@i1, @i2]
+    Vocalist[@v2].instrument_pks.should == [@i3]
+    Instrument.order(:id).select_map([:first, :last]).should == [@v1, @v1, @v2]
+  end
+
+  specify "should set associated pks correctly with left-side cpks for a many_to_many association" do
+    Vocalist.use_transactions = true
+    Vocalist.many_to_many :instruments, :join_table=>:vocalists_instruments, :left_key=>[:first, :last], :order=>:id
+
+    @db[:vocalists_instruments].filter([:first, :last]=>[@v1]).select_order_map(:instrument_id).should == [@i1, @i2, @i3]
+    Vocalist[@v1].instrument_pks = [@i1, @i3]
+    @db[:vocalists_instruments].filter([:first, :last]=>[@v1]).select_order_map(:instrument_id).should == [@i1, @i3]
+    Vocalist[@v1].instrument_pks = []
+    @db[:vocalists_instruments].filter([:first, :last]=>[@v1]).select_order_map(:instrument_id).should == []
+
+    @db[:vocalists_instruments].filter([:first, :last]=>[@v2]).select_order_map(:instrument_id).should == [@i2]
+    Vocalist[@v2].instrument_pks = [@i1, @i2]
+    @db[:vocalists_instruments].filter([:first, :last]=>[@v2]).select_order_map(:instrument_id).should == [@i1, @i2]
+    Vocalist[@v2].instrument_pks = []
+    @db[:vocalists_instruments].filter([:first, :last]=>[@v1]).select_order_map(:instrument_id).should == []
+
+    @db[:vocalists_instruments].filter([:first, :last]=>[@v3]).select_order_map(:instrument_id).should == []
+    Vocalist[@v3].instrument_pks = [@i1, @i3]
+    @db[:vocalists_instruments].filter([:first, :last]=>[@v3]).select_order_map(:instrument_id).should == [@i1, @i3]
+    Vocalist[@v3].instrument_pks = []
+    @db[:vocalists_instruments].filter([:first, :last]=>[@v1]).select_order_map(:instrument_id).should == []
+  end
+
+  specify "should set associated right-side cpks correctly with left-side cpks for a one_to_many association" do
+    Vocalist.use_transactions = true
+    Vocalist.one_to_many :hits, :key=>[:first, :last], :order=>:week
+    Vocalist.order(:first, :last).all.map{|a| a.hit_pks.sort}.should == [[@h1, @h2, @h3], [], []]
+
+    Vocalist[@v2].hit_pks = [@h1, @h3]
+    Vocalist[@v1].hit_pks.should == [@h2]
+    Hit.order(:year, :week).select_map([:first, :last]).should == [@v2, @v1, @v2]
+
+    Vocalist[@v1].hit_pks = [@h1]
+    Vocalist[@v2].hit_pks.should == [@h3]
+    Hit.order(:year, :week).select_map([:first, :last]).should == [@v1, [nil, nil], @v2]
+
+    Vocalist[@v1].hit_pks = [@h1, @h2]
+    Vocalist[@v2].hit_pks.should == [@h3]
+    Hit.order(:year, :week).select_map([:first, :last]).should == [@v1, @v1, @v2]
+  end
+
+  specify "should set associated right-side cpks correctly with left-side cpks for a many_to_many association" do
+    Vocalist.use_transactions = true
+    Vocalist.many_to_many :hits, :join_table=>:vocalists_hits, :left_key=>[:first, :last], :right_key=>[:year, :week], :order=>:week
+
+    @db[:vocalists_hits].filter([:first, :last]=>[@v1]).select_order_map([:year, :week]).should == [@h1, @h2, @h3]
+    Vocalist[@v1].hit_pks = [@h1, @h3]
+    @db[:vocalists_hits].filter([:first, :last]=>[@v1]).select_order_map([:year, :week]).should == [@h1, @h3]
+    Vocalist[@v1].hit_pks = []
+    @db[:vocalists_hits].filter([:first, :last]=>[@v1]).select_order_map([:year, :week]).should == []
+
+    @db[:vocalists_hits].filter([:first, :last]=>[@v2]).select_order_map([:year, :week]).should == [@h2]
+    Vocalist[@v2].hit_pks = [@h1, @h2]
+    @db[:vocalists_hits].filter([:first, :last]=>[@v2]).select_order_map([:year, :week]).should == [@h1, @h2]
+    Vocalist[@v2].hit_pks = []
+    @db[:vocalists_hits].filter([:first, :last]=>[@v1]).select_order_map([:year, :week]).should == []
+
+    @db[:vocalists_hits].filter([:first, :last]=>[@v3]).select_order_map([:year, :week]).should == []
+    Vocalist[@v3].hit_pks = [@h1, @h3]
+    @db[:vocalists_hits].filter([:first, :last]=>[@v3]).select_order_map([:year, :week]).should == [@h1, @h3]
+    Vocalist[@v3].hit_pks = []
+    @db[:vocalists_hits].filter([:first, :last]=>[@v1]).select_order_map([:year, :week]).should == []
+  end
+end
 
 describe "List plugin without a scope" do
   before(:all) do
-    @db = INTEGRATION_DB
+    @db = DB
     @db.create_table!(:sites) do
       primary_key :id
       String :name
@@ -1178,7 +1398,7 @@ describe "List plugin without a scope" do
     @c.plugin :list
   end
   before do
-    @c.delete
+    @c.dataset.delete
     @c.create :name => "abc"
     @c.create :name => "def"
     @c.create :name => "hig"
@@ -1241,7 +1461,7 @@ end
 
 describe "List plugin with a scope" do
   before(:all) do
-    @db = INTEGRATION_DB
+    @db = DB
     @db.create_table!(:pages) do
       primary_key :id
       String :name
@@ -1253,7 +1473,7 @@ describe "List plugin with a scope" do
     @c.plugin :list, :field => :pos, :scope => :parent_id
   end
   before do
-    @c.delete
+    @c.dataset.delete
     p1 = @c.create :name => "Hm", :parent_id => 0
     p2 = @c.create :name => "Ps", :parent_id => p1.id
     @c.create :name => "P1", :parent_id => p2.id
@@ -1265,7 +1485,7 @@ describe "List plugin with a scope" do
     @db.drop_table?(:pages)
   end
 
-  it "should return rows in order of position" do
+  specify "should return rows in order of position" do
     @c.map(:name).should == %w[ Hm Ps Au P1 P2 P3 ]
   end
 
@@ -1286,7 +1506,7 @@ describe "List plugin with a scope" do
     @c[:name => "P3"].next.should == nil
   end
 
-  it "should define move_to" do
+  specify "should define move_to" do
     @c[:name => "P2"].move_to(1)
     @c.map(:name).should == %w[ Hm Ps Au P2 P1 P3 ]
 
@@ -1297,7 +1517,7 @@ describe "List plugin with a scope" do
     proc { @c[:name => "P2"].move_to(10) }.should raise_error(Sequel::Error)
   end
 
-  it "should define move_to_top and move_to_bottom" do
+  specify "should define move_to_top and move_to_bottom" do
     @c[:name => "Au"].move_to_top
     @c.map(:name).should == %w[ Hm Au Ps P1 P2 P3 ]
 
@@ -1305,7 +1525,7 @@ describe "List plugin with a scope" do
     @c.map(:name).should == %w[ Hm Ps Au P1 P2 P3 ]
   end
 
-  it "should define move_up and move_down" do
+  specify "should define move_up and move_down" do
     @c[:name => "P2"].move_up
     @c.map(:name).should == %w[ Hm Ps Au P2 P1 P3 ]
 
@@ -1319,7 +1539,7 @@ end
 
 describe "Sequel::Plugins::Tree" do
   before(:all) do
-    @db = INTEGRATION_DB
+    @db = DB
   end
 
   describe "with natural database order" do
@@ -1484,7 +1704,7 @@ end
 
 describe "Sequel::Plugins::PreparedStatements" do
   before(:all) do
-    @db = INTEGRATION_DB
+    @db = DB
     @db.create_table!(:ps_test) do
       primary_key :id
       String :name
@@ -1494,7 +1714,7 @@ describe "Sequel::Plugins::PreparedStatements" do
     @c.plugin :prepared_statements_with_pk
   end
   before do
-    @c.delete
+    @c.dataset.delete
     @foo = @c.create(:name=>'foo', :i=>10)
     @bar = @c.create(:name=>'bar', :i=>20)
   end
@@ -1557,7 +1777,7 @@ end
 
 describe "Caching plugins" do
   before(:all) do
-    @db = INTEGRATION_DB
+    @db = DB
     @db.drop_table?(:albums, :artists)
     @db.create_table(:artists) do
       primary_key :id
@@ -1617,3 +1837,273 @@ describe "Caching plugins" do
     it_should_behave_like "a caching plugin"
   end
 end
+
+describe "Sequel::Plugins::ConstraintValidations" do
+  before(:all) do
+    @db = DB
+    @db.extension(:constraint_validations)
+    @db.create_constraint_validations_table
+    @ds = @db[:cv_test]
+    @regexp = regexp = @db.dataset.supports_regexp?
+    @validation_opts = {}
+    opts_proc = proc{@validation_opts}
+    @validate_block = proc do |opts|
+      opts = opts_proc.call
+      presence :pre, opts.merge(:name=>:p)
+      exact_length 5, :exactlen, opts.merge(:name=>:el)
+      min_length 5, :minlen, opts.merge(:name=>:minl)
+      max_length 5, :maxlen, opts.merge(:name=>:maxl)
+      length_range 3..5, :lenrange, opts.merge(:name=>:lr)
+      if regexp
+        format(/^foo\d+/, :form, opts.merge(:name=>:f))
+      end
+      like 'foo%', :lik, opts.merge(:name=>:l)
+      ilike 'foo%', :ilik, opts.merge(:name=>:il)
+      includes %w'abc def', :inc, opts.merge(:name=>:i)
+      unique :uniq, opts.merge(:name=>:u)
+      max_length 6, :minlen, opts.merge(:name=>:maxl2)
+    end
+    @valid_row = {:pre=>'a', :exactlen=>'12345', :minlen=>'12345', :maxlen=>'12345', :lenrange=>'1234', :lik=>'fooabc', :ilik=>'FooABC', :inc=>'abc', :uniq=>'u'}
+    @violations = [
+      [:pre, [nil, '', ' ']],
+      [:exactlen, [nil, '', '1234', '123456']],
+      [:minlen, [nil, '', '1234']],
+      [:maxlen, [nil, '123456']],
+      [:lenrange, [nil, '', '12', '123456']],
+      [:lik, [nil, '', 'fo', 'fotabc', 'FOOABC']],
+      [:ilik, [nil, '', 'fo', 'fotabc']],
+      [:inc, [nil, '', 'ab', 'abcd']],
+    ]
+
+    if @regexp
+      @valid_row[:form] = 'foo1'
+      @violations << [:form, [nil, '', 'foo', 'fooa']]
+    end
+  end
+  after(:all) do
+    @db.drop_constraint_validations_table
+  end
+
+  shared_examples_for "constraint validations" do
+    cspecify "should set up constraints that work even outside the model", :mysql do 
+      proc{@ds.insert(@valid_row)}.should_not raise_error
+
+      # Test for unique constraint
+      proc{@ds.insert(@valid_row)}.should raise_error(Sequel::DatabaseError)
+
+      @ds.delete
+      @violations.each do |col, vals|
+        try = @valid_row.dup
+        vals += ['1234567'] if col == :minlen
+        vals.each do |val|
+          next if val.nil? && @validation_opts[:allow_nil]
+          try[col] = val
+          proc{@ds.insert(try)}.should raise_error(Sequel::DatabaseError)
+        end
+      end
+
+      # Test for dropping of constraint
+      @db.alter_table(:cv_test){validate{drop :maxl2}}
+      proc{@ds.insert(@valid_row.merge(:minlen=>'1234567'))}.should_not raise_error
+    end
+
+    it "should set up automatic validations inside the model" do 
+      c = Class.new(Sequel::Model(@ds))
+      c.plugin :constraint_validations
+      c.dataset.delete
+      proc{c.create(@valid_row)}.should_not raise_error
+
+      # Test for unique validation 
+      c.new(@valid_row).should_not be_valid
+
+      c.dataset.delete
+      @violations.each do |col, vals|
+        try = @valid_row.dup
+        vals.each do |val|
+          next if val.nil? && @validation_opts[:allow_nil]
+          try[col] = val
+          c.new(try).should_not be_valid
+        end
+      end
+      c.db.constraint_validations = nil
+    end
+  end
+
+  describe "via create_table" do
+    before(:all) do
+      @table_block = proc do
+        regexp = @regexp
+        validate_block = @validate_block
+        @db.create_table!(:cv_test) do
+          primary_key :id
+          String :pre
+          String :exactlen
+          String :minlen
+          String :maxlen
+          String :lenrange
+          if regexp
+            String :form
+          end
+          String :lik
+          String :ilik
+          String :inc
+          String :uniq, :null=>false
+          validate(&validate_block)
+        end
+      end
+    end
+    after(:all) do
+      @db.drop_table?(:cv_test)
+      @db.drop_constraint_validations_for(:table=>:cv_test)
+    end
+
+    describe "with :allow_nil=>true" do
+      before(:all) do
+        @validation_opts = {:allow_nil=>true}
+        @table_block.call
+      end
+      it_should_behave_like "constraint validations"
+    end
+    describe "with :allow_nil=>false" do
+      before(:all) do
+        @table_block.call
+      end
+      it_should_behave_like "constraint validations"
+    end
+  end
+
+  describe "via alter_table" do
+    before(:all) do
+      @table_block = proc do
+        regexp = @regexp
+        validate_block = @validate_block
+        @db.create_table!(:cv_test) do
+          primary_key :id
+          String :lik
+          String :ilik
+          String :inc
+          String :uniq, :null=>false
+        end
+        @db.alter_table(:cv_test) do
+          add_column :pre, String
+          add_column :exactlen, String
+          add_column :minlen, String
+          add_column :maxlen, String
+          add_column :lenrange, String
+          if regexp
+            add_column :form, String
+          end
+          validate(&validate_block)
+        end
+      end
+    end
+    after(:all) do
+      @db.drop_table?(:cv_test)
+      @db.drop_constraint_validations_for(:table=>:cv_test)
+    end
+
+    describe "with :allow_nil=>true" do
+      before(:all) do
+        @validation_opts = {:allow_nil=>true}
+        @table_block.call
+      end
+      it_should_behave_like "constraint validations"
+    end
+    describe "with :allow_nil=>false" do
+      before(:all) do
+        @table_block.call
+      end
+      it_should_behave_like "constraint validations"
+    end
+  end
+end
+
+describe "date_arithmetic extension" do
+  asd = begin
+    require 'active_support/duration'
+    require 'active_support/inflector'
+    require 'active_support/core_ext/string/inflections'
+    true
+  rescue LoadError
+    false
+  end
+
+  before(:all) do
+    @db = DB
+    @db.extension(:date_arithmetic)
+    if @db.database_type == :sqlite
+      @db.use_timestamp_timezones = false
+    end
+    @date = Date.civil(2010, 7, 12)
+    @dt = Time.local(2010, 7, 12)
+    if asd
+      @d0 = ActiveSupport::Duration.new(0, [[:days, 0]])
+      @d1 = ActiveSupport::Duration.new(1, [[:days, 1]])
+      @d2 = ActiveSupport::Duration.new(1, [[:years, 1], [:months, 1], [:days, 1], [:minutes, 61], [:seconds, 1]])
+    end
+    @h0 = {:days=>0}
+    @h1 = {:days=>1, :years=>nil, :hours=>0}
+    @h2 = {:years=>1, :months=>1, :days=>1, :hours=>1, :minutes=>1, :seconds=>1}
+    @a1 = Time.local(2010, 7, 13)
+    @a2 = Time.local(2011, 8, 13, 1, 1, 1)
+    @s1 = Time.local(2010, 7, 11)
+    @s2 = Time.local(2009, 6, 10, 22, 58, 59)
+    @check = lambda do |meth, in_date, in_interval, should|
+      output = @db.get(Sequel.send(meth, in_date, in_interval))
+      output = Time.parse(output.to_s) unless output.is_a?(Time) || output.is_a?(DateTime)
+      output.year.should == should.year
+      output.month.should == should.month
+      output.day.should == should.day
+      output.hour.should == should.hour
+      output.min.should == should.min
+      output.sec.should == should.sec
+    end
+  end
+  after(:all) do
+    if @db.database_type == :sqlite
+      @db.use_timestamp_timezones = true
+    end
+  end
+
+  if asd
+    specify "be able to use Sequel.date_add to add ActiveSupport::Duration objects to dates and datetimes" do
+      @check.call(:date_add, @date, @d0, @dt)
+      @check.call(:date_add, @date, @d1, @a1)
+      @check.call(:date_add, @date, @d2, @a2)
+
+      @check.call(:date_add, @dt, @d0, @dt)
+      @check.call(:date_add, @dt, @d1, @a1)
+      @check.call(:date_add, @dt, @d2, @a2)
+    end
+
+    specify "be able to use Sequel.date_sub to subtract ActiveSupport::Duration objects from dates and datetimes" do
+      @check.call(:date_sub, @date, @d0, @dt)
+      @check.call(:date_sub, @date, @d1, @s1)
+      @check.call(:date_sub, @date, @d2, @s2)
+
+      @check.call(:date_sub, @dt, @d0, @dt)
+      @check.call(:date_sub, @dt, @d1, @s1)
+      @check.call(:date_sub, @dt, @d2, @s2)
+    end
+  end
+
+  specify "be able to use Sequel.date_add to add interval hashes to dates and datetimes" do
+    @check.call(:date_add, @date, @h0, @dt)
+    @check.call(:date_add, @date, @h1, @a1)
+    @check.call(:date_add, @date, @h2, @a2)
+
+    @check.call(:date_add, @dt, @h0, @dt)
+    @check.call(:date_add, @dt, @h1, @a1)
+    @check.call(:date_add, @dt, @h2, @a2)
+  end
+
+  specify "be able to use Sequel.date_sub to subtract interval hashes from dates and datetimes" do
+    @check.call(:date_sub, @date, @h0, @dt)
+    @check.call(:date_sub, @date, @h1, @s1)
+    @check.call(:date_sub, @date, @h2, @s2)
+
+    @check.call(:date_sub, @dt, @h0, @dt)
+    @check.call(:date_sub, @dt, @h1, @s1)
+    @check.call(:date_sub, @dt, @h2, @s2)
+  end
+end
