@@ -115,6 +115,49 @@ describe Sequel::Model::Associations::AssociationReflection, "#reciprocal" do
     ParParentThree.association_reflection(:par_parents).keys.should_not include(:reciprocal)
     ParParentThree.association_reflection(:par_parents).reciprocal.should == :par_parent_threes
   end
+
+  it "should handle ambiguous reciprocals where only one doesn't have conditions/blocks" do
+    ParParent.many_to_one :par_parent_two, :class=>ParParentTwo, :key=>:par_parent_two_id
+    ParParent.many_to_one :par_parent_two2, :clone=>:par_parent_two, :conditions=>{:id=>:id}
+    ParParentTwo.one_to_many :par_parents
+    ParParent.many_to_many :par_parent_threes, :class=>ParParentThree, :right_key=>:par_parent_three_id
+    ParParent.many_to_many :par_parent_threes2, :clone=>:par_parent_threes do |ds|
+      ds
+    end
+    ParParentThree.many_to_many :par_parents
+
+    ParParentTwo.association_reflection(:par_parents).reciprocal.should == :par_parent_two
+    ParParentThree.association_reflection(:par_parents).reciprocal.should == :par_parent_threes
+  end
+
+  it "should handle ambiguous reciprocals where only one has matching primary keys" do
+    ParParent.many_to_one :par_parent_two, :class=>ParParentTwo, :key=>:par_parent_two_id
+    ParParent.many_to_one :par_parent_two2, :clone=>:par_parent_two, :primary_key=>:foo
+    ParParentTwo.one_to_many :par_parents, :class=>ParParent, :key=>:par_parent_two_id
+    ParParentTwo.one_to_many :par_parents2, :clone=>:par_parents, :primary_key=>:foo
+    ParParent.many_to_many :par_parent_threes, :class=>ParParentThree, :right_key=>:par_parent_three_id
+    ParParent.many_to_many :par_parent_threes2, :clone=>:par_parent_threes, :right_primary_key=>:foo
+    ParParent.many_to_many :par_parent_threes3, :clone=>:par_parent_threes, :left_primary_key=>:foo
+    ParParentThree.many_to_many :par_parents
+
+    ParParent.association_reflection(:par_parent_two).reciprocal.should == :par_parents
+    ParParent.association_reflection(:par_parent_two2).reciprocal.should == :par_parents2
+    ParParentTwo.association_reflection(:par_parents).reciprocal.should == :par_parent_two
+    ParParentTwo.association_reflection(:par_parents2).reciprocal.should == :par_parent_two2
+    ParParentThree.association_reflection(:par_parents).reciprocal.should == :par_parent_threes
+  end
+
+  specify "should handle reciprocals where current association has conditions/block" do
+    ParParent.many_to_one :par_parent_two, :conditions=>{:id=>:id}
+    ParParentTwo.one_to_many :par_parents
+    ParParent.many_to_many :par_parent_threes do |ds|
+      ds
+    end
+    ParParentThree.many_to_many :par_parents
+
+    ParParent.association_reflection(:par_parent_two).reciprocal.should == :par_parents
+    ParParent.association_reflection(:par_parent_threes).reciprocal.should == :par_parents
+  end
 end
 
 describe Sequel::Model::Associations::AssociationReflection, "#select" do
@@ -134,9 +177,9 @@ describe Sequel::Model::Associations::AssociationReflection, "#select" do
   it "should be the associated_table.* if :select is not present for a many_to_many associaiton" do
     @c.many_to_many :cs, :class=>'ParParent'
     @c.association_reflection(:cs).keys.should_not include(:select)
-    @c.association_reflection(:cs).select.should == :par_parents.*
+    @c.association_reflection(:cs).select.should == Sequel::SQL::ColumnAll.new(:par_parents)
   end
-  it "should be if :select is not present for a many_to_one and one_to_many associaiton" do
+  it "should be blank if :select is not present for a many_to_one and one_to_many associaiton" do
     @c.one_to_many :cs, :class=>'ParParent'
     @c.association_reflection(:cs).keys.should_not include(:select)
     @c.association_reflection(:cs).select.should == nil
@@ -190,7 +233,12 @@ end
 describe Sequel::Model::Associations::AssociationReflection do
   before do
     @c = Class.new(Sequel::Model(:foo))
-    @c.meta_def(:name){"C"}
+    def @c.name() "C" end
+  end
+
+  it "#eager_loading_predicate_key should be an alias of predicate_key for backwards compatibility" do
+    @c.one_to_many :cs, :class=>@c
+    @c.dataset.literal(@c.association_reflection(:cs).eager_loading_predicate_key).should == 'foo.c_id'
   end
 
   it "one_to_many #qualified_primary_key should be a qualified version of the primary key" do
@@ -239,7 +287,7 @@ describe Sequel::Model::Associations::AssociationReflection, "#eager_limit_strat
     @c = Class.new(Sequel::Model(:a))
   end
   after do
-    Sequel::Model.default_eager_limit_strategy = nil
+    Sequel::Model.default_eager_limit_strategy = true
   end
 
   it "should be nil by default for *_one associations" do
@@ -271,13 +319,13 @@ describe Sequel::Model::Associations::AssociationReflection, "#eager_limit_strat
   end
 
   it "should use :distinct_on for one_to_one associations if picking and the association dataset supports ordered distinct on" do
-    @c.dataset.meta_def(:supports_ordered_distinct_on?){true}
+    def (@c.dataset).supports_ordered_distinct_on?() true end
     @c.one_to_one :c, :class=>@c, :eager_limit_strategy=>true
     @c.association_reflection(:c).eager_limit_strategy.should == :distinct_on
   end
 
   it "should use :window_function for associations if picking and the association dataset supports window functions" do
-    @c.dataset.meta_def(:supports_window_functions?){true}
+    def (@c.dataset).supports_window_functions?() true end
     @c.one_to_one :c, :class=>@c, :eager_limit_strategy=>true
     @c.association_reflection(:c).eager_limit_strategy.should == :window_function
     @c.one_to_many :cs, :class=>@c, :eager_limit_strategy=>true, :limit=>1
@@ -309,17 +357,13 @@ describe Sequel::Model::Associations::AssociationReflection, "#eager_limit_strat
     c.dataset = :a
     c.one_to_many :cs, :class=>c, :limit=>1
     c.association_reflection(:cs).eager_limit_strategy.should == :ruby
-    c.dataset.meta_def(:supports_window_functions?){true}
+    def (c.dataset).supports_window_functions?() true end
     c.many_to_many :cs, :class=>c, :limit=>1
     c.association_reflection(:cs).eager_limit_strategy.should == :window_function
-
-    c.default_eager_limit_strategy = :correlated_subquery
-    c.many_to_many :cs, :class=>c, :limit=>1
-    c.association_reflection(:cs).eager_limit_strategy.should == :correlated_subquery
   end
 
   it "should ignore Model.default_eager_limit_strategy for one_to_one associations" do
-    @c.default_eager_limit_strategy = :correlated_subquery
+    @c.default_eager_limit_strategy = :window_function
     @c.one_to_one :c, :class=>@c
     @c.association_reflection(:c).eager_limit_strategy.should be_nil
   end
@@ -331,7 +375,7 @@ describe Sequel::Model, " association reflection methods" do
       def self.name; 'Node'; end
       def self.to_s; 'Node'; end
     end
-    MODEL_DB.reset
+    DB.reset
   end
   
   it "#all_association_reflections should include all association reflection hashes" do
