@@ -6,7 +6,7 @@ describe "Model attribute setters" do
       columns :id, :x, :y, :"x y"
     end
     @o = @c.new
-    MODEL_DB.reset
+    DB.reset
   end
 
   specify "refresh should return self" do
@@ -33,7 +33,7 @@ describe "Model attribute setters" do
     @o.changed_columns.should == [:x, :y]
   end
 
-  it "should have columns that can't be called like normal ruby methods" do
+  it "should handle columns that can't be called like normal ruby methods" do
     @o.send(:"x y=", 3)
     @o.changed_columns.should == [:"x y"]
     @o.values.should == {:"x y"=>3}
@@ -47,7 +47,7 @@ describe "Model.def_column_alias" do
       columns :id
       def_column_alias(:id2, :id)
     end.load(:id=>1)
-    MODEL_DB.reset
+    DB.reset
   end
 
   it "should create an getter alias for the column" do
@@ -182,11 +182,21 @@ describe Sequel::Model, ".dataset_module" do
     @c.dataset.return_3.should == 3
   end
 
+  it "should also extend the instance_dataset with the module if the model has a dataset" do
+    @c.dataset_module{def return_3() 3 end}
+    @c.instance_dataset.return_3.should == 3
+  end
+
   it "should add methods defined in the module to the class" do
     @c.dataset_module{def return_3() 3 end}
     @c.return_3.should == 3
   end
 
+  it "should add methods defined in the module outside the block to the class" do
+    @c.dataset_module.module_eval{def return_3() 3 end}
+    @c.return_3.should == 3
+  end
+
   it "should cache calls and readd methods if set_dataset is used" do
     @c.dataset_module{def return_3() 3 end}
     @c.set_dataset :items
@@ -251,6 +261,12 @@ describe Sequel::Model, ".dataset_module" do
     Object.new.extend(@c.dataset_module Module.new{def return_3() 3 end}).return_3.should == 3
   end
 
+  it "should have dataset_module support a subset method" do
+    @c.dataset_module{subset :released, :released}
+    @c.released.sql.should == 'SELECT * FROM items WHERE released'
+    @c.where(:foo).released.sql.should == 'SELECT * FROM items WHERE (foo AND released)'
+  end
+
   it "should raise error if called with both an argument and ablock" do
     proc{@c.dataset_module(Module.new{def return_3() 3 end}){}}.should raise_error(Sequel::Error)
   end
@@ -335,12 +351,12 @@ describe "Model.qualified_primary_key_hash" do
   end
   
   specify "should handle a single primary key" do
-    @c.qualified_primary_key_hash(1).should == {:id.qualify(:items)=>1}
+    @c.qualified_primary_key_hash(1).should == {Sequel.qualify(:items, :id)=>1}
   end
 
   specify "should handle a composite primary key" do
     @c.set_primary_key([:id1, :id2])
-    @c.qualified_primary_key_hash([1, 2]).should == {:id1.qualify(:items)=>1, :id2.qualify(:items)=>2}
+    @c.qualified_primary_key_hash([1, 2]).should == {Sequel.qualify(:items, :id1)=>1, Sequel.qualify(:items, :id2)=>2}
   end
 
   specify "should raise an error for no primary key" do
@@ -349,9 +365,9 @@ describe "Model.qualified_primary_key_hash" do
   end
 
   specify "should allow specifying a different qualifier" do
-    @c.qualified_primary_key_hash(1, :apple).should == {:id.qualify(:apple)=>1}
+    @c.qualified_primary_key_hash(1, :apple).should == {Sequel.qualify(:apple, :id)=>1}
     @c.set_primary_key([:id1, :id2])
-    @c.qualified_primary_key_hash([1, 2], :bear).should == {:id1.qualify(:bear)=>1, :id2.qualify(:bear)=>2}
+    @c.qualified_primary_key_hash([1, 2], :bear).should == {Sequel.qualify(:bear, :id1)=>1, Sequel.qualify(:bear, :id2)=>2}
   end
 end
 
@@ -369,7 +385,7 @@ describe "Model.db" do
     Sequel::Model.db = @model_db
   end
 
-  specify "should be required when create named model classes" do
+  specify "should be required when creating named model classes" do
     begin
       proc{class ModelTest < Sequel::Model; end}.should raise_error(Sequel::Error)
     ensure
@@ -396,6 +412,8 @@ describe "Model.db" do
       class ModelTest2 < Sequel::Model(@db[:foo])
         db.should == @db
       end
+      ModelTest.instance_variable_set(:@db, nil)
+      ModelTest.db.should == @db
     ensure
       Object.send(:remove_const, :ModelTest)
       Object.send(:remove_const, :ModelTest2)
@@ -446,14 +464,6 @@ describe Sequel::Model, ".(allowed|restricted)_columns " do
     @c.allowed_columns.should == [:x, :y]
   end
 
-  it "should set the restricted columns correctly" do
-    @c.restricted_columns.should == nil
-    @c.set_restricted_columns :x
-    @c.restricted_columns.should == [:x]
-    @c.set_restricted_columns :x, :y
-    @c.restricted_columns.should == [:x, :y]
-  end
-
   it "should only set allowed columns by default" do
     @c.set_allowed_columns :x, :y
     i = @c.new(:x => 1, :y => 2, :z => 3)
@@ -465,36 +475,7 @@ describe Sequel::Model, ".(allowed|restricted)_columns " do
     i = @c.new
     i.update(:x => 7, :z => 9)
     i.values.should == {:x => 7}
-    MODEL_DB.sqls.should == ["INSERT INTO blahblah (x) VALUES (7)", "SELECT * FROM blahblah WHERE (id = 10) LIMIT 1"]
-  end
-
-  it "should not set restricted columns by default" do
-    @c.set_restricted_columns :z
-    i = @c.new(:x => 1, :y => 2, :z => 3)
-    i.values.should == {:x => 1, :y => 2}
-    i.set(:x => 4, :y => 5, :z => 6)
-    i.values.should == {:x => 4, :y => 5}
-
-    @c.instance_dataset._fetch = @c.dataset._fetch = {:x => 7}
-    i = @c.new
-    i.update(:x => 7, :z => 9)
-    i.values.should == {:x => 7}
-    MODEL_DB.sqls.should == ["INSERT INTO blahblah (x) VALUES (7)", "SELECT * FROM blahblah WHERE (id = 10) LIMIT 1"]
-  end
-
-  it "should have allowed take precedence over restricted" do
-    @c.set_allowed_columns :x, :y
-    @c.set_restricted_columns :y, :z
-    i = @c.new(:x => 1, :y => 2, :z => 3)
-    i.values.should == {:x => 1, :y => 2}
-    i.set(:x => 4, :y => 5, :z => 6)
-    i.values.should == {:x => 4, :y => 5}
-
-    @c.instance_dataset._fetch = @c.dataset._fetch = {:y => 7}
-    i = @c.new
-    i.update(:y => 7, :z => 9)
-    i.values.should == {:y => 7}
-    MODEL_DB.sqls.should == ["INSERT INTO blahblah (y) VALUES (7)", "SELECT * FROM blahblah WHERE (id = 10) LIMIT 1"]
+    DB.sqls.should == ["INSERT INTO blahblah (x) VALUES (7)", "SELECT * FROM blahblah WHERE (id = 10) LIMIT 1"]
   end
 end
 
@@ -540,7 +521,7 @@ describe Sequel::Model, ".strict_param_setting" do
   before do
     @c = Class.new(Sequel::Model(:blahblah)) do
       columns :x, :y, :z, :id
-      set_restricted_columns :z
+      set_allowed_columns :x, :y
     end
   end
   
@@ -553,13 +534,11 @@ describe Sequel::Model, ".strict_param_setting" do
     proc{@c.create(:z=>1)}.should raise_error(Sequel::Error)
     c = @c.new
     proc{c.set(:z=>1)}.should raise_error(Sequel::Error)
-    proc{c.set_all(:id=>1)}.should raise_error(Sequel::Error)
+    proc{c.set_all(:use_after_commit_rollback => false)}.should raise_error(Sequel::Error)
     proc{c.set_only({:x=>1}, :y)}.should raise_error(Sequel::Error)
-    proc{c.set_except({:x=>1}, :x)}.should raise_error(Sequel::Error)
     proc{c.update(:z=>1)}.should raise_error(Sequel::Error)
-    proc{c.update_all(:id=>1)}.should raise_error(Sequel::Error)
+    proc{c.update_all(:use_after_commit_rollback=>false)}.should raise_error(Sequel::Error)
     proc{c.update_only({:x=>1}, :y)}.should raise_error(Sequel::Error)
-    proc{c.update_except({:x=>1}, :x)}.should raise_error(Sequel::Error)
   end
 
   it "should be disabled by strict_param_setting = false" do
@@ -571,10 +550,10 @@ end
 
 describe Sequel::Model, ".require_modification" do
   before do
-    @ds1 = MODEL_DB[:items]
-    @ds1.meta_def(:provides_accurate_rows_matched?){false}
-    @ds2 = MODEL_DB[:items]
-    @ds2.meta_def(:provides_accurate_rows_matched?){true}
+    @ds1 = DB[:items]
+    def @ds1.provides_accurate_rows_matched?() false end
+    @ds2 = DB[:items]
+    def @ds2.provides_accurate_rows_matched?() true end
   end
   after do
     Sequel::Model.require_modification = nil
@@ -598,7 +577,7 @@ end
 
 describe Sequel::Model, ".[] optimization" do
   before do
-    @db = MODEL_DB.clone
+    @db = DB.clone
     @db.quote_identifiers = true
     @c = Class.new(Sequel::Model(@db))
   end
@@ -608,15 +587,13 @@ describe Sequel::Model, ".[] optimization" do
     @c.simple_pk.should == '"id"'
     @c.set_primary_key :b
     @c.simple_pk.should == '"b"'
-    @c.set_primary_key :b__a.identifier
+    @c.set_primary_key Sequel.identifier(:b__a)
     @c.simple_pk.should == '"b__a"'
   end
 
   it "should have simple_pk be blank if compound or no primary key" do
     @c.no_primary_key
     @c.simple_pk.should == nil
-    @c.set_primary_key :b, :a
-    @c.simple_pk.should == nil
     @c.set_primary_key [:b, :a]
     @c.simple_pk.should == nil
   end
@@ -639,7 +616,7 @@ describe Sequel::Model, ".[] optimization" do
     @c.simple_table.should == '"b"."a"'
   end
 
-  it "should simple_pk and simple_table respect dataset's identifier input methods" do
+  it "should have simple_pk and simple_table respect dataset's identifier input methods" do
     ds = @db[:ab]
     ds.identifier_input_method = :reverse
     @c.set_dataset ds
@@ -656,7 +633,7 @@ describe Sequel::Model, ".[] optimization" do
     @c.simple_table.should == nil
   end
 
-  it "should have simple_table superclasses setting if inheriting" do
+  it "should have simple_table inherit superclass's setting" do
     Class.new(@c).simple_table.should == nil
     @c.set_dataset :a
     Class.new(@c).simple_table.should == '"a"'
@@ -677,45 +654,80 @@ describe Sequel::Model, ".[] optimization" do
   end
 end
 
-describe "Model datasets #with_pk" do
+describe "Model datasets #with_pk with #with_pk!" do
   before do
     @c = Class.new(Sequel::Model(:a))
     @ds = @c.dataset
     @ds._fetch = {:id=>1}
-    MODEL_DB.reset
+    DB.reset
+  end
+
+  it "should be callable on the model class with optimized SQL" do
+    @c.with_pk(1).should == @c.load(:id=>1)
+    DB.sqls.should == ["SELECT * FROM a WHERE id = 1"]
+    @c.with_pk!(1).should == @c.load(:id=>1)
+    DB.sqls.should == ["SELECT * FROM a WHERE id = 1"]
   end
 
   it "should return the first record where the primary key matches" do
     @ds.with_pk(1).should == @c.load(:id=>1)
-    MODEL_DB.sqls.should == ["SELECT * FROM a WHERE (a.id = 1) LIMIT 1"]
+    DB.sqls.should == ["SELECT * FROM a WHERE (a.id = 1) LIMIT 1"]
+    @ds.with_pk!(1).should == @c.load(:id=>1)
+    DB.sqls.should == ["SELECT * FROM a WHERE (a.id = 1) LIMIT 1"]
   end
 
   it "should handle existing filters" do
     @ds.filter(:a=>2).with_pk(1)
-    MODEL_DB.sqls.should == ["SELECT * FROM a WHERE ((a = 2) AND (a.id = 1)) LIMIT 1"]
+    DB.sqls.should == ["SELECT * FROM a WHERE ((a = 2) AND (a.id = 1)) LIMIT 1"]
+    @ds.filter(:a=>2).with_pk!(1)
+    DB.sqls.should == ["SELECT * FROM a WHERE ((a = 2) AND (a.id = 1)) LIMIT 1"]
   end
 
   it "should work with string values" do
     @ds.with_pk("foo")
-    MODEL_DB.sqls.should == ["SELECT * FROM a WHERE (a.id = 'foo') LIMIT 1"]
+    DB.sqls.should == ["SELECT * FROM a WHERE (a.id = 'foo') LIMIT 1"]
+    @ds.with_pk!("foo")
+    DB.sqls.should == ["SELECT * FROM a WHERE (a.id = 'foo') LIMIT 1"]
   end
 
   it "should handle an array for composite primary keys" do
-    @c.set_primary_key :id1, :id2
+    @c.set_primary_key [:id1, :id2]
     @ds.with_pk([1, 2])
-    sqls = MODEL_DB.sqls
+    sqls = DB.sqls
+    ["SELECT * FROM a WHERE ((a.id1 = 1) AND (a.id2 = 2)) LIMIT 1",
+    "SELECT * FROM a WHERE ((a.id2 = 2) AND (a.id1 = 1)) LIMIT 1"].should include(sqls.pop)
+    sqls.should == []
+
+    @ds.with_pk!([1, 2])
+    sqls = DB.sqls
     ["SELECT * FROM a WHERE ((a.id1 = 1) AND (a.id2 = 2)) LIMIT 1",
     "SELECT * FROM a WHERE ((a.id2 = 2) AND (a.id1 = 1)) LIMIT 1"].should include(sqls.pop)
     sqls.should == []
   end
 
+  it "should have with_pk return nil and with_pk! raise if no rows match" do
+    @ds._fetch = []
+    @ds.with_pk(1).should == nil
+    DB.sqls.should == ["SELECT * FROM a WHERE (a.id = 1) LIMIT 1"]
+    proc{@ds.with_pk!(1)}.should raise_error(Sequel::NoMatchingRow)
+    DB.sqls.should == ["SELECT * FROM a WHERE (a.id = 1) LIMIT 1"]
+  end
+
+  it "should have with_pk return nil and with_pk! raise if no rows match when calling the class method" do
+    @ds._fetch = []
+    @c.with_pk(1).should == nil
+    DB.sqls.should == ["SELECT * FROM a WHERE id = 1"]
+    proc{@c.with_pk!(1)}.should raise_error(Sequel::NoMatchingRow)
+    DB.sqls.should == ["SELECT * FROM a WHERE id = 1"]
+  end
+
   it "should have #[] consider an integer as a primary key lookup" do
     @ds[1].should == @c.load(:id=>1)
-    MODEL_DB.sqls.should == ["SELECT * FROM a WHERE (a.id = 1) LIMIT 1"]
+    DB.sqls.should == ["SELECT * FROM a WHERE (a.id = 1) LIMIT 1"]
   end
 
   it "should not have #[] consider a string as a primary key lookup" do
     @ds['foo'].should == @c.load(:id=>1)
-    MODEL_DB.sqls.should == ["SELECT * FROM a WHERE (foo) LIMIT 1"]
+    DB.sqls.should == ["SELECT * FROM a WHERE (foo) LIMIT 1"]
   end
 end
