@@ -20,42 +20,42 @@ describe "Sequel::Model()" do
   end
 
   it "should return a model subclass with a dataset with the default database and given table name if given a LiteralString" do
-    c = Sequel::Model('blah'.lit)
+    c = Sequel::Model(Sequel.lit('blah'))
     c.superclass.should == Sequel::Model
     c.db.should == @db
-    c.table_name.should == 'blah'.lit
+    c.table_name.should == Sequel.lit('blah')
   end
 
   it "should return a model subclass with a dataset with the default database and given table name if given an SQL::Identifier" do
-    c = Sequel::Model(:blah.identifier)
+    c = Sequel::Model(Sequel.identifier(:blah))
     c.superclass.should == Sequel::Model
     c.db.should == @db
-    c.table_name.should == :blah.identifier
+    c.table_name.should == Sequel.identifier(:blah)
   end
 
   it "should return a model subclass with a dataset with the default database and given table name if given an SQL::QualifiedIdentifier" do
-    c = Sequel::Model(:blah.qualify(:boo))
+    c = Sequel::Model(Sequel.qualify(:boo, :blah))
     c.superclass.should == Sequel::Model
     c.db.should == @db
-    c.table_name.should == :blah.qualify(:boo)
+    c.table_name.should == Sequel.qualify(:boo, :blah)
   end
 
   it "should return a model subclass with a dataset with the default database and given table name if given an SQL::AliasedExpression" do
-    c = Sequel::Model(:blah.as(:boo))
+    c = Sequel::Model(Sequel.as(:blah, :boo))
     c.superclass.should == Sequel::Model
     c.db.should == @db
     c.table_name.should == :boo
   end
 
   it "should return a model subclass with the given dataset if given a dataset using an SQL::Identifier" do
-    ds = @db[:blah.identifier]
+    ds = @db[Sequel.identifier(:blah)]
     c = Sequel::Model(ds)
     c.superclass.should == Sequel::Model
     c.dataset.should == ds
   end
 
   it "should return a model subclass associated to the given database if given a database" do
-    db = Sequel::Database.new
+    db = Sequel.mock
     c = Sequel::Model(db)
     c.superclass.should == Sequel::Model
     c.db.should == db
@@ -68,10 +68,10 @@ describe "Sequel::Model()" do
 
   describe "reloading" do
     before do
-      Sequel::Model.cache_anonymous_models = true
+      Sequel.cache_anonymous_models = true
     end
     after do
-      Sequel::Model.cache_anonymous_models = false
+      Sequel.cache_anonymous_models = false
       Object.send(:remove_const, :Album) if defined?(::Album)
     end
 
@@ -84,29 +84,29 @@ describe "Sequel::Model()" do
 
     it "should work without raising an exception with an SQL::Identifier " do
       proc do
-        class ::Album < Sequel::Model(:table.identifier); end
-        class ::Album < Sequel::Model(:table.identifier); end
+        class ::Album < Sequel::Model(Sequel.identifier(:table)); end
+        class ::Album < Sequel::Model(Sequel.identifier(:table)); end
       end.should_not raise_error
     end
 
     it "should work without raising an exception with an SQL::QualifiedIdentifier " do
       proc do
-        class ::Album < Sequel::Model(:table.qualify(:schema)); end
-        class ::Album < Sequel::Model(:table.qualify(:schema)); end
+        class ::Album < Sequel::Model(Sequel.qualify(:schema, :table)); end
+        class ::Album < Sequel::Model(Sequel.qualify(:schema, :table)); end
       end.should_not raise_error
     end
 
     it "should work without raising an exception with an SQL::AliasedExpression" do
       proc do
-        class ::Album < Sequel::Model(:table.as(:alias)); end
-        class ::Album < Sequel::Model(:table.as(:alias)); end
+        class ::Album < Sequel::Model(Sequel.as(:table, :alias)); end
+        class ::Album < Sequel::Model(Sequel.as(:table, :alias)); end
       end.should_not raise_error
     end
 
     it "should work without raising an exception with an LiteralString" do
       proc do
-        class ::Album < Sequel::Model('table'.lit); end
-        class ::Album < Sequel::Model('table'.lit); end
+        class ::Album < Sequel::Model(Sequel.lit('table')); end
+        class ::Album < Sequel::Model(Sequel.lit('table')); end
       end.should_not raise_error
     end
 
@@ -126,16 +126,16 @@ describe "Sequel::Model()" do
 
     it "should work without raising an exception with a dataset with an SQL::Identifier" do
       proc do
-        class ::Album < Sequel::Model(@db[:table.identifier]); end
-        class ::Album < Sequel::Model(@db[:table.identifier]); end
+        class ::Album < Sequel::Model(@db[Sequel.identifier(:table)]); end
+        class ::Album < Sequel::Model(@db[Sequel.identifier(:table)]); end
       end.should_not raise_error
     end
 
     it "should raise an exception if anonymous model caching is disabled" do
-      Sequel::Model.cache_anonymous_models = false
+      Sequel.cache_anonymous_models = false
       proc do
-        class ::Album < Sequel::Model(@db[:table.identifier]); end
-        class ::Album < Sequel::Model(@db[:table.identifier]); end
+        class ::Album < Sequel::Model(@db[Sequel.identifier(:table)]); end
+        class ::Album < Sequel::Model(@db[Sequel.identifier(:table)]); end
       end.should raise_error
     end
   end
@@ -150,12 +150,12 @@ describe Sequel::Model do
   end
 
   it "should be associated with a dataset" do
-    model_a = Class.new(Sequel::Model) { set_dataset MODEL_DB[:as] }
+    model_a = Class.new(Sequel::Model) { set_dataset DB[:as] }
 
     model_a.dataset.should be_a_kind_of(Sequel::Mock::Dataset)
     model_a.dataset.opts[:from].should == [:as]
 
-    model_b = Class.new(Sequel::Model) { set_dataset MODEL_DB[:bs] }
+    model_b = Class.new(Sequel::Model) { set_dataset DB[:bs] }
 
     model_b.dataset.should be_a_kind_of(Sequel::Mock::Dataset)
     model_b.dataset.opts[:from].should == [:bs]
@@ -164,12 +164,12 @@ describe Sequel::Model do
   end
 end
 
-describe Sequel::Model, "dataset & schema" do
+describe Sequel::Model do
   before do
     @model = Class.new(Sequel::Model(:items))
   end
 
-  it "creates dynamic model subclass with set table name" do
+  it "has table_name return name of table" do
     @model.table_name.should == :items
   end
 
@@ -183,37 +183,37 @@ describe Sequel::Model, "dataset & schema" do
   end
 
   it "allows dataset change" do
-    @model.set_dataset(MODEL_DB[:foo])
+    @model.set_dataset(DB[:foo])
     @model.table_name.should == :foo
   end
 
-  it "set_dataset should take a Symbol" do
-    @model.db = MODEL_DB
+  it "allows set_dataset to accept a Symbol" do
+    @model.db = DB
     @model.set_dataset(:foo)
     @model.table_name.should == :foo
   end
 
-  it "set_dataset should take a LiteralString" do
-    @model.db = MODEL_DB
-    @model.set_dataset('foo'.lit)
-    @model.table_name.should == 'foo'.lit
+  it "allows set_dataset to accept a LiteralString" do
+    @model.db = DB
+    @model.set_dataset(Sequel.lit('foo'))
+    @model.table_name.should == Sequel.lit('foo')
   end
 
-  it "set_dataset should take an SQL::Identifier" do
-    @model.db = MODEL_DB
-    @model.set_dataset(:foo.identifier)
-    @model.table_name.should == :foo.identifier
+  it "allows set_dataset to acceptan SQL::Identifier" do
+    @model.db = DB
+    @model.set_dataset(Sequel.identifier(:foo))
+    @model.table_name.should == Sequel.identifier(:foo)
   end
 
-  it "set_dataset should take an SQL::QualifiedIdentifier" do
-    @model.db = MODEL_DB
-    @model.set_dataset(:foo.qualify(:bar))
-    @model.table_name.should == :foo.qualify(:bar)
+  it "allows set_dataset to acceptan SQL::QualifiedIdentifier" do
+    @model.db = DB
+    @model.set_dataset(Sequel.qualify(:bar, :foo))
+    @model.table_name.should == Sequel.qualify(:bar, :foo)
   end
 
-  it "set_dataset should take an SQL::AliasedExpression" do
-    @model.db = MODEL_DB
-    @model.set_dataset(:foo.as(:bar))
+  it "allows set_dataset to acceptan SQL::AliasedExpression" do
+    @model.db = DB
+    @model.set_dataset(Sequel.as(:foo, :bar))
     @model.table_name.should == :bar
   end
 
@@ -227,14 +227,14 @@ describe Sequel::Model, "dataset & schema" do
   end
 
   it "set_dataset should add the destroy method to the dataset that destroys each object" do
-    ds = MODEL_DB[:foo]
+    ds = DB[:foo]
     ds.should_not respond_to(:destroy)
     @model.set_dataset(ds)
     ds.should respond_to(:destroy)
-    MODEL_DB.sqls
+    DB.sqls
     ds._fetch = [{:id=>1}, {:id=>2}]
     ds.destroy.should == 2
-    MODEL_DB.sqls.should == ["SELECT * FROM foo", "DELETE FROM foo WHERE id = 1", "DELETE FROM foo WHERE id = 2"]
+    DB.sqls.should == ["SELECT * FROM foo", "DELETE FROM foo WHERE id = 1", "DELETE FROM foo WHERE id = 2"]
   end
 
   it "set_dataset should add the destroy method that respects sharding with transactions" do
@@ -248,24 +248,43 @@ describe Sequel::Model, "dataset & schema" do
   end
 
   it "should raise an error on set_dataset if there is an error connecting to the database" do
-    @model.meta_def(:columns){raise Sequel::DatabaseConnectionError}
+    def @model.columns() raise Sequel::DatabaseConnectionError end
     proc{@model.set_dataset(Sequel::Database.new[:foo].join(:blah))}.should raise_error
   end
 
   it "should not raise an error if there is a problem getting the columns for a dataset" do
-    @model.meta_def(:columns){raise Sequel::Error}
-    proc{@model.set_dataset(MODEL_DB[:foo].join(:blah))}.should_not raise_error
+    def @model.columns() raise Sequel::Error end
+    proc{@model.set_dataset(DB[:foo].join(:blah))}.should_not raise_error
   end
 
   it "doesn't raise an error on set_dataset if there is an error raised getting the schema" do
-    @model.meta_def(:get_db_schema){raise Sequel::Error}
-    proc{@model.set_dataset(MODEL_DB[:foo])}.should_not raise_error
+    def @model.get_db_schema(*) raise Sequel::Error end
+    proc{@model.set_dataset(DB[:foo])}.should_not raise_error
   end
 
   it "doesn't raise an error on inherited if there is an error setting the dataset" do
-    @model.meta_def(:set_dataset){raise Sequel::Error}
+    def @model.set_dataset(*) raise Sequel::Error end
     proc{Class.new(@model)}.should_not raise_error
   end
+
+  it "should raise if bad inherited instance variable value is used" do
+    def @model.inherited_instance_variables() super.merge(:@a=>:foo) end
+    @model.instance_eval{@a=1}
+    proc{Class.new(@model)}.should raise_error(Sequel::Error)
+  end
+
+  it "copy inherited instance variables into subclass if set" do
+    def @model.inherited_instance_variables() super.merge(:@a=>nil, :@b=>:dup, :@c=>:hash_dup, :@d=>proc{|v| v * 2}) end
+    @model.instance_eval{@a=1; @b=[2]; @c={3=>[4]}; @d=10}
+    m = Class.new(@model)
+    @model.instance_eval{@a=5; @b << 6; @c[3] << 7; @c[8] = [9]; @d=40}
+    m.instance_eval do
+      @a.should == 1
+      @b.should == [2]
+      @c.should == {3=>[4]}
+      @d.should == 20
+    end
+  end
 end
 
 describe Sequel::Model, "constructors" do
@@ -289,7 +308,7 @@ describe Sequel::Model, "constructors" do
   end
   
   it "should have dataset row_proc create an existing object" do
-    @m.dataset = Sequel::Dataset.new(nil)
+    @m.dataset = Sequel.mock.dataset
     o = @m.dataset.row_proc.call(:a=>1)
     o.should be_a_kind_of(@m)
     o.values.should == {:a=>1}
@@ -309,19 +328,12 @@ describe Sequel::Model, "constructors" do
     o.values.should == {:a=>1}
     o.new?.should be_false
   end
-  
-  it "should have .new with a second true argument create an existing object" do
-    o = @m.new({:a=>1}, true)
-    o.should be_a_kind_of(@m)
-    o.values.should == {:a=>1}
-    o.new?.should be_false
-  end
 end
 
 describe Sequel::Model, "new" do
   before do
     @m = Class.new(Sequel::Model) do
-      set_dataset MODEL_DB[:items]
+      set_dataset DB[:items]
       columns :x, :id
     end
   end
@@ -355,18 +367,18 @@ end
 describe Sequel::Model, ".subset" do
   before do
     @c = Class.new(Sequel::Model(:items))
-    MODEL_DB.reset
+    DB.reset
   end
 
   specify "should create a filter on the underlying dataset" do
     proc {@c.new_only}.should raise_error(NoMethodError)
     
-    @c.subset(:new_only) {:age.sql_number < 'new'}
+    @c.subset(:new_only){age < 'new'}
     
     @c.new_only.sql.should == "SELECT * FROM items WHERE (age < 'new')"
     @c.dataset.new_only.sql.should == "SELECT * FROM items WHERE (age < 'new')"
     
-    @c.subset(:pricey) {:price.sql_number > 100}
+    @c.subset(:pricey){price > 100}
     
     @c.pricey.sql.should == "SELECT * FROM items WHERE (price > 100)"
     @c.dataset.pricey.sql.should == "SELECT * FROM items WHERE (price > 100)"
@@ -376,7 +388,7 @@ describe Sequel::Model, ".subset" do
   end
 
   specify "should not override existing model methods" do
-    @c.meta_def(:active){true}
+    def @c.active() true end
     @c.subset(:active, :active)
     @c.active.should == true
   end
@@ -386,29 +398,29 @@ describe Sequel::Model, ".find" do
   before do
     @c = Class.new(Sequel::Model(:items))
     @c.dataset._fetch = {:name => 'sharon', :id => 1}
-    MODEL_DB.reset
+    DB.reset
   end
   
   it "should return the first record matching the given filter" do
     @c.find(:name => 'sharon').should be_a_kind_of(@c)
-    MODEL_DB.sqls.should == ["SELECT * FROM items WHERE (name = 'sharon') LIMIT 1"]
+    DB.sqls.should == ["SELECT * FROM items WHERE (name = 'sharon') LIMIT 1"]
 
-    @c.find(:name.like('abc%')).should be_a_kind_of(@c)
-    MODEL_DB.sqls.should == ["SELECT * FROM items WHERE (name LIKE 'abc%') LIMIT 1"]
+    @c.find(Sequel.expr(:name).like('abc%')).should be_a_kind_of(@c)
+    DB.sqls.should == ["SELECT * FROM items WHERE (name LIKE 'abc%' ESCAPE '\\') LIMIT 1"]
   end
   
   specify "should accept filter blocks" do
-    @c.find{:id.sql_number > 1}.should be_a_kind_of(@c)
-    MODEL_DB.sqls.should == ["SELECT * FROM items WHERE (id > 1) LIMIT 1"]
+    @c.find{id > 1}.should be_a_kind_of(@c)
+    DB.sqls.should == ["SELECT * FROM items WHERE (id > 1) LIMIT 1"]
 
-    @c.find {(:x.sql_number > 1) & (:y.sql_number < 2)}.should be_a_kind_of(@c)
-    MODEL_DB.sqls.should == ["SELECT * FROM items WHERE ((x > 1) AND (y < 2)) LIMIT 1"]
+    @c.find{(x > 1) & (y < 2)}.should be_a_kind_of(@c)
+    DB.sqls.should == ["SELECT * FROM items WHERE ((x > 1) AND (y < 2)) LIMIT 1"]
   end
 end
 
 describe Sequel::Model, ".fetch" do
   before do
-    MODEL_DB.reset
+    DB.reset
     @c = Class.new(Sequel::Model(:items))
   end
   
@@ -429,19 +441,19 @@ describe Sequel::Model, ".find_or_create" do
       set_primary_key :id
       columns :x
     end
-    MODEL_DB.reset
+    DB.reset
   end
 
   it "should find the record" do
     @c.find_or_create(:x => 1).should == @c.load(:x=>1, :id=>1)
-    MODEL_DB.sqls.should == ["SELECT * FROM items WHERE (x = 1) LIMIT 1"]
+    DB.sqls.should == ["SELECT * FROM items WHERE (x = 1) LIMIT 1"]
   end
   
   it "should create the record if not found" do
     @c.instance_dataset._fetch = @c.dataset._fetch = [[], {:x=>1, :id=>1}]
     @c.instance_dataset.autoid = @c.dataset.autoid = 1
     @c.find_or_create(:x => 1).should == @c.load(:x=>1, :id=>1)
-    MODEL_DB.sqls.should == ["SELECT * FROM items WHERE (x = 1) LIMIT 1",
+    DB.sqls.should == ["SELECT * FROM items WHERE (x = 1) LIMIT 1",
       "INSERT INTO items (x) VALUES (1)",
       "SELECT * FROM items WHERE (id = 1) LIMIT 1"]
   end
@@ -450,7 +462,7 @@ describe Sequel::Model, ".find_or_create" do
     @c.instance_dataset._fetch = @c.dataset._fetch = [[], {:x=>1, :id=>1}]
     @c.instance_dataset.autoid = @c.dataset.autoid = 1
     @c.find_or_create(:x => 1){|x| x[:y] = 2}.should == @c.load(:x=>1, :id=>1)
-    sqls = MODEL_DB.sqls
+    sqls = DB.sqls
     sqls.first.should == "SELECT * FROM items WHERE (x = 1) LIMIT 1"
     ["INSERT INTO items (x, y) VALUES (1, 2)", "INSERT INTO items (y, x) VALUES (2, 1)"].should include(sqls[1])
     sqls.last.should == "SELECT * FROM items WHERE (id = 1) LIMIT 1"
@@ -470,7 +482,7 @@ describe Sequel::Model, "A model class without a primary key" do
       columns :x
       no_primary_key
     end
-    MODEL_DB.reset
+    DB.reset
   end
 
   it "should be able to insert records without selecting them back" do
@@ -479,7 +491,7 @@ describe Sequel::Model, "A model class without a primary key" do
     i.class.should be(@c)
     i.values.to_hash.should == {:x => 1}
 
-    MODEL_DB.sqls.should == ['INSERT INTO items (x) VALUES (1)']
+    DB.sqls.should == ['INSERT INTO items (x) VALUES (1)']
   end
 
   it "should raise when deleting" do
@@ -494,32 +506,33 @@ describe Sequel::Model, "A model class without a primary key" do
     o = @c.new(:x => 2)
     o.should be_new
     o.save
-    MODEL_DB.sqls.should == ['INSERT INTO items (x) VALUES (2)']
+    DB.sqls.should == ['INSERT INTO items (x) VALUES (2)']
   end
 end
 
 describe Sequel::Model, "attribute accessors" do
   before do
     db = Sequel.mock
+    def db.supports_schema_parsing?() true end
     def db.schema(*)
-      [[:x, {:type=>:integer}], [:y, {:type=>:integer}]]
+      [[:x, {:type=>:integer}], [:z, {:type=>:integer}]]
     end
-    @dataset = db[:items].columns(:x, :y)
+    @dataset = db[:items].columns(:x, :z)
     @c = Class.new(Sequel::Model)
-    MODEL_DB.reset
+    DB.reset
   end
 
   it "should be created on set_dataset" do
-    %w'x y x= y='.each do |x|
-      @c.instance_methods.collect{|y| y.to_s}.should_not include(x)
+    %w'x z x= z='.each do |x|
+      @c.instance_methods.collect{|z| z.to_s}.should_not include(x)
     end
     @c.set_dataset(@dataset)
-    %w'x y x= y='.each do |x|
-      @c.instance_methods.collect{|y| y.to_s}.should include(x)
+    %w'x z x= z='.each do |x|
+      @c.instance_methods.collect{|z| z.to_s}.should include(x)
     end
     o = @c.new
-    %w'x y x= y='.each do |x|
-      o.methods.collect{|y| y.to_s}.should include(x)
+    %w'x z x= z='.each do |x|
+      o.methods.collect{|z| z.to_s}.should include(x)
     end
 
     o.x.should be_nil
@@ -538,7 +551,7 @@ describe Sequel::Model, "attribute accessors" do
   end
 
   it "should have a working typecasting setter even if the column is not selected" do
-    @c.set_dataset(@dataset.select(:y).columns(:y))
+    @c.set_dataset(@dataset.select(:z).columns(:z))
     o = @c.new
 
     o.x = '34'
@@ -546,7 +559,7 @@ describe Sequel::Model, "attribute accessors" do
   end
 
   it "should typecast if the new value is the same as the existing but has a different class" do
-    @c.set_dataset(@dataset.select(:y).columns(:y))
+    @c.set_dataset(@dataset.select(:z).columns(:z))
     o = @c.new
 
     o.x = 34
@@ -561,46 +574,38 @@ describe Sequel::Model, ".[]" do
   before do
     @c = Class.new(Sequel::Model(:items))
     @c.dataset._fetch = {:name => 'sharon', :id => 1}
-    MODEL_DB.reset
+    DB.reset
   end
 
   it "should return the first record for the given pk" do
     @c[1].should == @c.load(:name => 'sharon', :id => 1)
-    MODEL_DB.sqls.should == ["SELECT * FROM items WHERE id = 1"]
+    DB.sqls.should == ["SELECT * FROM items WHERE id = 1"]
     @c[9999].should == @c.load(:name => 'sharon', :id => 1)
-    MODEL_DB.sqls.should == ["SELECT * FROM items WHERE id = 9999"]
+    DB.sqls.should == ["SELECT * FROM items WHERE id = 9999"]
   end
 
   it "should have #[] return nil if no rows match" do
     @c.dataset._fetch = []
     @c[1].should == nil
-    MODEL_DB.sqls.should == ["SELECT * FROM items WHERE id = 1"]
+    DB.sqls.should == ["SELECT * FROM items WHERE id = 1"]
   end
 
   it "should work correctly for custom primary key" do
     @c.set_primary_key :name
     @c['sharon'].should == @c.load(:name => 'sharon', :id => 1)
-    MODEL_DB.sqls.should == ["SELECT * FROM items WHERE name = 'sharon'"]
+    DB.sqls.should == ["SELECT * FROM items WHERE name = 'sharon'"]
   end
 
   it "should return the first record for the given pk for a filtered dataset" do
     @c.dataset = @c.dataset.filter(:active=>true)
     @c[1].should == @c.load(:name => 'sharon', :id => 1)
-    MODEL_DB.sqls.should == ["SELECT * FROM items WHERE ((active IS TRUE) AND (id = 1)) LIMIT 1"]
+    DB.sqls.should == ["SELECT * FROM items WHERE ((active IS TRUE) AND (id = 1)) LIMIT 1"]
   end
 
   it "should work correctly for composite primary key specified as array" do
     @c.set_primary_key [:node_id, :kind]
     @c[3921, 201].should be_a_kind_of(@c)
-    sqls = MODEL_DB.sqls
-    sqls.length.should == 1
-    sqls.first.should =~ /^SELECT \* FROM items WHERE \((\(node_id = 3921\) AND \(kind = 201\))|(\(kind = 201\) AND \(node_id = 3921\))\) LIMIT 1$/
-  end
-  
-  it "should work correctly for composite primary key specified as separate arguments" do
-    @c.set_primary_key :node_id, :kind
-    @c[3921, 201].should be_a_kind_of(@c)
-    sqls = MODEL_DB.sqls
+    sqls = DB.sqls
     sqls.length.should == 1
     sqls.first.should =~ /^SELECT \* FROM items WHERE \((\(node_id = 3921\) AND \(kind = 201\))|(\(kind = 201\) AND \(node_id = 3921\))\) LIMIT 1$/
   end
@@ -618,11 +623,24 @@ describe "Model.db_schema" do
       def self.columns; orig_columns; end
     end
     @db = Sequel.mock
+    def @db.supports_schema_parsing?() true end
     @dataset = @db[:items]
   end
   
-  specify "should use the database's schema_for_table and set the columns and dataset columns" do
-    d = @dataset.db
+  specify "should not call database's schema if it isn't supported" do
+    def @db.supports_schema_parsing?() false end
+    def @db.schema(table, opts = {})
+      raise Sequel::Error
+    end
+    @dataset.instance_variable_set(:@columns, [:x, :y])
+
+    @c.dataset = @dataset
+    @c.db_schema.should == {:x=>{}, :y=>{}}
+    @c.columns.should == [:x, :y]
+    @c.dataset.instance_variable_get(:@columns).should == [:x, :y]
+  end
+
+  specify "should use the database's schema and set the columns and dataset columns" do
     def @db.schema(table, opts = {})
       [[:x, {:type=>:integer}], [:y, {:type=>:string}]]
     end
@@ -652,7 +670,7 @@ describe "Model.db_schema" do
   specify "should automatically set a singular primary key based on the schema" do
     ds = @dataset
     d = ds.db
-    d.meta_def(:schema){|table, *opts| [[:x, {:primary_key=>true}]]}
+    def d.schema(table, *opts) [[:x, {:primary_key=>true}]] end
     @c.primary_key.should == :id
     @c.dataset = ds
     @c.db_schema.should == {:x=>{:primary_key=>true}}
@@ -662,7 +680,7 @@ describe "Model.db_schema" do
   specify "should automatically set the composite primary key based on the schema" do
     ds = @dataset
     d = ds.db
-    d.meta_def(:schema){|table, *opts| [[:x, {:primary_key=>true}], [:y, {:primary_key=>true}]]}
+    def d.schema(table, *opts) [[:x, {:primary_key=>true}], [:y, {:primary_key=>true}]] end
     @c.primary_key.should == :id
     @c.dataset = ds
     @c.db_schema.should == {:x=>{:primary_key=>true}, :y=>{:primary_key=>true}}
@@ -672,7 +690,7 @@ describe "Model.db_schema" do
   specify "should automatically set no primary key based on the schema" do
     ds = @dataset
     d = ds.db
-    d.meta_def(:schema){|table, *opts| [[:x, {:primary_key=>false}], [:y, {:primary_key=>false}]]}
+    def d.schema(table, *opts) [[:x, {:primary_key=>false}], [:y, {:primary_key=>false}]] end
     @c.primary_key.should == :id
     @c.dataset = ds
     @c.db_schema.should == {:x=>{:primary_key=>false}, :y=>{:primary_key=>false}}
@@ -682,7 +700,7 @@ describe "Model.db_schema" do
   specify "should not modify the primary key unless all column schema hashes have a :primary_key entry" do
     ds = @dataset
     d = ds.db
-    d.meta_def(:schema){|table, *opts| [[:x, {:primary_key=>false}], [:y, {}]]}
+    def d.schema(table, *opts) [[:x, {:primary_key=>false}], [:y, {}]] end
     @c.primary_key.should == :id
     @c.dataset = ds
     @c.db_schema.should == {:x=>{:primary_key=>false}, :y=>{}}
